// -------------------------------------------------------------- -*- C++ -*-
// File: examples/src/color.cpp
// --------------------------------------------------------------------------
// Copyright (C) 1990-2005 by ILOG.
// All Rights Reserved.
//
// Permission is expressly granted to use this example in the
// course of developing applications that use ILOG products.
// --------------------------------------------------------------------------

/* ------------------------------------------------------------

Problem Description
-------------------

A map-coloring problem involves choosing colors for the 
countries on a map in such a way that at most four colors are 
used and no two neighboring countries are the same color.  

For our example, we will consider six countries: Belgium, 
Denmark, France, Germany, Netherlands, and Luxembourg.  The 
colors can be blue, white, red or green.

------------------------------------------------------------ */

#include <ilconcert/ilomodel.h>
#include <ilconcert/iloxmlcontext.h>
#include <ilconcert/ilxml/iloxmltool.h>



ILOSTLBEGIN

void serialize(IloModel model, char* fileName){
	// To write a Model in XML or Read it, you must use the IloXmlContext object.
	//To write a Model, use the writeModel method with a IloModel object and a file name.
	// By default, if the Model contains a predicate extractable or an unknown one, it will create a Xml tag <Unknown name="the-name-of-the-unknown/>
	// You can change the behavior with the setWriteMode method with NoUnknown mode for example.

	IloEnv env = model.getEnv();
	IloModel model2(env);
	IloXmlContext context(env);
	context.setWriteMode(NoUnknown);


	IloBool res = context.writeModel(model, fileName);
	// You can check the writting status with the following:
	if (!res){
		env.out() << "Model has not been written\n";
		env.out() << "The problem comes from the class: " << (char*)context.getWriteError()<< "\n";
	}
	else{
		res = context.readModel(model2, fileName);

		// Checks wether an error occured while reading or not.
		if (!res){ 
			env.out() << "An Error occured.\n";
			env.out() << "The Id of the Father: " <<  context.getParentIdReadError() << "\n";
			env.out() << "The Id of the Tag: " <<  context.getChildIdReadError() << "\n";
			env.out() << "The Father's tag: " <<  context.getParentTagReadError() << "\n";
			env.out() << "The tag: " <<  context.getChildTagReadError() << "\n";
			IloIntArray error_id = context.getIdListReadError();
			IloAnyArray error_tag = context.getTagListReadError();
			if (error_tag.getSize() > 0){
				env.out() << "Here is the list of tags from father to son that throwed an error.";
				for (int i=0; i < error_tag.getSize(); i++){
					env.out() << (char*)error_tag[i] << "\n";
				}
			}
			if (error_id.getSize() > 0){
				env.out() << "Here is the list of ids from father to son that throwed an error.";
				for (int i=0; i < error_id.getSize(); i++){
					env.out() << error_id[i] << "\n";
				}
			}
		} 
	}
	model2.end();
	context.end();
}




const char* Names[] = {"blue", "white", "red", "green"}; 


int main(){
	IloEnv env;
	try {
		IloModel model(env);

		IloIntVar Belgium(env, 0, 3), Denmark(env, 0, 3), 
			France(env, 0, 3), Germany(env, 0, 3), 
			Netherlands(env, 0, 3), Luxembourg(env, 0, 3);


		// Constraints
		model.add(Belgium != France); 
		model.add(Belgium != Germany); 
		model.add(Denmark != Germany ); 
		model.add(France != Germany); 
		model.add(Belgium != Netherlands); 
		model.add(Germany != Netherlands); 
		model.add(France != Luxembourg);  
		model.add(Luxembourg != Germany);
		model.add(Luxembourg != Belgium);

		serialize(model, "color.xml");
	}
	catch (IloException& ex) {
		cout << "Error: " << ex << endl;
	}
	env.end();
	return 0;
}

