// -------------------------------------------------------------- -*- C++ -*-
// File: examples/src/narin.cpp
// --------------------------------------------------------------------------
// Copyright (C) 1990-2005 by ILOG.
// All Rights Reserved.
//
// Permission is expressly granted to use this example in the
// course of developing applications that use ILOG products.
// --------------------------------------------------------------------------

/* ------------------------------------------------------------

Problem Description
-------------------

x and y are two constrained floating-point variables
k is a constrained integer variable

x is in [ -5 , 10^8 ]
y is in [  0 , 10^8 ]
k is in [ -1000 , 1000 ]

Solve the system of equations:

x^3 +10x = y^x - 2^k
kx + 7.7y = 2.4
(k -1)^(y+1) <= 10
((log(y + 2x + 12) <= k + 5) or (y >= k^2)) => (x <= 0 and y <= 1)
(x <= 0) => (k > 3)

------------------------------------------------------------ */

#include <ilconcert/ilomodel.h>
#include <ilconcert/iloxmlcontext.h>
#include <ilconcert/ilxml/iloxmltool.h>



ILOSTLBEGIN

void serialize(IloModel model, const char* fileName){
	// To write a Model in XML or Read it, you must use the IloXmlContext object.
	//To write a Model, use the writeModel method with a IloModel object and a file name.
	// By default, if the Model contains a predicate extractable or an unknown one, it will create a Xml tag <Unknown name="the-name-of-the-unknown/>
	// You can change the behavior with the setWriteMode method with NoUnknown mode for example.

	IloEnv env = model.getEnv();
	IloModel model2(env);
	IloXmlContext context(env);
	context.setWriteMode(NoUnknown);


	IloBool res = context.writeModel(model, fileName);
	// You can check the writting status with the following:
	if (!res){
		env.out() << "Model has not been written\n";
		env.out() << "The problem comes from the class: " << (char*)context.getWriteError()<< "\n";
	}
	else{
		res = context.readModel(model2, fileName);

		// Checks wether an error occured while reading or not.
		if (!res){ 
			env.out() << "An Error occured.\n";
			env.out() << "The Id of the Father: " <<  context.getParentIdReadError() << "\n";
			env.out() << "The Id of the Tag: " <<  context.getChildIdReadError() << "\n";
			env.out() << "The Father's tag: " <<  context.getParentTagReadError() << "\n";
			env.out() << "The tag: " <<  context.getChildTagReadError() << "\n";
			IloIntArray error_id = context.getIdListReadError();
			IloAnyArray error_tag = context.getTagListReadError();
			if (error_tag.getSize() > 0){
				env.out() << "Here is the list of tags from father to son that throwed an error.";
				for (int i=0; i < error_tag.getSize(); i++){
					env.out() << (char*)error_tag[i] << "\n";
				}
			}
			if (error_id.getSize() > 0){
				env.out() << "Here is the list of ids from father to son that throwed an error.";
				for (int i=0; i < error_id.getSize(); i++){
					env.out() << error_id[i] << "\n";
				}
			}
		} 
	}
	model2.end();
	context.end();
}




int main(){    
	IloEnv env;
	try {
		IloModel model(env);

		IloNumVar x(env, -5, 1e8);
		IloNumVar y(env, 0, 1e8);
		IloIntVar k(env, -1000, 1000);

		model.add(IloPower(x,3) + 10*x == IloPower(y, x) - IloPower(2, k));
		model.add(k*x + 7.7*y == 2.4);
		model.add(IloPower(k-1, y+1) <= 10);
		model.add(IloIfThen(env, IloLog(y + 2*x + 12) <= k + 5 || y >= k*k, x <= 0 && y <= 1));
		model.add(IloIfThen(env, x <= 0, k > 3));

		serialize(model, "narin.xml");
	}
	catch (IloException& ex) {
		cout << "Error: " << ex << endl;
	}
	env.end();
	return 0;
}
/*
x = [-1.285057857942743..-1.285057857928474]
y = [0.9792508352918879..0.979250835302613]
k = [4]
*/
