// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __CONCERT_iloadvcollectionH
#define __CONCERT_iloadvcollectionH

#if !defined(__CONCERT_ilocollectioniH)
#include <ilconcert/ilocollectioni.h>
#endif


template <class IloHandle, class IloImpl> class IloHandleSet : public IloAnyCollection {
public:
	virtual ~IloHandleSet(){}
	IloHandleSet(const IloEnv env, const IloAnyArray array, IloBool withIndex=IloFalse)
		: IloAnyCollection(new (env.getImpl()) IloAnySetI(env.getImpl(), array, withIndex)) {}

	IloHandleSet(IloEnvI* env, IloAnySetI* impl)
		: IloAnyCollection(new (env) IloAnySetI(env, impl)) {}

		IloHandleSet(const IloEnv env, IloBool withIndex=IloFalse) : IloAnyCollection(new (env.getImpl()) IloAnySetI(env.getImpl(), withIndex)) {}

		IloHandleSet(IloAnySetI* impl=0) : IloAnyCollection(impl) {}

		IloAnySetI* getImpl() const {
			return (IloAnySetI*)IloAnyCollection::getImpl();
		}

		IloHandleSet copy() const {
			IloTestAndRaise(getImpl() != 0, "IloSet: Using empty handle");
			return IloHandleSet(getImpl()->getEnv(), getImpl());
		}

		IloInt getSize() const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->getSize();
		}

		void add(IloHandle elt){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->add(elt.getImpl());
		}
		void add(IloHandleSet set){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->add((IloAnySetI*)set.getImpl());
		}

		void remove(IloHandle elt) {
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->remove(elt.getImpl());
		}

		void remove(IloHandleSet set){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->remove((IloAnySetI*)set.getImpl());
		}

		void setIntersection(IloHandle elt){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->setIntersection(elt.getImpl());
		}

		void setIntersection(IloHandleSet set){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->setIntersection((IloAnySetI*)set.getImpl());
		}

		void empty(){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->empty();
		}

		IloBool contains(IloHandle elt) const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->contains(elt.getImpl());
		}

		IloBool contains(IloHandleSet set) const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->contains((IloAnySetI*)set.getImpl());
		}

		IloBool intersects(IloHandleSet set) const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->intersects((IloAnySetI*)set.getImpl());
		}

		IloHandle getFirst() const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)_impl)->getFirst());
		}

		IloHandle getLast() const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)_impl)->getLast());
		}

		IloHandle getNext(IloHandle value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)_impl)->getNext(value.getImpl(),n));
		}
		IloHandle getPrevious(IloHandle value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)_impl)->getPrevious(value.getImpl(),n));
		}
		IloHandle getNextC(IloHandle value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)_impl)->getNextC(value.getImpl(),n));
		}
		IloHandle getPreviousC(IloHandle value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)_impl)->getPreviousC(value.getImpl(),n));
		}

		virtual void display(ILOSTD(ostream)& os) const{
			os << "[ ";
			for (IloInt i=0; i< this->getSize(); i++){
				os << IloHandle((IloImpl*)(this->operator[](i).getImpl())) << " ";
			}
			os << "]";
		}

		class Iterator : public IloAnyDefaultDataIterator {
		public:
			Iterator(const IloAnySetI* coll = 0) : IloAnyDefaultDataIterator(coll->getEnv()->getGeneralAllocator(), coll) {
				reset();
			}
			Iterator(const IloHandleSet coll): IloAnyDefaultDataIterator(coll.getImpl()->getEnv()->getGeneralAllocator(),
				coll.getImpl()) {
					reset();
				}

				Iterator(IloGenAlloc* heap, const IloAnySetI* coll): IloAnyDefaultDataIterator(heap, coll) {
					reset();
				}

				virtual ~Iterator(){}

				IloHandle operator*() {
					return IloHandle((IloImpl*)IloAnyDefaultDataIterator::operator*());
				}
		};

		const IloHandle operator[] (IloInt i) const{
			IloAssert(getImpl() != 0, "IloDiscreteDataCollection: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)getImpl())->operator[](i));
		}
		IloHandle operator[] (IloInt i){
			IloAssert(getImpl() != 0, "IloDiscreteDataCollection: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)getImpl())->operator[](i));
		}
		const IloHandle getValue(IloInt i) const{
			IloAssert(getImpl() != 0, "IloDiscreteDataCollection: Using empty handle");
			return IloHandle((IloImpl*)((IloAnySetI*)getImpl())->getValue(i));
		}
};




template <class IloType> class IloSimpleSet : public IloAnyCollection {
public:
	virtual ~IloSimpleSet(){}
	IloSimpleSet(const IloEnv env, const IloAnyArray array, IloBool withIndex=IloFalse)
		: IloAnyCollection(new (env.getImpl()) IloAnySetI(env.getImpl(), array, withIndex)) {}

		IloSimpleSet(const IloEnv env, IloBool withIndex=IloFalse) : IloAnyCollection(new (env.getImpl()) IloAnySetI(env.getImpl(), withIndex)) {}

		IloSimpleSet(IloAnySetI* impl=0) : IloAnyCollection(impl) {}

		IloAnySetI* getImpl() const {
			return (IloAnySetI*)IloAnyCollection::getImpl();
		}

		IloSimpleSet copy() const {
			IloTestAndRaise(getImpl() != 0, "IloSet: Using empty handle");
			return IloSimpleSet(new (getImpl()->getEnv()) IloAnySetI(getImpl()->getEnv(), getImpl()));
		}

		IloInt getSize() const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->getSize();
		}

		void add(IloType* elt){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->add(elt);
		}

		void add(IloSimpleSet set){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->add((IloAnySetI*)set.getImpl());
		}

		void remove(IloType* elt) {
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->remove(elt);
		}

		void remove(IloSimpleSet set){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->remove((IloAnySetI*)set.getImpl());
		}

		void setIntersection(IloType* elt){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->setIntersection(elt);
		}

		void setIntersection(IloSimpleSet set){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->setIntersection((IloAnySetI*)set.getImpl());
		}

		void empty(){
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			getImpl()->empty();
		}

		IloBool contains(IloType* elt) const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->contains(elt);
		}

		IloBool contains(IloSimpleSet set) const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->contains((IloAnySetI*)set.getImpl());
		}

		IloBool intersects(IloSimpleSet set) const{
			IloTestAndRaise(getImpl(), "Using empty IloSet handle.");
			return getImpl()->intersects((IloAnySetI*)set.getImpl());
		}

		IloType getNext(IloType* value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return ((IloAnySetI*)_impl)->getNext(value, n);
		}

		IloType getPrevious(IloType* value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return ((IloAnySetI*)_impl)->getPrevious(value, n);
		}

		IloType getNextC(IloType* value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return ((IloAnySetI*)_impl)->getNextC(value, n);
		}

		IloType getPreviousC(IloType* value, IloInt n=1) const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return ((IloAnySetI*)_impl)->getPreviousC(value, n);
		}

		IloType getFirst() const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return ((IloAnySetI*)_impl)->getFirst();
		}
		IloType getLast() const{
			IloAssert(getImpl() != 0, "IloSet: Using empty handle");
			return ((IloAnySetI*)_impl)->getLast();
		}

		class Iterator : public IloAnyDefaultDataIterator {
		public:
			Iterator() : IloAnyDefaultDataIterator() {}
			Iterator(const IloAnySetI* coll) : IloAnyDefaultDataIterator(coll->getEnv()->getGeneralAllocator(), coll) {
				reset();
			}
			Iterator(const IloSimpleSet coll): IloAnyDefaultDataIterator(coll.getImpl()->getEnv()->getGeneralAllocator(),
				coll.getImpl()) {
					reset();
				}

				Iterator(IloGenAlloc* heap, const IloAnySetI* coll): IloAnyDefaultDataIterator(heap, coll) {
					reset();
				}

				virtual ~Iterator(){}

				IloType operator*() {
					return IloType(IloAnyDefaultDataIterator::operator*());
				}
		};

		const IloType* operator[] (IloInt i) const{
			IloAssert(getImpl() != 0, "IloDiscreteDataCollection: Using empty handle");
			return (IloType*)((IloAnySetI*)getImpl())->operator[](i);
		}
};


#endif

