// -------------------------------------------------------*- C++ -*----------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E  
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloanyexpriH
#define __CONCERT_iloanyexpriH

#if !defined(__CONCERT_iloextractableH)
# include <ilconcert/iloextractable.h>
#endif
#if !defined(__CONCERT_iloexpressionH)
# include <ilconcert/iloexpression.h>
#endif

class IloAnySetI;
//---------------------------------------------------------------------------
// IloEncoder
//---------------------------------------------------------------------------
class IloEncoderI {
public:
  IloEncoderI() {}
  virtual ~IloEncoderI();
  virtual IloInt anyToInt(IloAny e) const=0;
  virtual IloAny intToAny(IloInt e) const=0;
  virtual IloBool isSymbolEncoder() const;
  virtual void display(ILOSTD(ostream)& out, IloAny e) const=0;
};


//---------------------------------------------------------------------------
// IloAnyExpr
//---------------------------------------------------------------------------
class IloAnyExprI : public IloExtractableI {
  ILOEXTRDECL
protected:
  IloInt _exprUsed;
public:
  IloAnyExprI(IloEnvI* env, const char* name=0)
    :IloExtractableI(env, name), _exprUsed(0) {}
  virtual ~IloAnyExprI();
  virtual IloEncoderI* getEncoder() const=0;
  IloAnyExprI* lockExpr() { _exprUsed++;; return this; }
  IloAnyExprI* unlockExpr() {_exprUsed--; return this; }
  virtual void end();
  virtual void release();

  IloInt getExprUsed() const { return _exprUsed; }

  //AFC
  virtual IloAnyExprI* makeImage() const;
};

class IloAnyConstI : public IloAnyExprI {
  ILOEXTRDECL
private:
  IloEncoderI* _encoder;
  IloAny _value;
public:
  IloAnyConstI(IloEnvI* env, IloEncoderI* encoder, IloAny value, const char* name=0); 
  virtual ~IloAnyConstI();
  virtual IloEncoderI* getEncoder() const;
  IloAny getValue() const {return _value;}
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  ILOEXTROTHERDECL
};

//---------------------------------------------------------------------------
class IloAnyEqI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr1;
  IloAnyExprI* _expr2;
public:
  IloAnyEqI(IloEnvI* env, IloAnyExprI* expr1, IloAnyExprI* expr2, const char* name=0);
  virtual ~IloAnyEqI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr1() const { return _expr1; } 
  IloAnyExprI* getExpr2() const { return _expr2; }
  ILOEXTROTHERDECL
};

class IloAnyNeqI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr1;
  IloAnyExprI* _expr2;
public:
  IloAnyNeqI(IloEnvI* env, IloAnyExprI* expr1, IloAnyExprI* expr2, const char* name=0);
  virtual ~IloAnyNeqI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr1() const { return _expr1; } 
  IloAnyExprI* getExpr2() const { return _expr2; }
  ILOEXTROTHERDECL
};

class IloAnyEqCstI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr;
  IloAny _value;
public:
  IloAnyEqCstI(IloEnvI* env, IloAnyExprI* expr, IloAny value, const char* name=0);
  virtual ~IloAnyEqCstI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr() const { return _expr; } 
  IloAny getConstant() const { return _value; }
  ILOEXTROTHERDECL
};

class IloAnyNeqCstI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr;
  IloAny _value;
public:
  IloAnyNeqCstI(IloEnvI* env, IloAnyExprI* expr, IloAny value, const char* name=0);
  virtual ~IloAnyNeqCstI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr() const { return _expr; } 
  IloAny getConstant() const { return _value; }
  ILOEXTROTHERDECL
};

//-------------------------

class IloSymbolLTI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr1;
  IloAnyExprI* _expr2;
public:
  IloSymbolLTI(IloEnvI* env, IloAnyExprI* expr1, IloAnyExprI* expr2, const char* name=0);
  virtual ~IloSymbolLTI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr1() const { return _expr1; } 
  IloAnyExprI* getExpr2() const { return _expr2; }
  ILOEXTROTHERDECL
};

class IloSymbolLTCstI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr;
  IloAny _value;
public:
  IloSymbolLTCstI(IloEnvI* env, IloAnyExprI* expr, IloAny value, const char* name=0);
  virtual ~IloSymbolLTCstI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr() const { return _expr; } 
  IloAny getConstant() const { return _value; }
  ILOEXTROTHERDECL
};

class IloSymbolGTI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr1;
  IloAnyExprI* _expr2;
public:
  IloSymbolGTI(IloEnvI* env, IloAnyExprI* expr1, IloAnyExprI* expr2, const char* name=0);
  virtual ~IloSymbolGTI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr1() const { return _expr1; } 
  IloAnyExprI* getExpr2() const { return _expr2; }
  ILOEXTROTHERDECL
};

class IloSymbolGTCstI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr;
  IloAny _value;
public:
  IloSymbolGTCstI(IloEnvI* env, IloAnyExprI* expr, IloAny value, const char* name=0);
  virtual ~IloSymbolGTCstI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr() const { return _expr; } 
  IloAny getConstant() const { return _value; }
  ILOEXTROTHERDECL
};

class IloSymbolLeI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr1;
  IloAnyExprI* _expr2;
public:
  IloSymbolLeI(IloEnvI* env, IloAnyExprI* expr1, IloAnyExprI* expr2, const char* name=0);
  virtual ~IloSymbolLeI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr1() const { return _expr1; } 
  IloAnyExprI* getExpr2() const { return _expr2; }
  ILOEXTROTHERDECL
};

class IloSymbolLeCstI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr;
  IloAny _value;
public:
  IloSymbolLeCstI(IloEnvI* env, IloAnyExprI* expr, IloAny value, const char* name=0);
  virtual ~IloSymbolLeCstI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr() const { return _expr; } 
  IloAny getConstant() const { return _value; }
  ILOEXTROTHERDECL
};

class IloSymbolGeI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr1;
  IloAnyExprI* _expr2;
public:
  IloSymbolGeI(IloEnvI* env, IloAnyExprI* expr1, IloAnyExprI* expr2, const char* name=0);
  virtual ~IloSymbolGeI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr1() const { return _expr1; } 
  IloAnyExprI* getExpr2() const { return _expr2; }
  ILOEXTROTHERDECL
};

class IloSymbolGeCstI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloAnyExprI* _expr;
  IloAny _value;
public:
  IloSymbolGeCstI(IloEnvI* env, IloAnyExprI* expr, IloAny value, const char* name=0);
  virtual ~IloSymbolGeCstI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloAnyExprI* getExpr() const { return _expr; } 
  IloAny getConstant() const { return _value; }
  ILOEXTROTHERDECL
};

#endif
