// -------------------------------------------------------*- C++ -*----------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilonumcollexpriH
#define __CONCERT_ilonumcollexpriH

#if !defined(__CONCERT_iloforallbaseH)
# include <ilconcert/iloforallbase.h>
#endif

#define IloNumCollectionExprArgI IloNumExprI
#define IloNumCollectionExprI IloNumExprI

class IloTupleIndexI;

class IloNumCollectionExprGeneratorI;

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumCollectionIndexI </p>
**/ 
class IloNumCollectionIndexI : public IloNumExprI {
  ILOEXTRDECL
  void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloNumCollectionIndexI(IloEnvI* env, const char* name=0); 
  virtual ~IloNumCollectionIndexI();
  IloInt getKey() const;
  virtual void errorNotSubstituted() const;
  IloNum eval(const IloAlgorithm alg) const;
  ILOEXTROTHERDECL 
};

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumCollectionTupleCellIndexI </p>
**/
class IloNumCollectionTupleCellExprI : public IloNumCollectionExprI {
	ILOEXTRDECL
private:
	IloTupleExprI* _tuple;
	IloInt _colIndex;
	IloSymbolI* _colName;
	void visitSubExtractables(IloExtractableVisitor* v);
public:
	/* <p> . </p> */
	IloNumCollectionTupleCellExprI(IloEnvI* env, IloTupleExprI* tuple, IloInt colIndex);
	/* <p> . </p> */
	IloNumCollectionTupleCellExprI(IloEnvI* env, IloTupleExprI* tuple, IloSymbolI* colName);
  /* <p> . </p> */
  virtual ~IloNumCollectionTupleCellExprI();
	/* <p> . </p> */
	IloTupleExprI* getTuple() const { return _tuple; }
	/* <p> . </p> */
	IloInt getColumnIndex() const { return _colIndex; }
	/* <p> . </p> */
	IloSymbolI* getColumnName() const { return _colName; }
	/* <p> . </p> */
	IloNum eval(const IloAlgorithm alg) const;

	ILOEXTROTHERDECL
};

/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumCollectionExprGeneratorI </p>
**/
class IloNumCollectionExprGeneratorI : public IloNumGeneratorI {
	ILOEXTRDECL
private:
	IloNumCollectionExprArgI* _coll;
	void visitSubExtractables(IloExtractableVisitor* v);
public:
	/* <p> . </p> */
	IloNumCollectionExprGeneratorI(IloEnvI* env, IloNumIndexI* x, IloNumCollectionExprArgI* expr);
	/* <p> . </p> */
	IloNumCollectionExprArgI* getCollection() const { return _coll; }
	/* <p> . </p> */
	virtual IloBool generatesDuplicates() const;
	ILOEXTROTHERDECL
};


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloEvalNumCollectionExprIIterator </p>
**/
class IloEvalFloatCollectionExprI;
class IloEvalNumCollectionExprIIterator : public IloNumDefaultDataIterator {
	IloEvalFloatCollectionExprI*  _expr;
public:
	/** <p> . </p> **/
	IloEvalNumCollectionExprIIterator(IloGenAlloc* heap, 
		IloEvalFloatCollectionExprI* expr);
	/** <p> . </p> **/
	virtual ~IloEvalNumCollectionExprIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	/* <p> . </p> */
	//virtual IloNum recomputeLB() const;
	/* <p> . </p> */
	//virtual IloNum recomputeUB() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloNum numLB, IloNum numUB, IloBool catchInvalidCollection = IloFalse);
};	


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloIntRangeExprI </p>
**/
class IloNumRangeExprI : public IloNumCollectionExprArgI {
	ILOEXTRDECL
	IloNumExprI* _lb;
	IloNumExprI* _ub;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	/** <p> . </p> */
	IloNumRangeExprI(IloEnvI* env, IloNumExprI* lb, IloNumExprI* ub) : 
	  IloNumCollectionExprArgI(env), _lb(lb->lockExpr()), _ub(ub->lockExpr()) {}
	  virtual ~IloNumRangeExprI();
	  /** <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /** <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /** <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  /** <p> . </p> */
	  IloNumExprI* getLB() const { return _lb; }
	  /** <p> . </p> */
	  IloNumExprI* getUB() const { return _ub; }
};


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumRangeExprIIterator </p>
**/
class IloNumRangeExprIIterator : public IloNumDataIterator {
	IloNumRangeExprI*  _rangeexpr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> */
	IloNumRangeExprIIterator(IloGenAlloc* heap, 
		const IloNumRangeExprI* rangeexpr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> */
	virtual ~IloNumRangeExprIIterator();
	/** <p> . </p> */
	virtual IloBool next();
	/** <p> . </p> */
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	/** <p> . </p> */
	virtual void reset(IloNum min, IloNum max, IloBool catchInvalidCollection = IloFalse);
	/** <p> . </p> */
	virtual IloNum recomputeMin() const;
	/** <p> . </p> */
	virtual IloNum recomputeMax() const;
};	



/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumCollectionTupleCellExprIIterator </p>
**/
class IloNumCollectionTupleCellExprIIterator : public IloNumDefaultDataIterator {
	IloNumCollectionTupleCellExprI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> **/
	IloNumCollectionTupleCellExprIIterator(IloGenAlloc* heap, 
		const IloNumCollectionTupleCellExprI* rangeexpr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> **/
	virtual ~IloNumCollectionTupleCellExprIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	/* <p> . </p> */
	//virtual IloNum recomputeLB() const;
	/* <p> . </p> */
	//virtual IloNum recomputeUB() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloNum numLB, IloNum numUB, IloBool catchInvalidCollection = IloFalse);
};	



/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumCollectionIndexIIterator </p>
**/
class IloNumCollectionIndexIIterator : public IloNumDefaultDataIterator {
	IloNumCollectionIndexI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> **/
	IloNumCollectionIndexIIterator(IloGenAlloc* heap, 
		const IloNumCollectionIndexI* rangeexpr, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> **/
	virtual ~IloNumCollectionIndexIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	/* <p> . </p> */
	//virtual IloNum recomputeLB() const;
	/* <p> . </p> */
	//virtual IloNum recomputeUB() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloNum numLB, IloNum numUB, IloBool catchInvalidCollection = IloFalse);
};	



class IloNumCollectionSubMapExprI;
/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloSymbolCollectionIndexIIterator </p>
**/
class IloNumCollectionSubMapExprIIterator : public IloNumDefaultDataIterator {
	const IloNumCollectionSubMapExprI*  _expr;
	IloModelEvaluatorI* _evaluator;
public:
	/** <p> . </p> **/
	IloNumCollectionSubMapExprIIterator(IloGenAlloc* heap, 
		const IloNumCollectionSubMapExprI* map, 
		const IloModelEvaluatorI* eval);
	/** <p> . </p> **/
	virtual ~IloNumCollectionSubMapExprIIterator();
	/* <p> . </p> */
	//virtual IloBool next();
	/* <p> . </p> */
	//virtual IloNum recomputeLB() const;
	/* <p> . </p> */
	//virtual IloNum recomputeUB() const;
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloNum numLB, IloNum numUB, IloBool catchInvalidCollection = IloFalse);
};	


/**
*  @group optim.concert.collexpr
*  @show internal
*  <p> IloNumAggregateSetExprI </p>
**/
class IloNumAggregateSetExprI : public IloNumCollectionExprArgI {
	ILOEXTRDECL
	IloExtendedComprehensionI* _comp;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	// no need to lock e, this is done in the comprehension.
	IloNumAggregateSetExprI(IloEnvI* env, IloNumExprI* e, IloGeneratorArray g);
	IloNumAggregateSetExprI(IloExtendedComprehensionI* comp);

	virtual ~IloNumAggregateSetExprI();
	
	  IloExtendedComprehensionI* getComprehension() const { return _comp; }

	  /** <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /** <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /** <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  IloNumExprI* getBody() const { return (IloNumExprI*)_comp->getExtent(); }
};
//

#endif
