// -------------------------------------------------------------- -*- C++ -*-
// File: ilconcert/ilodatasource.h
// --------------------------------------------------------------------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

// -*- C++ -*- header file.
#ifndef __CONCERT_ilodatasourceH
#define __CONCERT_ilodatasourceH

#ifndef __CONCERT_ilodatasourceiH
#include <ilconcert/ilsource/ilodatasourcei.h>
#endif



class IloStringDataSource {
private:
	IloStringDataSourceI* _impl;
public:
	IloStringDataSource(IloEnv env);
	IloStringDataSource(): _impl(0) {}

	IloStringDataSourceI* getImpl() const { return _impl; }
	IloStringDataSource(IloStringDataSourceI* impl): _impl(impl) {}

	void end(){
		if (_impl){
			delete _impl;
			_impl = 0;
		}
	}

	IloIntArray getIntArray(const char* str);

	IloNumArray getNumArray(const char* str);
	IloSymbolArray getSymbolArray(const char* str);

	IloIntSet getIntSet(const char* str);

	IloNumSet getNumSet(const char* str);
	IloSymbolSet getSymbolSet(const char* str);

	IloInt getInt(const char* str);
	IloNum getNum(const char* str);
	IloSymbol getSymbol(const char* str);

	IloIntArray2 getIntArray2(const char* str);
	IloNumArray2 getNumArray2(const char* str);

	IloSymbolArray2 getSymbolArray2(const char* str);

	IloIntSetArray getIntSetArray(const char* str);
	IloNumSetArray getNumSetArray(const char* str);

	IloSymbolSetArray getSymbolSetArray(const char* str);
};

class IloCsvDataSource {
private:
	IloCsvDataSourceI* _impl;
public:
	IloCsvDataSource(IloEnv env, const char* fileName);
	IloCsvDataSource(): _impl(0) {}

	IloCsvDataSourceI* getImpl() const { return _impl; }
	IloCsvDataSource(IloCsvDataSourceI* impl): _impl(impl) {}


	void end(){
		if (_impl){
			delete _impl;
			_impl = 0;
		}
	}

	IloIntSet getIntSetFromLine(const char* tableName, IloInt line = 0);
	IloIntSet getIntSetFromColumn(const char* tableName, IloInt column = 1);

	IloNumSet getNumSetFromLine(const char* tableName, IloInt line = 0);
	IloNumSet getNumSetFromColumn(const char* tableName, IloInt column = 1);
	IloSymbolSet getSymbolSetFromLine(const char* tableName, IloInt line = 0);
	IloSymbolSet getSymbolSetFromColumn(const char* tableName, IloInt column = 1);


	IloIntSet getIntSetFromLine(IloInt tableIndex, IloInt line = 0);
	IloIntSet getIntSetFromColumn(IloInt tableIndex, IloInt column = 1);

	IloNumSet getNumSetFromLine(IloInt tableIndex, IloInt line = 0);
	IloNumSet getNumSetFromColumn(IloInt tableIndex, IloInt column = 1);
	IloSymbolSet getSymbolSetFromLine(IloInt tableIndex, IloInt line = 0);
	IloSymbolSet getSymbolSetFromColumn(IloInt tableIndex, IloInt column = 1);


	IloIntArray getIntArrayFromLine(const char* tableName, IloInt line = 0);
	IloIntArray getIntArrayFromColumn(const char* tableName, IloInt column = 1);

	IloNumArray getNumArrayFromLine(const char* tableName, IloInt line = 0);
	IloNumArray getNumArrayFromColumn(const char* tableName, IloInt column = 1);
	IloSymbolArray getSymbolArrayFromLine(const char* tableName, IloInt line = 0);
	IloSymbolArray getSymbolArrayFromColumn(const char* tableName, IloInt column = 1);

	IloIntArray getIntArrayFromLine(IloInt tableIndex, IloInt line = 0);
	IloIntArray getIntArrayFromColumn(IloInt tableIndex, IloInt column = 1);

	IloNumArray getNumArrayFromLine(IloInt tableIndex, IloInt line = 0);
	IloNumArray getNumArrayFromColumn(IloInt tableIndex, IloInt column = 1);
	IloSymbolArray getSymbolArrayFromLine(IloInt tableIndex, IloInt line = 0);
	IloSymbolArray getSymbolArrayFromColumn(IloInt tableIndex, IloInt column = 1);

	////////////////////

	IloInt getIntFromTable(const char* tableName, IloInt x, IloInt y);

	///** <p> Gets an IloAny from a Table of the table </p> **/
	//IloAny getAnyFromTable(const char* tableName, IloInt x, IloInt y);

	IloNum getNumFromTable(const char* tableName, IloInt x, IloInt y);
	IloSymbol getSymbolFromTable(const char* tableName, IloInt x, IloInt y);

	IloInt getIntFromTable(IloInt tableIndex, IloInt x, IloInt y);

	IloNum getNumFromTable(IloInt tableIndex, IloInt x, IloInt y);
	IloSymbol getSymbolFromTable(IloInt tableIndex, IloInt x, IloInt y);
	IloTupleSet getTupleSet(IloTupleSchema schema, const char* tableName);

	IloIntArray2 getIntArray2ByLine(const char* tableName);
	IloNumArray2 getNumArray2ByLine(const char* tableName);

	IloSymbolArray2 getSymbolArray2ByLine(const char* tableName);

	IloIntSetArray getIntSetArrayByLine(const char* tableName);
	IloNumSetArray getNumSetArrayByLine(const char* tableName);

	IloSymbolSetArray getSymbolSetArrayByLine(const char* tableName);

	IloIntArray2 getIntArray2ByColumn(const char* tableName);
	IloNumArray2 getNumArray2ByColumn(const char* tableName);

	IloSymbolArray2 getSymbolArray2ByColumn(const char* tableName);

	IloIntSetArray getIntSetArrayByColumn(const char* tableName);
	IloNumSetArray getNumSetArrayByColumn(const char* tableName);

	IloSymbolSetArray getSymbolSetArrayByColumn(const char* tableName);
};


#ifdef ILO_MSVC
#ifndef ILO_WIN64
/////////////////

class IloExcelDataSource {
private:
	IloExcelDataSourceI* _impl;
public:
	IloExcelDataSource(IloEnv env, const char* fileName, IloBool readBolny = IloFalse);
	IloExcelDataSource(): _impl(0) {}

	IloExcelDataSourceI* getImpl() const { return _impl; }
	IloExcelDataSource(IloExcelDataSourceI* impl): _impl(impl) {}


	void end(){
		if (_impl){
			delete _impl;
			_impl = 0;
		}
	}
	IloIntArray getIntArrayFromLine(const char* range, IloInt line = 0);
	IloIntArray getIntArrayFromColumn(const char* range, IloInt column = 0);

	IloNumArray getNumArrayFromLine(const char* range, IloInt line = 0);
	IloNumArray getNumArrayFromColumn(const char* range, IloInt column = 0);
	IloSymbolArray getSymbolArrayFromLine(const char* range, IloInt line = 0);
	IloSymbolArray getSymbolArrayFromColumn(const char* range, IloInt column = 0);

	IloIntSet getIntSetFromLine(const char* range, IloInt line = 0);
	IloIntSet getIntSetFromColumn(const char* range, IloInt column = 0);

	IloNumSet getNumSetFromLine(const char* range, IloInt line = 0);
	IloNumSet getNumSetFromColumn(const char* range, IloInt column = 0);
	IloSymbolSet getSymbolSetFromLine(const char* range, IloInt line = 0);
	IloSymbolSet getSymbolSetFromColumn(const char* range, IloInt column = 0);

	IloInt getIntFromTable(const char* range, IloInt x, IloInt y);

	IloNum getNumFromTable(const char* range, IloInt x, IloInt y);
	IloSymbol getSymbolFromTable(const char* range, IloInt x, IloInt y);

	IloTupleSet getTupleSet(IloTupleSchema schema, const char* range);

	IloIntArray2 getIntArray2ByLine(const char* range);
	IloNumArray2 getNumArray2ByLine(const char* range);

	IloSymbolArray2 getSymbolArray2ByLine(const char* range);

	IloIntSetArray getIntSetArrayByLine(const char* range);
	IloNumSetArray getNumSetArrayByLine(const char* range);

	IloSymbolSetArray getSymbolSetArrayByLine(const char* range);

	IloIntArray2 getIntArray2ByColumn(const char* range);
	IloNumArray2 getNumArray2ByColumn(const char* range);

	IloSymbolArray2 getSymbolArray2ByColumn(const char* range);

	IloIntSetArray getIntSetArrayByColumn(const char* range);
	IloNumSetArray getNumSetArrayByColumn(const char* range);

	IloSymbolSetArray getSymbolSetArrayByColumn(const char* range);
};

#endif //ILO_WIN64
#endif //ILO_MSVC

#endif


