// --------------------------------------------------*- C++ -*---------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilodiffiH
#define __CONCERT_ilodiffiH

#if !defined(__CONCERT_ilomodelH)
# include <ilconcert/ilomodel.h>
#endif

class IloDiffI : public IloConstraintI {
  ILOEXTRDECL

  IloNumExprI*_expr1;
  IloNumExprI*_expr2;
  IloNum  _cnst;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloDiffI (IloEnvI*  m,
	    IloNumExprI* expr1,
	    IloNumExprI* expr2,
            const char* name);
  IloDiffI (IloEnvI*  m,
	    IloNumExprI* expr1,
            IloNum      val,
            const char* name);

  virtual ~IloDiffI();

  IloNumExprI* getExpr1() const {return _expr1;}
  IloNumExprI* getExpr2() const {return _expr2;}
  IloNum getConst() const { return _cnst;}

  ILOEXTROTHERDECL
};

class IloAllDiffRemoveVar : public IloChange {
  ILOCHANGEDECL

  IloIntExprI* _expr;
  IloAllDiffI* _alldiff;
public:
  IloAllDiffRemoveVar(IloAllDiffI* d, IloIntExprI* expr) 
    : IloChange(), _expr(expr), _alldiff(d) {}
  IloIntExprI* getExpr() const     { return (_expr); }
  IloAllDiffI*  getAllDiff() const { return (_alldiff); }
};

class IloAllDiffI : public IloConstraintI {
  friend class IloAddVarToAllDiff;
  ILOEXTRMEMBERS1DECL(IloAllDiffI, IloConstraintI,
                      IloIntExprArray, _exprArray)
  virtual void visitSubExtractables(IloExtractableVisitor* v);
protected:
  virtual void atRemove(IloExtractableI* user=0, IloAny info = 0);
public:
  void addExpr(IloIntExprI*);
  void removeExpr(IloIntExprI*);
  IloAllDiffI(IloEnvI* env, const char* name);
  virtual ~IloAllDiffI();
  IloIntExprArray getExprArray() const {return _exprArray;}
};

class IloAllMinDistanceI : public IloConstraintI {
  friend class IloAddVarToAllMinDistance;
  ILOEXTRMEMBERS2DECL(IloAllMinDistanceI, IloConstraintI,
                      IloIntExprArray, _exprArray,
                      IloInt, k)
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
   void addExpr(const IloIntExprI*);
   void removeExpr(const IloIntExprI*);
  IloAllMinDistanceI(IloEnvI* env, IloInt k, const char* name);
  virtual ~IloAllMinDistanceI();
  IloIntExprArray getExprArray() const {return _exprArray;}
  IloInt getK() const {return k;}
};

#ifdef ADV_CONCERT_WITH_NEW_CP_API
#if !defined(__CONCERT_ilomapH)
# include <ilconcert/ilomap.h>
#endif
class IloAllDiffIndexSubSetMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloIndexSubSet _subset;
  IloIntExprMap _exprMap;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloAllDiffIndexSubSetMapI(IloEnvI* env,
		       IloIndexSubSet s,
		       IloIntExprMap map,
		       const char* name = 0);
  virtual ~IloAllDiffIndexSubSetMapI();
  IloIndexSubSet getIndexSubSet() const {return _subset;}
  IloIntExprMap getExprMap() const {return _exprMap;}
  ILOEXTROTHERDECL
};

class IloAllMinDistanceIndexSubSetMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloIndexSubSet _subset;
  IloIntExprMap _exprMap;
  IloOplIntExpr _offset;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloAllMinDistanceIndexSubSetMapI(IloEnvI* env,
				   IloIndexSubSet s,
				   IloIntExprMap map,
				   IloOplIntExpr offset,
				   const char* name = 0);
  virtual ~IloAllMinDistanceIndexSubSetMapI();
  IloIndexSubSet getIndexSubSet() const {return _subset;}
  IloIntExprMap getExprMap() const {return _exprMap;}
  IloOplIntExpr getOffset() const {return _offset;}
  ILOEXTROTHERDECL
};
#endif //ifdef ADV_CONCERT_WITH_NEW_CP_API

class IloAddVarToAllMinDistance : public IloAddNumVar {
  ILOCOLUMNDECL
  
  IloAllMinDistanceI* _minDistance;

  const IloExtractableI* what() const {return _minDistance;}
  void destroy();

public:
  virtual ~IloAddVarToAllMinDistance();

  IloAddVarToAllMinDistance(IloAllMinDistanceI* minDistance) 
    : _minDistance(minDistance) {}

  void apply() const;

  IloEnvI*   getEnv() const     { return (_minDistance->getEnv()); }

  IloAllMinDistanceI* getAllMinDistanceI() const  { return (_minDistance); }
};

class IloPackI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloIntExprArray _load;
  IloIntExprArray _where;
  IloIntArray     _weight;
  IloIntExpr      _used;
  IloIntSetVar    _usedSet;

  friend class IloPack; // for access to following ctor
  IloPackI(IloEnvI * env,
           IloIntExprArray load,
           IloIntExprArray where,
           IloIntArray weight,
           IloIntExpr used,
           IloIntSetVar usedSet,
           const char * name);

  IloArray<IloBoolArray> _incompat;
  IloPackI(IloEnvI * env,
           IloIntExprArray load,
           IloIntExprArray where,
           IloIntArray weight,
           IloIntExpr used,
           IloIntSetVar usedSet,
           IloArray<IloBoolArray> incompat,
           const char * name);

public:
  IloPackI(IloEnvI * env,
           IloIntExprArray load,
           IloIntExprArray where,
           IloIntArray weight,
           const char * name);
  IloPackI(IloEnvI * env,
           IloIntExprArray load,
           IloIntExprArray where,
           IloIntArray weight,
           IloIntExpr used,
           const char * name);
  // Experimental
  IloPackI(IloEnvI * env,
           IloIntExprArray load,
           IloIntExprArray where,
           IloIntArray weight,
           IloIntExpr used,
           IloArray<IloBoolArray> incompat,
           const char * name);
  IloPackI(IloEnvI * env,
           IloIntExprArray load,
           IloIntExprArray where,
           IloIntArray weight,
           IloIntSetVar usedSet,
           const char * name);

  virtual ~IloPackI();
  IloIntExprArray getLoadArray() const { return _load; }
  IloIntArray     getWeightArray() const { return _weight; }
  IloIntExprArray getWhereArray() const { return _where; }
  IloIntExpr      getUsed() const { return _used; }
  IloIntSetVar    getUsedSet() const { return _usedSet; }
  IloArray<IloBoolArray> getIncompat() const { return _incompat; }

  IloExtractableI* makeClone(IloEnvI *) const;
  void display(ILOSTD(ostream)& out) const;

  virtual void visitSubExtractables(IloExtractableVisitor* v);
};

#endif
