// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloforallH
#define __CONCERT_iloforallH

#if !defined(__CONCERT_ilosetH)
# include <ilconcert/iloset.h>
#endif
#if !defined(__CONCERT_ilosymbolH)
# include <ilconcert/ilosymbol.h>
#endif
#if !defined(__CONCERT_ilotupleH)
# include <ilconcert/ilotuple.h>
#endif
#if !defined(__CONCERT_ilocollectionH)
# include <ilconcert/ilocollection.h>
#endif
#if !defined(__CONCERT_ilomemberH)
# include <ilconcert/ilomember.h>
#endif
#if !defined(__CONCERT_iloordH)
# include <ilconcert/iloord.h>
#endif
#if !defined(__CONCERT_iloforalliH)
# include <ilconcert/iloforalli.h>
#endif

class IloIntGenerator;
class IloNumGenerator;
class IloSymbolGenerator;
class IloTupleGenerator;
class IloTuplePattern;

/////////////////////////////////////////////////////////////////////////////
//   Formal Indexs
/////////////////////////////////////////////////////////////////////////////
//----------------------------------------------
class ILO_EXPORTED IloIntIndex : public IloIntExprArg {
	ILOEXTRHANDLE(IloIntIndex, IloIntExprArg)
public:
	IloIntIndex(IloEnv env, const char* name=0);
	IloIntGenerator within(IloIntCollection coll) const;
	IloIntGenerator within(IloIntSetVar var) const;

	IloIntGenerator within(IloIntCollectionExprArg expr) const;
	IloIntGenerator within(IloIntExprArg lb, IloIntExprArg ub) const;
	IloIntGenerator within(IloInt lb, IloIntExprArg ub) const;
	IloIntGenerator within(IloIntExprArg lb, IloInt ub) const;

	IloIntGenerator within(IloInt min, IloInt max) const;

	class NotSubstituted : public IloException {
	private:
		const IloIntIndexI* _idx;
	public:
		NotSubstituted(const IloIntIndexI* idx);
		NotSubstituted(const IloIntIndex idx);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};
};

//----------------------------------------------
class ILO_EXPORTED IloNumIndex : public IloNumExprArg {
	ILOEXTRHANDLE(IloNumIndex, IloNumExprArg)
public:
	IloNumIndex(IloEnv env, const char* name=0);
	IloNumGenerator within(IloNumCollection coll) const;

	IloNumGenerator within(IloNumCollectionExprArg expr) const;

	class NotSubstituted : public IloException {
	private:
		const IloNumIndexI* _idx;
	public:
		NotSubstituted(const IloNumIndexI* idx);
		NotSubstituted(const IloNumIndex idx);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};
};

//----------------------------------------------
class ILO_EXPORTED IloSymbolIndex : public IloSymbolExprArg {
	ILOEXTRHANDLE(IloSymbolIndex, IloSymbolExprArg)
public:
	IloSymbolIndex(IloEnv env, const char* name=0);
	IloSymbolGenerator within(IloAnyCollection coll) const;

	IloSymbolGenerator within(IloSymbolCollectionExprArg expr) const;

	class NotSubstituted : public IloException {
	private:
		const IloSymbolIndexI* _idx;
	public:
		NotSubstituted(const IloSymbolIndexI* idx);
		NotSubstituted(const IloSymbolIndex idx);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};
};

//----------------------------------------------
class ILO_EXPORTED IloTupleIndex : public IloTupleExprArg {
	ILOEXTRHANDLE(IloTupleIndex, IloTupleExprArg)
public:
	IloTupleIndex(IloEnv env, const char* name=0);
	IloTupleGenerator within(IloTupleSet coll) const;

	IloTupleGenerator within(IloTupleSetExprArg expr) const;

	class NotSubstituted : public IloException {
	private:
		const IloTupleIndexI* _idx;
	public:
		NotSubstituted(const IloTupleIndexI* idx);
		NotSubstituted(const IloTupleIndex idx);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};
};
//
//----------------------------------------------
class ILO_EXPORTED IloTuplePattern : public IloExtractable {
	ILOEXTRHANDLE(IloTuplePattern, IloExtractable)
public:
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3, IloTuplePatternItem a4);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3, IloTuplePatternItem a4,
		IloTuplePatternItem a5);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3, IloTuplePatternItem a4,
		IloTuplePatternItem a5, IloTuplePatternItem a6);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3, IloTuplePatternItem a4,
		IloTuplePatternItem a5, IloTuplePatternItem a6,
		IloTuplePatternItem a7);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3, IloTuplePatternItem a4,
		IloTuplePatternItem a5, IloTuplePatternItem a6,
		IloTuplePatternItem a7, IloTuplePatternItem a8);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3, IloTuplePatternItem a4,
		IloTuplePatternItem a5, IloTuplePatternItem a6,
		IloTuplePatternItem a7, IloTuplePatternItem a8,
		IloTuplePatternItem a9);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItem a1, IloTuplePatternItem a2,
		IloTuplePatternItem a3, IloTuplePatternItem a4,
		IloTuplePatternItem a5, IloTuplePatternItem a6,
		IloTuplePatternItem a7, IloTuplePatternItem a8,
		IloTuplePatternItem a9, IloTuplePatternItem a10);
	IloTuplePattern(IloEnv env,
		IloTuplePatternItemArray args);
	IloTupleGenerator within(IloTupleSet coll) const;
	IloTupleGenerator within(IloTupleSetExprArg coll) const;

public:
	//-------------------------------------------
	class NoArityMatching : public IloException {
	private:
		const IloTuplePatternI* _pattern;
		IloTupleSchema _schema;
	public:
		NoArityMatching(const IloTuplePatternI* p, IloTupleSchema s);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};

	class NoColumnMatching : public IloException {
	private:
		const IloTuplePatternI* _pattern;
		const IloTuplePatternItem* _item;
		const IloColumnDefinitionI* _column;
		IloInt _pos;
	public:
		NoColumnMatching(const IloTuplePatternI*, const IloTuplePatternItem* item, const IloColumnDefinitionI*, IloInt);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};

	class InvalidPatternItem : public IloException {
	private:
		const IloTuplePatternI* _pattern;
		const IloTuplePatternItem* _item;
	public:
		InvalidPatternItem(const IloTuplePatternI*, const IloTuplePatternItem*);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};

	class SchemaGeneration : public IloException {
	private:
		const IloTuplePatternI* _pattern;
		const IloTuplePatternItem* _item;
	public:
		SchemaGeneration(const IloTuplePatternI*, const IloTuplePatternItem*);
		virtual const char* getMessage() const;
		void print(ILOSTD(ostream)& out) const;
	};
};

//inline IloIndex::IloIndex(IloTuplePattern x):_impl(x.getImpl()) {}
//
///////////////////////////////////////////////////////////////////////////////
//// Generators
///////////////////////////////////////////////////////////////////////////////


//---------------------------------------------
class ILO_EXPORTED IloIntGenerator : public IloGenerator {
public:
	IloIntGenerator():IloGenerator() {}
	IloIntGenerator(IloIntGeneratorI* impl):IloGenerator(impl) {}
	IloIntGeneratorI* getImpl() const {
		return (IloIntGeneratorI*)IloGenerator::getImpl();
	}
	IloIntIndex getIndex() const {
		IloAssert(getImpl(), "Using empty IloIntGenerator handle.");
		return (getImpl()->getIndex());
	}
};

//---------------------------------------------
class ILO_EXPORTED IloNumGenerator : public IloGenerator {
public:
	IloNumGenerator():IloGenerator() {}
	IloNumGenerator(IloNumGeneratorI* impl):IloGenerator(impl) {}
	IloNumGeneratorI* getImpl() const {
		return (IloNumGeneratorI*)IloGenerator::getImpl();
	}
	IloNumIndex getIndex() const {
		IloAssert(getImpl(), "Using empty IloNumGenerator handle.");
		return (getImpl()->getIndex());
	}
};

//---------------------------------------------
class ILO_EXPORTED IloSymbolGenerator : public IloGenerator {
public:
	IloSymbolGenerator():IloGenerator() {}
	IloSymbolGenerator(IloSymbolGeneratorI* impl):IloGenerator(impl) {}
	IloSymbolGeneratorI* getImpl() const {
		return (IloSymbolGeneratorI*)IloGenerator::getImpl();
	}
	IloSymbolIndex getIndex() const {
		IloAssert(getImpl(), "Using empty IloSymbolGenerator handle.");
		return (IloSymbolIndexI*)getImpl()->getIndex();
	}
};

//---------------------------------------------
class ILO_EXPORTED IloTupleGenerator : public IloGenerator {
public:
	IloTupleGenerator():IloGenerator() {}
	IloTupleGenerator(IloTupleGeneratorI* impl):IloGenerator(impl) {}
	IloTupleGeneratorI* getImpl() const {
		return (IloTupleGeneratorI*)IloGenerator::getImpl();
	}
	IloTupleIndex getIndex() const {
		IloAssert(getImpl(), "Using empty IloTupleGenerator handle.");
		return (IloTupleIndexI*)getImpl()->getIndex();
	}
};

/////////////////////////////////////////////////////////////////////////////
inline IloIntGenerator IloIntIndex::within(IloIntCollection coll) const {
	IloAssert(getImpl(), "Using empty IloIntIndex handle.");
	IloAssert(coll.getImpl(), "Using empty IloIntCollection handle.");
	return (getImpl()->within((IloIntCollectionI*)coll.getImpl()));
}
inline IloIntGenerator IloIntIndex::within(IloIntSetVar var) const {
	IloAssert(getImpl(), "Using empty IloIntIndex handle.");
	IloAssert(var.getImpl(), "Using empty IloIntSetVar handle.");
	return (getImpl()->within(var.getImpl()));
}
inline IloIntGenerator IloIntIndex::within(IloInt min, IloInt max) const {
	IloAssert(getImpl(), "Using empty IloIntIndex handle.");
	return (getImpl()->within(IloIntRange(getImpl()->getEnv(), min, max).getImpl()));
}

/////////////////////////////////////////////////////////////////////////////
inline IloNumGenerator IloNumIndex::within(IloNumCollection coll) const {
	IloAssert(getImpl(), "Using empty IloIntIndex handle.");
	IloAssert(coll.getImpl(), "Using empty IloNumCollection handle.");
	return (getImpl()->within((IloNumCollectionI*)coll.getImpl()));
}

/////////////////////////////////////////////////////////////////////////////
inline IloSymbolGenerator IloSymbolIndex::within(IloAnyCollection coll) const {
	IloAssert(getImpl(), "Using empty IloIntIndex handle.");
	IloAssert(coll.getImpl(), "Using empty IloAnyCollection handle.");
	return (IloSymbolGeneratorI*)getImpl()->within((IloAnyCollectionI*)coll.getImpl());
}

/////////////////////////////////////////////////////////////////////////////
inline IloTupleGenerator IloTupleIndex::within(IloTupleSet coll) const {
	IloAssert(getImpl(), "Using empty IloIntIndex handle.");
	IloAssert(coll.getImpl(), "Using empty IloTupleSet handle.");
	return getImpl()->within((IloTupleSetI*)coll.getImpl());
}
inline IloTupleGenerator IloTupleIndex::within(IloTupleSetExprArg coll) const {
	IloAssert(getImpl(), "Using empty IloIntIndex handle.");
	IloAssert(coll.getImpl(), "Using empty IloTupleSet handle.");
	return getImpl()->within((IloTupleSetExprArgI*)coll.getImpl());
}

/////////////////////////////////////////////////////////////////////////////
inline IloTupleGenerator IloTuplePattern::within(IloTupleSet coll) const {
	IloAssert(getImpl(), "Using empty IloTuplePattern handle.");
	IloAssert(coll.getImpl(), "Using empty IloTupleSet handle.");
	return getImpl()->copy()->within(coll.getImpl());
}
inline IloTupleGenerator IloTuplePattern::within(IloTupleSetExprArg coll) const {
	IloAssert(getImpl(), "Using empty IloTuplePattern handle.");
	IloAssert(coll.getImpl(), "Using empty IloTupleSet handle.");
	return getImpl()->copy()->within(coll.getImpl());
}


/////////////////////////////////////////////////////////////////////////////
// For-all
/////////////////////////////////////////////////////////////////////////////

ILO_EXPORTEDFUNCTION(IloConstraint) IloForAll(IloGenerator g,
											  IloConstraint ct,
											  const char* name=0);
ILO_EXPORTEDFUNCTION(IloConstraint) IloForAll(IloGenerator g1, IloGenerator g2,
											  IloConstraint ct,
											  const char* name=0);
ILO_EXPORTEDFUNCTION(IloConstraint) IloForAll(IloGenerator g1, IloGenerator g2, IloGenerator g3,
											  IloConstraint ct,
											  const char* name=0);

ILO_EXPORTEDFUNCTION(IloConstraint) IloForAll(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
											  IloConstraint ct,
											  const char* name=0);

ILO_EXPORTEDFUNCTION(IloConstraint) IloForAll(IloGeneratorArray gs,
											  IloConstraint ct,
											  const char* name=0);


/////////////////////////////////////////////////////////////////////////////
// Aggregation operators
/////////////////////////////////////////////////////////////////////////////
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloSum(IloGenerator g1,
										   IloIntExprArg e);
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloSum(IloGenerator g1, IloGenerator g2,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloSum(IloGenerator g1, IloGenerator g2, IloGenerator g3,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloSum(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloSum(IloGeneratorArray g,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloProd(IloGenerator g1,
											IloIntExprArg e);
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloProd(IloGenerator g1, IloGenerator g2,
											IloIntExprArg e);
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloProd(IloGenerator g1, IloGenerator g2, IloGenerator g3,
											IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloProd(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
											IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloProd(IloGeneratorArray g,
											IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMin(IloGenerator g1,
										   IloIntExprArg e);
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMin(IloGenerator g1, IloGenerator g2,
										   IloIntExprArg e);
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMin(IloGenerator g1, IloGenerator g2, IloGenerator g3,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMin(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMin(IloGeneratorArray g,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMax(IloGenerator g1,
										   IloIntExprArg e);
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMax(IloGenerator g1, IloGenerator g2,
										   IloIntExprArg e);
ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMax(IloGenerator g1, IloGenerator g2, IloGenerator g3,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMax(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloIntExprArg) IloMax(IloGeneratorArray g,
										   IloIntExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloSum(IloGenerator g1,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloSum(IloGenerator g1, IloGenerator g2,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloSum(IloGenerator g1, IloGenerator g2, IloGenerator g3,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloSum(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
										   IloNumExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloSum(IloGeneratorArray g,
										   IloNumExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloProd(IloGenerator g1,
											IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloProd(IloGenerator g1, IloGenerator g2,
											IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloProd(IloGenerator g1, IloGenerator g2, IloGenerator g3,
											IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloProd(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
											IloNumExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloProd(IloGeneratorArray g,
											IloNumExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMin(IloGenerator g1,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMin(IloGenerator g1, IloGenerator g2,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMin(IloGenerator g1, IloGenerator g2, IloGenerator g3,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMin(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
										   IloNumExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMin(IloGeneratorArray g,
										   IloNumExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMax(IloGenerator g1,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMax(IloGenerator g1, IloGenerator g2,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMax(IloGenerator g1, IloGenerator g2, IloGenerator g3,
										   IloNumExprArg e);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMax(IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4,
										   IloNumExprArg e);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloMax(IloGeneratorArray g,
										   IloNumExprArg e);

/////////////////////////////////////////////////////////////////////////////
// Aggregate Piecewise linear function
/////////////////////////////////////////////////////////////////////////////
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   const IloGenerator g1,
													   const IloNumExprArg point,
													   const IloNumExprArg slope,
													   const IloNumExprArg lastSlope,
													   const IloNumExprArg a,
													   const IloNumExprArg fa);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   IloGenerator g1, IloGenerator g2,
													   const IloNumExprArg point,
													   const IloNumExprArg slope,
													   const IloNumExprArg lastSlope,
													   const IloNumExprArg a,
													   const IloNumExprArg fa);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   IloGenerator g1, IloGenerator g2,
													   IloGenerator g3,
													   const IloNumExprArg point,
													   const IloNumExprArg slope,
													   const IloNumExprArg lastSlope,
													   const IloNumExprArg a,
													   const IloNumExprArg fa);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   IloGenerator g1, IloGenerator g2,
													   IloGenerator g3, IloGenerator g4,
													   const IloNumExprArg point,
													   const IloNumExprArg slope,
													   const IloNumExprArg lastSlope,
													   const IloNumExprArg a,
													   const IloNumExprArg fa);
ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   IloGeneratorArray generators,
													   const IloNumExprArg point,
													   const IloNumExprArg slope,
													   const IloNumExprArg lastSlope,
													   const IloNumExprArg a,
													   const IloNumExprArg fa);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   const IloNumExprArg firstSlope,
													   IloGenerator g1,
													   const IloNumExprArg x,
													   const IloNumExprArg y,
													   const IloNumExprArg lastSlope);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   const IloNumExprArg firstSlope,
													   IloGenerator g1, IloGenerator g2,
													   const IloNumExprArg x,
													   const IloNumExprArg y,
													   const IloNumExprArg lastSlope);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
													   const IloNumExprArg firstSlope,
													   IloGenerator g1, IloGenerator g2,
													   IloGenerator g3,
													   const IloNumExprArg x,
													   const IloNumExprArg y,
													   const IloNumExprArg lastSlope);


ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
                                const IloNumExprArg firstSlope,
                                IloGenerator g1, IloGenerator g2,
                                IloGenerator g3, IloGenerator g4,
                                const IloNumExprArg x,
                                const IloNumExprArg y,
                                const IloNumExprArg lastSlope);


ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg node,
                                const IloNumExprArg firstSlope,
                                IloGeneratorArray generators,
                                const IloNumExprArg x,
                                const IloNumExprArg y,
                                const IloNumExprArg lastSlope);

ILO_EXPORTEDFUNCTION(IloNumExprArg) IloPiecewiseLinear(const IloNumExprArg& nodeExpr,
                                 const IloNumExprArray& breakPointExprs,
                                 const IloNumExprArray& slopeExprs,
                                 const IloNumExprArg& aExpr,
                                 const IloNumExprArg& faExpr);

#endif
