// --------------------------------------------------------------------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloiteratorH
#define __CONCERT_iloiteratorH

#if !defined(__CONCERT_iloextractableH)
#include <ilconcert/iloextractable.h>
#endif

class IloBaseIterator {
  private:
    IloEnvI*                     _env;
    class IloTypedList*          _list;
    class IloTypedList*          _current;

    IloBool                      _withSubClasses;
  public:
    IloBaseIterator(const IloEnv env, IloBool withSubClasses = IloTrue)
      : _env(env.getImpl())
      , _list(0)
      , _current(0)
      , _withSubClasses(withSubClasses)
    {}
    void goToNextNonEmptyList(IloLinkedList<IloExtractableI>::Iterator& _iter);
    void init(IloRtti::TypeInfo typeInfo, IloLinkedList<IloExtractableI>::Iterator &_iter);
};

template <class E>
class IloIterator : public IloBaseIterator {
private:
    IloExtractableList::Iterator _iter;

#if defined(ILO_HP_aCC) || defined(ILO_LINUX)
  typedef typename E::ImplClass EImplClass;
#endif

  void init() {
#if defined(ILO_HP_aCC) || defined(ILO_LINUX)
    IloBaseIterator::init(EImplClass::GetTypeInfo(), _iter);
#else
    IloBaseIterator::init(E::ImplClass::GetTypeInfo(), _iter);
#endif
  }

public:
  IloIterator(const IloEnv env, IloBool withSubClasses = IloTrue)
    : IloBaseIterator(env,withSubClasses),
      _iter(IloExtractableList())
  {
    init();
  }

  IloBool ok() { return _iter.ok(); }
  void operator++() {
    ++_iter;
    if (!_iter.ok()) { IloBaseIterator::goToNextNonEmptyList(_iter); }
  }

  E operator*() {
#if defined(ILO_HP_aCC) || defined(ILO_LINUX)
    EImplClass* ext = (EImplClass*)*_iter;
    IloAssert(ext->isType(EImplClass::GetTypeInfo()),
              "Bad extractable type in IloKindOfIterator::operator*");
    return (EImplClass*)ext;
#elif defined(ILO_RS6000)
    typename E::ImplClass *ext = (typename E::ImplClass*)*_iter;
    IloAssert(ext->isType(E::ImplClass::GetTypeInfo()),
              "Bad extractable type in IloKindOfIterator::operator*");
    return (typename E::ImplClass*)ext;
#else
    E::ImplClass* ext = (E::ImplClass*)*_iter;
    IloAssert(ext->isType(E::ImplClass::GetTypeInfo()),
              "Bad extractable type in IloKindOfIterator::operator*");
    return (E::ImplClass*)ext;
#endif
  }
};

#endif

