// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilolinearH
#define __CONCERT_ilolinearH

#if !defined(__CONCERT_iloexpressionH)
#include <ilconcert/iloexpression.h>
#endif

////////////////////////////////////////////////////////////////////////
//
//  IloRangeI
//
////////////////////////////////////////////////////////////////////////
class IloIntSet;
class IloNumSet;
class IloIntSetI;
class IloNumSetI;

class IloNumLinTermI;
class IloIntLinTermI;

class IloRangeI : public IloConstraintI {
  ILOEXTRDECL
  friend class IloRangeArray;
  friend class IloRange;
  IloNumLinTermI* _expr;
  IloNum  _lhs, _rhs;

  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloRangeI(IloEnvI*    env,
            IloNum      lhs,
            IloNum      rhs,
            const char* name=0);
  IloRangeI(IloEnvI* env,
            IloNum lhs,
	    IloNumExprI*  expr,
            IloNum rhs=IloInfinity,
            const char* name=0);
  virtual ~IloRangeI ();

  ILOEXTROTHERDECL

  IloNumLinTermI* getExpr() const { return _expr; }

  IloNum getLb () const                  { return (_lhs); }
  IloNum getUb () const                  { return (_rhs); }

  void addVar(IloNumVarI* var, IloNum val);

  IloInt normalize();

  void setCoef(const IloNumVar, IloNum);
  inline void setBounds(IloNum, IloNum);
  inline void setUb(IloNum);
  inline void setLb(IloNum);
  void setExpr(const IloNumExprArg);
  void setCoef(const IloNumVarArray, const IloNumArray);

  class LinearIterator {
    IloNumLinExprTermI* _iter;
  public:
    LinearIterator (const IloNumLinTermI* expr);
    LinearIterator (const IloIntLinTermI* expr=0);
    IloBool           ok () const;
    void              operator++ ();
    const IloNumVarI* getVar     () const;
    IloNum            getVal     () const;
  };
};

////////////////////////////////////////////////////////////////////////
//
//  IloAddValueToRange
//
////////////////////////////////////////////////////////////////////////

class IloAddValueToRange : public IloAddNumVar {
  ILOCOLUMNDECL

  IloRangeI* _rng;
  IloNum     _value;

  friend class IloNumColumnI;

  const IloExtractableI* what() const { return _rng; }

public:
  IloAddValueToRange (IloRangeI* rng, IloNum val)
    : _rng(rng), _value(val) {}

  const IloAddValueToRange* getNext() const {
    return ((const IloAddValueToRange*)IloAddNumVar::getNext());
  }
  IloNum       getValue() const  { return (_value); }
  IloRangeI*   getRange() const  { return (_rng); }

  friend inline const IloAddValueToRange& operator+ (const IloAddValueToRange& one,
                                                     const IloAddValueToRange& two);
  void apply() const;
};


inline const IloAddValueToRange& operator+ (const IloAddValueToRange& one,
                                            const IloAddValueToRange& two) {
  return ((const IloAddValueToRange&)(((const IloAddNumVar&)one) +
                                      ((const IloAddNumVar&)two)  ));
}

////////////////////////////////////////////////////////////////////////
//
//  IloRange
//
////////////////////////////////////////////////////////////////////////

class IloRangeBase;

class ILO_EXPORTED IloRange : public IloConstraint {
  ILOEXTRHANDLE(IloRange,IloConstraint)
public:
  IloRange(const IloEnv env,
           IloNum lb,
           IloNum ub,
           const char* name=0)
  : IloConstraint (new (env.getImpl()) IloRangeI(env.getImpl(),lb,ub,name)) {}


  IloRange(const IloEnv env,
           IloNum lhs,
           const IloNumExprArg expr,
           IloNum rhs = IloInfinity,
           const char* name = 0)
  : IloConstraint (new (env.getImpl()) IloRangeI(env.getImpl(), lhs, expr.getImpl(), rhs, name)) {
    IloAssert (expr.getImpl(), "Using empty IloNumExpr handle.");
  }


  IloRange(const IloEnv env,
           const IloNumExprArg expr,
           IloNum rhs = IloInfinity,
           const char* name = 0)
  : IloConstraint (new (env.getImpl()) IloRangeI(env.getImpl(), -IloInfinity, expr.getImpl(), rhs, name)) {
    IloAssert (expr.getImpl(), "Using empty IloNumExpr handle.");
  }

#if defined(ILOBADRANGE)
  IloRange(const IloRangeBase b);
#endif

  IloNum getLb() const {
     IloAssert (getImpl(), "Using empty IloRange handle.");
     return (getImpl()->getLb());
  }
  IloNum getLB() const { return getLb(); }
  IloNum getUb() const {
     IloAssert (getImpl(), "Using empty IloRange handle.");
     return (getImpl()->getUb());
  }
  IloNum getUB() const { return getUb(); }
  IloNumExprArg getExpr() const {
    IloAssert(getImpl(), "IloRange: using empty handle");
    return (IloNumExprI*)(void*)(getImpl()->getExpr());
  }
  IloAddValueToRange operator() () const {
    IloAssert(getImpl(), "IloRange: using empty handle");
    return IloAddValueToRange(getImpl(), 1.0);
  }
  IloAddValueToRange operator() (IloNum value) const {
    IloAssert(getImpl(), "IloRange: using empty handle");
    return IloAddValueToRange(getImpl(), value);
  }

  IloInt normalize() const {
    IloAssert(getImpl(), "Using empty IloRange handle");
    return getImpl()->normalize();
  }

  // Change methods use notification
  void setCoef(const IloNumVar var, IloNum coef) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setCoef(var, coef);
  }
  void setLinearCoef(const IloNumVar var, IloNum value) {
    setCoef(var, value);
  }
  void setCoef(const IloNumVarArray vars,
               const IloNumArray coefs) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    IloAssert(vars.getImpl(), "Using empty IloNumVarArray handle.");
    IloAssert(coefs.getImpl(), "Using empty IloNumArray handle.");
    getImpl()->setCoef(vars, coefs);
  }
  void setLinearCoefs(const IloNumVarArray vars, const IloNumArray values) {
    setCoef(vars, values);
  }
  void setExpr(const IloNumExprArg expr) {
    IloAssert(getImpl(), "Trying changing IloRange through 0 handle");
    getImpl()->setExpr(expr);
  }
  void setBounds(IloNum lb, IloNum ub) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setBounds(lb, ub);
  }
  void setUb(IloNum ub) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setUb(ub);
  }
  void setUB(IloNum ub) { setUb(ub); }
  void setLb(IloNum lb) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setLb(lb);
  }
  void setLB(IloNum lb) { setLb(lb); }
};

////////////////////////////////////////////////////////////////////////
//
//  IloRangeArray
//
////////////////////////////////////////////////////////////////////////

class ILO_EXPORTED IloRangeArray : public IloConstraintArray {
public:
  typedef IloDefaultArrayI ImplClass;
  IloRangeArray(IloDefaultArrayI* i=0) : IloConstraintArray(i) {}
  IloRangeArray(const IloEnv env, IloInt n = 0)
    : IloConstraintArray(env, n)     {}
  IloRangeArray(const IloEnv env, IloInt n, const IloRange v0);
#ifdef ILOALLOWVARARGS
#ifdef ILOVARARGS
  IloRangeArray(const IloEnv env, IloInt n, const IloRange v0, IloVarArg* v1 ...);
#else
  IloRangeArray(const IloEnv env, IloInt n, const IloRange v0, const IloRange v1 ...);
#endif
#else // ILOALLOWVARARGS
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2,
	  const IloRange v3);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2,
	  const IloRange v3, const IloRange v4);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2,
	  const IloRange v3, const IloRange v4, const IloRange v5);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2,
	  const IloRange v3, const IloRange v4, const IloRange v5,
	  const IloRange v6);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2,
	  const IloRange v3, const IloRange v4, const IloRange v5,
	  const IloRange v6, const IloRange v7);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2,
	  const IloRange v3, const IloRange v4, const IloRange v5,
	  const IloRange v6, const IloRange v7, const IloRange v8);
  IloRangeArray(const IloEnv env, IloInt n,
	  const IloRange v0, const IloRange v1, const IloRange v2,
	  const IloRange v3, const IloRange v4, const IloRange v5,
	  const IloRange v6, const IloRange v7, const IloRange v8,
	  const IloRange v9);
#endif

  IloRangeArray(const IloEnv env, IloInt n, IloNum lb, IloNum ub);

  IloRangeArray(const IloEnv       env,
                const IloNumArray  lbs,
                const IloNumExprArray rows,
                const IloNumArray  ubs);
  IloRangeArray(const IloEnv       env,
                IloNum             lb,
                const IloNumExprArray rows,
                const IloNumArray  ubs);
  IloRangeArray(const IloEnv       env,
                const IloNumArray  lbs,
                const IloNumExprArray rows,
                IloNum             ub);
  IloRangeArray(const IloEnv        env,
                IloNum              lb,
                const IloNumExprArray  rows,
                IloNum              ub);

  IloRangeArray(const IloEnv       env,
                const IloIntArray  lbs,
                const IloNumExprArray rows,
                const IloIntArray  ubs);
  IloRangeArray(const IloEnv       env,
                IloNum             lb,
                const IloNumExprArray rows,
                const IloIntArray  ubs);
  IloRangeArray(const IloEnv       env,
                const IloIntArray  lbs,
                const IloNumExprArray rows,
                IloNum             ub);

  IloRangeArray(const IloEnv      env,
                const IloNumArray lbs,
                const IloNumArray ubs);

  IloRangeArray(const IloEnv      env,
                const IloIntArray lbs,
                const IloIntArray ubs);

  IloRangeArray(const IloEnv      env,
                IloNum            lb,
                const IloNumArray ubs);
  IloRangeArray(const IloEnv      env,
                const IloNumArray lbs,
                IloNum            ub);
  IloRangeArray(const IloEnv      env,
                IloNum            lb,
                const IloIntArray ubs);
  IloRangeArray(const IloEnv      env,
                const IloIntArray lbs,
                IloNum            ub);

  void add(const IloRangeArray array)         { IloExtractableArray::add (array); }
  void add(const IloRange range)              { IloExtractableArray::add (range); }
  void add(IloInt more, const IloRange range) { IloExtractableArray::add (more, range); }

  IloRange& operator[] (IloInt i) {
    return (*(IloRange*)&(IloExtractableArray::operator[](i)));
  }
  IloRange  operator[] (IloInt i) const {
    return ((IloRangeI*)((IloExtractableArray::operator[](i)).getImpl()));
  }

  IloNumColumn operator() (const IloIntArray vals);
  IloNumColumn operator() (const IloNumArray vals);

  // Change methods use notification
  void setBounds(const IloNumArray lbs, const IloNumArray ubs);
  void setBounds(const IloIntArray lbs, const IloIntArray ubs);
};

typedef IloDefaultArrayI IloRangeArrayI;

////////////////////////////////////////////////////////////////////////
//
//  Changes for IloRange
//
////////////////////////////////////////////////////////////////////////

class IloSetRangeBounds : public IloChange {
  ILOCHANGEDECL

  IloRangeI* _rng;
  IloNum   _lb, _ub;

  IloSetRangeBounds ();
  IloSetRangeBounds (IloRangeI* rng, IloNum lb, IloNum ub)
    : _rng(rng), _lb(lb), _ub(ub)   {}

  friend class IloRangeI;

public:

   IloRangeI* getRange() const          { return (_rng); }
   IloNum     getLb() const             { return (_lb); }
   IloNum     getUb() const             { return (_ub); }
};

void IloRangeI::setBounds(IloNum lb, IloNum ub) {
  if (_lhs != lb || _rhs != ub) {
    _lhs = lb;
    _rhs = ub;
    notify(IloSetRangeBounds(this, lb, ub));
  }
}

void IloRangeI::setUb(IloNum val) {
  if (_rhs != val) {
    _rhs = val;
    notify(IloSetRangeBounds(this, _lhs, val));
  }
}

void IloRangeI::setLb(IloNum val) {
  if (_lhs != val) {
    _lhs = val;
    notify(IloSetRangeBounds(this, val, _rhs));
  }
}

class IloSetRangeArrayBounds : public IloChange {
  ILOCHANGEDECL

  IloRangeArray     _rng;
  const IloNumArray _lb;
  const IloNumArray _ub;

public:
  IloSetRangeArrayBounds (const IloRangeArray rng,
                          const IloNumArray lb,
                          const IloNumArray ub)
    : _rng(rng), _lb(lb), _ub(ub)        {}

  const IloRangeArray& getRanges () const  { return (_rng); }
  const IloNumArray& getLbs () const     { return (_lb); }
  const IloNumArray& getUbs () const     { return (_ub); }
};


class IloSetRangeCoef : public IloChange {
  ILOCHANGEDECL

  IloRangeI*        _rng;
  const IloNumVarI* _var;
  IloNum            _val;
  IloNum            _old;

public:
  IloSetRangeCoef (IloRangeI* rng, IloNumVarI* var, IloNum val, IloNum old)
    : _rng(rng), _var(var), _val(val), _old(old)    {}

  const IloRangeI*  getRange () const   { return (_rng); }
  const IloNumVarI* getVar   () const   { return (_var); }
  IloNum            getValue () const   { return (_val); }
  IloNum            getOldVal() const   { return (_old); }
};

class IloSetRangeCoefs : public IloChange {
  ILOCHANGEDECL

  IloRangeI*           _rng;
  const IloNumVarArray _vars;
  const IloNumArray    _vals;
  const IloNumArray    _old;

public:
  IloSetRangeCoefs(const IloRange rng,
                   const IloNumVarArray vars,
                   const IloNumArray vals,
                   const IloNumArray old)
    : _rng(rng.getImpl()), _vars(vars), _vals(vals), _old(old)     {}

  const IloRangeI*      getRange() const    { return (_rng); }
  const IloNumVarArray& getVars () const    { return (_vars); }
  const IloNumArray&    getVals () const    { return (_vals); }
  const IloNumArray&    getOldVals() const  { return (_old); }
};


class IloSetRangeExpr : public IloChange {
  ILOCHANGEDECL

  IloRangeI*      _rng;
  IloNumLinTermI* _oldPosExpr;
  IloNumLinTermI* _newPosExpr;

public:
  IloSetRangeExpr(IloRangeI*      rng,
		  IloNumLinTermI* oldPosExpr,
		  IloNumLinTermI*    newPosExpr)
    : _rng(rng), _oldPosExpr(oldPosExpr), _newPosExpr(newPosExpr) {}

  IloRangeI*       getRange() const      { return (_rng); }
  IloNumLinTermI*  getOldPosExpr() const { return (_oldPosExpr); }
  IloNumLinTermI*  getNewPosExpr() const { return (_newPosExpr); }
};

/////////////////////////////////////////////////////////////////////////
//
// helper class
//

class ILO_EXPORTED IloInvalidInequalityException : public IloException {
public:
  IloInvalidInequalityException()
    :  IloException("Strict Inequality is defined only on Integer Expressions")
    {}
};

//  void VerifyIntRange(IloRangeI* rng);

//  IloRange IloVerifiedIntRange(const IloEnv env,
//                               IloNum lhs,
//                               const IloNumExprArg expr,
//                               IloNum rhs);
//  IloRange IloVerifiedIntRange(const IloEnv env,
//                               IloNum lhs,
//                               const IloNumExprArg posexpr,
//                               const IloNumExprArg negexpr,
//			                       IloNum rhs);

class IloRangeBase : public IloExtractable {
  IloNum   _cnst;
public:
  IloRangeBase (IloRangeI* rng, IloNum cst=0.0)
    : IloExtractable(rng), _cnst (cst)  {}
  IloRangeI* getImpl() const {
    return ((IloRangeI*)IloExtractable::getImpl());
  }
  IloNum getConst() const { return _cnst; }
#if defined(ILOBADRANGE)
  operator IloConstraint() { return (getImpl()); }
#else
  operator IloRange() { return (getImpl()); }
#endif
};

class IloBinaryNumConstraintI : public IloConstraintI {
  ILOEXTRDECL
protected:
  IloNumExprI*_expr1;
  IloNumExprI*_expr2;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloInt _state;
public:
  IloBinaryNumConstraintI (IloEnvI*  e,
			   IloNumExprI* expr1,
			   IloNumExprI* expr2,
			   const char* name);
  virtual ~IloBinaryNumConstraintI();

  IloNumExprI* getExpr1() const {return _expr1;}
  IloNumExprI* getExpr2() const {return _expr2;}
  IloBool isVar1() const { return (_state & 1) == 0; }
  IloBool isVar2() const { return (_state & 2) == 0; }
  ILOEXTROTHERDECL
};

class IloNumEqI : public IloBinaryNumConstraintI {
  ILOEXTRDECL
public:
  IloNumEqI (IloEnvI*  e, IloNumExprI* expr1, IloNumExprI* expr2, const char* name=0);
  ILOEXTROTHERDECL
};

class IloNumGeI : public IloBinaryNumConstraintI {
  ILOEXTRDECL
public:
  IloNumGeI (IloEnvI*  e, IloNumExprI* expr1, IloNumExprI* expr2, const char* name=0);
  ILOEXTROTHERDECL
};

class IloNumLeI : public IloBinaryNumConstraintI {
  ILOEXTRDECL
public:
  IloNumLeI (IloEnvI*  e, IloNumExprI* expr1, IloNumExprI* expr2, const char* name=0);
  ILOEXTROTHERDECL
};

class IloNumGTI : public IloBinaryNumConstraintI {
  ILOEXTRDECL
public:
  IloNumGTI (IloEnvI*  e, IloNumExprI* expr1, IloNumExprI* expr2, const char* name=0);
  ILOEXTROTHERDECL
};

class IloNumLTI : public IloBinaryNumConstraintI {
  ILOEXTRDECL
public:
  IloNumLTI (IloEnvI*  e, IloNumExprI* expr1, IloNumExprI* expr2, const char* name=0);
  ILOEXTROTHERDECL
};



ILO_EXPORTEDFUNCTION(IloConstraint) operator== (IloNumExprArg base, IloNumExprArg expr);
ILO_EXPORTEDFUNCTION(IloRange) operator== (IloNumExprArg base, IloNum val);
ILO_EXPORTEDFUNCTION(IloRange) operator== (IloNum val, IloNumExprArg eb);

/////////////////////////////////////////////////////////////////////////
//
// operator>= and operator>
//

//
//  IloNumExpr   x, y;
//  IloNum  a, b;
//
//  x + a >= y + b gets normalized to
//
//  x - y >= b - a, i.e                 b-a <= x-y <= IloInfinity.
//
//  Special case: constant sides are not moved, i.e.
//
//  a >= b + x     becomes              x   <= a-b
//  x + a >= b     becomes              b-a <= x
//
//  reason:  for x >= a and a >= x, a should stay where it was.
//

ILO_EXPORTEDFUNCTION(IloConstraint) operator>= (IloNumExprArg base, IloNumExprArg base2);
ILO_EXPORTEDFUNCTION(IloRange) operator>= (IloNumExprArg expr, IloNum val);
ILO_EXPORTEDFUNCTION(IloRange) operator>= (IloNum val, IloNumExprArg eb);

ILO_EXPORTEDFUNCTION(IloConstraint) operator> (IloNumExprArg base, IloNumExprArg base2);

ILO_EXPORTEDFUNCTION(IloConstraint) operator> (IloNumExprArg base, IloNum val);

ILO_EXPORTEDFUNCTION(IloConstraint) operator> (IloNum val, IloNumExprArg eb);

ILO_EXPORTEDFUNCTION(IloConstraint) operator> (IloIntExprArg base, IloIntExprArg base2);

ILO_EXPORTEDFUNCTION(IloConstraint) operator> (IloIntExprArg base, IloInt val);

ILO_EXPORTEDFUNCTION(IloConstraint) operator> (IloInt val, IloIntExprArg eb);
/////////////////////////////////////////////////////////////////////////
//
// operator<= and operator<
//
//
//  IloNumExpr   x, y;
//  IloNum  a, b;
//
//  x + a <= y + b gets normalized to
//
//  x - y <= b - a, i.e                 -IloInfinity <= x-y <= b-a.
//
//  Special case: constant sides are not moved, i.e.
//
//  a <= b + x     becomes              a-b <= x
//  x + a <= b     becomes              x   <= b-a
//
//  reason:  for x <= a and a <= x, a should stay where it was.
//

ILO_EXPORTEDFUNCTION(IloConstraint) operator<= (IloNumExprArg base, IloNumExprArg base2);

ILO_EXPORTEDFUNCTION(IloRange) operator<= (IloNumExprArg base, IloNum val);

ILO_EXPORTEDFUNCTION(IloRangeBase) operator<= (IloNum val, const IloNumExprArg expr);

ILO_EXPORTEDFUNCTION(IloRange) operator<=(const IloRangeBase base, IloNum val);

ILO_EXPORTEDFUNCTION(IloConstraint) operator< (IloNumExprArg base, IloNumExprArg base2);
ILO_EXPORTEDFUNCTION(IloConstraint) operator< (IloNumExprArg base, IloNum val);
ILO_EXPORTEDFUNCTION(IloConstraint) operator< (IloNum val, const IloNumExprArg expr);
ILO_EXPORTEDFUNCTION(IloConstraint) operator< (IloIntExprArg base, IloIntExprArg base2);
ILO_EXPORTEDFUNCTION(IloConstraint) operator< (IloIntExprArg base, IloInt val);

// commented doc comment removed because it was causing problem for generation

//ILO_EXPORTEDFUNCTION(IloConstraint) operator< (IloInt val, IloIntExprArg base);




//////////////////////////////
//
//  ILOSOS1
//

class IloSOS1I: public IloConstraintI {
  ILOEXTRDECL

  IloNumVarArray _varArray;
  IloNumArray    _valArray;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
protected:
  virtual void atRemove(IloExtractableI* user=0, IloAny info = 0);
public:
  virtual ~IloSOS1I();
  IloSOS1I(IloEnvI* m, const char* name=0);
  IloSOS1I(IloEnvI* m, const IloNumVarArray vars, const char* name=0);
  IloSOS1I(IloEnvI* m, const IloNumVarArray vars, const IloNumArray vals,
           const char* name=0);

  void addVar(IloNumVar var);
  void removeVar(IloNumVar);
  void addVal(IloNum    val);
  IloNum getMaxVal() const;

  ILOEXTROTHERDECL

  IloNumVarArray getVarArray() const {return _varArray;}
  IloNumArray    getValArray() const {return _valArray;}
};

class IloSOS1RemoveVar : public IloChange {
  ILOCHANGEDECL

  IloNumVarI* _var;
  IloSOS1I* _sos;
public:
  IloSOS1RemoveVar(IloSOS1I* sos, IloNumVarI* var)
    : IloChange(), _var(var), _sos(sos) {}
  IloNumVarI* getVar() const     { return (_var); }
  IloSOS1I*  getSOS1() const { return (_sos); }
};

class IloAddVarToSOS1 : public IloAddNumVar {
  ILOCOLUMNDECL

  IloSOS1I* _sos1;
  IloNum    _val;

  const IloExtractableI* what() const {return _sos1;}
  void destroy();

public:
  virtual ~IloAddVarToSOS1();
  IloAddVarToSOS1(IloSOS1I* sos1, IloNum val) : _sos1(sos1), _val(val) {}

  void apply() const;

  IloEnvI*  getEnv() const    { return (_sos1->getEnv()); }
  IloSOS1I* getSOS1I() const  { return (_sos1); }
  IloNum    getValue() const  { return (_val); }
};

class ILO_EXPORTED IloSOS1 : public IloConstraint {
  ILOEXTRHANDLE(IloSOS1,IloConstraint)
public:
  IloSOS1(const IloEnv env, const char* name = 0);
  IloSOS1(const IloEnv env, const IloNumVarArray vars, const char* name = 0)
    : IloConstraint(new (env.getImpl()) IloSOS1I(env.getImpl(), vars.copy(), name)) {
    IloAssert(vars.getImpl(), "Using empty IloNumVarArray handle.");
  }

  IloSOS1(const IloEnv env, const IloNumVarArray vars, const IloNumArray vals,
          const char* name = 0)
    : IloConstraint(new (env.getImpl()) IloSOS1I(env.getImpl(),
						 vars.copy(),
						 vals.copy(), name)) {
    IloAssert(vars.getImpl(), "Using empty IloNumVarArray handle.");
    IloAssert(vals.getImpl(), "Using empty IloNumArray handle.");
    IloAssert(vars.getSize() == vals.getSize(),
              "variable and value array sizes does not match for SOS1");
  }

  IloAddVarToSOS1 operator() () {
    return (IloAddVarToSOS1(getImpl(), -1.0));
  }

  IloAddVarToSOS1 operator() (IloNum val) {
    return (IloAddVarToSOS1(getImpl(), val));
  }

  void getValues(IloNumArray values) const {
    IloAssert(getImpl(), "Using empty IloSOS1 handle.");
    IloAssert(values.getImpl(), "Using empty IloNumArray handle.");
    values.add(getImpl()->getValArray());
  }
  void getVariables(IloNumVarArray variables) const {
    IloAssert(getImpl(), "Using empty IloSOS1 handle.");
    IloAssert(variables.getImpl(), "Using empty IloNumVarArray handle.");
    variables.add(getImpl()->getVarArray());
  }
  void getNumVars(IloNumVarArray variables) const { getVariables(variables); }
};

class ILO_EXPORTED IloSOS1Array : public IloConstraintArray {
public:
  typedef IloDefaultArrayI ImplClass;
  IloSOS1Array(IloDefaultArrayI* i=0) : IloConstraintArray(i) {}
  IloSOS1Array(const IloEnv env, IloInt n = 0) : IloConstraintArray(env, n) {}
  IloSOS1Array(const IloEnv env, IloInt n, const IloSOS1 v0);
#ifdef ILOALLOWVARARGS
#ifdef ILOVARARGS
  IloSOS1Array(const IloEnv env, IloInt n, const IloSOS1 v0, IloVarArg* v1 ...);
#else
  IloSOS1Array(const IloEnv env, IloInt n, const IloSOS1 v0, const IloSOS1 v1 ...);
#endif
#else // ILOALLOWVARARGS
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2,
	  const IloSOS1 v3);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2,
	  const IloSOS1 v3, const IloSOS1 v4);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2,
	  const IloSOS1 v3, const IloSOS1 v4, const IloSOS1 v5);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2,
	  const IloSOS1 v3, const IloSOS1 v4, const IloSOS1 v5,
	  const IloSOS1 v6);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2,
	  const IloSOS1 v3, const IloSOS1 v4, const IloSOS1 v5,
	  const IloSOS1 v6, const IloSOS1 v7);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2,
	  const IloSOS1 v3, const IloSOS1 v4, const IloSOS1 v5,
	  const IloSOS1 v6, const IloSOS1 v7, const IloSOS1 v8);
  IloSOS1Array(const IloEnv env, IloInt n,
	  const IloSOS1 v0, const IloSOS1 v1, const IloSOS1 v2,
	  const IloSOS1 v3, const IloSOS1 v4, const IloSOS1 v5,
	  const IloSOS1 v6, const IloSOS1 v7, const IloSOS1 v8,
	  const IloSOS1 v9);
#endif

  void add(const IloSOS1Array& x)         { IloExtractableArray::add (x); }
  void add(const IloSOS1& x)              { IloExtractableArray::add (x); }
  void add(IloInt more, const IloSOS1& x) { IloExtractableArray::add (more, x); }

  IloSOS1& operator[] (IloInt i) {
    return (*(IloSOS1*)&(IloExtractableArray::operator[](i)));
  }
  IloSOS1  operator[] (IloInt i) const {
    return ((IloSOS1I*)((IloExtractableArray::operator[](i)).getImpl()));
  }
};

typedef IloDefaultArrayI IloSOS1ArrayI;


//////////////////////////////
//
//  ILOSOS2
//

class IloSOS2I: public IloConstraintI {
  ILOEXTRDECL

  IloNumVarArray _varArray;
  IloNumArray    _valArray;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
protected:
  virtual void atRemove(IloExtractableI* user=0, IloAny info = 0);
public:
  virtual ~IloSOS2I();
  IloSOS2I(IloEnvI* m, const char* name=0);
  IloSOS2I(IloEnvI* m, const IloNumVarArray vars, const char* name=0);
  IloSOS2I(IloEnvI* m, const IloNumVarArray vars, const IloNumArray vals,
           const char* name=0);

  void addVar(IloNumVar var);
  void removeVar(IloNumVar);
  void addVal(IloNum    val);
  IloNum getMaxVal() const;

  ILOEXTROTHERDECL

  IloNumVarArray getVarArray() const {return _varArray;}
  IloNumArray    getValArray() const {return _valArray;}
};

class IloSOS2RemoveVar : public IloChange {
  ILOCHANGEDECL

  IloNumVarI* _var;
  IloSOS2I* _sos;
public:
  IloSOS2RemoveVar(IloSOS2I* sos, IloNumVarI* var) : IloChange(), _var(var), _sos(sos) {}
  IloNumVarI* getVar() const     { return (_var); }
  IloSOS2I*  getSOS2() const { return (_sos); }
};


class IloAddVarToSOS2 : public IloAddNumVar {
  ILOCOLUMNDECL

  IloSOS2I* _sos2;
  IloNum    _val;

  const IloExtractableI* what() const {return _sos2;}
  void destroy();

public:
  virtual ~IloAddVarToSOS2();
  IloAddVarToSOS2(IloSOS2I* sos2, IloNum val) : _sos2(sos2), _val(val) {}

  void apply() const;

  IloEnvI*  getEnv() const    { return (_sos2->getEnv()); }
  IloSOS2I* getSOS2I() const  { return (_sos2); }
  IloNum    getValue() const  { return (_val); }
};


class ILO_EXPORTED IloSOS2 : public IloConstraint {
  ILOEXTRHANDLE(IloSOS2,IloConstraint)
public:
  IloSOS2(const IloEnv env, const char* name = 0);
  IloSOS2(const IloEnv env, const IloNumVarArray vars, const char* name = 0)
    : IloConstraint(new (env.getImpl()) IloSOS2I(env.getImpl(), vars.copy(), name)) {
    IloAssert(vars.getImpl(), "Using empty IloNumVarArray handle.");
  }
  IloSOS2(const IloEnv env, const IloNumVarArray vars, const IloNumArray vals,
          const char* name = 0)
    : IloConstraint(new (env.getImpl()) IloSOS2I(env.getImpl(),
						 vars.copy(),
						 vals.copy(), name)) {
    IloAssert(vars.getImpl(), "Using empty IloNumVarArray handle.");
    IloAssert(vals.getImpl(), "Using empty IloNumArray handle.");
    IloAssert (vars.getSize() == vals.getSize(),
               "Inconsistent variable and value set for SOS2");
  }

  IloAddVarToSOS2 operator() () {
    return (IloAddVarToSOS2(getImpl(), -1.0));
  }

  IloAddVarToSOS2 operator() (IloNum val) {
    return (IloAddVarToSOS2(getImpl(), val));
  }

  void getValues(IloNumArray values) const {
    IloAssert(getImpl(), "Using empty IloSOS2 handle.");
    IloAssert(values.getImpl(), "Using empty IloNumArray handle.");
    values.add(getImpl()->getValArray());
  }
  void getVariables(IloNumVarArray variables) const {
    IloAssert(getImpl(), "Using empty IloSOS2 handle.");
    IloAssert(variables.getImpl(), "Using empty IloNumVarArray handle.");
    variables.add(getImpl()->getVarArray());
  }
  void getNumVars(IloNumVarArray variables) const { getVariables(variables); }
};

class ILO_EXPORTED IloSOS2Array : public IloConstraintArray {
public:
  typedef IloDefaultArrayI ImplClass;
  IloSOS2Array(IloDefaultArrayI* i=0) : IloConstraintArray(i) {}
  IloSOS2Array(const IloEnv env, IloInt num = 0) : IloConstraintArray(env, num) {}
  IloSOS2Array(const IloEnv env, IloInt num, const IloSOS2 v0);
#ifdef ILOALLOWVARARGS
#ifdef ILOVARARGS
  IloSOS2Array(const IloEnv env, IloInt num, const IloSOS2 v0, IloVarArg* v1, ...);
#else
  IloSOS2Array(const IloEnv env, IloInt num, const IloSOS2 v0, const IloSOS2 v1, ...);
#endif
#else // ILOALLOWVARARGS
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2,
	  const IloSOS2 v3);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2,
	  const IloSOS2 v3, const IloSOS2 v4);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2,
	  const IloSOS2 v3, const IloSOS2 v4, const IloSOS2 v5);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2,
	  const IloSOS2 v3, const IloSOS2 v4, const IloSOS2 v5,
	  const IloSOS2 v6);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2,
	  const IloSOS2 v3, const IloSOS2 v4, const IloSOS2 v5,
	  const IloSOS2 v6, const IloSOS2 v7);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2,
	  const IloSOS2 v3, const IloSOS2 v4, const IloSOS2 v5,
	  const IloSOS2 v6, const IloSOS2 v7, const IloSOS2 v8);
  IloSOS2Array(const IloEnv env, IloInt num,
	  const IloSOS2 v0, const IloSOS2 v1, const IloSOS2 v2,
	  const IloSOS2 v3, const IloSOS2 v4, const IloSOS2 v5,
	  const IloSOS2 v6, const IloSOS2 v7, const IloSOS2 v8,
	  const IloSOS2 v9);
#endif

  void add(const IloSOS2Array array)     { IloExtractableArray::add (array); }
  void add(const IloSOS2 x)              { IloExtractableArray::add (x); }
  void add(IloInt more, const IloSOS2 x) { IloExtractableArray::add (more, x); }

  IloSOS2& operator[] (IloInt i) {
    return (*(IloSOS2*)&(IloExtractableArray::operator[](i)));
  }
  IloSOS2  operator[] (IloInt i) const {
    return ((IloSOS2I*)((IloExtractableArray::operator[](i)).getImpl()));
  }
};

class IloAddValueToObj : public IloAddNumVar {
  ILOCOLUMNDECL

  IloNum _value;
  IloObjectiveI* _obj;

  const IloExtractableI* what() const;

public:
  IloAddValueToObj (IloObjective obj, IloNum val);

  void apply    () const;

  IloNum getValue()            const { return (_value); }
  IloObjectiveI* getObjective() const { return (_obj); }
};

typedef IloDefaultArrayI IloSOS2ArrayI;

class ILO_EXPORTED IloObjective : public IloExtractable {
  ILOEXTRHANDLE(IloObjective, IloExtractable)
public:
  enum Sense {
     Minimize = 1,
     Maximize = -1
  };

  IloObjective(const IloEnv env,
               IloNum       constant = 0.0,
               Sense        sense = Minimize,
               const char*  name = 0);

  inline IloObjective(const IloEnv     env,
               const IloNumExprArg expr,
               Sense            sense = Minimize,
               const char*      name = 0);


  Sense  getSense () const;
  IloNum getConst () const;
  IloNum getConstant() const { return getConst(); }
  operator IloNumExprArg() const;

  IloAddValueToObj operator() () {
    return (IloAddValueToObj (*this, 1.0));
  }

  IloAddValueToObj operator() (IloNum value) {
    return (IloAddValueToObj (*this, value));
  }


  // Change methods use notification
  void setSense(IloObjective::Sense sense);
  void setConst(IloNum);
  void setConstant(IloNum constant) { setConst(constant); }
  IloNumExprArg getExpr() const;
  void setExpr(const IloNumExprArg);
  void setCoef(const IloNumVar, IloNum);
  void setLinearCoef(const IloNumVar var, IloNum value) {
    setCoef(var, value);
  }
  void setCoef(const IloNumVarArray, const IloNumArray);
  void setLinearCoefs(const IloNumVarArray vars, const IloNumArray values) {
    setCoef(vars, values);
  }
};


class ILO_EXPORTED IloObjectiveI : public IloExtractableI {
  ILOEXTRDECL
  IloObjective::Sense _sense;
  IloNumLinTermI* _expr;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual void whenAddedToCP(const IloAlgorithm alg, IloAny ptr) const;
  virtual void whenRemovedFromCP(const IloAlgorithm alg, IloAny ptr) const;
  static IloAddToCP _addToCP;
  static IloRemoveFromCP _removeFromCP;
public:
  IloObjectiveI(IloEnvI*            env,
                IloNum              cnst,
                IloObjective::Sense sense,
                const char*         name=0);

  IloObjectiveI(IloEnvI*            env,
                IloNumExprI*    expr,
                IloObjective::Sense sense,
                const char*         name=0);

  virtual ~IloObjectiveI();

  IloNumLinTermI* getExpr () const  { return (_expr); }
  IloObjective::Sense getSense() const  { return (_sense); }

  void addVar(IloNumVarI* var, IloNum val);

  void setSense(IloObjective::Sense);
  void setConst(IloNum);
  void setCoef(const IloNumVar, IloNum);
  void setExpr(const IloNumExprArg);
  void setCoef(const IloNumVarArray, const IloNumArray);

  IloInt normalize();

  ILOEXTROTHERDECL
};

/////////////////////////////////////////////////////////////////////////
//
// changes
//

class IloSetObjSense : public IloChange {
  ILOCHANGEDECL

  IloObjectiveI*      _obj;
  IloObjective::Sense _sense;

public:
  IloSetObjSense(IloObjectiveI* obj, IloObjective::Sense sense)
    : _obj(obj), _sense(sense)        {}

  IloObjectiveI*      getObjective() const    { return (_obj); }
  IloObjective::Sense getSense() const        { return (_sense); }
};


class IloSetObjConst : public IloChange {
  ILOCHANGEDECL

  IloObjectiveI* _obj;
  IloNum _val;

public:
  IloSetObjConst(IloObjectiveI* obj, IloNum val)
    : _obj(obj), _val(val)    {}

  IloObjectiveI* getObjective() const    { return (_obj); }
  IloNum getConst() const        { return (_val); }
};

class IloSetObjCoef : public IloChange {
  ILOCHANGEDECL

  IloObjectiveI*    _obj;
  const IloNumVarI* _var;
  IloNum            _val;
  IloNum            _old;

public:
  IloSetObjCoef(IloObjectiveI* obj, const IloNumVar var, IloNum val, IloNum old)
    : _obj (obj)
    , _var (var.getImpl())
    , _val (val)
    , _old(old)    {}

  const IloObjectiveI*    getObjective() const { return (_obj); }
  const IloNumVarI* getVar() const       { return (_var); }
  IloNum            getVal() const       { return (_val); }
  IloNum            getOldVal() const   { return (_old); }
};


class IloSetObjCoefs : public IloChange {
  ILOCHANGEDECL

  IloObjectiveI*    _obj;
  IloNumVarArray    _vars;
  const IloNumArray _vals;
  const IloNumArray _old;

public:
  IloSetObjCoefs (IloObjectiveI*    obj,
                  IloNumVarArray    vars,
                  const IloNumArray vals,
                  const IloNumArray old)
  : _obj (obj)
  , _vars (vars)
  , _vals (vals)
  , _old(old)     {}

  const IloObjectiveI*  getObjective() const { return (_obj); }
  const IloNumVarArray& getVars() const      { return (_vars); }
  const IloNumArray&    getVals() const      { return (_vals); }
  const IloNumArray&    getOldVals() const   { return (_old); }
};

class IloSetObjExpr : public IloChange {
  ILOCHANGEDECL

  IloObjectiveI* _obj;
  IloNumLinTermI* _oldExpr;
  IloNumLinTermI* _newExpr;

public:
  IloSetObjExpr(IloObjectiveI* obj, IloNumLinTermI* oldExpr,
                                    IloNumLinTermI* newExpr)
    : _obj(obj), _oldExpr(oldExpr), _newExpr(newExpr) {}

  IloObjectiveI*  getObjective() const { return (_obj); }
  IloNumLinTermI* getOldExpr() const   { return (_oldExpr); }
  IloNumLinTermI* getNewExpr() const   { return (_newExpr); }
};

/////////////////////////////
//
// IloObjective inline methods
//

ILO_EXPORTED IloObjective::IloObjective(const IloEnv env,
                           const IloNumExprArg expr,
                           Sense sense,
                           const char* name)
  : IloExtractable(new (env.getImpl()) IloObjectiveI(env.getImpl(), expr.getImpl(), sense, name)) {
  IloAssert(env.getImpl(), "Using empty IloEnv handle");
  IloAssert(expr.getImpl(), "Using empty IloNumExpr handle");
}


/////////////////////////////////////////////////////////////////////////
//
// IloMinimize
//
/////////////////////////////////////////////////////////////////////////

inline IloObjective IloMinimize(const IloEnv env,
                                IloNum constant=0.0,
                                const char* name=0) {
  IloAssert (0L != env.getImpl(), "Using empty handle");
  return IloObjective(env, constant, IloObjective::Minimize, name);
}
inline IloObjective IloMinimize(const IloEnv     env,
                                const IloNumExprArg expr,
                                const char*      name=0) {
  IloAssert (0L != env.getImpl(), "Using empty handle");
  return IloObjective(env, expr, IloObjective::Minimize, name);
}

/////////////////////////////////////////////////////////////////////////
//
// IloMaximize
//
/////////////////////////////////////////////////////////////////////////

inline IloObjective IloMaximize(const IloEnv env,
                                IloNum constant=0.0,
                                const char* name=0) {
  IloAssert (0L != env.getImpl(), "Using empty handle");
  return IloObjective(env, constant, IloObjective::Maximize, name);
}
inline IloObjective IloMaximize(const IloEnv env,
                                const IloNumExprArg expr,
                                const char* name=0) {
  IloAssert (0L != env.getImpl(), "Using empty handle");
  return IloObjective(env, expr, IloObjective::Maximize, name);
}

///////////////////////////////////////////
//
// IloForAllRange
//
class IloForAllI;
class IloForAllRangeI : public IloConstraintI {
   ILOEXTRMEMBERS2DECL(IloForAllRangeI, IloConstraintI,
            IloForAllI*, forall,
            IloInt, position)
   virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
   IloForAllI* getForAll() const   { return forall;}
   IloInt   getPosition() const         { return position;}
   IloConstraintI* getExpanded() const { return 0;} // TO BE REMOVED
   inline void setLinearCoef(const IloNumVar, IloNum);
   inline void setBounds(IloNum, IloNum);
   inline void setUB(IloNum);
   inline void setLB(IloNum);
   inline void setLinearCoefs(const IloNumVarArray, const IloNumArray);
};

class IloSetForAllRangeBounds : public IloChange {
  ILOCHANGEDECL

  IloForAllRangeI* _rng;
  IloNum   _lb, _ub;

  IloSetForAllRangeBounds ();
  IloSetForAllRangeBounds (IloForAllRangeI* rng, IloNum lb, IloNum ub)
    : _rng(rng), _lb(lb), _ub(ub)   {}

  friend class IloForAlllRangeI;

public:

   IloForAllRangeI* getForAllRange() const { return (_rng); }
   IloNum     getLb() const             { return (_lb); }
   IloNum     getUb() const             { return (_ub); }
};

void IloForAllRangeI::setBounds(IloNum lb, IloNum ub) {
  notify(IloSetForAllRangeBounds(this, lb, ub));
}

class IloSetForAllRangeUB : public IloChange {
  ILOCHANGEDECL

  IloForAllRangeI* _rng;
  IloNum   _ub;

  IloSetForAllRangeUB ();
  IloSetForAllRangeUB (IloForAllRangeI* rng, IloNum ub)
    : _rng(rng), _ub(ub)   {}

  friend class IloForAlllRangeI;

public:

   IloForAllRangeI* getForAllRange() const { return (_rng); }
   IloNum     getUB() const             { return (_ub); }
};

void IloForAllRangeI::setUB(IloNum val) {
  notify(IloSetForAllRangeUB(this, val));
}

class IloSetForAllRangeLB : public IloChange {
  ILOCHANGEDECL

  IloForAllRangeI* _rng;
  IloNum   _lb;

  IloSetForAllRangeLB ();
  IloSetForAllRangeLB (IloForAllRangeI* rng, IloNum lb)
    : _rng(rng), _lb(lb)   {}

  friend class IloForAlllRangeI;

public:

   IloForAllRangeI* getForAllRange() const { return (_rng); }
   IloNum     getLB() const             { return (_lb); }
};

void IloForAllRangeI::setLB(IloNum val) {
  notify(IloSetForAllRangeLB(this, val));
}

class IloSetForAllRangeLinearCoef : public IloChange {
  ILOCHANGEDECL

  IloForAllRangeI*  _rng;
  const IloNumVarI* _var;
  IloNum            _val;

public:
  IloSetForAllRangeLinearCoef (IloForAllRangeI* rng, IloNumVarI* var, IloNum val)
    : _rng(rng), _var(var), _val(val) {}

  const IloForAllRangeI*  getForAllRange () const   { return (_rng); }
  const IloNumVarI* getVar   () const   { return (_var); }
  IloNum            getValue () const   { return (_val); }
};

void IloForAllRangeI::setLinearCoef(const IloNumVar var, IloNum coef) {
  notify(IloSetForAllRangeLinearCoef(this, var.getImpl(), coef));
}

class IloSetForAllRangeLinearCoefs : public IloChange {
  ILOCHANGEDECL

  const IloForAllRangeI* _rng;
  const IloNumVarArray   _vars;
  const IloNumArray      _vals;

public:
  IloSetForAllRangeLinearCoefs(const IloForAllRangeI* rng,
                   const IloNumVarArray vars,
                   const IloNumArray vals)
    : _rng(rng), _vars(vars), _vals(vals) {}

  const IloForAllRangeI* getForAllRange() const    { return (_rng); }
  const IloNumVarArray& getVars () const    { return (_vars); }
  const IloNumArray&    getVals () const    { return (_vals); }
};


void IloForAllRangeI::setLinearCoefs(const IloNumVarArray vars, const IloNumArray coefs) {
  notify(IloSetForAllRangeLinearCoefs(this, vars, coefs));
}

class IloForAllRange : public IloConstraint {
   ILOEXTRHANDLE(IloForAllRange,IloConstraint)
public:
   IloForAllRange(IloEnv      env,
                  IloForAllI* forall,
                  IloInt      position,
                  char*       name = 0);
  void setLinearCoef(const IloNumVar var, IloNum value) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setLinearCoef(var, value);
  }
  void setLinearCoefs(const IloNumVarArray vars, const IloNumArray values) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    IloAssert(vars.getImpl(), "Using empty IloNumVarArray handle.");
    IloAssert(values.getImpl(), "Using empty IloNumArray handle.");
    getImpl()->setLinearCoefs(vars, values);
  }

  void setBounds(IloNum lb, IloNum ub) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setBounds(lb, ub);
  }
  void setUB(IloNum ub) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setUB(ub);
  }
  void setLB(IloNum lb) {
    IloAssert(getImpl(), "Using empty IloRange handle.");
    getImpl()->setLB(lb);
  }
};
typedef IloArray<IloForAllRange> IloForAllRangeArray;

class IloRangeWithExprBoundsI : public IloRangeI {
  ILOEXTRDECL
  IloNumExprI* _left;
  IloNumExprI* _right;

  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloRangeWithExprBoundsI(IloEnvI* env,
            IloNumExprI* left,
	          IloNumExprI*  expr,
            IloNumExprI* right,
            const char* name=0);
  virtual ~IloRangeWithExprBoundsI ();

  ILOEXTROTHERDECL

  IloNumExprI* getLeft() const { return _left; }
  IloNumExprI* getRight() const { return _right; }

  IloBool updateBounds(IloModelEvaluatorI*);
};

#endif
