// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilomapextrH
#define __CONCERT_ilomapextrH

#if !defined(__CONCERT_ilomapH)
#include <ilconcert/ilomap.h>
#endif
#if !defined(__CONCERT_iloanycollexpriH)
#include <ilconcert/ilocollexpr/iloanycollexpri.h>
#endif

/////////////////////////////////////////////////////////////////////////////
//   Macros
/////////////////////////////////////////////////////////////////////////////
#define DEFINE_INDEX_UTILITIES_AUX()\
  IloBool isIntIndex() const { return getIndex()->isIntIndex(); }\
  IloBool isNumIndex() const { return getIndex()->isNumIndex(); }\
  IloBool isAnyIndex() const { return getIndex()->isAnyIndex(); }\
  IloBool isExtractableIndex() const { return getIndex()->isExtractableIndex(); }\
  IloBool isIntExprIndex() const { return getIndex()->isIntExprIndex(); }\
  IloBool isNumExprIndex() const { return getIndex()->isNumExprIndex(); }\
  IloBool isAnyExprIndex() const { return getIndex()->isAnyExprIndex(); }\
  virtual IloBool hasRangeIndex() const { return getIndexer()->isIntRange(); }\
  IloInt getOffset() const {\
    IloAssert(hasRangeIndex(), "Cannot get index offset from an intSet");\
    return ((IloIntRangeI*)getIndexer())->getLB();\
  }\
  virtual IloIntArray getIndexerIntArray() const {\
    IloDiscreteDataCollectionI* indexer = getIndexer();\
    IloAssert(indexer->isIntSet(), "Cannot get an indexer int array");\
    return ((IloIntSetI*)indexer)->toArray();\
  }\
  virtual IloNumArray getIndexerNumArray() const {\
    IloDiscreteDataCollectionI* indexer = getIndexer();\
    IloAssert(indexer->isNumSet(), "Cannot get an indexer num array");\
    return ((IloNumSetI*)indexer)->toArray();\
  }\
  virtual IloAnyArray getIndexerAnyArray() const {\
    IloDiscreteDataCollectionI* indexer = getIndexer();\
    IloAssert(indexer->isAnySet(), "Cannot get an indexer any array");\
    return ((IloAnySetI*)indexer)->toArray();\
  }

#define DEFINE_INDEX_UTILITIES()\
  IloMapExtractIndexI* getIndex() const { return _index; }\
  DEFINE_INDEX_UTILITIES_AUX()

#define DEFINE_MAP_UTILITIES()\
  DEFINE_INDEX_UTILITIES()\
 protected:\
  virtual void visitSubExtractables(IloExtractableVisitor* v);\
 public:\
  virtual IloDiscreteDataCollectionI* getIndexer() const {\
    return getMap().getImpl()->getIndexer(_currentDim -1);\
  }\
  virtual IloBool isRoot() const { return IloFalse; }\
  virtual IloInt getCurrentDim() const { return _currentDim; }\
  IloBool isLastDimension() const { return _currentDim == getMap().getNbDim(); }\
  virtual IloExtractableI* makeClone(IloEnvI*) const = 0;\
  virtual void display(ILOSTD(ostream)& out) const = 0;

/////////////////////////////////////////////////////////////////////////////
//   IloMap
/////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
//   IloIntMap
//---------------------------------------------------------------------------
class IloIntSubMapExprI : public IloIntExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
  IloIntSubMapExprI(IloEnvI* env, IloMapExtractIndexI* index, IloInt dim);
  virtual ~IloIntSubMapExprI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloIntMap evalMap(const IloAlgorithm alg) const;
  virtual IloIntMap getMap() const = 0;
  virtual IloIntMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloIntSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloIntSubMapRootI : public IloIntSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloIntMap _map;
public:
  IloIntSubMapRootI(IloEnvI* env,
		    IloMapExtractIndexI* index,
		    IloIntMap m,
		    IloBool withCopy);
  IloIntSubMapRootI(IloEnvI* env,
		    IloIntExprI* index,
		    IloIntMap m,
		    IloBool withCopy);
  IloIntSubMapRootI(IloEnvI* env,
		    IloNumExprI* index,
		    IloIntMap m,
		    IloBool withCopy);
  IloIntSubMapRootI(IloEnvI* env,
		    IloAnyExprI* index,
		    IloIntMap m,
		    IloBool withCopy);
  virtual ~IloIntSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloIntMap getMap() const { return _map; }
  virtual IloIntMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloIntSubMapSubI : public IloIntSubMapExprI {
  ILOEXTRDECL
private:
  IloIntSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
		   IloIntSubMapExprI* owner, IloInt dim);
  IloIntSubMapSubI(IloEnvI* env, IloIntExprI* index,
		   IloIntSubMapExprI* owner, IloInt dim);
  IloIntSubMapSubI(IloEnvI* env, IloNumExprI* index,
		   IloIntSubMapExprI* owner, IloInt dim);
  IloIntSubMapSubI(IloEnvI* env, IloAnyExprI* index,
		   IloIntSubMapExprI* owner, IloInt dim);
  virtual ~IloIntSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloIntSubMapExprI* getOwner() const { return _owner; }
  virtual IloIntMap getMap() const { return _owner->getMap(); }
  virtual IloIntMap getEvaluatedMap(const IloAlgorithm alg) const;
};


//---------------------------------------------------------------------------
//   IloIntExprMap
//---------------------------------------------------------------------------
class IloIntExprSubMapExprI : public IloIntExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
  IloIntExprSubMapExprI(IloEnvI* env, IloMapExtractIndexI* index, IloInt dim);
  virtual ~IloIntExprSubMapExprI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloIntExprMap evalMap(const IloAlgorithm alg) const;
  virtual IloIntExprMap getMap() const = 0;
  virtual IloIntExprMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloIntExprSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloIntExprSubMapRootI : public IloIntExprSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloIntExprMap _map;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntExprSubMapRootI(IloEnvI* env,
			IloMapExtractIndexI* index,
			IloIntExprMap m,
			IloBool withCopy);
  IloIntExprSubMapRootI(IloEnvI* env,
			IloIntExprI* index,
			IloIntExprMap m,
			IloBool withCopy);
  IloIntExprSubMapRootI(IloEnvI* env,
			IloNumExprI* index,
			IloIntExprMap m,
			IloBool withCopy);
  IloIntExprSubMapRootI(IloEnvI* env,
			IloAnyExprI* index,
			IloIntExprMap m,
			IloBool withCopy);
  virtual ~IloIntExprSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloIntExprMap getMap() const { return _map; }
  virtual IloIntExprMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloIntExprSubMapSubI : public IloIntExprSubMapExprI {
  ILOEXTRDECL
private:
  IloIntExprSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntExprSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
		       IloIntExprSubMapExprI* owner, IloInt dim);
  IloIntExprSubMapSubI(IloEnvI* env, IloIntExprI* index,
		       IloIntExprSubMapExprI* owner, IloInt dim);
  IloIntExprSubMapSubI(IloEnvI* env, IloNumExprI* index,
		       IloIntExprSubMapExprI* owner, IloInt dim);
  IloIntExprSubMapSubI(IloEnvI* env, IloAnyExprI* index,
		       IloIntExprSubMapExprI* owner, IloInt dim);
  virtual ~IloIntExprSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloIntExprSubMapExprI* getOwner() const { return _owner; }
  virtual IloIntExprMap getMap() const { return _owner->getMap(); }
  virtual IloIntExprMap getEvaluatedMap(const IloAlgorithm alg) const;
};

//---------------------------------------------------------------------------
//   IloNumMap
//---------------------------------------------------------------------------

class IloNumSubMapExprI : public IloNumExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
  IloNumSubMapExprI(IloEnvI* env, IloMapExtractIndexI* index, IloInt dim);
  virtual ~IloNumSubMapExprI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloNumMap evalMap(const IloAlgorithm alg) const;
  virtual IloNumMap getMap() const = 0;
  virtual IloNumMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloNumSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloNumSubMapRootI : public IloNumSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloNumMap _map;
public:
  IloNumSubMapRootI(IloEnvI* env,
		    IloMapExtractIndexI* index,
		    IloNumMap m,
		    IloBool withCopy);
  IloNumSubMapRootI(IloEnvI* env,
		    IloIntExprI* index,
		    IloNumMap m,
		    IloBool withCopy);
  IloNumSubMapRootI(IloEnvI* env,
		    IloNumExprI* index,
		    IloNumMap m,
		    IloBool withCopy);
  IloNumSubMapRootI(IloEnvI* env,
		    IloAnyExprI* index,
		    IloNumMap m,
		    IloBool withCopy);
  virtual ~IloNumSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloNumMap getMap() const { return _map; }
  virtual IloNumMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloNumSubMapSubI : public IloNumSubMapExprI {
  ILOEXTRDECL
private:
  IloNumSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
		   IloNumSubMapExprI* owner, IloInt dim);
  IloNumSubMapSubI(IloEnvI* env, IloIntExprI* index,
		   IloNumSubMapExprI* owner, IloInt dim);
  IloNumSubMapSubI(IloEnvI* env, IloNumExprI* index,
		   IloNumSubMapExprI* owner, IloInt dim);
  IloNumSubMapSubI(IloEnvI* env, IloAnyExprI* index,
		   IloNumSubMapExprI* owner, IloInt dim);
  virtual ~IloNumSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloNumSubMapExprI* getOwner() const { return _owner; }
  virtual IloNumMap getMap() const { return _owner->getMap(); }
  virtual IloNumMap getEvaluatedMap(const IloAlgorithm alg) const;
};


//---------------------------------------------------------------------------
//   IloNumExprMap
//---------------------------------------------------------------------------
class IloNumExprSubMapExprI : public IloNumExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
  IloNumExprSubMapExprI(IloEnvI* env, IloMapExtractIndexI* index, IloInt dim);
  virtual ~IloNumExprSubMapExprI();
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloNumExprMap evalMap(const IloAlgorithm alg) const;
  virtual IloNumExprMap getMap() const = 0;
  virtual IloNumExprMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloNumExprSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloNumExprSubMapRootI : public IloNumExprSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloNumExprMap _map;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumExprSubMapRootI(IloEnvI* env,
			IloMapExtractIndexI* index,
			IloNumExprMap m,
			IloBool withCopy);
  IloNumExprSubMapRootI(IloEnvI* env,
			IloIntExprI* index,
			IloNumExprMap m,
			IloBool withCopy);
  IloNumExprSubMapRootI(IloEnvI* env,
			IloNumExprI* index,
			IloNumExprMap m,
			IloBool withCopy);
  IloNumExprSubMapRootI(IloEnvI* env,
			IloAnyExprI* index,
			IloNumExprMap m,
			IloBool withCopy);
  virtual ~IloNumExprSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloNumExprMap getMap() const { return _map; }
  virtual IloNumExprMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloNumExprSubMapSubI : public IloNumExprSubMapExprI {
  ILOEXTRDECL
private:
  IloNumExprSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumExprSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
		       IloNumExprSubMapExprI* owner, IloInt dim);
  IloNumExprSubMapSubI(IloEnvI* env, IloIntExprI* index,
		       IloNumExprSubMapExprI* owner, IloInt dim);
  IloNumExprSubMapSubI(IloEnvI* env, IloNumExprI* index,
		       IloNumExprSubMapExprI* owner, IloInt dim);
  IloNumExprSubMapSubI(IloEnvI* env, IloAnyExprI* index,
		       IloNumExprSubMapExprI* owner, IloInt dim);
  virtual ~IloNumExprSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloNumExprSubMapExprI* getOwner() const { return _owner; }
  virtual IloNumExprMap getMap() const { return _owner->getMap(); }
  virtual IloNumExprMap getEvaluatedMap(const IloAlgorithm alg) const;
};

//---------------------------------------------------------------------------
//   IloSymbolMap
//---------------------------------------------------------------------------
class IloSymbolSubMapExprI : public IloSymbolExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
  IloSymbolSubMapExprI(IloEnvI* env, IloMapExtractIndexI* index, IloInt dim);
  virtual ~IloSymbolSubMapExprI();
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getSymbolEncoder();
  }
  virtual IloSymbolMap evalMap(const IloAlgorithm alg) const;
  virtual IloSymbolMap getMap() const = 0;
  virtual IloSymbolMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloSymbolSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloSymbolSubMapRootI : public IloSymbolSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloSymbolMap _map;
public:
  IloSymbolSubMapRootI(IloEnvI* env,
		       IloMapExtractIndexI* index,
		       IloSymbolMap m,
		       IloBool withCopy);
  IloSymbolSubMapRootI(IloEnvI* env,
		       IloIntExprI* index,
		       IloSymbolMap m,
		       IloBool withCopy);
  IloSymbolSubMapRootI(IloEnvI* env,
		       IloNumExprI* index,
		       IloSymbolMap m,
		       IloBool withCopy);
  IloSymbolSubMapRootI(IloEnvI* env,
		       IloAnyExprI* index,
		       IloSymbolMap m,
		       IloBool withCopy);
  virtual ~IloSymbolSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloSymbolMap getMap() const { return _map; }
  virtual IloSymbolMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloSymbolSubMapSubI : public IloSymbolSubMapExprI {
  ILOEXTRDECL
private:
  IloSymbolSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloSymbolSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
		      IloSymbolSubMapExprI* owner, IloInt dim);
  IloSymbolSubMapSubI(IloEnvI* env, IloIntExprI* index,
		      IloSymbolSubMapExprI* owner, IloInt dim);
  IloSymbolSubMapSubI(IloEnvI* env, IloNumExprI* index,
		      IloSymbolSubMapExprI* owner, IloInt dim);
  IloSymbolSubMapSubI(IloEnvI* env, IloAnyExprI* index,
		      IloSymbolSubMapExprI* owner, IloInt dim);
  virtual ~IloSymbolSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloSymbolSubMapExprI* getOwner() const { return _owner; }
  virtual IloSymbolMap getMap() const { return _owner->getMap(); }
  virtual IloSymbolMap getEvaluatedMap(const IloAlgorithm alg) const;
};

////////////////////////////////////////////////////////////
//  Extractables for IloMap[IloXXXIndex]
//
#define SUBMAP_ACCESSOR_IMPL(_subMapType) \
  _subMapType _subMapType::operator[](IloIntExprArg idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapIntExprIndexI* idxE = new (env) IloMapIntExprIndexI(env, idx.getImpl()); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloNumExprArg idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapNumExprIndexI* idxE = new (env) IloMapNumExprIndexI(env, idx.getImpl()); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloSymbolExprArg idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapAnyExprIndexI* idxE = new (env) IloMapAnyExprIndexI(env, idx.getImpl()); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloTupleExprArg idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapAnyExprIndexI* idxE = new (env) IloMapAnyExprIndexI(env, idx.getImpl()->makeImage()); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloTuplePattern idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapAnyExprIndexI* idxE = new (env) IloMapAnyExprIndexI(env, idx.getImpl()->copy()); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloInt idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapIntIndexI* idxE = new (env) IloMapIntIndexI(env, idx); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloNum idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapNumIndexI* idxE = new (env) IloMapNumIndexI(env, idx); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloSymbol idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapAnyIndexI* idxE = new (env) IloMapSymbolIndexI(env, idx.getImpl()); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](const char* idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapAnyIndexI* idxE = new (env) IloMapSymbolIndexI(env, env->makeSymbol(idx)); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } \
  _subMapType _subMapType::operator[](IloTuple idx) const { \
    IloEnvI* env = getEnv().getImpl(); \
    IloMapAnyIndexI* idxE = new (env) IloMapTupleIndexI(env, idx.getImpl()); \
    return _subMapType(getImpl()->makeSubMap(idxE)); \
  } 

//---------------------------------------------------------------------------
//   IloAnyCollectionMap
//---------------------------------------------------------------------------
class IloAnyCollectionSubMapExprI : public IloAnyCollectionExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
  IloAnyCollectionSubMapExprI(IloEnvI* env,
			      IloMapExtractIndexI* index,
			      IloInt dim);
  virtual ~IloAnyCollectionSubMapExprI();
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getSymbolEncoder();
  }
  virtual IloAnyCollectionMap evalMap(const IloAlgorithm alg) const;
  virtual IloAnyCollectionMap getMap() const = 0;
  virtual IloAnyCollectionMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloAnyCollectionSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloAnyCollectionSubMapRootI : public IloAnyCollectionSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloAnyCollectionMap _map;
public:
  IloAnyCollectionSubMapRootI(IloEnvI* env,
			      IloMapExtractIndexI* index,
			      IloAnyCollectionMap m,
			      IloBool withCopy);
  IloAnyCollectionSubMapRootI(IloEnvI* env,
			      IloIntExprI* index,
			      IloAnyCollectionMap m,
			      IloBool withCopy);
  IloAnyCollectionSubMapRootI(IloEnvI* env,
			      IloNumExprI* index,
			      IloAnyCollectionMap m,
			      IloBool withCopy);
  IloAnyCollectionSubMapRootI(IloEnvI* env,
			      IloAnyExprI* index,
			      IloAnyCollectionMap m,
			      IloBool withCopy);
  virtual ~IloAnyCollectionSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloAnyCollectionMap getMap() const { return _map; }
  virtual IloAnyCollectionMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloAnyCollectionSubMapSubI : public IloAnyCollectionSubMapExprI {
  ILOEXTRDECL
private:
  IloAnyCollectionSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloAnyCollectionSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
			     IloAnyCollectionSubMapExprI* owner,
			     IloInt dim);
  IloAnyCollectionSubMapSubI(IloEnvI* env, IloIntExprI* index,
			     IloAnyCollectionSubMapExprI* owner,
			     IloInt dim);
  IloAnyCollectionSubMapSubI(IloEnvI* env, IloNumExprI* index,
			     IloAnyCollectionSubMapExprI* owner,
			     IloInt dim);
  IloAnyCollectionSubMapSubI(IloEnvI* env, IloAnyExprI* index,
			     IloAnyCollectionSubMapExprI* owner,
			     IloInt dim);
  virtual ~IloAnyCollectionSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloAnyCollectionSubMapExprI* getOwner() const { return _owner; }
  virtual IloAnyCollectionMap getMap() const { return _owner->getMap(); }
  virtual IloAnyCollectionMap getEvaluatedMap(const IloAlgorithm alg) const;
};
//--------------------------------------------

//---------------------------------------------------------------------------
//   IloIntCollectionMap
//---------------------------------------------------------------------------
class IloIntCollectionSubMapExprI : public IloIntCollectionExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
  IloNum eval(const IloAlgorithm) const;
  
  IloIntCollectionSubMapExprI(IloEnvI* env,
			      IloMapExtractIndexI* index,
			      IloInt dim);
  virtual ~IloIntCollectionSubMapExprI();
  virtual IloIntCollectionMap evalMap(const IloAlgorithm alg) const;
  virtual IloIntCollectionMap getMap() const = 0;
  virtual IloIntCollectionMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloIntCollectionSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloIntCollectionSubMapRootI : public IloIntCollectionSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloIntCollectionMap _map;
public:
  IloIntCollectionSubMapRootI(IloEnvI* env,
			      IloMapExtractIndexI* index,
			      IloIntCollectionMap m,
			      IloBool withCopy);
  IloIntCollectionSubMapRootI(IloEnvI* env,
			      IloIntExprI* index,
			      IloIntCollectionMap m,
			      IloBool withCopy);
  IloIntCollectionSubMapRootI(IloEnvI* env,
			      IloNumExprI* index,
			      IloIntCollectionMap m,
			      IloBool withCopy);
  IloIntCollectionSubMapRootI(IloEnvI* env,
			      IloAnyExprI* index,
			      IloIntCollectionMap m,
			      IloBool withCopy);
  virtual ~IloIntCollectionSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloIntCollectionMap getMap() const { return _map; }
  virtual IloIntCollectionMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloIntCollectionSubMapSubI : public IloIntCollectionSubMapExprI {
  ILOEXTRDECL
private:
  IloIntCollectionSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntCollectionSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
			     IloIntCollectionSubMapExprI* owner,
			     IloInt dim);
  IloIntCollectionSubMapSubI(IloEnvI* env, IloIntExprI* index,
			     IloIntCollectionSubMapExprI* owner,
			     IloInt dim);
  IloIntCollectionSubMapSubI(IloEnvI* env, IloNumExprI* index,
			     IloIntCollectionSubMapExprI* owner,
			     IloInt dim);
  IloIntCollectionSubMapSubI(IloEnvI* env, IloAnyExprI* index,
			     IloIntCollectionSubMapExprI* owner,
			     IloInt dim);
  virtual ~IloIntCollectionSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloIntCollectionSubMapExprI* getOwner() const { return _owner; }
  virtual IloIntCollectionMap getMap() const { return _owner->getMap(); }
  virtual IloIntCollectionMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isInteger() const{ return IloTrue;}
};
//-----------------------------------

//---------------------------------------------------------------------------
//   IloNumCollectionMap
//---------------------------------------------------------------------------
class IloNumCollectionSubMapExprI : public IloNumCollectionExprI {
  ILOEXTRDECL
protected:
  IloMapExtractIndexI* _index;
  IloInt _currentDim;
public:
	IloNum eval(const IloAlgorithm) const;

  IloNumCollectionSubMapExprI(IloEnvI* env, IloMapExtractIndexI* index, IloInt dim);
  virtual ~IloNumCollectionSubMapExprI();
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getSymbolEncoder();
  }
  virtual IloNumCollectionMap evalMap(const IloAlgorithm alg) const;
  virtual IloNumCollectionMap getMap() const = 0;
  virtual IloNumCollectionMap getEvaluatedMap(const IloAlgorithm alg) const = 0;
  IloNumCollectionSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  DEFINE_MAP_UTILITIES()
};

class IloNumCollectionSubMapRootI : public IloNumCollectionSubMapExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloNumCollectionMap _map;
public:
  IloNumCollectionSubMapRootI(IloEnvI* env,
			      IloMapExtractIndexI* index,
			      IloNumCollectionMap m,
			      IloBool withCopy);
  IloNumCollectionSubMapRootI(IloEnvI* env,
			      IloIntExprI* index,
			      IloNumCollectionMap m,
			      IloBool withCopy);
  IloNumCollectionSubMapRootI(IloEnvI* env,
			      IloNumExprI* index,
			      IloNumCollectionMap m,
			      IloBool withCopy);
  IloNumCollectionSubMapRootI(IloEnvI* env,
			      IloAnyExprI* index,
			      IloNumCollectionMap m,
			      IloBool withCopy);
  virtual ~IloNumCollectionSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloNumCollectionMap getMap() const { return _map; }
  virtual IloNumCollectionMap getEvaluatedMap(const IloAlgorithm alg) const;
  virtual IloBool isRoot() const { return IloTrue; }
};

class IloNumCollectionSubMapSubI : public IloNumCollectionSubMapExprI {
  ILOEXTRDECL
private:
  IloNumCollectionSubMapExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumCollectionSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
			     IloNumCollectionSubMapExprI* owner,
			     IloInt dim);
  IloNumCollectionSubMapSubI(IloEnvI* env, IloIntExprI* index,
			     IloNumCollectionSubMapExprI* owner,
			     IloInt dim);
  IloNumCollectionSubMapSubI(IloEnvI* env, IloNumExprI* index,
			     IloNumCollectionSubMapExprI* owner,
			     IloInt dim);
  IloNumCollectionSubMapSubI(IloEnvI* env, IloAnyExprI* index,
			     IloNumCollectionSubMapExprI* owner,
			     IloInt dim);
  virtual ~IloNumCollectionSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloNumCollectionSubMapExprI* getOwner() const { return _owner; }
  virtual IloNumCollectionMap getMap() const { return _owner->getMap(); }
  virtual IloNumCollectionMap getEvaluatedMap(const IloAlgorithm alg) const;
};

#endif
