// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilomapiH
#define __CONCERT_ilomapiH

#define MAPLOCK

#if !defined(__CONCERT_ilocollectionH)
#include <ilconcert/ilocollection.h>
#endif
#if !defined(__CONCERT_ilocollexprbaseH)
#include <ilconcert/ilocollexprbase.h>
#endif
#if !defined(__CONCERT_ilotuplesetH)
#include <ilconcert/ilotupleset.h>
#endif
#if !defined(__CONCERT_iloextractableH)
# include <ilconcert/iloextractable.h>
#endif

class IloMapI;
class IloAbstractTupleMapI;
class IloMapVectorIndexI;
class IloAbstractMapI;

/////////////////////////////////////
// IloMapItem
//
class IloMapItem {
  friend class IloMapI;
public:
  //1: Extractable ?
  //2: Index ?
  //3: Int ?
  //4: Num ?
  //5: Symbol ?
  //6: Tuple ?
  //7: Pattern ?
  //8: IntCollection ?
  //9: NumCollection ?
  //10: AnyCollection ?
  //12: String ?
  //13: Constraint ?
  //16: SubMap ?
  enum Type {
    Anonymous=0,
    Extractable=1,
    IntConst =4,
    IntExpr  =5,
    IntIndex =7,
    NumConst =8,
    NumExpr  =9,
    NumIndex =11,
    SymbolConst=16,
    SymbolExpr = 17,
    SymbolIndex=19,
    TupleConst=32,
    TupleExpr = 33,
    TupleIndex=35,
#ifdef CONST_TUPLE_PATTERN
    TuplePatternConst=64,
    TuplePatternExpr=65,
#endif //ifdef CONST_TUPLE_PATTERN
    TuplePattern=67,
    IntCollectionConst=128,
    IntCollectionExpr=129,
    IntCollectionIndex=131,
    NumCollectionConst=256,
    NumCollectionExpr=257,
    NumCollectionIndex=259,
    AnyCollectionConst=512,
    AnyCollectionExpr=513,
    SymbolCollectionIndex=515,
    StringConst=2048,
    ConstraintExpr=4097,
    SubMap=32768
  };
private:
  Type   _type;
  union {
    IloInt _int;
    IloNum _num;
    IloAny _any;
  } _data;
  IloMapItem(Type t, IloAny e)
    :_type(t) { _data._any = e;}
  IloMapItem();
public:
  IloMapItem(const IloMapItem& i);

  IloMapItem(IloIntExprArg x);
  IloMapItem(IloIntVar x);
  IloMapItem(IloIntIndex x);
  IloMapItem(IloInt x);
  IloMapItem(const int x);
  IloMapItem(const char x);
  IloMapItem(const short x);
  IloMapItem(IloNumExprArg x);
  IloMapItem(IloNumVar x);
  IloMapItem(IloNumIndex x);
  IloMapItem(IloNum x);
  IloMapItem(const char* x);
  IloMapItem(IloSymbol x);
  IloMapItem(IloSymbolExprArg x);
  IloMapItem(IloSymbolIndex x);
  IloMapItem(IloTupleIndex x);
  IloMapItem(IloTupleExprArg x);
  IloMapItem(IloTuple x);
  IloMapItem(IloTuplePattern x);

  IloMapItem(IloIntCollectionExprArg x);
  IloMapItem(IloIntCollectionIndex x);
  IloMapItem(IloIntCollection x);
  IloMapItem(IloNumCollectionExprArg x);
  IloMapItem(IloNumCollectionIndex x);
  IloMapItem(IloNumCollection x);
  IloMapItem(IloSymbolCollectionExprArg x);
  IloMapItem(IloSymbolCollectionIndex x);
  IloMapItem(IloAnyCollection x);

  IloMapItem(IloConstraint x);
  IloMapItem(IloExtractable x);
  IloMapItem(IloMapI* x);

  ~IloMapItem() {}

  Type getType() const { return _type; }

  IloMapItem getClone(IloEnvI* env);
  void operator= (const IloMapItem& i);
  IloBool isAnonymous() const { return !_type; }
  IloBool isConstant() const { return !(_type & 1); }
  IloBool isExtractable() const { return (_type & 1); }
  IloBool isIndex() const { return (_type & 2); }
  IloBool isInt() const { return (_type & 4); }
  IloBool isNum() const { return (_type & 8); }
  IloBool isSymbol() const { return (_type & 16); }
  IloBool isTuple() const { return (_type & 32); }
  IloBool isPattern() const { return (_type & 64); }

  IloBool isIntCollection() const { return (_type & 128); }
  IloBool isNumCollection() const { return (_type & 256); }
  IloBool isAnyCollection() const { return (_type & 512); }

  IloBool isString() const { return (_type & 2048); }
  IloBool isConstraint() const { return (_type & 4096); }
  IloBool isSubMap() const { return (_type & 32768); }

  IloBool isAny() const{ return (!isInt() && !isNum());}
  IloBool isTypeCompatible(Type t) const;

  IloInt asInt() const {
    IloAssert(_type == IntConst, "Map Item is not an integer.");
    return _data._int;
  }
  void getValue(IloInt& res) { res = asInt(); }
  IloNum asNum() const {
    if (_type == NumConst)
      return _data._num;
    IloAssert(_type == IntConst, "Map Item is not a double nor an int.");
    IloNum res = _data._int; 
    return res;
  }
  void getValue(IloNum& res) {  res = asNum(); }
  const char* asString() const {
    if (isString())
      return (const char*)_data._any;
    else return asSymbol().getImpl()->getString();
  }
  IloSymbol asSymbol() const {
    IloAssert(_type == SymbolConst, "Map Item is not a symbol.");
    return (IloSymbolI*)_data._any;
  }
  void getValue(IloSymbol& res) {
    if (isSymbol())
      res = asSymbol();
    else res = res.getImpl()->getEnv()->makeSymbol(asString());
  }
  IloAny asAny() const{
    IloAssert((_type != NumConst) && (_type != IntConst),
	      "Map Item can not be used as an IloAny");
    return _data._any;
  }
  IloIntCollection asIntCollection() const {
    IloAssert(_type == IntCollectionConst,
	      "Map Item is not a collection of int.");
    return (IloIntCollectionI*)_data._any;
  }
  void getValue(IloIntCollection& res) { res = asIntCollection(); }

  IloIntSet asIntSet() const {
    IloAssert(_type == IntCollectionConst, "Map Item is not a collection of int.");
    IloIntCollectionI* res = (IloIntCollectionI*)_data._any;
	if (res!=NULL) IloTestAndRaise(res->isType(IloIntSetI::GetTypeInfo()), "Map Item is not an intSet");
	return (IloIntSetI*)res;
  }

  IloNumCollection asNumCollection() const {
    IloAssert(_type == NumCollectionConst,
	      "Map Item is not a collection of num.");
    return (IloNumCollectionI*)_data._any;
  }
  IloNumSet asNumSet() const {
    IloAssert(_type == NumCollectionConst, "Map Item is not a collection of int.");
    IloNumCollectionI* res = (IloNumCollectionI*)_data._any;
	if (res!=NULL) IloTestAndRaise(res->isType(IloNumSetI::GetTypeInfo()), "Map Item is not an numSet");
	return (IloNumSetI*)res;
  }
  
  void getValue(IloNumCollection& res) { res = asNumCollection(); }
  IloAnyCollection asAnyCollection() const {
    IloAssert(_type == AnyCollectionConst,
	      "Map Item is not a collection of any.");
    return (IloAnyCollectionI*)_data._any;
  }
  IloSymbolSet asSymbolSet() const {
    IloAssert(_type == AnyCollectionConst, "Map Item is not a collection of int.");
    IloAnyCollectionI* res = (IloAnyCollectionI*)_data._any;
	if (res!=NULL) IloTestAndRaise(res->isType(IloAnySetI::GetTypeInfo()) && res->isSymbolSet(), "Map Item is not an symbolSet");
	return (IloAnySetI*)res;
  }
  IloTupleCollection asTupleCollection() const {
    IloAssert(_type == AnyCollectionConst, "Map Item is not a collection of int.");
    IloAnyCollectionI* res = (IloAnyCollectionI*)_data._any;
	if (res!=NULL) IloTestAndRaise(res->isType(IloTupleCollectionI::GetTypeInfo()), "Map Item is not an tupleCollection");
	return (IloTupleCollectionI*)res;
  }
  IloTupleSet asTupleSet() const {
    IloAssert(_type == AnyCollectionConst, "Map Item is not a collection of int.");
    IloAnyCollectionI* res = (IloAnyCollectionI*)_data._any;
	if (res!=NULL) IloTestAndRaise(res->isType(IloTupleSetI::GetTypeInfo()), "Map Item is not an tupleSet");
	return (IloTupleSetI*)res;
  }

  void getValue(IloAnyCollection& res) { res = asAnyCollection(); }
  IloTuple asTuple() const {
    IloAssert(_type == TupleConst, "Map Item is not a tuple.");
    return (IloTupleI*)_data._any;
  }
  void getValue(IloTuple& res) { res = asTuple(); }
  IloExtractable asExtractable() const {
    IloAssert(isExtractable(), "Map Item is not an extractable.");
    return (IloExtractableI*)_data._any;
  }
  void getValue(IloExtractable& res) { res = asExtractable(); }
  IloIntExprI* asIntExpr() const {
    IloAssert(isInt() && isExtractable(),
	      "Map Item is not an integer expression.");
    return (IloIntExprI*)_data._any;
  }
  void getValue(IloIntExpr& res) { res = asIntExpr(); }
  IloNumExprI* asNumExpr() const {
    IloAssert(isNum() && isExtractable(),
	      "Map Item is not a numeric expression.");
    return (IloNumExprI*)_data._any;
  }
  void getValue(IloNumExpr& res) { res = asNumExpr(); }
  IloIntCollectionExprI* asIntCollectionExpr() const {
    IloAssert(isIntCollection() && isExtractable(),
	      "Map Item is not an intcollection expression.");
    return (IloIntCollectionExprI*)_data._any;
  }
  void getValue(IloIntCollectionExprI*& res) { res = asIntCollectionExpr(); }
  IloNumCollectionExprI* asNumCollectionExpr() const {
    IloAssert(isNumCollection() && isExtractable(),
	      "Map Item is not a numcollection expression.");
    return (IloNumCollectionExprI*)_data._any;
  }
  void getValue(IloNumCollectionExprI*& res) { res = asNumCollectionExpr(); }
  IloAnyCollectionExprI* asAnyCollectionExpr() const {
    IloAssert(isAnyCollection() && isExtractable(),
	      "Map Item is not an anycollection expression.");
    return (IloAnyCollectionExprI*)_data._any;
  }
  IloTuplePatternI* asPattern() const {
    IloAssert(isPattern(), "Map Item is not a tuple pattern.");
    return (IloTuplePatternI*)_data._any;
  }
  void getValue(IloTuplePattern& res) { res = asPattern(); }
  IloTupleIndexI* asTupleIndex() const {
    IloAssert(_type == TupleIndex, "Map Item is not a tuple index.");
    return (IloTupleIndexI*)_data._any;
  }
  void getValue(IloTupleIndex& res) { res = asTupleIndex(); }
  IloAnyExprI* asAnyExpr() const {
    IloAssert(isAny() && isExtractable() && !isConstraint(),
	      "Map Item is not an any expression.");
    return (IloAnyExprI*)_data._any;
  }
  void getValue(IloAnyExprI*& res) { res = asAnyExpr(); }
  IloConstraint asConstraint() const {
    IloAssert(isConstraint(), "Map Item is not a constraint.");
    return (IloConstraintI*)_data._any;
  }
  void getValue(IloConstraint& res) { res = asConstraint(); }
  IloMapI* asSubMap() const {
    if (!isSubMap()) {
      throw IloWrongUsage("Map Item is not a submap.");
    }
    IloAssert(isSubMap(), "Map Item is not a submap.");
    return (IloMapI*)_data._any;
  }
  void getValue(IloMapI*& res) { res = asSubMap(); }

  void display(ILOSTD(ostream)& out) const;

  static Type GetType(IloInt) { return IntConst; }
  static Type GetType(IloNum) { return NumConst; }
  static Type GetType(IloSymbol) { return SymbolConst; }
  static Type GetType(IloTuple) { return TupleConst; }
  static Type GetType(IloIntCollection) { return IntCollectionConst; }
  static Type GetType(IloNumCollection) { return NumCollectionConst; }
  static Type GetType(IloAnyCollection) { return AnyCollectionConst; }
  static Type GetType(const char*) { return StringConst; }

  static Type GetType(IloIntExprArg) { return IntExpr; }
  static Type GetType(IloIntIndex) { return IntIndex; }
  static Type GetType(IloNumExprArg) { return NumExpr; }
  static Type GetType(IloNumIndex) { return NumIndex; }
  static Type GetType(IloSymbolExprArg) { return SymbolExpr; }
  static Type GetType(IloSymbolIndex) { return SymbolIndex; }
  static Type GetType(IloTupleExprArg) { return TupleExpr; }
  static Type GetType(IloTupleIndex) { return TupleIndex; }
#ifdef CONST_TUPLE_PATTERN
  static Type GetType(IloTuplePattern p) {
    return ((p.getImpl()->isConst())
	    ? TuplePatternConst
	    : ((p.getImpl()->isExpr()) ? TuplePatternExpr : TuplePattern)); }
#else
  static Type GetType(IloTuplePattern) { return TuplePattern; }
#endif //ifdef CONST_TUPLE_PATTERN
  static Type GetType(IloIntCollectionExprArg) { return IntCollectionExpr; }
  static Type GetType(IloIntCollectionIndex) { return IntCollectionIndex; }
  static Type GetType(IloNumCollectionExprArg) { return NumCollectionExpr; }
  static Type GetType(IloNumCollectionIndex) { return NumCollectionIndex; }
  static Type GetType(IloSymbolCollectionExprArg) { return AnyCollectionExpr; }
  static Type GetType(IloSymbolCollectionIndex) { return SymbolCollectionIndex; }

  static Type GetType(IloExtractable) { return Extractable; }
  static Type GetType(IloConstraint) { return ConstraintExpr; }
  static Type GetType(IloMapI*) { return SubMap; }
};

inline ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, const IloMapItem & ext) {
  ext.display(out);
  return (out);
}


//--------------------------------------------------------------------
// MapItem arrays
//--------------------------------------------------------------------
typedef IloArray<IloMapItem> IloMapItemArrayBase;

class ILO_EXPORTED IloMapItemArray : public IloMapItemArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;
  IloMapItemArray(IloDefaultArrayI* i=0) : IloMapItemArrayBase(i) {}
  IloMapItemArray(const IloMapItemArray& copy) : IloMapItemArrayBase(copy) {}
  IloMapItemArray(const IloEnv env, IloInt n = 0) : IloMapItemArrayBase(env, n) {}
  const IloMapItem& operator[] (IloInt i) const {
    return IloMapItemArrayBase::operator[](i);
  }
  IloMapItem& operator[] (IloInt i) {
    return IloMapItemArrayBase::operator[](i);
  }
};

class IloMapException  : public IloException {
  const IloAbstractMapI* _map;
  const IloExtractableI* _obj;
public:
  IloMapException (const IloAbstractMapI*);
  IloMapException (const IloAbstractMapI*, const IloExtractableI*);
  virtual void print(ILOSTD(ostream)& out) const;
  const IloAbstractMapI* getMap() const { return (_map); }
  const IloExtractableI* getExtractable() const { return (_obj); }
  void setContext(IloExtractableI* context) { _obj = context; }
};

class IloMapOutOfBoundException  : public IloMapException {
  const IloMapItem _index;
public:
  IloMapOutOfBoundException (const IloAbstractMapI*, const IloMapItem, const IloExtractableI*);
  IloMapOutOfBoundException (const IloAbstractMapI*, const IloMapItem);
  void print(ILOSTD(ostream)& out) const;
  const IloMapItem getMapItem() const { return (_index); }
};

class IloMapUnboundIndexException  : public IloMapException {
public:
  IloMapUnboundIndexException (const IloAbstractMapI*, const IloExtractableI*);
  void print(ILOSTD(ostream)& out) const;
};

class IloLockedMapException  : public IloMapException {
public:
  IloLockedMapException (const IloAbstractMapI*);
  void print(ILOSTD(ostream)& out) const;
};

/////////////////////////////////////
// IloMapIndex
//
#define IloMapIndex IloMapItem
typedef IloArray<IloMapIndex> IloMapIndexArrayBase;

class IloMapIndexArray : public IloMapIndexArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;
  IloMapIndexArray(IloDefaultArrayI* i=0)
    :IloMapIndexArrayBase(i) {}
  IloMapIndexArray(const IloMapIndexArray& copy)
    :IloMapIndexArrayBase(copy) {}
  IloMapIndexArray(const IloEnv env, IloInt n=0)
    :IloMapIndexArrayBase(env, n) {}
  IloBool isConstant() const;
};


/////////////////////////////////////
// IloMapExtractIndexI
//
// IloIntExprI, IloNumExprI, IloSymbolIndexI, IloTupleIndexI, IloTuplePattern
// IloInt, IloNum, IloAny
class IloMapExtractIndexI : public IloExtractableI {
  ILOEXTRDECL
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  IloMapExtractIndexI(IloEnvI* env) : IloExtractableI(env) {}
  virtual ~IloMapExtractIndexI(){}
public:
  static IloMapExtractIndexI* make(IloEnvI*, IloMapIndex midx);
  virtual IloExtractableI* makeClone(IloEnvI*) const = 0;
  virtual void display(ILOSTD(ostream)& out) const = 0;
  virtual IloNum eval(const IloAlgorithm alg) const = 0;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloMapI* m) const = 0;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloAbstractTupleMapI* m,
				   IloInt dim=0) const = 0;
  virtual IloInt getAbsoluteIndex(const IloMapI* m) const = 0;
  virtual IloInt getAbsoluteIndex(const IloAbstractTupleMapI* m,
				  IloInt dim=0) const = 0;
  virtual IloBool isIntIndex() const;
  virtual IloBool isNumIndex() const;
  virtual IloBool isAnyIndex() const;
  virtual IloBool isSymbolIndex() const;
  virtual IloBool isTupleIndex() const;
  virtual IloBool isExtractableIndex() const;
  virtual IloBool isIntExprIndex() const;
  virtual IloBool isNumExprIndex() const;
  virtual IloBool isAnyExprIndex() const;
  virtual IloInt getIntIndex() const;
  virtual IloNum getNumIndex() const;
  virtual IloAny getAnyIndex() const;
  virtual IloSymbolI* getSymbolIndex() const;
  virtual IloTupleI* getTupleIndex() const;
  virtual IloIntExprI* getIntExprIndex() const;
  virtual IloNumExprI* getNumExprIndex() const;
  virtual IloAnyExprI* getAnyExprIndex() const;
  virtual IloExtractableI* getExtractableIndex() const;
};

class IloMapIntIndexI : public IloMapExtractIndexI {
  ILOEXTRDECL
private:
  IloInt _value;
public:
  IloMapIntIndexI(IloEnvI* env, IloInt value) : IloMapExtractIndexI(env), _value(value) {}
  virtual ~IloMapIntIndexI();
  virtual IloBool isIntIndex() const;
  virtual IloInt getIntIndex() const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloInt getAbsoluteIndex(const IloMapI* m) const;
  virtual IloInt getAbsoluteIndex(const IloAbstractTupleMapI* m, IloInt dim=0) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg, IloMapI* m) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloAbstractTupleMapI* m,
				   IloInt dim=0) const;
};

class IloMapNumIndexI : public IloMapExtractIndexI {
  ILOEXTRDECL
private:
  IloNum _value;
public:
  IloMapNumIndexI(IloEnvI* env, IloNum value) : IloMapExtractIndexI(env), _value(value) {}
  virtual ~IloMapNumIndexI();
  virtual IloBool isNumIndex() const;
  virtual IloNum getNumIndex() const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloInt getAbsoluteIndex(const IloMapI* m) const;
  virtual IloInt getAbsoluteIndex(const IloAbstractTupleMapI* m, IloInt dim=0) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg, IloMapI* m) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloAbstractTupleMapI* m,
				   IloInt dim=0) const;
};

class IloMapAnyIndexI : public IloMapExtractIndexI {
  ILOEXTRDECL
protected:
  IloAny _value;
  IloEncoderI* _encoder;
public:
  IloMapAnyIndexI(IloEnvI* env, IloAny value, IloEncoderI* encoder)
    : IloMapExtractIndexI(env), _value(value), _encoder(encoder) {}
  virtual ~IloMapAnyIndexI();
  virtual IloEncoderI* getEncoder() const;
  virtual IloBool isAnyIndex() const;
  virtual IloAny getAnyIndex() const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloInt getAbsoluteIndex(const IloMapI* m) const;
  virtual IloInt getAbsoluteIndex(const IloAbstractTupleMapI* m, IloInt dim=0) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg, IloMapI* m) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloAbstractTupleMapI* m,
				   IloInt dim=0) const;
};

class IloMapSymbolIndexI : public IloMapAnyIndexI {
  ILOEXTRDECL
public:
  IloMapSymbolIndexI(IloEnvI* env, IloSymbolI* value)
    : IloMapAnyIndexI(env, value, env->getSymbolEncoder()) {}
  virtual ~IloMapSymbolIndexI();
  virtual IloBool isSymbolIndex() const;
  virtual IloSymbolI* getSymbolIndex() const;
  virtual IloInt getAbsoluteIndex(const IloMapI* m) const;
  virtual IloInt getAbsoluteIndex(const IloAbstractTupleMapI* m, IloInt dim=0) const;
};

class IloMapTupleIndexI : public IloMapAnyIndexI {
  ILOEXTRDECL
public:
  IloMapTupleIndexI(IloEnvI* env, IloTupleI* value)
    : IloMapAnyIndexI(env, value, env->getTupleEncoder()) {}
  virtual ~IloMapTupleIndexI();
  virtual IloBool isTupleIndex() const;
  virtual IloTupleI* getTupleIndex() const;
  virtual IloInt getAbsoluteIndex(const IloMapI* m) const;
  virtual IloInt getAbsoluteIndex(const IloAbstractTupleMapI* m, IloInt dim=0) const;
};

class IloMapExtractableIndexI : public IloMapExtractIndexI {
  ILOEXTRDECL
protected:
  IloExtractableI* _expr;
  IloMapExtractableIndexI(IloEnvI* env, IloExtractableI* expr);
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  virtual ~IloMapExtractableIndexI();
  virtual IloBool isExtractableIndex() const;
  virtual IloExtractableI* getExtractableIndex() const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloInt getAbsoluteIndex(const IloMapI* m) const;
  virtual IloInt getAbsoluteIndex(const IloAbstractTupleMapI* m, IloInt dim=0) const;
  virtual IloExtractableI* lock();
  virtual void release();
};

class IloMapNumExprIndexI : public IloMapExtractableIndexI {
  ILOEXTRDECL
//protected:
//  IloMapNumExprIndexI(IloEnvI* env, IloExtractableI* expr)
//    : IloMapExtractableIndexI(env, expr) {}
public:
  IloMapNumExprIndexI(IloEnvI* env, IloNumExprI* expr);
  virtual ~IloMapNumExprIndexI();
  virtual IloBool isNumExprIndex() const;
  virtual IloNumExprI* getNumExprIndex() const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg, IloMapI* m) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloAbstractTupleMapI* m,
				   IloInt dim=0) const;
};

class IloMapIntExprIndexI : public IloMapNumExprIndexI {
  ILOEXTRDECL
//protected:
//  IloMapIntExprIndexI(IloEnvI* env, IloExtractableI* expr)
//    : IloMapNumExprIndexI(env, expr) {}
public:
  IloMapIntExprIndexI(IloEnvI* env, IloIntExprI* expr)
    : IloMapNumExprIndexI(env, expr) {}
  virtual ~IloMapIntExprIndexI();
  virtual IloBool isIntExprIndex() const;
  virtual IloIntExprI* getIntExprIndex() const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg, IloMapI* m) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloAbstractTupleMapI* m,
				   IloInt dim=0) const;
};

class IloMapAnyExprIndexI : public IloMapExtractableIndexI {
  ILOEXTRDECL
//protected:
//  IloMapAnyExprIndexI(IloEnvI* env, IloExtractableI* expr)
//    : IloMapExtractableIndexI(env, expr) {}
public:
  IloMapAnyExprIndexI(IloEnvI* env, IloAnyExprI* expr);
  virtual ~IloMapAnyExprIndexI();
  virtual IloBool isAnyExprIndex() const;
  virtual IloAnyExprI* getAnyExprIndex() const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg, IloMapI* m) const;
  virtual IloInt evalAbsoluteIndex(const IloAlgorithm alg,
				   IloAbstractTupleMapI* m,
				   IloInt dim=0) const;
};

/////////////////////////////////////
typedef IloArray<IloDiscreteDataCollectionI*> IloMapIndexerI;

/////////////////////////////////////
//
// IloAbstractMapI
//
class IloAbstractMapI : public IloRttiEnvObjectI {
  ILORTTIDECL
public:
  IloAbstractMapI(IloEnvI*);
  virtual IloDiscreteDataCollectionI* getIndexer() const = 0;
  virtual IloDiscreteDataCollectionI* getIndexer(IloInt i) const = 0;
  virtual void lock() = 0;
  virtual void unlock() = 0;
  virtual IloBool isLocked() const = 0;
  virtual const char* getName() const = 0;
  virtual void setName(const char* name) = 0;
  virtual IloInt getNbDim() const = 0;
  virtual IloInt getTotalSize() const = 0;
  virtual IloInt getSize() const = 0;
  virtual IloMapItem getAt(IloMapIndexArray indices) const = 0;
  virtual void setAt(IloMapIndexArray indices, IloMapItem value) = 0;
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value) = 0;
  virtual IloMapItem getAtAbsoluteIndex(IloIntArray indices) const = 0;
  virtual void display(ILOSTD(ostream)& out) const = 0;
};

/////////////////////////////////////
//
// IloMapI
//
class IloMapI : public IloAbstractMapI {
  ILORTTIDECL
private:
  IloBool _locked;
  IloBool _validIdx;
  IloInt _nbDim;
  IloArrayI* _values;
  IloDiscreteDataCollectionI* _indexer;
  char* _name;
  IloTupleSchemaI* _auxData;
  IloMapItem::Type _itemType;
  void* getData(IloInt i) const { return  _values->getData(i); }  
  IloMapItem getAt(IloMapIndex index) const {
    IloInt absIdx = getAbsoluteIndex(index);
    IloMapItem item = getAtAbsoluteIndex(absIdx);
    return item;
  }
  IloMapItem getSubMapExpr(IloMapIndexArray indices, IloInt currIdx) const;
public:
  void setAtAbsoluteIndex(IloInt i, IloMapItem value);
  IloMapItem getAtAbsoluteIndex(IloInt i) const;
  void setType(IloMapItem::Type type);
public:
  IloMapI(IloEnvI*, IloInt, IloDiscreteDataCollectionI*, IloInt, IloBool);
  IloMapI(IloEnvI*, IloInt, IloDiscreteDataCollectionI*, IloArrayI*, IloBool);
  IloMapI(IloEnvI*, const IloMapI*);
  virtual ~IloMapI();
  IloMapI* getCopy() const;
  IloMapI* getLazyCopy() const;
  virtual void lock();
  virtual void unlock();
  virtual IloBool isLocked() const;
  IloMapItem::Type getItemType() const { return _itemType; }
  IloMapItem::Type getLastItemType() const;
  IloTupleSchemaI* getAuxData() const {return (IloTupleSchemaI*)_auxData;}
  void setAuxData(IloTupleSchemaI* auxData) { _auxData = auxData; }
  void zeroData() {_values->zeroData(); }
  IloInt getNbDim() const { return _nbDim; }
  IloInt getTotalSize() const;
  IloInt getSize() const { return _indexer->getSize(); }
  IloInt getNbElt() const { return _validIdx ? getSize() : 1; }
  void* getBase(IloInt i) const { return _values->getBase(i); }
  IloInt getMod() const { return _values->getMod(); }
  const char* getName() const { return _name; }
  void setName(const char* name);
  IloArrayI* getValues() const { return _values; }
  IloDiscreteDataCollectionI* getIndexer() const { return _indexer; }
  IloDiscreteDataCollectionI* getIndexer(IloInt i) const;

  IloBool hasValidIndexer() const { return _validIdx; }

  IloBool isIndexerIntCompatible() const {
    return ((_indexer->isIntRange())
	    || (_indexer->isIntSet())
	    || (_indexer->isNumSet()));
  }
  IloBool isIndexerNumCompatible() const {
    return ((_indexer->isNumSet())
	    || (_indexer->isIntRange())
	    || (_indexer->isIntSet()));
  }
  IloBool isIndexerSymbolCompatible() const {
    return (_indexer->isSymbolSet());
  }
  IloBool isIndexerTupleCompatible() const {
    return (_indexer->isTupleSet());
  }

  IloInt getAbsoluteIndex(IloAny idx) const {
    IloTestAndRaise(_indexer, "Using IloMap without indexer");
    return _indexer->getIndex(idx);
  }
  IloInt getAbsoluteIndex(IloMapIndex idx) const;
  IloMapVectorIndexI* getVectorIndex() const;
  using IloAbstractMapI::setAt;
  using IloAbstractMapI::getAt;
  virtual IloMapItem getAt(IloMapIndexArray indices) const;
  virtual void setAt(IloMapIndexArray indices, IloMapItem value);
  using IloAbstractMapI::setAtAbsoluteIndex;
  using IloAbstractMapI::getAtAbsoluteIndex;
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value);
  virtual IloMapItem getAtAbsoluteIndex(IloIntArray indices) const;
  void display(ILOSTD(ostream)& out) const;
};

/////////////////////////////////////
// IloMapVectorIndexI
//
class IloMapIterator;
class IloMapVectorIndexI {
  friend class IloMapIterator;
  IloMapIndexerI _indexers;
  IloIntArray _absVector;
  IloMapVectorIndexI(const IloMapI* m, IloIntArray index);
public:
  IloMapVectorIndexI(const IloMapI* m);
  ~IloMapVectorIndexI();
  IloInt getIntAt(IloInt dim) const;
  IloNum getNumAt(IloInt dim) const;
  IloAny getAnyAt(IloInt dim) const;
  IloSymbol getSymbolAt(IloInt dim) const {
    return IloSymbol((IloSymbolI*)getAnyAt(dim));
  }
  const char* getStringAt(IloInt dim) const {
    return ((IloSymbolI*)getAnyAt(dim))->getString();
  }
  void display(ILOSTD(ostream)& out) const;
};

/////////////////////////////////////
// IloMapIterator
//
class IloMapIterator {
protected:
  IloGenAlloc* _heap;
  const IloMapI* _map;
  IloIntArray _size;
  IloIntArray _index;
  IloArray<IloArrayI*> _currSubMap;
  IloBool _ok;
  IloMapVectorIndexI * _vi;
  virtual IloBool next(IloInt);
  void resetIndex(IloInt i);
  void checkVectorIndex();
public:
  virtual IloBool next();

  virtual void reset();

  IloMapIterator(IloGenAlloc* heap, const IloMapI* m);

  IloMapIterator(const IloMapI* m = 0);

  virtual ~IloMapIterator();

  virtual IloBool isOpen() const;

  IloGenAlloc* getHeap() const { return _heap; }

  const IloMapI* getMapI() const { return _map; }

  IloBool ok() const {
    return _ok;
  }

  void setNotOk() { _ok = IloFalse; }

  void operator++() {
    _ok = next();
  }

  void operator delete(void *p, size_t sz);
#ifdef ILODELETEOPERATOR
  void operator delete(void *p, const IloEnvI *);
  void operator delete(void *p, const IloEnv &);
#endif
  void displayCurrentAbsoluteVectorIndex(ILOSTD(ostream)& out) const;

  IloArrayI* getCurrentEltArray() const {
    return _currSubMap[_map->getNbDim()-1];
  }

  IloInt getCurrentDeepestIndex() const {
    return _index[_map->getNbDim()-1];
  }
  IloMapVectorIndexI* getVectorIndex() const;
  void fillVectorIndex(IloMapVectorIndexI* vi) const;
  IloInt vectorIndexGetIntAt(IloInt dim);
  IloNum vectorIndexGetNumAt(IloInt dim);
  IloAny vectorIndexGetAnyAt(IloInt dim);
  IloSymbol vectorIndexGetSymbolAt(IloInt dim);
  const char* vectorIndexGetStringAt(IloInt dim);
  void displayVectorIndex(ILOSTD(ostream)& out);
};

/////////////////////////////////////
//
// Handle of SubMap extractables
//

#define ILOSUBMAPHANDLE(_typeElt, _super) \
class name2(_typeElt, SubMapExprI); \
class name2(_typeElt, SubMapExpr) : public _super { \
  ILOEXTRHANDLE(name2(_typeElt, SubMapExpr), _super) \
public: \
  name2(_typeElt, SubMapExpr) operator[](IloInt idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloNum idx) const; \
  name2(_typeElt, SubMapExpr) operator[](const char* idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloSymbol idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloTuple idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloIntExprArg idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloNumExprArg idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloSymbolExprArg idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloTupleExprArg idx) const; \
  name2(_typeElt, SubMapExpr) operator[](IloTuplePattern idx) const; \
  name2(_typeElt, SubMapExpr) operator[](int idx) const {\
        return this->operator[]((IloInt)idx);\
  }\
  name2(_typeElt, SubMapExpr) operator[](char idx) const {\
        return this->operator[]((IloInt)idx);\
  }\
  name2(_typeElt, SubMapExpr) operator[](short idx) const {\
        return this->operator[]((IloInt)idx);\
  }\
};

ILOSUBMAPHANDLE(IloInt, IloIntExprArg)
ILOSUBMAPHANDLE(IloIntExpr, IloIntExprArg)
ILOSUBMAPHANDLE(IloNum, IloNumExprArg)
ILOSUBMAPHANDLE(IloNumExpr, IloNumExprArg)
ILOSUBMAPHANDLE(IloSymbol, IloSymbolExprArg)

#ifdef ADV_CONCERT_WITH_NEW_CP_API
/////////////////////////////////////
// IloOplIntExpr
//
class IloOplIntExpr {
public:
  enum Type {
    Invalid  =0,
    IntConst =1,
    IntExpr  =2,
    IntIndex =3
  };
private:
  Type   _type;
  union {
    IloInt _int;
    IloIntExprI* _intExpr;
  } _data;
  IloOplIntExpr(Type t, IloIntExprI* x) :_type(t) { _data._intExpr = x;}
public:
  IloOplIntExpr();
  IloOplIntExpr(const IloOplIntExpr& i);
  IloOplIntExpr(IloIntExprArg x);
  IloOplIntExpr(IloIntVar x);
  IloOplIntExpr(IloIntIndex x);
  IloOplIntExpr(IloInt x);
  IloOplIntExpr(const int x);
  IloOplIntExpr(const char x);
  IloOplIntExpr(const short x);
  ~IloOplIntExpr() {}

  Type getType() const { return _type; }

  IloOplIntExpr getClone(IloEnvI* env);
  void operator= (const IloOplIntExpr& i);
  IloBool isValid() const { return !(Invalid == _type); }
  IloBool isConstant() const { return !(_type & 2); }
  IloBool isExtractable() const { return (_type & 2); }
  IloBool isIndex() const { return (IntIndex == _type); }

  IloInt asInt() const {
    IloAssert(_type == IntConst, "OPL int expr Item is not an integer.");
    return _data._int;
  }
  void getValue(IloInt& res) { res = asInt(); }
  IloIntExprI* asIntExpr() const {
    IloAssert(isExtractable(), "OPL int expr is not an integer expression.");
    return (IloIntExprI*)_data._intExpr;
  }
  void getValue(IloIntExpr& res) { res = asIntExpr(); }
  void display(ILOSTD(ostream)& out) const;
  void accept(IloExtractableI* parent,IloExtractableVisitor* v);
  static Type GetType(IloInt) { return IntConst; }
  static Type GetType(IloIntExprArg) { return IntExpr; }
  static Type GetType(IloIntIndex) { return IntIndex; }
  IloInt makeValue(IloModelEvaluatorI*) const;
};

inline ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, const IloOplIntExpr & ext) {
  ext.display(out);
  return (out);
}
#endif //ifdef ADV_CONCERT_WITH_NEW_CP_API

#endif
