#ifndef __CONCERT_ilomemmgrH
#define __CONCERT_ilomemmgrH

#ifndef __CONCERT_ilosysH
# include <ilconcert/ilosys.h>
#endif
#ifndef __CONCERT_ilorttiH
# include <ilconcert/ilortti.h>
#endif

class IloGenAlloc : public IloRtti {
  ILORTTIDECL
public:
  IloGenAlloc() { }
  virtual ~IloGenAlloc();
  virtual void * alloc(size_t size) const = 0;
  virtual void free(void * ptr, size_t size) const = 0;
  virtual void saveValue(IloAny * slot) const = 0;
  virtual void saveValue(IloInt * slot) const = 0;
  virtual void saveValue(IloNum * slot) const = 0;
};

inline void * operator new (size_t sz, const IloGenAlloc * gen) {
  return gen->alloc(sz);
}
#ifdef ILODELETEOPERATOR
inline void operator delete(void*, const IloGenAlloc*) { }
#endif
#if defined(ILONEWOPERATOR)
inline void* operator new [] (size_t sz, const IloGenAlloc * gen) {
  return gen->alloc(sz);
}
# if defined(ILODELETEOPERATOR)
inline void operator delete[] (void*, const IloGenAlloc*) { }
# endif
#endif

class IloConcertAlloc : public IloGenAlloc {
  ILORTTIDECL
private:
  IloEnvI * _env;
public:
  IloConcertAlloc(IloEnvI * env);
  IloEnvI* getEnvI() const { return _env; }
  void * alloc(size_t size) const;
  void free(void * ptr, size_t size) const;
  void saveValue(IloAny * slot) const;
  void saveValue(IloInt * slot) const;
  void saveValue(IloNum * slot) const;
};

class IloAllocationStack;

class IloStackGenAlloc : public IloGenAlloc {
  ILORTTIDECL
private:
  IloAllocationStack* _stack;
public:
  IloStackGenAlloc(IloAllocationStack* s)
    :IloGenAlloc(),_stack(s) {}
  void * alloc(size_t size) const;
  void free(void * ptr, size_t size) const;
  void saveValue(IloAny * slot) const;
  void saveValue(IloInt * slot) const;
  void saveValue(IloNum * slot) const;
};

typedef IloGenAlloc IloMemoryManagerI;

//----------------------------------------------------------------------
class IloMemoryManager {
private:
  IloMemoryManagerI * _impl;
public:
  IloMemoryManager(IloMemoryManagerI * impl = 0) : _impl(impl) { }
  IloMemoryManagerI * getImpl() const { return _impl; }
  void * alloc(size_t size) const {
    IloAssert(getImpl() != 0, "IloMemoryManager : empty handle");
    return getImpl()->alloc(size);
  }
  void free(void * ptr, size_t size) const {
    IloAssert(getImpl() != 0, "IloMemoryManager : empty handle");
    getImpl()->free(ptr, size);
  }
  void saveValue(IloAny * slot) const {
    IloAssert(getImpl() != 0, "IloMemoryManager : empty handle");
    getImpl()->saveValue(slot);
  }
  void saveValue(IloInt * slot) const {
    IloAssert(getImpl() != 0, "IloMemoryManager : empty handle");
    getImpl()->saveValue(slot);
  }
  void saveValue(IloNum * slot) const {
    IloAssert(getImpl() != 0, "IloMemoryManager : empty handle");
    getImpl()->saveValue(slot);
  }
};

inline void* operator new (size_t sz, const IloMemoryManager& mm) {
  IloAssert(mm.getImpl(), "Using empty IloMemoryManager handle.");
  return mm.getImpl()->alloc(sz);
}
#ifdef ILODELETEOPERATOR
inline void operator delete(void*, const IloMemoryManager&) {}
inline void operator delete(void*, IloMemoryManager) {}
#endif

#if defined(ILONEWOPERATOR)
inline void* operator new[] (size_t s, const IloMemoryManager& mm) {
  IloAssert(mm.getImpl(), "Using empty IloMemoryManager handle.");
  return (operator new (s, mm.getImpl()));
}
# ifdef ILODELETEOPERATOR
inline void operator delete[] (void*, const IloMemoryManager&) {}
inline void operator delete[] (void*, IloMemoryManager) {}
# endif
#endif

class IloEnv IloGetEnv(IloMemoryManager mm);

#endif /* __CONCERT_ilomemgrH */
