// --------------------------------------------------*- C++ -*---------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilooplcpiH
#define __CONCERT_ilooplcpiH

#if !defined(__CONCERT_ilomodelH)
# include <ilconcert/ilomodel.h>
#endif

#if !defined(__CONCERT_ilomapH)
# include <ilconcert/ilomap.h>
#endif

#ifdef ADV_CONCERT_WITH_NEW_CP_API

class IloAtLeastAtMostIndexSubSetMapI : public IloConstraintI {
  ILOEXTRDECL
public:
  enum Type {
    Exactly,
    AtLeast,
    AtMost
  };
private:
  Type _type;
  IloIndexSubSet _subset;
  IloIntExprMap  _exprMap;
  IloOplIntExpr  _value; // IloInt || IloIntIndex
  IloOplIntExpr  _count; // IloInt || IloIntIndex || IloIntExpr
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloAtLeastAtMostIndexSubSetMapI(IloEnvI* env,
				  Type t,
				  IloIndexSubSet s,
				  IloIntExprMap map,
				  IloOplIntExpr v,
				  IloOplIntExpr c,
				  const char* name = 0);
  virtual ~IloAtLeastAtMostIndexSubSetMapI();
  Type getType() const {return _type;}
  IloIndexSubSet getIndexSubSet() const {return _subset;}
  IloIntExprMap getExprMap() const {return _exprMap;}
  IloOplIntExpr getValueExpr() const {return _value;}
  IloOplIntExpr getCountExpr() const {return _count;}
  ILOEXTROTHERDECL
};

class IloInverseMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloIntExprMap _exprMap;
  IloIntExprMap _exprMapInv;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloInverseMapI(IloEnvI* env, IloIntExprMap map1, IloIntExprMap map2,
		 const char* name = 0);
  virtual ~IloInverseMapI();
  IloIntExprMap getExprMap() const {return _exprMap;}
  IloIntExprMap getExprMapInv() const {return _exprMapInv;}
  ILOEXTROTHERDECL
};

class IloPackMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloIntExprMap _load;
  IloIntExprMap _where;
  IloIntMap     _weight;
  IloOplIntExpr _numBinUsed;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloPackMapI(IloEnvI* env, IloIntExprMap ld, IloIntExprMap wh, IloIntMap we,
	      const char* name = 0);
  IloPackMapI(IloEnvI* env, IloIntExprMap ld, IloIntExprMap wh, IloIntMap we,
	      IloIntExprArg numBinUsed, const char* name = 0);
  IloPackMapI(IloEnvI* env, IloIntExprMap ld, IloIntExprMap wh, IloIntMap we,
	      IloOplIntExpr numBinUsed, const char* name = 0);
  IloPackMapI(IloEnvI* env, IloIntExprMap ld, IloIntExprMap wh, IloIntMap we,
	      IloInt numBinUsed, const char* name = 0);
  virtual ~IloPackMapI();
  IloIntExprMap getLoadMap() const {return _load;}
  IloIntExprMap getWhereMap() const {return _where;}
  IloIntMap getWeightMap() const {return _weight;}
  IloOplIntExpr getNumBinUsed() const {return _numBinUsed;}
  IloBool withNumBinUsed() const {return _numBinUsed.isValid();} 
  ILOEXTROTHERDECL
};

class IloTableConstraintMapI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloIndexSubSet _subset;
  IloIntExprMap  _exprMap;
  IloIntMap      _values;
  IloBool        _compatible;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTableConstraintMapI(IloEnvI* env, IloIndexSubSet s, IloIntExprMap m,
			 IloIntMap vals, IloBool compat, const char* name = 0);
  virtual ~IloTableConstraintMapI();
  IloIndexSubSet getIndexSubSet() const {return _subset;}
  IloIntExprMap getExprMap() const {return _exprMap;}
  IloIntMap getValues() const {return _values;}
  IloBool isCompatible() const {return _compatible; }
  ILOEXTROTHERDECL
};

#endif //ifdef ADV_CONCERT_WITH_NEW_CP_API

#endif

