// -------------------------------------------------------------- -*- C++ -*-
// File: ilconcert/ilorandom.h
// --------------------------------------------------------------------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

// -*- C++ -*- header file.
#ifndef __CONCERT_ilorandomH
#define __CONCERT_ilorandomH

#include <ilconcert/iloenv.h>

#ifdef ILONEWRAND
#define ILORANDOM_SIZE 624
#else
#define ILORANDOM_SIZE 532
#endif

//-----------------------------------------------------------------------

class IloRandomStateI {
public:
#ifdef ILONEWRAND
  typedef IloUInt Type;
#else
  typedef IloNum Type;
#endif
protected:
  IloInt  _index;
  Type  * _values;
public:
  IloRandomStateI();
  virtual ~IloRandomStateI();
  void display(ILOSTD(ostream) & out) const;
  void copy(const IloRandomStateI * state);
  Type getElement(IloInt i) const { return _values[i]; }
  IloInt getIndex() const { return _index; }
  void setElement(IloInt i, Type val) { _values[i] = val; }
  void setIndex(IloInt i) { _index = i; }
};

class IloConcertRandomStateI : public IloRandomStateI {
private:
  IloEnvI * _env;
public:
  typedef IloRandomStateI::Type Type;
  IloConcertRandomStateI(IloEnvI * env);
  virtual ~IloConcertRandomStateI();

  IloEnvI * getEnv() const { return _env; }
  void operator delete(void *, size_t);
#ifdef ILODELETEOPERATOR
  void operator delete(void *, const IloEnvI *);
  void operator delete(void *, const IloEnv &);
#endif
};

class IloRandomBaseI {
protected:
  IloRandomStateI * _state;
public:
  IloRandomBaseI(IloRandomStateI * state = 0);
  virtual ~IloRandomBaseI();
  void advance(IloInt count = 1);
  void copy(const IloRandomBaseI * rb) { _state->copy(rb->_state); }
  void copy(const IloRandomStateI * state) { _state->copy(state); }
  void displayState(ILOSTD(ostream) & out) const { _state->display(out); }
  virtual IloNum getFloat() = 0;
  IloNum getFloatAux();
  IloInt getInt(IloInt n);
  void reSeed(IloInt);

  IloRandomStateI::Type getElement(IloInt i) const {
    return _state->getElement(i);
  }
  IloInt getIndex() const { return _state->getIndex(); }
  void setElement(IloInt i, IloRandomStateI::Type value) {
    _state->setElement(i, value);
  }
  void setIndex(IloInt index) { _state->setIndex(index); }

#ifdef ILONEWRAND
private:
  unsigned long get();
#endif
};

class IloRandomI : public IloRandomBaseI {
private:
  IloEnvI * _env;
public:
  IloRandomI(IloEnvI *env, IloInt seed = 0);
  virtual ~IloRandomI();
  virtual IloBool canDelete() const;
  void display(ILOSTD(ostream) &os) const;
  IloEnvI * getEnv() { return _env; }
  IloNum getFloat();
  IloRandomI * makeClone(IloEnvI * env) const;
  void operator delete(void *, size_t);
#ifdef ILODELETEOPERATOR
  void operator delete(void *, const IloEnvI *);
  void operator delete(void *, const IloEnv &);
#endif
  ILOEXTENSIONI
};

class IloUniqueRandomI : public IloRandomI {
public:
  IloUniqueRandomI(IloEnvI * env, IloInt seed = 0);
  IloBool canDelete() const;
};

class ILO_EXPORTED IloRandom {
protected:
  IloRandomI *_impl;
public:
  IloRandom() : _impl(0) { }
  IloRandom(const IloEnv env, IloInt seed = 0);
  IloRandom(IloRandomI *impl) : _impl(impl) { }
  IloRandom(const IloRandom& rand) : _impl(rand._impl) { }

  IloRandomI *getImpl() const { return _impl; }

  void copy(IloRandom r) {
    assert(_impl != 0);
    assert(r._impl != 0);
    _impl->copy(r._impl);
  }
  void advance(IloInt n = 1) const {
    assert(_impl != 0);
    _impl->advance(n);
  }
  void display(ILOSTD(ostream) &os) const {
    assert(_impl != 0);
    _impl->display(os);
  }

  void end() {
    if (_impl != 0 && _impl->canDelete())
      delete _impl;
    _impl = 0;
  }

  IloNum getFloat() const {
    assert(_impl != 0);
    return _impl->getFloat();
  }

  IloInt getInt(IloInt n) const {
    assert(_impl != 0);
    return _impl->getInt(n);
  }

  IloEnv getEnv() const {
    assert(_impl != 0);
    return _impl->getEnv();
  }

  IloRandom makeClone(IloEnv env) const {
    assert(_impl != 0);
    assert(env.getImpl() != 0);
    return _impl->makeClone(env.getImpl());
  }

  void reSeed(IloInt seed) {
    assert(_impl != 0);
    _impl->reSeed(seed);
  }

  ILOEXTENSIONH(IloRandom)
};


ILOSTD(ostream)& operator << (ILOSTD(ostream) &os, const IloRandom& r);

#endif
