// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __XML_iloreaderH
#define __XML_iloreaderH


#if !defined(__CONCERT_ilosysH)
#include <ilconcert/ilosys.h>
#endif


#if !defined(__CONCERT_iloenvH)
#include <ilconcert/iloenv.h>
#endif

#if !defined(__CONCERT_ilosolutionH)
#include <ilconcert/ilosolution.h>
#endif

#if !defined(__CONCERT_iloanyH)
#include <ilconcert/iloany.h>
#endif

#if !defined(__CONCERT_ilosetH)
#include <ilconcert/iloset.h>
#endif

#if !defined(__XML_iloxmlexceptionH)
#include <ilconcert/ilxml/iloxmlexception.h>
#endif

#if !defined(__XML_iloxmlhashH)
#include <ilconcert/ilxml/iloxmlhash.h>
#endif



#ifdef ILO_RS6000
void* operator new (size_t sz, const IloEnv& env);
void* operator new[] (size_t s, const IloEnv& env);
#endif

#if !defined(__XML_xmlH)
#include <ilconcert/ilxml/iloxml.h>
#endif
#if !defined(__XML_parserH)
#include <ilconcert/ilxml/iloparser.h>
#endif
#if !defined(__XML_listH)
#include <ilconcert/ilxml/ilolist.h>
#endif
#if !defined(__XML_memoryH)
#include <ilconcert/ilxml/ilomemory.h>
#endif
#if !defined(__XML_poolH)
#include <ilconcert/ilxml/ilopool.h>
#endif

#if defined(IL_STD)
IL_STDUSE
#endif /* !ILO_STD */

class IloXmlReaderI;


class ILO_EXPORTED IloXmlReader {
private:
  IloXmlReaderI* _impl;

public:

  const IloXmlReaderI* getImpl();

  void end();

  IloXmlObjectHandler& getObjectHandler();

  IloBool isSerialized(IloInt id);

  IloIntArray* getSerialized();

  IloIntArray* getSolutionSerialized();

  void addLink2Obj(IloInt id, void* object);

  IloAny getObjectById(IloInt id);

  IloXmlReader(IloEnv env, const char* fileName=0);

  IloXmlReader(IloXmlReaderI* impl): _impl(impl) {};

  void deleteAllocatedMemory(char* pointer);

  void deleteAllocatedMemory(const char* pointer);


  void setfileName(const char* fileName);

  IloXmlElement* getRoot();

  IloEnv getEnv();

  IloEnvI* getEnvImpl();


  IloBool getIntAttribute(IloXmlElement* element, const char* attribute, IloInt &value);

  IloBool getNumAttribute(IloXmlElement* element, const char* attribute, IloNum &value);

  IloInt string2Int(const char* str);

  IloNum string2Num(const char* str);

  IloIntArray string2IntArray(const char* str);

  IloNumArray string2NumArray(const char* str);

  IloIntSet string2IntSet(const char* str);

  IloNumSet string2NumSet(const char* str);

  IloXmlElement* getFirstSubElement(IloXmlElement* element);

  const char* readAttribute(IloXmlElement* element, const char* attribute);

  const char* readComment(IloXmlElement* element);

  const char* readText(IloXmlElement* element);

  const char* readData(IloXmlElement* element);

  const char* readCData(IloXmlElement* element);

  IloInt getChildrenCardinal(IloXmlElement* element);

  IloXmlElement* findElement(IloXmlElement* root, const char* tag, const char* attribute, const char* value);

  IloXmlElement* findElementByTag(IloXmlElement* element, const char* tag);

  IloBool openDocument();
  IloBool openStream(istream& file);

  IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloInt Xml_Id);

  IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloRtti* exprI);

  IloBool checkTypeOfObjectById(IloTypeInfo type, IloRtti* exprI);

  IloBool checkTypeOfObjectById(IloTypeInfo type, IloInt Xml_Id);

};



class ILO_EXPORTED IloReaderError : public IloException {
public:
  IloReaderError(const char* message);
  virtual ~IloReaderError();
};


//VB
class IloXmlReaderI : public IloEnvObjectI {
private:
  char* _fileName;
  IloXmlDocument* _document;
  IloXmlElement* _root;

  IloXmlObjectHandler _voidHandler;

private:
  IloBool checkStream(istream& file);
  IloBool checkIfSolutionObject(IloInt id);

public:
  ~IloXmlReaderI();

  void addLink2Obj(IloInt id, void* object);
  IloAny getObjectById(IloInt id);

  IloXmlObjectHandler& getObjectHandler() {
      return _voidHandler;
  }

  IloBool isSerialized(IloInt id);
  IloIntArray* getSerialized();
  IloIntArray* getSolutionSerialized();


  IloXmlReaderI(IloEnv env, const char* fileName=0);
  void deleteAllocatedMemory(char* pointer);
  void deleteAllocatedMemory(const char* pointer);
  void setfileName(const char* fileName);
  IloXmlElement* getRoot();

  IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloInt Xml_Id);
  IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloRtti* exprI);
  IloBool checkTypeOfObjectById(IloTypeInfo type, IloRtti* exprI);
  IloBool checkTypeOfObjectById(IloTypeInfo type, IloInt Xml_Id);

  IloBool getIntAttribute(IloXmlElement* element, const char* attribute, IloInt &value);
  IloBool getNumAttribute(IloXmlElement* element, const char* attribute, IloNum &value);
  IloInt string2Int(const char* string);
  IloNum string2Num(const char* string);
  IloIntArray string2IntArray(const char* string);
  IloNumArray string2NumArray(const char* string);
  IloIntSet string2IntSet(const char* string);
  IloNumSet string2NumSet(const char* string);
  IloXmlElement* getFirstSubElement(IloXmlElement* element);
  const char* readAttribute(IloXmlElement* element, const char* attribute);
  const char* readComment(IloXmlElement* element);
  const char* readText(IloXmlElement* element);
  const char* readData(IloXmlElement* element);
  const char* readCData(IloXmlElement* element);
  IloInt getChildrenCardinal(IloXmlElement* element);
  IloXmlElement* findElement(IloXmlElement* root, const char* tag, const char* attribute, const char* value);
  IloXmlElement* findElementByTag(IloXmlElement* element, const char* tag);
  IloBool openDocument();
  IloBool openStream(istream& file);
};

#endif
