// ---------------------------------------------------------------- -*- C++ -*-
// File: ilconcert/iloskiplist.h
// ----------------------------------------------------------------------------
//  Copyright (C) 1999-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// ----------------------------------------------------------------------------

#ifndef __CONCERT_IloSkipListH
#define __CONCERT_IloSkipListH

#ifndef __CONCERT_IloParamIH
#include <ilconcert/iloparami.h>
#endif

//------------------------------------------------------------
// IloSkipListNodeI
//------------------------------------------------------------

class IloSkipListNodeI {

  friend class IloSkipListI;
  friend class IloIntervalI;
  friend class IloFunctionSegmentI;
  friend class IloIntervalListI;
  friend class IloSegmentedFunctionI;
  friend class IloAnySetSegmentI;
  friend class IloNumToAnySetStepFunctionI;

 private:
  IloInt             _level;
  IloNum             _start;
  IloSkipListNodeI** _forward;
  IloSkipListNodeI*  _prev;

 public:
  IloSkipListNodeI();
  IloSkipListNodeI(IloEnvI* env, IloInt level);  
  ~IloSkipListNodeI() {}

  void end(IloEnvI* env);

  IloInt getLevel() const { return _level; }
  IloNum getStart() const { return _start; }
  IloSkipListNodeI* getNext() const { return _forward[0]; }
  IloSkipListNodeI* getPrev() const { return _prev; }
  IloBool isHeader() const { return _start == - IloInfinity; }
  IloBool isTail() const { return _start == IloInfinity; } 
  void setStart(IloNum start) { _start = start; }
  IloNum getXLeft() const { return _start; }
#if defined(ILO_MSVC)
  IloNum getXRight() const;
#else
  IloNum getXRight() const { return _forward[0]->_start; }
#endif
};

//------------------------------------------------------------
// IloSkipListI
//------------------------------------------------------------

class IloRandomI;

#define ILOSKIPLISTMAXLEVEL 20

class IloSkipListI : public IloParameterI {
  friend class IloIntervalListI;
  friend class IloSegmentedFunctionI;
  friend class IloNumToAnySetStepFunctionI;

private:
  IloEnvI*           _env;
  IloInt             _level;
  IloSkipListNodeI*  _header;
  IloSkipListNodeI*  _tail;
  IloSkipListNodeI*  _recycle;
  IloSkipListNodeI** _slice0;
  IloSkipListNodeI** _slice1;
  IloSkipListNodeI** _sliceEnd;
  IloSkipListNodeI** _slice0R;
  IloInt             _randomBits;
  IloInt             _randomsLeft;
  IloInt             _BitsInRandom;
  IloInt             _MaxLevel;
  IloInt             _RandomType;
  IloInt             _RandomMask;
	IloRandomI*        _random;

public:
  IloSkipListI(IloEnvI* env, const char* name =0);  
  virtual ~IloSkipListI();  

  void init(IloInt maxLevel =ILOSKIPLISTMAXLEVEL);
  void reset();
  IloBool removeItem(IloNum start);
  IloSkipListNodeI* search       (IloNum start);
  IloSkipListNodeI* insert       (IloNum start);
  IloSkipListNodeI* insertFirst  (IloNum start);
  IloSkipListNodeI* insertLast   (IloNum start);
  IloSkipListNodeI* insertSlice0 (IloNum start);
  IloSkipListNodeI* searchSlice0 (IloNum start);
  IloSkipListNodeI* searchSlice1 (IloNum start);
  void removeFirst   ();
  void removeLast    ();
  void removeSlice0R ();
  void nextSlice0    ();
  void nextSlice1    ();
  void remove01      ();
  IloSkipListNodeI* getHeader() const { return _header; }
  IloSkipListNodeI* getTail() const { return _tail; }
  IloInt getLevel() const { return _level; }
  IloBool check() const;
  void setMaxLevel(IloInt maxLevel) {
    if (maxLevel < _MaxLevel)
      _MaxLevel = maxLevel;
  }
  void setRandomMode16() {
    _RandomType = 4;
    _RandomMask = 15;
  }
  void setRandomMode8() {
    _RandomType = 3;
    _RandomMask = 7;
  }
  void setRandomMode4() {
    _RandomType = 2;
    _RandomMask = 3;
  }
  void setRandomMode2() {
    _RandomType = 1;
    _RandomMask = 1;
  }

protected:
  typedef enum { rawNode=0, headNode, tailNode } NodeType;
  virtual IloSkipListNodeI* allocateNode(NodeType type,
                                         IloInt level);
  virtual void destroyNode(IloSkipListNodeI* node);
  IloSkipListNodeI* destroyAndGetNext(IloSkipListNodeI* node);

  IloBool isDestroyed() const { return (_env==0); }
  void setDestroyed() { _env = 0; }
  void destroyAll();  // to be called by each subclass's destructor

private:
  IloInt randomLevel();
  void recycle(IloSkipListNodeI* p);
  IloSkipListNodeI* newNode();
};

#endif
