// -------------------------------------------------------*- C++ -*----------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotupleH
#define __CONCERT_ilotupleH

#if !defined(__CONCERT_ilotuplecollectionH)
# include <ilconcert/ilotuplecollection.h>
#endif

#if !defined(__CONCERT_ilostringH)
# include <ilconcert/ilostring.h>
#endif
#if !defined(__CONCERT_ilosymbolH)
# include <ilconcert/ilosymbol.h>
#endif


class IloTuple;

class IloTupleCollectionI;
class IloTupleDataColumnI;
class IloTupleBufferI;
class IloTuplePatternI;

class IloIntCollectionExprArg;
class IloNumCollectionExprArg;
class IloSymbolCollectionExprArg;

class IloIntMap;
class IloNumMap;

//-------------------------------------------------
// Tuple knows the collection + a row index
//-------------------------------------------------
class IloTupleI : public IloRttiEnvObjectI {
  ILORTTIDECL
protected:
  IloTupleCollectionI* _coll;
  IloInt _index;
  IloIntArray makeColumnIndexArray(IloStringArray name) const;

  friend class IloTupleIterator;
  void reuse(IloTupleCollectionI* coll) { _coll = coll; _index = 0; }
public:
  friend class IloTupleBufferI;
  virtual ~IloTupleI(){  }

  IloTuplePatternI* makeConstPattern() const;

  /* <p> . </p> **/
  IloBool equal(IloTupleI*);

  IloTupleI(IloTupleCollectionI* coll, IloInt i=-1);

  void setCollection(IloTupleCollectionI* coll){ _coll = coll;}
  const IloTupleCollectionI* getCollection() const { return _coll; }

  IloTupleSchemaI* getSchema() const;

  IloInt getColumnIndex(const char* name) const;
  IloInt getColumnIndex(const IloSymbolI* s) const;

  IloInt getIndex() const { return _index; }
  void setIndex(IloInt i);
  //
  virtual IloInt getIntValue(const char* col) const;
  virtual IloNum getNumValue(const char* col) const;
  virtual IloAny getAnyValue(const char* col) const;
  virtual IloSymbol getSymbolValue(const char* col) const;
  virtual const char* getStringValue(const char* col) const;
  virtual IloTuple makeTupleValue(const char* col) const;

  virtual IloIntCollectionI* getIntCollectionValue(const char* col) const;
  virtual IloNumCollectionI* getNumCollectionValue(const char* col) const;
  virtual IloAnyCollectionI* getAnyCollectionValue(const char* col) const;

  virtual IloInt getIntValue(IloInt index) const;
  virtual IloNum getNumValue(IloInt index) const;
  virtual IloAny getAnyValue(IloInt index) const;

  virtual IloIntCollectionI* getIntCollectionValue(IloInt index) const;
  virtual IloNumCollectionI* getNumCollectionValue(IloInt index) const;
  virtual IloAnyCollectionI* getAnyCollectionValue(IloInt index) const;

  virtual IloSymbol getSymbolValue(IloInt index) const;
  virtual const char* getStringValue(IloInt index) const;
  virtual IloTuple makeTupleValue(IloInt index) const;

  //
  virtual IloInt getIntValue(IloIntArray path) const;
  virtual IloNum getNumValue(IloIntArray path) const;
  virtual IloAny getAnyValue(IloIntArray path) const;

  virtual IloIntCollectionI* getIntCollectionValue(IloIntArray path) const;
  virtual IloNumCollectionI* getNumCollectionValue(IloIntArray path) const;
  virtual IloAnyCollectionI* getAnyCollectionValue(IloIntArray path) const;


  virtual IloSymbol getSymbolValue(IloIntArray path) const;
  virtual const char* getStringValue(IloIntArray path) const;
  virtual IloTuple makeTupleValue(IloIntArray path) const;

  virtual IloInt getIntValue(IloStringArray path) const;
  virtual IloNum getNumValue(IloStringArray path) const;
  virtual IloAny getAnyValue(IloStringArray path) const;
  virtual IloSymbol getSymbolValue(IloStringArray path) const;
  virtual const char* getStringValue(IloStringArray path) const;
  virtual IloTuple makeTupleValue(IloStringArray path) const;

   virtual IloIntCollectionI* getIntCollectionValue(IloStringArray path) const;
   virtual IloNumCollectionI* getNumCollectionValue(IloStringArray path) const;
   virtual IloAnyCollectionI* getAnyCollectionValue(IloStringArray path) const;

  virtual void display(ILOSTD(ostream)& out) const;
};

#define IloTupleExprI IloAnyExprI

//-------------------------------------------
class IloTupleConstI : public IloTupleExprI {
  ILOEXTRDECL
private:
  IloTupleI* _tuple;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleConstI(IloEnvI* env, IloTupleI* tuple);
  virtual ~IloTupleConstI();
  IloTupleI* getTuple() const { return _tuple; }
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getTupleEncoder();
  }
  ILOEXTROTHERDECL
};


class ILO_EXPORTED IloTuple {
protected:
   IloTupleI* _impl;
 public:

	 virtual ~IloTuple(){}
   IloTuple():_impl(0){}
   IloTuple(IloTupleI* impl):_impl(impl){}
   IloTupleI* getImpl() const { return _impl; }
   void end(){
     if (_impl){
       delete _impl; _impl = 0;
     }
   }

   IloEnv getEnv() const;

    void display(ILOSTD(ostream)& outs) const;

   IloInt getIndex() const;

   IloTupleCollection getCollection() const;

   IloTupleSchema getSchema() const { 
     IloAssert(getImpl(), "Using empty IloTuple handle");
     return getImpl()->getSchema();
   }

   void setIndex(IloInt i);

   IloInt getIntValue(IloInt index) const;

   IloNum getNumValue(IloInt index) const;

   IloAny getAnyValue(IloInt index) const;

   IloIntCollection getIntCollectionValue(IloInt index) const;

   IloNumCollection getNumCollectionValue(IloInt index) const;

   IloIntSet getIntSetValue(IloInt index) const;
   IloNumSet getNumSetValue(IloInt index) const;
   IloSymbolSet getSymbolSetValue(IloInt index) const;

   IloIntMap getIntMapValue(IloInt index) const;

   IloNumMap getNumMapValue(IloInt index) const;

   IloAnyCollection getAnyCollectionValue(IloInt index) const;

   IloSymbol getSymbolValue(IloInt index) const;

   const char* getStringValue(IloInt index) const;

   IloTuple makeTupleValue(IloInt index) const;


   IloInt getIntValue(IloIntArray path) const;

   IloNum getNumValue(IloIntArray path) const;

   IloAny getAnyValue(IloIntArray path) const;

   IloIntCollection getIntCollectionValue(IloIntArray path) const;

   IloIntSet getIntSetValue(IloIntArray path) const;
   IloNumSet getNumSetValue(IloIntArray path) const;
   IloSymbolSet getSymbolSetValue(IloIntArray path) const;

   IloIntMap getIntMapValue(IloIntArray path) const;

   IloNumCollection getNumCollectionValue(IloIntArray path) const;

   IloNumMap getNumMapValue(IloIntArray path) const;

   IloAnyCollection getAnyCollectionValue(IloIntArray path) const;

   IloSymbol getSymbolValue(IloIntArray path) const;

   const char* getStringValue(IloIntArray path) const;

   IloTuple makeTupleValue(IloIntArray path) const;



   IloInt getIntValue(IloInt idx1, IloInt idx2) const;

   IloNum getNumValue(IloInt idx1, IloInt idx2) const;

   IloAny getAnyValue(IloInt idx1, IloInt idx2) const;

   IloSymbol getSymbolValue(IloInt idx1, IloInt idx2) const;

   const char* getStringValue(IloInt idx1, IloInt idx2) const;

   IloTuple makeTupleValue(IloInt idx1, IloInt idx2) const;

   IloIntCollection getIntCollectionValue(IloInt idx1, IloInt idx2) const;

   IloIntSet getIntSetValue(IloInt idx1, IloInt idx2) const;
   IloNumSet getNumSetValue(IloInt idx1, IloInt idx2) const;
   IloSymbolSet getSymbolSetValue(IloInt idx1, IloInt idx2) const;

   IloIntMap getIntMapValue(IloInt idx1, IloInt idx2) const;

   IloNumCollection getNumCollectionValue(IloInt idx1, IloInt idx2) const;

   IloNumMap getNumMapValue(IloInt idx1, IloInt idx2) const;

   IloAnyCollection getAnyCollectionValue(IloInt idx1, IloInt idx2) const;



   IloInt getIntValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloNum getNumValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloAny getAnyValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloSymbol getSymbolValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   const char* getStringValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloTuple makeTupleValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloIntCollection getIntCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloIntSet getIntSetValue(IloInt idx1, IloInt idx2, IloInt idx3) const;
   IloNumSet getNumSetValue(IloInt idx1, IloInt idx2, IloInt idx3) const;
   IloSymbolSet getSymbolSetValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloIntMap getIntMapValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloNumCollection getNumCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloNumMap getNumMapValue(IloInt idx1, IloInt idx2, IloInt idx3) const;

   IloAnyCollection getAnyCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3) const;



   IloInt getIntValue(const char* col) const;

   IloNum getNumValue(const char* col) const;

   IloAny getAnyValue(const char* col) const;

   IloSymbol getSymbolValue(const char* col) const;

   const char* getStringValue(const char* col) const;

   IloTuple makeTupleValue(const char* col) const;

   IloIntCollection getIntCollectionValue(const char* col) const;

   IloIntSet getIntSetValue(const char* col) const;
   IloNumSet getNumSetValue(const char* col) const;
   IloSymbolSet getSymbolSetValue(const char* col) const;

   IloIntMap getIntMapValue(const char* col) const;

   IloNumCollection getNumCollectionValue(const char* col) const;

   IloNumMap getNumMapValue(const char* col) const;

   IloAnyCollection getAnyCollectionValue(const char* col) const;



   IloInt getIntValue(IloStringArray path) const;

   IloNum getNumValue(IloStringArray path) const;

   IloAny getAnyValue(IloStringArray path) const;

   IloSymbol getSymbolValue(IloStringArray path) const;

   const char* getStringValue(IloStringArray path) const;

   IloTuple makeTupleValue(IloStringArray path) const;

   IloIntCollection getIntCollectionValue(IloStringArray col) const;

   IloIntSet getIntSetValue(IloStringArray col) const;
   IloNumSet getNumSetValue(IloStringArray col) const;
   IloSymbolSet getSymbolSetValue(IloStringArray col) const;

   IloIntMap getIntMapValue(IloStringArray col) const;

   IloNumCollection getNumCollectionValue(IloStringArray col) const;

   IloNumMap getNumMapValue(IloStringArray col) const;

   IloAnyCollection getAnyCollectionValue(IloStringArray col) const;



   IloInt getIntValue(const char* col1, const char* col2) const;

   IloNum getNumValue(const char* col1, const char* col2) const;

   IloAny getAnyValue(const char* col1, const char* col2) const;

   IloSymbol getSymbolValue(const char* col1, const char* col2) const;

   const char* getStringValue(const char* col1, const char* col2) const;

   IloTuple makeTupleValue(const char* col1, const char* col2) const;

   IloIntCollection getIntCollectionValue(const char* col1, const char* col2) const;

   IloIntSet getIntSetValue(const char* col1, const char* col2) const;
   IloNumSet getNumSetValue(const char* col1, const char* col2) const;
   IloSymbolSet getSymbolSetValue(const char* col1, const char* col2) const;


   IloIntMap getIntMapValue(const char* col1, const char* col2) const;

   IloNumCollection getNumCollectionValue(const char* col1, const char* col2) const;

   IloNumMap getNumMapValue(const char* col1, const char* col2) const;

   IloAnyCollection getAnyCollectionValue(const char* col1, const char* col2) const;



   IloInt getIntValue(const char* col1, const char* col2, const char* col3) const;

   IloNum getNumValue(const char* col1, const char* col2, const char* col3) const;

   IloAny getAnyValue(const char* col1, const char* col2, const char* col3) const;

   IloSymbol getSymbolValue(const char* col1, const char* col2, const char* col3) const;

   const char* getStringValue(const char* col1, const char* col2, const char* col3) const;

   IloTuple makeTupleValue(const char* col1, const char* col2, const char* col3) const;

   IloIntCollection getIntCollectionValue(const char* col1, const char* col2, const char* col3) const;

   IloIntSet getIntSetValue(const char* col1, const char* col2, const char* col3) const;
   IloNumSet getNumSetValue(const char* col1, const char* col2, const char* col3) const;
   IloSymbolSet getSymbolSetValue(const char* col1, const char* col2, const char* col3) const;

   IloIntMap getIntMapValue(const char* col1, const char* col2, const char* col3) const;

   IloNumCollection getNumCollectionValue(const char* col1, const char* col2, const char* col3) const;

   IloNumMap getNumMapValue(const char* col1, const char* col2, const char* col3) const;

   IloAnyCollection getAnyCollectionValue(const char* col1, const char* col2, const char* col3) const;

 };

inline ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, const IloTuple& t){
  IloAssert(t.getImpl(), "Use empty handle.");
  t.getImpl()->display(out);
  return (out);
}


//-------------------------------------------------
// Tuple knows the collection + a row index
//-------------------------------------------------
class IloTupleBufferI : public IloTupleI {
  ILORTTIDECL
private:
  IloTuplePathBuffer _buffer;
protected:
  //using IloTupleI::isIndexEqual;
  //virtual IloBool isIndexEqual(IloTupleI*);
public:
  virtual ~IloTupleBufferI(){
    _buffer.end();
  }

  void endAll();

  void display(ILOSTD(ostream)& out) const;
  void import(IloTupleI* tuple);
  void add(IloTupleBufferI* tuple);

  IloTupleBufferI(IloTupleCollectionI* coll, IloTupleCellArray values);

  IloTupleBufferI(IloTupleCollectionI* coll, IloInt i=-1);

  void setIntValue(IloInt columnIndex, IloInt value);
  void setNumValue(IloInt columnIndex, IloNum value);
  void setAnyValue(IloInt columnIndex, IloAny value);
  void setSymbolValue(IloInt columnIndex, IloSymbol value);
  void setSymbolValue(IloInt columnIndex, const char* value){
		setSymbolValue(columnIndex, getEnv()->makeSymbol(value));
  }
  void setTupleValue(IloInt columnIndex, IloTuple value);

  void setIntCollectionValue(IloInt columnIndex, IloIntCollectionI* value);
  void setNumCollectionValue(IloInt columnIndex, IloNumCollectionI* value);
  void setAnyCollectionValue(IloInt columnIndex, IloAnyCollectionI* value);

  void setIntValue(const char* col, IloInt value);
  void setNumValue(const char* col, IloNum value);
  void setAnyValue(const char* col, IloAny value);
  void setSymbolValue(const char* col, IloSymbol value);
  void setSymbolValue(const char* col, const char* value){
	  setSymbolValue(col, getEnv()->makeSymbol(value));
  }
  void setTupleValue(const char* col, IloTuple value);

  void setIntCollectionValue(const char* col, IloIntCollectionI* value);
  void setNumCollectionValue(const char* col, IloNumCollectionI* value);
  void setAnyCollectionValue(const char* col, IloAnyCollectionI* value);


  //
  void setIntValue(IloIntArray path, IloInt value);
  void setNumValue(IloIntArray path, IloNum value);
  void setAnyValue(IloIntArray path, IloAny value);

  void setIntCollectionValue(IloIntArray path, IloIntCollectionI* value);
  void setNumCollectionValue(IloIntArray path, IloNumCollectionI* value);
  void setAnyCollectionValue(IloIntArray path, IloAnyCollectionI* value);

  void setSymbolValue(IloIntArray path, IloSymbol value);
  void setSymbolValue(IloIntArray path, const char* value){
	  setSymbolValue(path, getEnv()->makeSymbol(value));
  }

  void setTupleValue(IloIntArray path, IloTuple value);

  void setIntValue(IloStringArray path, IloInt value);
  void setNumValue(IloStringArray path, IloNum value);
  void setAnyValue(IloStringArray path, IloAny value);
  void setSymbolValue(IloStringArray path, IloSymbol value);
  void setSymbolValue(IloStringArray path, const char* value){
	  setSymbolValue(path, getEnv()->makeSymbol(value));
  }

  void setIntCollectionValue(IloStringArray path, IloIntCollectionI* value);
  void setNumCollectionValue(IloStringArray path, IloNumCollectionI* value);
  void setAnyCollectionValue(IloStringArray path, IloAnyCollectionI* value);


  void setTupleValue(IloStringArray path, IloTuple value);

  void setIntValue(IloInt idx1, IloInt idx2, IloInt value);
  void setNumValue(IloInt idx1, IloInt idx2, IloNum value);
  void setAnyValue(IloInt idx1, IloInt idx2, IloAny value);
  void setSymbolValue(IloInt idx1, IloInt idx2, IloSymbol value);
  void setSymbolValue(IloInt idx1, IloInt idx2, const char* value){
	  setSymbolValue(idx1, idx2, getEnv()->makeSymbol(value));
  }

  void setIntCollectionValue(IloInt idx1, IloInt idx2, IloIntCollectionI* value);
  void setNumCollectionValue(IloInt idx1, IloInt idx2, IloNumCollectionI* value);
  void setAnyCollectionValue(IloInt idx1, IloInt idx2, IloAnyCollectionI* value);

  void setTupleValue(IloInt idx1, IloInt idx2, IloTuple value);

  void setIntValue(const char* col1, const char* col2, IloInt value);
  void setNumValue(const char* col1, const char* col2, IloNum value);
  void setAnyValue(const char* col1, const char* col2, IloAny value);
  void setSymbolValue(const char* col1, const char* col2, IloSymbol value);
  void setSymbolValue(const char* col1, const char* col2, const char* value){
	  setSymbolValue(col1, col2, getEnv()->makeSymbol(value));
  }

  void setIntCollectionValue(const char* col1, const char* col2, IloIntCollectionI* value);
  void setNumCollectionValue(const char* col1, const char* col2, IloNumCollectionI* value);
  void setAnyCollectionValue(const char* col1, const char* col2, IloAnyCollectionI* value);

  void setTupleValue(const char* col1, const char* col2, IloTuple value);

  //
  void setIntValue(IloInt idx1, IloInt idx2, IloInt idx3, IloInt value);
  void setNumValue(IloInt idx1, IloInt idx2, IloInt idx3, IloNum value);
  void setAnyValue(IloInt idx1, IloInt idx2, IloInt idx3, IloAny value);
  void setSymbolValue(IloInt idx1, IloInt idx2, IloInt idx3, IloSymbol value);
  void setSymbolValue(IloInt idx1, IloInt idx2, IloInt idx3, const char* value){
	  setSymbolValue(idx1, idx2, idx3, getEnv()->makeSymbol(value));
  }

  void setIntCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3, IloIntCollectionI* value);
  void setNumCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3, IloNumCollectionI* value);
  void setAnyCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3, IloAnyCollectionI* value);

  void setTupleValue(IloInt idx1, IloInt idx2, IloInt idx3, IloTuple value);

  void setIntValue(const char* col1, const char* col2, const char* col3, IloInt value);
  void setNumValue(const char* col1, const char* col2, const char* col3, IloNum value);
  void setAnyValue(const char* col1, const char* col2, const char* col3, IloAny value);
  void setSymbolValue(const char* col1, const char* col2, const char* col3, IloSymbol value);
  void setSymbolValue(const char* col1, const char* col2, const char* col3, const char* value){
		setSymbolValue(col1, col2, col3, getEnv()->makeSymbol(value));
  }

  void setIntCollectionValue(const char* col1, const char* col2, const char* col3, IloIntCollectionI* value);
  void setNumCollectionValue(const char* col1, const char* col2, const char* col3, IloNumCollectionI* value);
  void setAnyCollectionValue(const char* col1, const char* col2, const char* col3, IloAnyCollectionI* value);

  void setTupleValue(const char* col1, const char* col2, const char* col3, IloTuple value);

  void setIntValue(IloInt pathSize, IloInt* path, IloInt value);
  void setNumValue(IloInt pathSize, IloInt* path, IloNum value);
  void setAnyValue(IloInt pathSize, IloInt* path, IloAny value);
  void setSymbolValue(IloInt pathSize, IloInt* path, IloSymbol value);
  void setSymbolValue(IloInt pathSize, IloInt* path, const char* value){
	  setSymbolValue(pathSize, path, getEnv()->makeSymbol(value));
  }

  void setTupleValue(IloInt pathSize, IloInt* path, IloTuple value);

  void setIntCollectionValue(IloInt pathSize, IloInt* path, IloIntCollectionI* value);
  void setNumCollectionValue(IloInt pathSize, IloInt* path, IloNumCollectionI* value);
  void setAnyCollectionValue(IloInt pathSize, IloInt* path, IloAnyCollectionI* value);

  virtual IloInt getIntValue(const char* col) const;
  virtual IloNum getNumValue(const char* col) const;
  virtual IloAny getAnyValue(const char* col) const;
  virtual IloSymbol getSymbolValue(const char* col) const;
  virtual const char* getStringValue(const char* col) const;
  virtual IloTuple makeTupleValue(const char* col) const;

  virtual IloIntCollectionI* getIntCollectionValue(const char* col) const;
  virtual IloNumCollectionI* getNumCollectionValue(const char* col) const;
  virtual IloAnyCollectionI* getAnyCollectionValue(const char* col) const;

  virtual IloInt getIntValue(IloInt index) const;
  virtual IloNum getNumValue(IloInt index) const;
  virtual IloAny getAnyValue(IloInt index) const;

  virtual IloIntCollectionI* getIntCollectionValue(IloInt index) const;
  virtual IloNumCollectionI* getNumCollectionValue(IloInt index) const;
  virtual IloAnyCollectionI* getAnyCollectionValue(IloInt index) const;

  virtual IloSymbol getSymbolValue(IloInt index) const;
  virtual const char* getStringValue(IloInt index) const;
  virtual IloTuple makeTupleValue(IloInt index) const;

  //
  virtual IloInt getIntValue(IloIntArray path) const;
  virtual IloNum getNumValue(IloIntArray path) const;
  virtual IloAny getAnyValue(IloIntArray path) const;

  virtual IloIntCollectionI* getIntCollectionValue(IloIntArray path) const;
  virtual IloNumCollectionI* getNumCollectionValue(IloIntArray path) const;
  virtual IloAnyCollectionI* getAnyCollectionValue(IloIntArray path) const;


  virtual IloSymbol getSymbolValue(IloIntArray path) const;
  virtual const char* getStringValue(IloIntArray path) const;
  virtual IloTuple makeTupleValue(IloIntArray path) const;

  virtual IloInt getIntValue(IloStringArray path) const;
  virtual IloNum getNumValue(IloStringArray path) const;
  virtual IloAny getAnyValue(IloStringArray path) const;
  virtual IloSymbol getSymbolValue(IloStringArray path) const;
  virtual const char* getStringValue(IloStringArray path) const;
  virtual IloTuple makeTupleValue(IloStringArray path) const;

   virtual IloIntCollectionI* getIntCollectionValue(IloStringArray path) const;
   virtual IloNumCollectionI* getNumCollectionValue(IloStringArray path) const;
   virtual IloAnyCollectionI* getAnyCollectionValue(IloStringArray path) const;

  //
  IloTuplePathBuffer getLocalBuffer();
  IloInt commit();
  void clear();
};


class ILO_EXPORTED IloTupleBuffer : public IloTuple {
public:
  void clear();

  void display(ILOSTD(ostream)& outs) const;

  IloTupleBuffer():IloTuple(0){}
  IloTupleBuffer(IloTupleBufferI* impl):IloTuple(impl){}
  IloTupleBufferI* getImpl() const {
    return (IloTupleBufferI*)IloTuple::getImpl();
  }
  void setIntValue(IloInt columnIndex, IloInt value);
  void setNumValue(IloInt columnIndex, IloNum value);
  void setAnyValue(IloInt columnIndex, IloAny value);

  void setIntCollectionValue(IloInt columnIndex, IloIntCollection value);
  void setNumCollectionValue(IloInt columnIndex, IloNumCollection value);
  void setAnyCollectionValue(IloInt columnIndex, IloAnyCollection value);

  void setIntMapValue(IloInt columnIndex, IloIntMap value);
  void setNumMapValue(IloInt columnIndex, IloNumMap value);

  void setSymbolValue(IloInt columnIndex, const char* value);
  void setSymbolValue(IloInt columnIndex, IloSymbol value);
  void setTupleValue(IloInt columnIndex, IloTuple value);

  void setIntValue(const char* col, IloInt value);
  void setNumValue(const char* col, IloNum value);
  void setAnyValue(const char* col, IloAny value);
  void setSymbolValue(const char* col, const char* value);
  void setSymbolValue(const char* col, IloSymbol value);
  void setTupleValue(const char* col, IloTuple value);

  void setIntCollectionValue(const char* col, IloIntCollection value);
  void setNumCollectionValue(const char* col, IloNumCollection value);
  void setAnyCollectionValue(const char* col, IloAnyCollection value);

  void setIntMapValue(const char* col, IloIntMap value);
  void setNumMapValue(const char* col, IloNumMap value);

  void setIntCollectionValue(IloIntArray path, IloIntCollection value);
  void setNumCollectionValue(IloIntArray path, IloNumCollection value);
  void setAnyCollectionValue(IloIntArray path, IloAnyCollection value);

  void setIntMapValue(IloIntArray path, IloIntMap value);
  void setNumMapValue(IloIntArray path, IloNumMap value);

  //
  void setIntValue(IloIntArray path, IloInt value);
  void setNumValue(IloIntArray path, IloNum value);
  void setAnyValue(IloIntArray path, IloAny value);
  void setSymbolValue(IloIntArray path, const char* value);
  void setSymbolValue(IloIntArray path, IloSymbol value);
  void setTupleValue(IloIntArray path, IloTuple value);

  void setIntValue(IloStringArray path, IloInt value);
  void setNumValue(IloStringArray path, IloNum value);
  void setAnyValue(IloStringArray path, IloAny value);
  void setSymbolValue(IloStringArray path, const char* value);
  void setSymbolValue(IloStringArray path, IloSymbol value);
  void setTupleValue(IloStringArray path, IloTuple value);

  void setIntCollectionValue(IloStringArray path, IloIntCollection value);
  void setNumCollectionValue(IloStringArray path, IloNumCollection value);
  void setAnyCollectionValue(IloStringArray path, IloAnyCollection value);

  void setIntMapValue(IloStringArray path, IloIntMap value);
  void setNumMapValue(IloStringArray path, IloNumMap value);

  void setIntValue(IloInt idx1, IloInt idx2, IloInt value);
  void setNumValue(IloInt idx1, IloInt idx2, IloNum value);
  void setAnyValue(IloInt idx1, IloInt idx2, IloAny value);
  void setSymbolValue(IloInt idx1, IloInt idx2, const char* value);
  void setSymbolValue(IloInt idx1, IloInt idx2, IloSymbol value);
  void setTupleValue(IloInt idx1, IloInt idx2, IloTuple value);

  void setIntCollectionValue(IloInt idx1, IloInt idx2, IloIntCollection value);
  void setNumCollectionValue(IloInt idx1, IloInt idx2, IloNumCollection value);
  void setAnyCollectionValue(IloInt idx1, IloInt idx2, IloAnyCollection value);

  void setIntMapValue(IloInt idx1, IloInt idx2, IloIntMap value);
  void setNumMapValue(IloInt idx1, IloInt idx2, IloNumMap value);

  void setIntValue(const char* col1, const char* col2, IloInt value);
  void setNumValue(const char* col1, const char* col2, IloNum value);
  void setAnyValue(const char* col1, const char* col2, IloAny value);
  void setSymbolValue(const char* col1, const char* col2, const char* value);
  void setSymbolValue(const char* col1, const char* col2, IloSymbol value);
  void setTupleValue(const char* col1, const char* col2, IloTuple value);

  void setIntCollectionValue(const char* col1, const char* col2, IloIntCollection value);
  void setNumCollectionValue(const char* col1, const char* col2, IloNumCollection value);
  void setAnyCollectionValue(const char* col1, const char* col2, IloAnyCollection value);

  void setIntMapValue(const char* col1, const char* col2, IloIntMap value);
  void setNumMapValue(const char* col1, const char* col2, IloNumMap value);

  //
  void setIntValue(IloInt idx1, IloInt idx2, IloInt idx3, IloInt value);
  void setNumValue(IloInt idx1, IloInt idx2, IloInt idx3, IloNum value);
  void setAnyValue(IloInt idx1, IloInt idx2, IloInt idx3, IloAny value);
  void setSymbolValue(IloInt idx1, IloInt idx2, IloInt idx3, const char* value);
  void setSymbolValue(IloInt idx1, IloInt idx2, IloInt idx3, IloSymbol value);
  void setTupleValue(IloInt idx1, IloInt idx2, IloInt idx3, IloTuple value);

  void setIntCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3, IloIntCollection value);
  void setNumCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3, IloNumCollection value);
  void setAnyCollectionValue(IloInt idx1, IloInt idx2, IloInt idx3, IloAnyCollection value);

  void setIntMapValue(IloInt idx1, IloInt idx2, IloInt idx3, IloIntMap value);
  void setNumMapValue(IloInt idx1, IloInt idx2, IloInt idx3, IloNumMap value);

  void setIntValue(const char* col1, const char* col2, const char* col3, IloInt value);
  void setNumValue(const char* col1, const char* col2, const char* col3, IloNum value);
  void setAnyValue(const char* col1, const char* col2, const char* col3, IloAny value);
  void setSymbolValue(const char* col1, const char* col2, const char* col3, const char* value);
  void setSymbolValue(const char* col1, const char* col2, const char* col3, IloSymbol value);
  void setTupleValue(const char* col1, const char* col2, const char* col3, IloTuple value);

  void setIntCollectionValue(const char* col1, const char* col2, const char* col3, IloIntCollection value);
  void setNumCollectionValue(const char* col1, const char* col2, const char* col3, IloNumCollection value);
  void setAnyCollectionValue(const char* col1, const char* col2, const char* col3, IloAnyCollection value);

  void setIntMapValue(const char* col1, const char* col2, const char* col3, IloIntMap value);
  void setNumMapValue(const char* col1, const char* col2, const char* col3, IloNumMap value);

  IloTuplePathBuffer getLocalBuffer();
  IloInt commit();
};


// --------------------------------------------------------------
// Tuple Expressions
// --------------------------------------------------------------
#define IloTupleExprArgI IloAnyExprI

//-----------------------------------------------------------------
class ILO_EXPORTED IloTupleExprArg : public IloExtractable {
  ILOEXTRHANDLE(IloTupleExprArg, IloExtractable)
public:
	IloIntExprArg getInt(IloInt colIndex) const;
	IloIntExprArg getInt(const char* colName) const;
	IloNumExprArg getNum(IloInt colIndex) const;
	IloNumExprArg getNum(const char* colName) const;
	IloSymbolExprArg getSymbol(IloInt colIndex) const;
	IloSymbolExprArg getSymbol(const char* colName) const;
	IloTupleExprArg getTuple(IloInt colIndex) const;
	IloTupleExprArg getTuple(const char* colName) const;

	/* <p> . </p> */
	IloIntCollectionExprArg getIntCollection(IloInt colIndex) const;
	/* <p> . </p> */
	IloIntCollectionExprArg getIntCollection(const char* colName) const;
	/* <p> . </p> */
	IloNumCollectionExprArg getNumCollection(IloInt colIndex) const;
	/* <p> . </p> */
	IloNumCollectionExprArg getNumCollection(const char* colName) const;
	/* <p> . </p> */
	IloSymbolCollectionExprArg getSymbolCollection(IloInt colIndex) const;
	/* <p> . </p> */
	IloSymbolCollectionExprArg getSymbolCollection(const char* colName) const;
};

class IloTupleEncoderI : public IloEncoderI {
public:
  IloTupleEncoderI():IloEncoderI(){}
  virtual IloInt anyToInt(IloAny e) const;
  virtual IloAny intToAny(IloInt e) const;
  virtual void display(ILOSTD(ostream)& out, IloAny e) const;
};

// --------------------------------------------------------------
// Tuple Constraints
// --------------------------------------------------------------

//-----------------------------------------------------------------
IloConstraint operator==(const IloTupleExprArg expr1,
			 const IloTupleExprArg expr2);

IloConstraint operator==(const IloTupleExprArg expr, IloTuple value);

inline IloConstraint operator==(IloTuple val, const IloTupleExprArg expr){
  return (expr == val);
}

IloConstraint operator!=(const IloTupleExprArg expr1,
			 const IloTupleExprArg expr2);

IloConstraint operator!=(const IloTupleExprArg expr, IloTuple value);

inline IloConstraint operator!=(IloTuple val, const IloTupleExprArg expr){
  return (expr != val);
}

// --------------------------------------------------------------
// HashTable with IloTuple keys
// --------------------------------------------------------------
extern IloInt  IloTupleHashFunction(IloTuple, IloInt size);
extern IloBool IloTupleCompFunction(IloTuple key1, IloTuple key2);

template <class ValueType>
class IloTupleHashTable : public IloEnvHashTable<IloTuple, ValueType> {
 public:
  IloTupleHashTable(IloEnv env, IloInt size=IloDefaultHashSize)
    :IloEnvHashTable<IloTuple, ValueType>(env,
					  IloTupleHashFunction,
					  IloTupleCompFunction,
					  size) {
  }
  virtual ~IloTupleHashTable() {
  }
};

// --------------------------------------------------------------
// Tuple Table
// --------------------------------------------------------------

typedef IloTupleHashTable<IloInt> IloTupleTableBase;

class IloTupleTable : public IloTupleTableBase {
private:
 public:
  IloTupleTable(IloEnv env, IloInt size=IloDefaultHashSize);
  virtual ~IloTupleTable();
  IloInt get(IloTuple) const;
  IloInt make(IloTuple);
};

#endif
