// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplecollectionH
#define __CONCERT_ilotuplecollectionH

#if !defined(__CONCERT_iloextractableH)
# include <ilconcert/iloextractable.h>
#endif
#if !defined(__CONCERT_ilodatacolumnH)
# include <ilconcert/iltuple/ilodatacolumn.h>
#endif
#if !defined(__CONCERT_ilotuplebufferH)
# include <ilconcert/iltuple/ilotuplebuffer.h>
#endif

class IloTuple;
class IloTupleBuffer;
class IloTupleSetI;
class IloTupleCollectionI;
class IloTupleSchemaI;
class IloTupleRequestI;
class IloTupleRequest;
class IloTupleSchema;
class IloTupleIndex;
class IloTuplePattern;
class IloGenerator;
class IloTupleIterator;
class IloTupleExprArg;
class IloTupleSetMap;

#define IloTupleSetExprArgI IloAnyExprI
#define IloTupleSetExprI IloAnyExprI

class IloColumnDefinitionI;
class IloColumnDefinition;
class IloColumnDefinitionArray;
class IloTupleSetArray;
class IloSymbolArray;

class IloTupleSchema  {
private:
  IloTupleSchemaI* _impl;
public:
  IloTupleSchema(IloEnv env ,const char* name = 0);

  IloTupleSchema(IloTupleSchemaI* impl);

  IloTupleSchemaI* getImpl() const {
    return _impl;
  }

  void end();

  IloInt getSize() const;

  IloColumnDefinition getColumn(IloInt i) const;

  const char* getColumnName(IloInt idx) const;

  IloEnv getEnv() const;

  void addIntColumn(const char* name =0);

  void addNumColumn(const char* name =0);

  void addAnyColumn(const char* name =0);

  void addSymbolColumn(const char* name =0);

  void addIntCollectionColumn(const char* name =0);

  void addNumCollectionColumn(const char* name =0);

  void addAnyCollectionColumn(const char* name =0);

  void addTupleColumn(IloTupleSchema ax, const char* name=0);

  void clear();

  void setName(const char* name);

  const char* getName();

  IloBool isSimpleTypedSchema() const;

  IloInt getTotalColumnNumber() const;

  IloInt getColumnIndex(const char* name) const;

  IloInt getColumnIndex(const IloSymbolI* name) const;

  IloColumnDefinitionI* getColumn(IloIntArray path);

  IloColumnDefinitionArray getArray() const;
  IloTupleSchemaI* getTupleColumn(IloInt colIndex) const;

  IloBool isInt(IloInt index);

  IloBool isNum(IloInt index);

  IloBool isAny(IloInt index);

  IloBool isSymbol(IloInt index);

  IloBool isTuple(IloInt index);

  IloBool isIntCollection(IloInt index);

  IloBool isNumCollection(IloInt index);

  IloBool isAnyCollection(IloInt index);

  IloBool isInt(IloIntArray path);

  IloBool isNum(IloIntArray path);

  IloBool isAny(IloIntArray path);

  IloBool isIntCollection(IloIntArray path);

  IloBool isNumCollection(IloIntArray path);

  IloBool isAnyCollection(IloIntArray path);

  IloBool isSymbol(IloIntArray path);

  IloBool isTuple(IloIntArray path);

  void setData(IloInt index, IloAny data);

  void setData(IloIntArray path, IloAny data);

  IloAny getData(IloInt index);

  IloAny getData(IloIntArray path);

  IloBool isInt(IloInt index1, IloInt index2);
  IloBool isNum(IloInt index1, IloInt index2);
  IloBool isAny(IloInt index1, IloInt index2);
  IloBool isIntCollection(IloInt index1, IloInt index2);
  IloBool isNumCollection(IloInt index1, IloInt index2);
  IloBool isAnyCollection(IloInt index1, IloInt index2);
  IloBool isSymbol(IloInt index1, IloInt index2);
  IloBool isTuple(IloInt index1, IloInt index2);

  IloBool isInt(IloInt index1, IloInt index2, IloInt index3);
  IloBool isNum(IloInt index1, IloInt index2, IloInt index3);
  IloBool isAny(IloInt index1, IloInt index2, IloInt index3);
  IloBool isIntCollection(IloInt index1, IloInt index2, IloInt index3);
  IloBool isNumCollection(IloInt index1, IloInt index2, IloInt index3);
  IloBool isAnyCollection(IloInt index1, IloInt index2, IloInt index3);
  IloBool isSymbol(IloInt index1, IloInt index2, IloInt index3);
  IloBool isTuple(IloInt index1, IloInt index2, IloInt index3);
};

ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, const IloTupleSchema& s);

IloBool operator==(const IloTupleSchema schema1, const IloTupleSchema schema2);
IloBool operator!=(const IloTupleSchema schema1, const IloTupleSchema schema2);

class IloTupleCollection : public IloAnyCollection {
public:
	IloTupleCollection(IloTupleCollectionI* impl);
	IloTupleCollectionI* getImpl() const;
	IloTupleCollection(){ _impl = 0; }
};

class IloTupleSet : public IloTupleCollection {
public:
	IloTupleSchema getSchema() const;

	void setName(const char* name);

	const char* getName() const;

	IloTupleSet(){ _impl = 0; }
	IloTupleSet(IloTupleSetI* impl);

	IloTupleSet(IloEnv env, const IloTupleSchema schema);

	//IloTupleSet(IloEnv env, const IloTupleSchema schema, IloIntArray names, IloTupleSetArray references);

	IloTuple makeNext(IloTuple value, IloInt n=0) const;
	IloTuple makePrevious(IloTuple value, IloInt n=0) const;

	IloTuple makeNextC(IloTuple value, IloInt n=0) const;
	IloTuple makePreviousC(IloTuple value, IloInt n=0) const;

	IloTuple makeFirst() const;
	IloTuple makeLast() const;

	IloTupleSet(const IloTupleSchema schema, IloTupleIndex e, IloGenerator g);
	IloTupleSet(const IloTupleSchema schema, IloTupleIndex e, IloGenerator g1, IloGenerator g2);
	IloTupleSet(const IloTupleSchema schema, IloTupleIndex e, IloGenerator g1, IloGenerator g2, IloGenerator g3);
	IloTupleSet(const IloTupleSchema schema, IloTupleIndex e, IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4);
	IloTupleSet(const IloTupleSchema schema, IloTupleIndex e, IloGeneratorArray generators);

	IloTupleSet(const IloTupleSchema schema, IloTuplePattern e, IloGenerator g);
	IloTupleSet(const IloTupleSchema schema, IloTuplePattern e, IloGenerator g1, IloGenerator g2);
	IloTupleSet(const IloTupleSchema schema, IloTuplePattern e, IloGenerator g1, IloGenerator g2, IloGenerator g3);
	IloTupleSet(const IloTupleSchema schema, IloTuplePattern e, IloGenerator g1, IloGenerator g2, IloGenerator g3, IloGenerator g4);
	IloTupleSet(const IloTupleSchema schema, IloTuplePattern e, IloGeneratorArray generators);

	//IloTupleSet(const IloTupleSchema schema, 
	//	IloIntArray names, IloTupleSetArray references, 
	//	IloTupleIndex e, IloGeneratorArray generators);
	//IloTupleSet(const IloTupleSchema schema, 
	//	IloIntArray names, IloTupleSetArray references, 
	//	IloTuplePattern e, IloGeneratorArray generators);

	IloTupleSetI* getImpl() const;

	const char* getColumnName(IloInt index) const;
	const char* getColumnName(IloIntArray path) const;

	void setColumnName(IloInt index, const char* name);
	void setColumnName(IloIntArray path, const char* name);

	IloTuple makeTuple(IloInt index) const;

	IloTupleBuffer makeTupleBuffer(IloInt index = -1) const;

	IloTuplePathBuffer makeLine(IloInt index) const;

	IloInt commit(IloTupleBuffer line, IloBool check = IloTrue);

	IloInt getLength() const;

	IloInt getSize() const;

	IloTupleIterator* iterator(IloGenAlloc* heap);
	IloTupleIterator* iterator();

	void clearSelectIndexes();
	void createSelectIndexes();

	void displayRow(IloInt i, ILOSTD(ostream)& out) const;

	///**
	//* @show internal
	//**/
	//void commit(IloTuplePattern exp);
};


class IloTupleIterator : public IloAnyDataIterator {
private:
	IloTupleRequestI* _request;
	IloIntArrayI* _reqResult;
  IloBool _ownsReqResult;
	IloInt _index;
	void initSlice();
public:
	IloInt getIndex() const {
		return _index;
	}
protected:
	IloTupleIterator(IloGenAlloc* heap);
  void initCollection(IloTupleSetI*);
public:
	virtual ~IloTupleIterator();

	IloTupleIterator(IloGenAlloc* heap, IloTupleSet coll);
	IloTupleIterator(IloTupleSet coll);

	IloTupleIterator(IloGenAlloc* heap, IloTupleSet coll, IloTupleRequest req, IloBool computeSlice=IloTrue);
	IloTupleIterator(IloTupleSet coll, IloTupleRequest req, IloBool computeSlice=IloTrue);

	IloTupleSet getTupleSet() const;
	virtual IloTupleSchemaI* getSchema() const;

	void initRequest(IloTupleRequest req, IloBool computeSlice=IloTrue);
	void clearReqResult();

	virtual IloBool next();
	virtual void reset(IloBool catchInvalidCollection = IloFalse);
	virtual void reset(IloAny val, IloBool catchInvalidCollection = IloFalse);

#define AFC_FIX_TUPLEITERATOR
#ifdef AFC_FIX_TUPLEITERATOR
	IloTuple operator*();
#endif

	void freeColumn();
};


//-----------------------------------------------------------------
class ILO_EXPORTED IloTupleSetExprArg : public IloExtractable {
	ILOEXTRHANDLE(IloTupleSetExprArg, IloExtractable)
};



//--------------------------------------------------------------------
// TupleSetExpr arrays
//--------------------------------------------------------------------
typedef IloArray<IloTupleSetExprArg> IloTupleSetExprArrayBase;

class ILO_EXPORTED IloTupleSetExprArray : public IloTupleSetExprArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;
  IloTupleSetExprArray(IloDefaultArrayI* i=0) : IloTupleSetExprArrayBase(i) {}
  IloTupleSetExprArray(const IloTupleSetExprArray& copy) : IloTupleSetExprArrayBase(copy) {}
  IloTupleSetExprArray(const IloEnv env, IloInt n = 0) : IloTupleSetExprArrayBase(env, n) {}
  const IloTupleSetExprArg& operator[] (IloInt i) const {
    return IloTupleSetExprArrayBase::operator[](i);
  }
  IloTupleSetExprArg& operator[] (IloInt i) {
    return IloTupleSetExprArrayBase::operator[](i);
  }
};


//
IloConstraint operator==(const IloTupleSetExprArg expr1, const IloTupleSetExprArg expr2);
IloConstraint operator==(const IloTupleSetExprArg expr, IloTupleSet val);
IloConstraint operator!=(const IloTupleSetExprArg expr1, const IloTupleSetExprArg expr2);
IloConstraint operator!=(const IloTupleSetExprArg expr, IloTupleSet val);

IloTupleSet IloUnion(IloTupleSchema schema, IloGeneratorArray gen, IloTupleExprArg arg);
IloTupleSet IloUnion(IloTupleSchema schema, IloGeneratorArray gen, IloTupleSetExprArg arg);



//IloTupleSet IloInter(IloTupleSchema schema, IloGeneratorArray gen, IloTupleExprArg arg);
IloTupleSet IloInter(IloTupleSchema schema, IloGeneratorArray gen, IloTupleSetExprArg arg);

//IloTupleSet IloExclude(IloTupleSchema schema, IloGeneratorArray gen, IloTupleExprArg arg);
IloTupleSet IloExclude(IloTupleSchema schema, IloGeneratorArray gen, IloTupleSetExprArg arg);


//IloTupleSet IloSymExclude(IloTupleSchema schema, IloGeneratorArray gen, IloTupleExprArg arg);
IloTupleSet IloSymExclude(IloTupleSchema schema, IloGeneratorArray gen, IloTupleSetExprArg arg);




IloIntExprArg IloOrd (IloTupleSetExprArg map, IloTupleExprArg y);
IloIntExprArg IloOrd(IloTupleSetExprArg x, IloTuple y);


IloConstraint IloOrdered(IloTupleSetExprArg coll, IloTupleExprArg exp1, IloTupleExprArg exp2);

IloConstraint IloOrdered(IloTupleSetExprArg coll, IloTupleExprArg arg, IloTuple set);

IloConstraint IloOrdered(IloTupleSetExprArg map, IloTuple set, IloTupleExprArg arg);




IloTupleExprArg IloPreviousC(IloTupleSetExprArg set, IloTupleExprArg value, IloIntExprArg n);
IloTupleExprArg IloNextC(IloTupleSetExprArg set, IloTupleExprArg value, IloIntExprArg n);

IloTupleExprArg IloPrevious(IloTupleSetExprArg set, IloTupleExprArg value, IloIntExprArg n);

IloTupleExprArg IloNext(IloTupleSetExprArg set, IloTupleExprArg value, IloIntExprArg n);


IloTupleExprArg IloItem(IloTupleSetExprArg set, IloIntExprArg n);


IloTupleExprArg IloPreviousC(IloTupleSetExprArg set, IloTupleExprArg value);

IloTupleExprArg IloNextC(IloTupleSetExprArg set, IloTupleExprArg value);

IloTupleExprArg IloPrevious(IloTupleSetExprArg set, IloTupleExprArg value);

IloTupleExprArg IloNext(IloTupleSetExprArg set, IloTupleExprArg value);


IloTupleExprArg IloFirst(IloTupleSetExprArg set);
IloTupleExprArg IloLast(IloTupleSetExprArg set);

IloTupleSetExprArg IloSymExclude(IloTupleSetExprArg expr1,IloTupleSetExprArg expr2);
IloTupleSetExprArg IloUnion(IloTupleSetExprArg expr1, IloTupleSetExprArg expr2);
IloTupleSetExprArg IloExclude(IloTupleSetExprArg expr1, IloTupleSetExprArg expr2);
IloTupleSetExprArg IloInter(IloTupleSetExprArg expr1, IloTupleSetExprArg expr2);

IloTupleSetExprArg IloSymExclude(IloTupleSet expr1,IloTupleSetExprArg expr2);
IloTupleSetExprArg IloUnion(IloTupleSet expr1, IloTupleSetExprArg expr2);
IloTupleSetExprArg IloExclude(IloTupleSet expr1, IloTupleSetExprArg expr2);
IloTupleSetExprArg IloInter(IloTupleSet expr1, IloTupleSetExprArg expr2);

IloTupleSetExprArg IloSymExclude(IloTupleSetExprArg expr1,IloTupleSet expr2);
IloTupleSetExprArg IloUnion(IloTupleSetExprArg expr1, IloTupleSet expr2);
IloTupleSetExprArg IloExclude(IloTupleSetExprArg expr1, IloTupleSet expr2);
IloTupleSetExprArg IloInter(IloTupleSetExprArg expr1, IloTupleSet expr2);

IloIntExprArg IloCard(const IloTupleSetExprArg e);

#endif
