// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplecollectioniH
#define __CONCERT_ilotuplecollectioniH

#if !defined(__CONCERT_ilotuplecollectionH)
# include <ilconcert/ilotuplecollection.h>
#endif
#if !defined(__CONCERT_ilotupleH)
# include <ilconcert/ilotuple.h>
#endif

#if !defined(__CONCERT_ilocollectioncolumniH)
# include <ilconcert/iltuple/ilocollectioncolumni.h>
#endif

#if !defined(__CONCERT_ilosetH)
# include <ilconcert/iloset.h>
#endif

#if !defined(__CONCERT_ilotuplecollectionhashH)
# include <ilconcert/iltuple/ilotuplecollectionhash.h>
#endif

#if !defined(__CONCERT_ilotuplerequestH)
# include <ilconcert/iltuple/ilotuplerequest.h>
#endif

#if !defined(__CONCERT_ilotupleschemaiH)
#include <ilconcert/iltuple/ilotupleschemai.h>
#endif


class IloTupleSet;
class IloTupleIterator;
class IloTupleSetArray;
class IloTupleI;
class IloTuple;
class IloTupleBuffer;
class IloTupleBufferI;
class IloTupleSchema;
class IloColumnDefinition;

class IloTuplePattern;
class IloTuplePatternI;

class IloIntCollectionColumnI;
class IloNumCollectionColumnI;
class IloAnyCollectionColumnI;
class IloExtendedComprehensionI;
class IloTupleSetArray;

class IloTupleException : public IloException {
public:
  IloTupleException(const char* m):IloException(m){}
};

class IloTupleDefinitionI : public IloColumnDefinitionI{
private:
  IloTupleSchema _schema;
public:
  IloTupleDefinitionI(IloEnv env, IloTupleSchema schema, const char* name = 0);
  virtual IloBool isTuple() const;
  IloTupleSchema getSchema() const;
  virtual void display(ILOSTD(ostream)& out) const;
};



class IloTupleCollectionI : public IloAnyCollectionI {
	ILORTTIDECL
protected:
	IloInt _size;
	IloIntArray _empty;
	IloDataCollectionArray _array;
	IloTupleSchema _schema;
	IloTupleCellArray _sharedTupleCells;
	IloTupleCellArray _sharedKeyCells;
	IloTupleCellArray _sharedRefSliceCells;

	// for internal use to avoid conflicts with sharedxxxCells
	IloTupleCellArray _internalTupleCells;
	//IloTupleCellArray _sharedKeyCells;

	// for reference resolution
	IloTupleCellArray _referenceCells;

	IloTupleCells2ArrayIndexHash* _hashForTupleCollection;
	IloAnyArray _pointers;
	IloIntArray _collectionColumnIdx;
	IloBool _hasReference;
	IloIntArray _referencedColumns;

	IloTupleCellArray getNormalizedCells(IloTupleCellArray line);
	void freeNormalizedCells(IloTupleCellArray line);

	void init(IloEnv env, IloTupleSchema schema, IloInt n = 0);
	void disableSelectIndexesOnColumn();
	void enableSelectIndexesOnColumn();
	IloBool isSelectIndexEnabledOnColumn() const;

	//virtual void addLine(IloTuplePathBuffer line);
	void removeValuesOnly(IloInt line);
	void initReferencedColumns();
	

public:
	void setLine(IloInt, IloTupleCellArray line);
	IloDiscreteDataCollectionI* makeColumnAsSet(IloInt idx) const;
	IloInt getRealIndex(IloInt columnIdx, IloInt idx);
	IloBool hasReference() const { return _hasReference;}
	IloIntArray getReferencedColumns() const{ 
		IloTestAndRaise(_referencedColumns.getImpl(), "IloTupleCollectionI::getReferencedColumns should not be called");
		return _referencedColumns;
	}
	virtual void setReferences(IloIntArray idx, IloArray<IloDiscreteDataCollectionI*> references);

	IloAnyArray makePointersToColumns() const;
	IloIntArray getCollectionColumnIndexes() const{ return _collectionColumnIdx;}
	
	void fillWithDefaultValue(IloTupleCellArray cells);
	void fillWithDefaultValue(IloTupleBufferI* buf);
	void setDefaultValue(IloTupleCellArray cells);
	void setDefaultValue(IloTupleBufferI* buf);

	IloInt getInternalId(IloIntArray path) const;
	IloInt getInternalSubId(IloIntArray path) const;
	void fillHashForTupleCollection();
	void createHashForTupleCollection();
	void deleteHashForTupleCollection();

	IloTupleCells2ArrayIndexHash* getHashForTupleCollection() { return _hashForTupleCollection; }

	IloAnyArray getSharedPointersToColumns() const{ return _pointers;}

	void setSize(IloInt size);
	friend class IloTupleIterator;
	/* Methodes pour Chris. */
	IloTupleCollectionI(IloEnv env, IloTupleSchema schema, IloInt numberOfLine, IloTupleCellArray defaultValue);
	IloTupleCollectionI(IloEnv env, IloTupleSchema schema, IloInt numberOfLine, IloTupleBufferI* defaultValue);
	IloTupleCollectionI(IloEnv env, IloTupleSchema schema, IloInt numberOfLine); /* initialise n lignes a 0. */
	IloTupleCollectionI(IloTupleCollectionI* set); /* par copie des data du tupleset. */
	IloDataCollectionArray getColumnArray() const { return _array; }
	virtual IloDiscreteDataCollectionI* copy() const;

	void clearSelectIndexes();
	void createSelectIndexes();
	void createSelectIndexes(IloInt idxCol);
	void createSelectIndexes(IloIntArray idxCol);

private:
	IloDiscreteDataCollectionI* findColumn(const char* name) const;

private:	
	void setValue(IloDiscreteDataCollection column, IloInt index, IloIntCollectionI* value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloNumCollectionI* value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloAnyCollectionI* value);

	void setValue(IloDiscreteDataCollection column, IloInt index, IloInt value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloNum value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloAny value);

	void setValue(IloIntArray path, IloInt index, IloInt value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloNum value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloAny value){
		setValue(getColumn(path), index, value);
	}

	void setValue(IloIntArray path, IloInt index, IloIntCollectionI* value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloNumCollectionI* value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloAnyCollectionI* value){
		setValue(getColumn(path), index, value);
	}

public:
	virtual void setLockable(IloBool flag);
	IloBool hasKey() const { 
		return (this->getTypeInfo() != IloTupleCollectionI::GetTypeInfo()) 
			&& getSchema().getImpl()->hasKey(); 
	}

protected:
    IloTupleCellArray makeKeyCells(IloInt line);
	IloTupleCellArray makeKeyCells(IloTupleCellArray);
	IloTupleCellArray getOrMakeSharedTupleCells(IloTuplePatternI*);
	IloTupleCellArray getOrMakeInternalTupleCells(IloInt);

public:
	IloTupleCellArray getOrMakeSharedTupleCells(IloTuplePathBuffer);
	IloTupleCellArray getOrMakeEmptySharedTupleCells();
	IloTupleCellArray getOrMakeSharedTupleCells(IloInt line);

	IloTupleCellArray getOrMakeSharedKeyCells(IloInt line);
	IloTupleCellArray getOrMakeSharedKeyCells(IloTupleCellArray);
	IloTupleCellArray getOrMakeEmptySharedKeyCells();

	IloTupleCellArray getOrMakeEmptyRefSliceCells();

public:

	IloIntArray makePathFromAbsolutePosition(IloInt position) const;

	virtual IloInt commit(IloTuplePatternI* exp);
	void commitWithoutSize(IloTupleCellArray array, IloInt startIndex);
public:
	virtual IloInt commit(IloTupleCellArray array);
	//void setLine(IloInt index, IloTuplePathBuffer buffer);
	virtual void remove(IloInt index);
	void remove(IloTuple buffer);
	virtual void remove(IloTupleCellArray buffer);

	void fillTupleCells(IloTupleCellArray array, IloInt line) const;
	void fillTupleBuffer(IloTupleBufferI* array, IloInt line) const;
	void fillTupleKeys(IloTupleCellArray array, IloInt line) const;
	IloTupleCellArray makeTupleCells(IloTuplePathBuffer buf) const;
	IloTupleCellArray makeTupleCells(IloInt line) const;

	const char* getColumnName(IloInt index) const{
		return this->getColumn(index).getImpl()->getName();
	}

	IloDiscreteDataCollection getColumn(IloInt index) const;

	IloTupleCollectionI* getTupleColumn(IloInt colIndex) const;

	IloInt getColumnIndex(const char* name) const;
	IloInt getColumnIndex(const IloSymbolI* s1) const;

	virtual IloBool supportDuplicates() const{ return IloTrue; }

	IloTupleSchema getSchema() const { return _schema;}

	IloInt getLength() const {
		return _array.getSize();
	}

	virtual IloBool includes(IloDataCollectionI* dom) const;
	virtual IloDataType getDataType() const;
	virtual void displayType(ILOSTD(ostream)& os) const;
	virtual IloAnyArray getArray() const;
	virtual IloBool isTupleCollection() const;

	IloInt getSize() const;

	virtual ~IloTupleCollectionI();

	IloDiscreteDataCollection getColumn(IloIntArray path) const;


	IloTuple makeTuple(IloInt index) const;
	IloTupleBuffer makeTupleBuffer(IloInt index = -1) const;
	IloTuplePathBuffer makeLine(IloInt index) const;
	virtual IloInt commit(IloTupleBuffer line);


	virtual void display(ILOSTD(ostream)& os) const;
	virtual void displayRow(IloInt i, ILOSTD(ostream)& out) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		displayRow(index, out);
	}
	IloDataIterator* iterator(IloGenAlloc* heap) const;

	IloDataIterator* iterator() const { return this->iterator(getEnv()->getGeneralAllocator()); }

	virtual IloInt getIndex(IloSymbol value) const; 
	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;

	virtual IloInt getIndex(IloAny value) const; /* provient de IloAnycollectionI... */
	virtual IloBool contains(IloAny elt) const;/* provient de IloAnycollectionI... */
	virtual IloAny getValue(IloInt index) const;/* provient de IloAnycollectionI... */

	virtual IloBool areEquivalents(IloInt index1, IloInt index2) const;
};

class IloTupleSetChangeListenerI : public IloDataChangeListenerI {
  ILORTTIDECL
  friend class IloTupleSetChangeNotifierI;
protected :
  virtual void change() const = 0;
  virtual void changeTuple(IloInt) const = 0;
  virtual void addTuple(IloInt) const = 0;
  virtual void removeTuple(IloInt) const = 0;
public:
  IloTupleSetChangeListenerI(IloEnvI* env) : IloDataChangeListenerI(env) {}
  virtual ~IloTupleSetChangeListenerI() {};
};

class IloTupleSetChangeNotifierI : public IloDataChangeNotifierI {
  ILORTTIDECL
  friend class IloTupleSetI;
  friend class IloTupleCollectionI;
  IloTupleSetI* _tupleSet;
protected:
  virtual void change() const;
  virtual void changeTuple(IloInt) const;
  virtual void addTuple(IloInt) const;
  virtual void removeTuple(IloInt) const;
public:
  IloTupleSetChangeNotifierI(IloEnvI*, IloTupleSetI*);
};

class IloTupleSetI : public IloTupleCollectionI {
	ILORTTIDECL
		virtual IloInt commit(IloTupleCellArray array){
			return commit(array, IloTrue);
		}
protected:
	IloInt commit2HashTable(IloTupleCellArray array, IloBool check);
	void fillColumns();

public:
	friend class IloTupleIterator;
public:
	virtual void setReferences(IloIntArray idx, IloArray<IloDiscreteDataCollectionI*> references);

	IloAnyArray makePointersToKeyColumns() const;


	IloIntArray intersectIndex(IloEnv env, IloIntArray set1, IloIntArray set2, IloInt v);
	IloIntArray intersectIndex(IloEnv env, IloIntArray set1, IloNumArray set2, IloNum v);
	IloIntArray intersectIndex(IloEnv env, IloIntArray set1, IloAnyArray set2, IloAny v);
	IloIntArray intersectIndex(IloEnv env, IloIntArray set1, IloIntArray set2);
	IloIntArray intersectOrderedIndex(IloEnv env, IloIntArray set1, IloIntArray set2);
	void intersectOrderedIndex(IloEnv env, IloIntArray set1, IloInt from1, IloInt to1, IloIntArray set2, IloInt& pos2, IloInt to2, IloIntArray intersection);


protected:
	enum Type { INT=1, NUM=2, ANY=3, SYMBOL=4};
	IloTupleSetI(IloEnv env, Type type, IloInt arity);
	IloVBHash* _VBhash;
	IloTupleCells2IndexHash* _tupleHash;  

private :
  IloTupleSetChangeNotifierI* _changeNotifier;
public: 
  IloTupleSetChangeNotifierI* getChangeNotifier();

private:  
	IloDiscreteDataCollectionI* findColumn(const char* name) const;
	IloIntArray makeTheQuery(IloTupleRequest query, IloBool& arrayCreated);

public:
	IloTupleSetI* makeTupleSetKey() const;

	void build(IloExtendedComprehensionI* comp, IloModelEvaluatorI* eval=0);
	IloTupleCells2IndexHash* getTupleCellsHash() const { return _tupleHash; }
	IloIntArray select(IloTupleRequest query, IloBool& arrayCreated);

	IloEnvObjectI* findVB(IloRequestSignatureI* req);
	void addVB(IloRequestSignatureI* req, IloEnvObjectI* a);

	IloTupleBuffer checkUnicityOfTuple(IloTupleRequest query);

private:
	IloIntArray select(IloInt value, IloDiscreteDataCollection column, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	IloIntArray select(IloNum value, IloDiscreteDataCollection column, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	IloIntArray select(IloAny value, IloDiscreteDataCollection column, IloBool& arrayCreated);

private:
	//void addLine(IloTuplePathBuffer buffer);

	IloIntArray select(IloTuplePath* path, IloBool& arrayCreated);

	IloIntArray select(IloInt value, IloIntArray path, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	IloIntArray select(IloNum value, IloIntArray path, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	IloIntArray select(IloAny value, IloIntArray path, IloBool& arrayCreated);

public:
	IloTupleSetI(IloTupleSetI* set);
	virtual IloTuple makeNext(IloTuple value, IloInt n=0) const;
	virtual IloTuple makePrevious(IloTuple value, IloInt n=0) const;

	virtual IloTuple makeNextC(IloTuple value, IloInt n=0) const;
	virtual IloTuple makePreviousC(IloTuple value, IloInt n=0) const;

	virtual IloTuple makeFirst() const;
	virtual IloTuple makeLast() const;

	virtual IloBool supportDuplicates() const;

	IloBool isIn(IloTupleBuffer buffer);
	IloBool isIn(IloTuplePathBuffer buffer);

	IloInt getIndex(IloTupleCellArray array);
	IloInt getIndex(IloTuplePatternI* exp);
	IloTupleI* find(IloTupleBufferI* buffer);
	IloTupleI* find(IloTupleBuffer buffer);
	IloTupleI* find(IloTuplePathBuffer buffer);

	void fillTupleHash();
	void createTupleHash();
	void clearTupleHash();

  void clearAllCaches();


	void createHashForSelect();

	void deleteHashForSelect(){
		if (_VBhash){
			for (IloVBHash::Iterator it(_VBhash); it.ok(); ++it){
				IloRequestSignatureI* sel = it.getKey();
				delete sel; sel=0;
				IloEnvObjectI* hash = (IloEnvObjectI*)*(it);
				delete hash; hash = 0;
			}
			delete _VBhash; _VBhash = 0;
		}
	}

	IloBool isHashForSelectCreated() const {
		return (_VBhash != 0);
	}

	IloInt getColumnIndex(const char* name) const;

	IloInt getColumnIndex(const IloSymbolI* s1) const;

	IloIntDataColumnI* getIntColumn(IloInt colIndex) const;
	IloIntDataColumnI* getIntColumn(const char* name) const;

	IloNumDataColumnI* getNumColumn(IloInt colIndex) const;
	IloNumDataColumnI* getNumColumn(const char* name) const;

	IloAnyDataColumnI* getAnyColumn(IloInt colIndex) const;
	IloAnyDataColumnI* getAnyColumn(const char* name) const;


	IloIntCollectionColumnI* getIntCollectionColumn(IloInt colIndex) const;
	IloIntCollectionColumnI* getIntCollectionColumn(const char* name) const;

	IloNumCollectionColumnI* getNumCollectionColumn(IloInt colIndex) const;
	IloNumCollectionColumnI* getNumCollectionColumn(const char* name) const;

	IloAnyCollectionColumnI* getAnyCollectionColumn(IloInt colIndex) const;
	IloAnyCollectionColumnI* getAnyCollectionColumn(const char* name) const;


	virtual IloDataType getDataType() const;
	virtual void displayType(ILOSTD(ostream)& os) const;
	virtual IloBool isTupleSet() const;

	IloTupleSetI(IloEnv env, IloTupleSchema schema);
	IloTupleSetI(IloEnv env, IloTupleSchema schema, IloTupleBufferI* defaultValue);
	IloTupleSetI(IloEnv env, IloTupleSchema schema, IloTupleCellArray defaultValue);

	virtual ~IloTupleSetI();


	void enableSelectIndexes(){
		//createHashForSelect();
		enableSelectIndexesOnColumn();
	}

	void disableSelectIndexes(){
		disableSelectIndexesOnColumn();
	}

	void setColumnName(IloIntArray path, const char* name){
		getColumn(path).getImpl()->setName(name);
	}
	const char* getColumnName(IloIntArray path) const{
		return this->getColumn(path).getImpl()->getName();
	}

	void setColumnName(IloInt index, const char* name){
		this->getColumn(index).getImpl()->setName(name);
	}
	const char* getColumnName(IloInt index) const{
		return this->getColumn(index).getImpl()->getName();
	}

	virtual IloInt commit(IloTupleBuffer line);
	IloInt commit(IloTupleCellArray line, IloBool check);

	IloInt commit(IloTupleBuffer line, IloBool check);
	IloInt commit(IloInt index, IloTuplePathBuffer line, IloBool check);

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	virtual IloDataIterator* iterator() const { return this->iterator(getEnv()->getGeneralAllocator()); }

	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloSymbol value) const;
	virtual IloInt getIndex(IloAny value) const;
	IloInt getIndex(IloTupleI* tuple) const;
	IloInt getIndex(IloTuple tuple) const;

	virtual IloInt commit(IloTuplePatternI*);
	IloInt commit(IloTuplePatternI* exp, IloBool check);
	virtual IloDiscreteDataCollectionI* copy() const;
	virtual void remove(IloInt index);
	//virtual void remove(IloTuple buffer);
	virtual void remove(IloTupleCellArray buffer);
	virtual IloBool areEquivalents(IloInt index1, IloInt index2) const;
};


//------------------


class IloTupleSetExprOperatorI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	IloTupleExprI* _expr;
	IloIntExprI* _offset;
	IloBool _circ;
	IloBool _sense;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getTupleCollection() const { return _coll; }
	IloTupleExprI* getTupleExpr() const { return _expr; }
	IloIntExprI* getOffSet() const { return _offset; }
	IloBool isCircular() const { return _circ; }
	IloBool isNext() const { return _sense; }
	IloTupleSetExprOperatorI(IloEnvI* env,
		IloTupleSetExprI* coll,
		IloTupleExprI* index,
		IloIntExprI* offset = 0,
		IloBool sense = IloTrue,
		IloBool circ = IloTrue);
	virtual ~IloTupleSetExprOperatorI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


class IloTupleSetExprItemI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	IloIntExprI* _offset;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getTupleCollection() const { return _coll; }
	IloIntExprI* getOffSet() const { return _offset; }
	IloTupleSetExprItemI(IloEnvI* env,
		IloTupleSetExprI* coll,
		IloIntExprI* offset = 0);
	virtual ~IloTupleSetExprItemI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


class IloTupleSetOrdI : public IloIntExprI {
  ILOEXTRDECL
  IloTupleSetExprI* _coll;
  IloTupleExprI* _expr;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleSetOrdI(IloEnvI* env, IloTupleSetExprI* coll, IloTupleExprI* expr) :
    IloIntExprI(env), _coll(coll->lockExpr()), _expr(expr->lockExpr()) {}
  virtual ~IloTupleSetOrdI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloTupleSetExprI* getCollection() const { return _coll; }
  IloTupleExprI* getExpr() const { return _expr; }
};




class IloTupleCollectionFirstI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getExpr() const { return _coll; }
	IloTupleCollectionFirstI(IloEnvI* env, IloTupleSetExprI* coll);
	virtual ~IloTupleCollectionFirstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionLastI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getExpr() const { return _coll; }
	IloTupleCollectionLastI(IloEnvI* env, IloTupleSetExprI* coll);
	virtual ~IloTupleCollectionLastI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionInterI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionInterI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionInterI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionInterCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionInterCstI(IloEnvI* env, IloTupleSetI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionInterCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionRecInterCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetI* getRight() const { return _right; }
	IloTupleCollectionRecInterCstI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetI* right);
	virtual ~IloTupleCollectionRecInterCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


//========================
class IloTupleCollectionUnionI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionUnionI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionUnionI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionUnionCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionUnionCstI(IloEnvI* env, IloTupleSetI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionUnionCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionRecUnionCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetI* getRight() const { return _right; }
	IloTupleCollectionRecUnionCstI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetI* right);
	virtual ~IloTupleCollectionRecUnionCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


//======================
class IloTupleCollectionExcludeI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionExcludeI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionExcludeI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionExcludeCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionExcludeCstI(IloEnvI* env, IloTupleSetI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionExcludeCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionRecExcludeCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetI* getRight() const { return _right; }
	IloTupleCollectionRecExcludeCstI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetI* right);
	virtual ~IloTupleCollectionRecExcludeCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

//===========================
class IloTupleCollectionSymExcludeI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionSymExcludeI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionSymExcludeI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionSymExcludeCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionSymExcludeCstI(IloEnvI* env, IloTupleSetI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionSymExcludeCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionRecSymExcludeCstI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetI* getRight() const { return _right; }
	IloTupleCollectionRecSymExcludeCstI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetI* right);
	virtual ~IloTupleCollectionRecSymExcludeCstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


class IloTupleCollectionCardI : public IloIntExprI {
	ILOEXTRDECL
		IloTupleSetExprI* _expr;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	/* <p> . </p> */
	IloTupleCollectionCardI(IloEnvI* env, IloTupleSetExprI* expr) :
	  IloIntExprI(env), _expr(expr->lockExpr()) {}
	  virtual ~IloTupleCollectionCardI();
	  /* <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /* <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /* <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  /* <p> . </p> */
	  IloTupleSetExprI* getExpr() const { return _expr; }
};



typedef IloArray<IloTupleSet> IloTupleSetArrayBase;

class ILO_EXPORTED IloTupleSetArray : public IloTupleSetArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;
  IloTupleSetArray(IloDefaultArrayI* i=0) : IloTupleSetArrayBase(i) {}
  IloTupleSetArray(const IloTupleSetArray& copy) : IloTupleSetArrayBase(copy) {}
  IloTupleSetArray(const IloEnv env) : IloTupleSetArrayBase(env, 0) {}
};


#endif
