// ------------------------------------------------------------*- C++ -*-----
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplesetH
#define __CONCERT_ilotuplesetH

#if !defined(__CONCERT_iloanysetH)
#include <ilconcert/iloanyset.h>
#endif
#if !defined(__CONCERT_ilotupleH)
#include <ilconcert/ilotuple.h>
#endif

//#define ILOHASTUPLESETI

class IloIntTupleSetI;


#define IloAnyTupleSetI IloIntTupleSetI


class IloIntTupleSet {
  IloIntTupleSetI* _impl;
public:
  operator IloExtractable();

  IloIntTupleSet(IloIntTupleSetI* impl = 0) : _impl(impl){}
  IloIntTupleSet(const IloEnv env, const IloInt arity);

  IloIntTupleSetI* getImpl() const {
    return _impl;
  }

  IloBool add(const IloIntArray tuple) const;
  IloBool remove(const IloIntArray tuple) const;
  IloBool isIn(const IloIntArray tuple) const;
  IloInt getArity()const;
};


ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out,
				   const IloIntTupleSet& dom);


class IloIntTupleSetIterator : public IloTupleIterator {
public:
  IloIntTupleSetIterator(const IloEnv env, IloIntTupleSet tset);

  IloIntTupleSetIterator(IloIntTupleSetI* tset);
  IloIntArray  operator*() const;

  void end() {/* For Compat only. */ }
};


class IloAnyTupleSet {
  IloAnyTupleSetI* _impl;
public:
  operator IloExtractable();

  IloAnyTupleSet(IloAnyTupleSetI* impl = 0) : _impl(impl){}
  IloAnyTupleSet(const IloEnv env, const IloInt arity);

  IloAnyTupleSetI* getImpl() const {
    return _impl;
  }

  IloBool add(const IloAnyArray tuple) const;
  IloBool remove(const IloAnyArray tuple) const;
  IloBool isIn(const IloAnyArray tuple) const;
  IloInt getArity()const;
};


ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out,
				   const IloAnyTupleSet& dom);


class IloAnyTupleSetIterator : public IloTupleIterator {
public:
  IloAnyTupleSetIterator(const IloEnv env, IloAnyTupleSet tset);

  IloAnyTupleSetIterator(IloAnyTupleSetI* tset);
  IloAnyArray  operator*() const;

  void end() {// For Compat only.
  }
};




//////////////////////////////////////
// PREDICATES / TABLE CONSTRAINTS


//////////////////////////////////////////////////////////////////////
// IloIntPredicate
//////////////////////////////////////////////////////////////////////

class IloIntBinaryPredicateI {
public:
  virtual ~IloIntBinaryPredicateI(){}
  virtual IloBool isTrue(const IloInt val1, const IloInt val2)=0;
};

class IloIntTernaryPredicateI {
public:
  virtual ~IloIntTernaryPredicateI(){}
  virtual IloBool isTrue(const IloInt val1,
			 const IloInt val2,
			 const IloInt val3)=0;
};

//
// API
//

class IloIntBinaryPredicate {
public:
  IloIntBinaryPredicate():_impl(0){}
  IloIntBinaryPredicate(IloIntBinaryPredicateI* impl):_impl(impl){}
  ~IloIntBinaryPredicate(){}
  IloIntBinaryPredicateI* getImpl() const {return _impl;}
  void operator=(const IloIntBinaryPredicate& h){_impl = h._impl;}
protected:
  IloIntBinaryPredicateI* _impl;
public:
  IloBool isTrue(const IloInt val1, const IloInt val2){
    assert(_impl !=0);
    return _impl->isTrue(val1,val2);
  }
};

class IloIntTernaryPredicate {
public:
  IloIntTernaryPredicate():_impl(0){}
  IloIntTernaryPredicate(IloIntTernaryPredicateI* impl):_impl(impl){}
  ~IloIntTernaryPredicate(){}
  IloIntTernaryPredicateI* getImpl() const {return _impl;}
  void operator=(const IloIntTernaryPredicate& h){_impl = h._impl;}
protected:
  IloIntTernaryPredicateI* _impl;
public:
  IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3){
    assert(_impl !=0);
    return _impl->isTrue(val1,val2,val3);
  }
};

//////////////////////////////////////////////////////////////////////
// IloAnyPredicate
//////////////////////////////////////////////////////////////////////

class IloAnyBinaryPredicateI : public IloIntBinaryPredicateI {
public:
	virtual ~IloAnyBinaryPredicateI(){}
  IloAnyBinaryPredicateI() : IloIntBinaryPredicateI() {}
  virtual IloBool isTrue(const IloAny val1, const IloAny val2)=0;
  virtual IloBool isTrue(const IloInt val1, const IloInt val2) {
    return isTrue((IloAny)val1, (IloAny)val2);
  }
};

class IloAnyTernaryPredicateI : public IloIntTernaryPredicateI {
public:
	virtual ~IloAnyTernaryPredicateI(){}
  IloAnyTernaryPredicateI() : IloIntTernaryPredicateI() {}
  virtual IloBool isTrue(const IloAny val1,
			 const IloAny val2,
			 const IloAny val3)=0;
  virtual IloBool isTrue(const IloInt val1, const IloInt val2,
			 const IloInt val3) {
    return isTrue((IloAny)val1, (IloAny)val2, (IloAny)val3);
  }
};

//
// API
//

class IloAnyBinaryPredicate {
public:
  IloAnyBinaryPredicate():_impl(0){}
  IloAnyBinaryPredicate(IloAnyBinaryPredicateI* impl):_impl(impl){}
  ~IloAnyBinaryPredicate(){}
  IloAnyBinaryPredicateI* getImpl() const {return _impl;}
  void operator=(const IloAnyBinaryPredicate& h){_impl = h._impl;}
protected:
  IloAnyBinaryPredicateI* _impl;
public:
  IloBool isTrue(const IloAny val1, const IloAny val2){
    assert(_impl !=0);
    return _impl->isTrue(val1,val2);
  }
};

class IloAnyTernaryPredicate {
public:
  IloAnyTernaryPredicate():_impl(0){}
  IloAnyTernaryPredicate(IloAnyTernaryPredicateI* impl):_impl(impl){}
  ~IloAnyTernaryPredicate(){}
  IloAnyTernaryPredicateI* getImpl() const {return _impl;}
  void operator=(const IloAnyTernaryPredicate& h){_impl = h._impl;}
protected:
  IloAnyTernaryPredicateI* _impl;
public:
  IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3){
    assert(_impl !=0);
    return _impl->isTrue(val1,val2,val3);
  }
};

//---------------------------------------------------------------------
// Macros
//---------------------------------------------------------------------

//
// Binary Predicate
//

//---------------------------------------------------------------------
// Macros
//---------------------------------------------------------------------

//
// Binary Predicate
//

// ILOANYBINARYPREDICATE0


#define ILOANYBINARYPREDICATENAME0(name, envName)\
class envName : public IloAnyBinaryPredicateI { \
  public:\
    envName():IloAnyBinaryPredicateI(){}\
    IloBool isTrue(const IloAny val1, const IloAny val2);\
};\
IloAnyBinaryPredicate name(IloEnv env){\
 return new (env) envName();\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2)

#define ILOANYBINARYPREDICATE0(name)\
ILOANYBINARYPREDICATENAME0(name, name2(name,I))

// ILOANYBINARYPREDICATE1

#define ILOANYBINARYPREDICATENAME1(name, envName, type1, nameArg1)\
class envName : public IloAnyBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    envName(type1 IlcArg1);\
    IloBool isTrue(const IloAny val1, const IloAny val2);\
};\
envName::envName(type1 IlcArg1)\
    :IloAnyBinaryPredicateI(),nameArg1(IlcArg1){}\
IloAnyBinaryPredicate name(IloEnv env,type1 arg1){\
 return new (env) envName(arg1);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2)

#define ILOANYBINARYPREDICATE1(name, type1, nameArg1)\
ILOANYBINARYPREDICATENAME1(name, name2(name,I), type1, nameArg1)

// ILOANYBINARYPREDICATE2

#define ILOANYBINARYPREDICATENAME2(name, envName, type1, nameArg1, type2, nameArg2)\
class envName : public IloAnyBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    envName(type1 IlcArg1, type2 IlcArg2);\
    IloBool isTrue(const IloAny val1, const IloAny val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2)\
   :IloAnyBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2){}\
IloAnyBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2){\
 return new (env) envName(arg1, arg2);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2)

#define ILOANYBINARYPREDICATE2(name, type1, nameArg1, type2, nameArg2)\
ILOANYBINARYPREDICATENAME2(name, name2(name,I), type1, nameArg1, type2, nameArg2)

// ILOANYBINARYPREDICATE3


#define ILOANYBINARYPREDICATENAME3(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
class envName : public IloAnyBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3);\
    IloBool isTrue(const IloAny val1, const IloAny val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3)\
   :IloAnyBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3){}\
IloAnyBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3){\
 return new (env) envName(arg1, arg2, arg3);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2)

#define ILOANYBINARYPREDICATE3(name, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
ILOANYBINARYPREDICATENAME3(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3)

// ILOANYBINARYPREDICATE4


#define ILOANYBINARYPREDICATENAME4(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
class envName : public IloAnyBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4);\
    IloBool isTrue(const IloAny val1, const IloAny val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4)\
   :IloAnyBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4){}\
IloAnyBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4){\
 return new (env) envName(arg1, arg2, arg3, arg4);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2)

#define ILOANYBINARYPREDICATE4(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
ILOANYBINARYPREDICATENAME4(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)


// ILOANYBINARYPREDICATE5


#define ILOANYBINARYPREDICATENAME5(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
class envName : public IloAnyBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5);\
    IloBool isTrue(const IloAny val1, const IloAny val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5)\
   :IloAnyBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5){}\
IloAnyBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2)

#define ILOANYBINARYPREDICATE5(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
ILOANYBINARYPREDICATENAME5(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)

// ILOANYBINARYPREDICATE6


#define ILOANYBINARYPREDICATENAME6(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
class envName : public IloAnyBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    type6 nameArg6; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6);\
    IloBool isTrue(const IloAny val1, const IloAny val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6)\
   :IloAnyBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5), nameArg6(IlcArg6){}\
IloAnyBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5, type6 arg6){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5, arg6);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2)

#define ILOANYBINARYPREDICATE6(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
ILOANYBINARYPREDICATENAME6(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)


//
// Ternary Predicate
//

// ILOANYTERNARYPREDICATE0

#define ILOANYTERNARYPREDICATENAME0(name, envName)\
class envName : public IloAnyTernaryPredicateI { \
  public:\
    envName():IloAnyTernaryPredicateI(){}\
    IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3);\
};\
IloAnyTernaryPredicate name(IloEnv env){\
 return new (env) envName();\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2, const IloAny val3)

#define ILOANYTERNARYPREDICATE0(name)\
ILOANYTERNARYPREDICATENAME0(name, name2(name,I))

// ILOANYTERNARYPREDICATE1

#define ILOANYTERNARYPREDICATENAME1(name, envName, type1, nameArg1)\
class envName : public IloAnyTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    envName(type1 IlcArg1);\
    IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3);\
};\
envName::envName(type1 IlcArg1)\
    :IloAnyTernaryPredicateI(),nameArg1(IlcArg1){}\
IloAnyTernaryPredicate name(IloEnv env,type1 arg1){\
 return new (env) envName(arg1);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2, const IloAny val3)

#define ILOANYTERNARYPREDICATE1(name, type1, nameArg1)\
ILOANYTERNARYPREDICATENAME1(name, name2(name,I), type1, nameArg1)

// ILOANYTERNARYPREDICATE2

#define ILOANYTERNARYPREDICATENAME2(name, envName, type1, nameArg1, type2, nameArg2)\
class envName : public IloAnyTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    envName(type1 IlcArg1, type2 IlcArg2);\
    IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2)\
   :IloAnyTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2){}\
IloAnyTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2){\
 return new (env) envName(arg1, arg2);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2, const IloAny val3)

#define ILOANYTERNARYPREDICATE2(name, type1, nameArg1, type2, nameArg2)\
ILOANYTERNARYPREDICATENAME2(name, name2(name,I), type1, nameArg1, type2, nameArg2)

// ILOANYTERNARYPREDICATE3

#define ILOANYTERNARYPREDICATENAME3(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
class envName : public IloAnyTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3);\
    IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3)\
   :IloAnyTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3){}\
IloAnyTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3){\
 return new (env) envName(arg1, arg2, arg3);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2, const IloAny val3)

#define ILOANYTERNARYPREDICATE3(name, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
ILOANYTERNARYPREDICATENAME3(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3)

// ILOANYTERNARYPREDICATE4

#define ILOANYTERNARYPREDICATENAME4(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
class envName : public IloAnyTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4);\
    IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4)\
   :IloAnyTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4){}\
IloAnyTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4){\
 return new (env) envName(arg1, arg2, arg3, arg4);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2, const IloAny val3)

#define ILOANYTERNARYPREDICATE4(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
ILOANYTERNARYPREDICATENAME4(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)


// ILOANYTERNARYPREDICATE5

#define ILOANYTERNARYPREDICATENAME5(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
class envName : public IloAnyTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5);\
    IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5)\
   :IloAnyTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5){}\
IloAnyTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2, const IloAny val3)

#define ILOANYTERNARYPREDICATE5(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
ILOANYTERNARYPREDICATENAME5(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)

// ILOANYTERNARYPREDICATE6

#define ILOANYTERNARYPREDICATENAME6(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
class envName : public IloAnyTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    type6 nameArg6; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6);\
    IloBool isTrue(const IloAny val1, const IloAny val2, const IloAny val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6)\
   :IloAnyTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5), nameArg6(IlcArg6){}\
IloAnyTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5, type6 arg6){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5, arg6);\
}\
IloBool envName ::isTrue(const IloAny val1, const IloAny val2, const IloAny val3)

#define ILOANYTERNARYPREDICATE6(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
ILOANYTERNARYPREDICATENAME6(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)

//---------------------------------------------------------------------
// Macros
//---------------------------------------------------------------------

//
// Binary Predicate
//

// ILOINTBINARYPREDICATE0

#define ILOINTBINARYPREDICATENAME0(name, envName)\
class envName : public IloIntBinaryPredicateI { \
  public:\
    envName():IloIntBinaryPredicateI(){}\
    IloBool isTrue(const IloInt val1, const IloInt val2);\
};\
IloIntBinaryPredicate name(IloEnv env){\
 return new (env) envName();\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2)

#define ILOINTBINARYPREDICATE0(name)\
ILOINTBINARYPREDICATENAME0(name, name2(name,I))

// ILOINTBINARYPREDICATE1

#define ILOINTBINARYPREDICATENAME1(name, envName, type1, nameArg1)\
class envName : public IloIntBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    envName(type1 IlcArg1);\
    IloBool isTrue(const IloInt val1, const IloInt val2);\
};\
envName::envName(type1 IlcArg1)\
    :IloIntBinaryPredicateI(),nameArg1(IlcArg1){}\
IloIntBinaryPredicate name(IloEnv env,type1 arg1){\
 return new (env) envName(arg1);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2)

#define ILOINTBINARYPREDICATE1(name, type1, nameArg1)\
ILOINTBINARYPREDICATENAME1(name, name2(name,I), type1, nameArg1)

// ILOINTBINARYPREDICATE2

#define ILOINTBINARYPREDICATENAME2(name, envName, type1, nameArg1, type2, nameArg2)\
class envName : public IloIntBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    envName(type1 IlcArg1, type2 IlcArg2);\
    IloBool isTrue(const IloInt val1, const IloInt val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2)\
   :IloIntBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2){}\
IloIntBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2){\
 return new (env) envName(arg1, arg2);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2)

#define ILOINTBINARYPREDICATE2(name, type1, nameArg1, type2, nameArg2)\
ILOINTBINARYPREDICATENAME2(name, name2(name,I), type1, nameArg1, type2, nameArg2)

// ILOINTBINARYPREDICATE3

#define ILOINTBINARYPREDICATENAME3(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
class envName : public IloIntBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3);\
    IloBool isTrue(const IloInt val1, const IloInt val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3)\
   :IloIntBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3){}\
IloIntBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3){\
 return new (env) envName(arg1, arg2, arg3);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2)

#define ILOINTBINARYPREDICATE3(name, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
ILOINTBINARYPREDICATENAME3(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3)

// ILOINTBINARYPREDICATE4

#define ILOINTBINARYPREDICATENAME4(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
class envName : public IloIntBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4);\
    IloBool isTrue(const IloInt val1, const IloInt val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4)\
   :IloIntBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4){}\
IloIntBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4){\
 return new (env) envName(arg1, arg2, arg3, arg4);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2)

#define ILOINTBINARYPREDICATE4(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
ILOINTBINARYPREDICATENAME4(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)


// ILOINTBINARYPREDICATE5

#define ILOINTBINARYPREDICATENAME5(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
class envName : public IloIntBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5);\
    IloBool isTrue(const IloInt val1, const IloInt val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5)\
   :IloIntBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5){}\
IloIntBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2)

#define ILOINTBINARYPREDICATE5(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
ILOINTBINARYPREDICATENAME5(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)

// ILOINTBINARYPREDICATE6

#define ILOINTBINARYPREDICATENAME6(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
class envName : public IloIntBinaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    type6 nameArg6; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6);\
    IloBool isTrue(const IloInt val1, const IloInt val2);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6)\
   :IloIntBinaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5), nameArg6(IlcArg6){}\
IloIntBinaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5, type6 arg6){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5, arg6);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2)

#define ILOINTBINARYPREDICATE6(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
ILOINTBINARYPREDICATENAME6(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)


//
// Ternary Predicate
//

// ILOINTTERNARYPREDICATE0

#define ILOINTTERNARYPREDICATENAME0(name, envName)\
class envName : public IloIntTernaryPredicateI { \
  public:\
    envName():IloIntTernaryPredicateI(){}\
    IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3);\
};\
IloIntTernaryPredicate name(IloEnv env){\
 return new (env) envName();\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2, const IloInt val3)

#define ILOINTTERNARYPREDICATE0(name)\
ILOINTTERNARYPREDICATENAME0(name, name2(name,I))

// ILOINTTERNARYPREDICATE1

#define ILOINTTERNARYPREDICATENAME1(name, envName, type1, nameArg1)\
class envName : public IloIntTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    envName(type1 IlcArg1);\
    IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3);\
};\
envName::envName(type1 IlcArg1)\
    :IloIntTernaryPredicateI(),nameArg1(IlcArg1){}\
IloIntTernaryPredicate name(IloEnv env,type1 arg1){\
 return new (env) envName(arg1);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2, const IloInt val3)

#define ILOINTTERNARYPREDICATE1(name, type1, nameArg1)\
ILOINTTERNARYPREDICATENAME1(name, name2(name,I), type1, nameArg1)

// ILOINTTERNARYPREDICATE2

#define ILOINTTERNARYPREDICATENAME2(name, envName, type1, nameArg1, type2, nameArg2)\
class envName : public IloIntTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    envName(type1 IlcArg1, type2 IlcArg2);\
    IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2)\
   :IloIntTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2){}\
IloIntTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2){\
 return new (env) envName(arg1, arg2);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2, const IloInt val3)

#define ILOINTTERNARYPREDICATE2(name, type1, nameArg1, type2, nameArg2)\
ILOINTTERNARYPREDICATENAME2(name, name2(name,I), type1, nameArg1, type2, nameArg2)

// ILOINTTERNARYPREDICATE3

#define ILOINTTERNARYPREDICATENAME3(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
class envName : public IloIntTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3);\
    IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3)\
   :IloIntTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3){}\
IloIntTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3){\
 return new (env) envName(arg1, arg2, arg3);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2, const IloInt val3)

#define ILOINTTERNARYPREDICATE3(name, type1, nameArg1, type2, nameArg2, type3, nameArg3)\
ILOINTTERNARYPREDICATENAME3(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3)

// ILOINTTERNARYPREDICATE4

#define ILOINTTERNARYPREDICATENAME4(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
class envName : public IloIntTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4);\
    IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4)\
   :IloIntTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4){}\
IloIntTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4){\
 return new (env) envName(arg1, arg2, arg3, arg4);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2, const IloInt val3)

#define ILOINTTERNARYPREDICATE4(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)\
ILOINTTERNARYPREDICATENAME4(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4)


// ILOINTTERNARYPREDICATE5

#define ILOINTTERNARYPREDICATENAME5(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
class envName : public IloIntTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5);\
    IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5)\
   :IloIntTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5){}\
IloIntTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2, const IloInt val3)

#define ILOINTTERNARYPREDICATE5(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)\
ILOINTTERNARYPREDICATENAME5(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5)

// ILOINTTERNARYPREDICATE6

#define ILOINTTERNARYPREDICATENAME6(name, envName, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
class envName : public IloIntTernaryPredicateI { \
  public:\
    type1 nameArg1; \
    type2 nameArg2; \
    type3 nameArg3; \
    type4 nameArg4; \
    type5 nameArg5; \
    type6 nameArg6; \
    envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6);\
    IloBool isTrue(const IloInt val1, const IloInt val2, const IloInt val3);\
};\
envName::envName(type1 IlcArg1, type2 IlcArg2, type3 IlcArg3, type4 IlcArg4, type5 IlcArg5, type6 IlcArg6)\
   :IloIntTernaryPredicateI(),nameArg1(IlcArg1), nameArg2(IlcArg2), nameArg3(IlcArg3), nameArg4(IlcArg4), nameArg5(IlcArg5), nameArg6(IlcArg6){}\
IloIntTernaryPredicate name(IloEnv env,type1 arg1, type2 arg2, type3 arg3, type4 arg4, type5 arg5, type6 arg6){\
 return new (env) envName(arg1, arg2, arg3, arg4, arg5, arg6);\
}\
IloBool envName ::isTrue(const IloInt val1, const IloInt val2, const IloInt val3)

#define ILOINTTERNARYPREDICATE6(name, type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)\
ILOINTTERNARYPREDICATENAME6(name, name2(name,I), type1, nameArg1, type2, nameArg2, type3, nameArg3, type4, nameArg4, type5, nameArg5, type6, nameArg6)

/////////////////////
//
// ILOTABLECONSTRAINT
//

// Int Table Constraint

IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVarArray vars,
					const IloIntTupleSet set,
          IloBool compatible);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVar var1,
					const IloIntVar var2,
					const IloIntTupleSet set,
          IloBool compatible);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVar var1,
					const IloIntVar var2,
					const IloIntVar var3,
					const IloIntTupleSet set,
          IloBool compatible);


IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVarArray vars,
					const IloIntTernaryPredicate pred);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVar var1,
					const IloIntVar var2,
					const IloIntVar var3,
					const IloIntTernaryPredicate pred);
IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVarArray vars,
					const IloIntBinaryPredicate pred);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVar var1,
					const IloIntVar var2,
					const IloIntBinaryPredicate pred);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloIntVar y,
					const IloIntArray a,
					const IloNumVar x);



// IloTableConstraint

// Num Table Constraint

// Any Table Constraint

IloConstraint IloTableConstraint(const IloEnv env,
					const IloAnyVarArray vars,
					const IloAnyTupleSet set,
          IloBool compatible);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloAnyVarArray vars,
					const IloAnyTernaryPredicate pred);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloAnyVarArray vars,
					const IloAnyBinaryPredicate pred);

IloConstraint IloTableConstraint(const IloEnv env,
					const IloAnyVar y,
					const IloAnyArray a,
					const IloNumVar x);


#endif // __CONCERT_ilotuplesetH
