// ------------------------------------------------------------*- C++ -*-----
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplesetiH
#define __CONCERT_ilotuplesetiH

#if !defined(__CONCERT_ilotuplesetH)
#include <ilconcert/ilotupleset.h>
#endif
#if !defined(__CONCERT_ilotuplecollectioniH)
#include <ilconcert/ilotuplecollectioni.h>
#endif




#define IloAnyTupleSetI IloIntTupleSetI


class IloIntTupleSetI : public IloTupleSetI {
	ILORTTIDECL
public:
  class ExtractorI : public IloExtractableI {
  private:
    ILOEXTRDECL
    IloTupleSetI* _coll;
    virtual void visitSubExtractables(IloExtractableVisitor* v);
  public:
    virtual ~ExtractorI(){}
    ExtractorI(IloTupleSetI* coll)
      : IloExtractableI(coll->getEnv()), _coll(coll){}
    IloTupleSetI* getCollection() const { return _coll; }
    ILOEXTROTHERDECL
  };
  class Extractor : public IloExtractable {
    ILOEXTRHANDLE(Extractor, IloExtractable)
  };
private:
  ExtractorI* _extractor;
public:
  virtual ~IloIntTupleSetI(){
    if (_extractor) { IloExtractable(_extractor).end(); _extractor = 0;}
  }
  IloIntTupleSetI(IloEnvI* env,const IloInt arity);

  IloIntTupleSetI* makeClone(IloEnvI* env) const;

  IloExtractableI* getExtractor() const {
    return _extractor;
  }

  IloInt getArity()const;
  IloBool add(const IloIntArray vals);
  IloBool remove(const IloIntArray);
  IloBool isIn(const IloIntArray) const;
};

class IloIntTupleSetAddTuple : public IloChange {
  ILOCHANGEDECL
  IloIntTupleSetI::ExtractorI* _set;
  IloIntArray      _elt;
public:
  IloIntTupleSetAddTuple (IloIntTupleSetI::ExtractorI* set, IloIntArray elt)
	  : _set(set), _elt(elt)   { }

  IloIntTupleSetI::ExtractorI* getSet() const { return (_set); }
  IloIntArray getElement()  const { return (_elt); }
};

class IloIntTupleSetRemoveTuple  : public IloChange {
  ILOCHANGEDECL
  IloIntTupleSetI::ExtractorI* _set;
  IloIntArray      _elt;
public:
  IloIntTupleSetRemoveTuple(IloIntTupleSetI::ExtractorI* set, IloIntArray elt)
    : _set(set), _elt(elt)   {}

  IloIntTupleSetI::ExtractorI* getSet() const    { return (_set); }
  IloIntArray    getElement()  const    { return (_elt); }
};



// INT table constraint

// Table constraint with TupleSet

class IloIntTableSetConstraintI : public IloConstraintI {
  ILOEXTRMEMBERS3DECL(IloIntTableSetConstraintI, IloConstraintI,
		    IloIntVarArray, _vars,
		    IloIntTupleSet, _set,
		    IloBool, _compatible)
  virtual ~IloIntTableSetConstraintI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntVarArray getVars() const {return _vars;}
  IloIntTupleSet getSet() const {return _set;}
  IloExtractableI* getExtractable() const {return _set.getImpl()->getExtractor();}

  IloBool getCompatible() const { return _compatible; }
};

class IloIntTableSetConstraint : public IloConstraint {
  ILOEXTRHANDLE(IloIntTableSetConstraint,IloConstraint)
public:
  IloIntTableSetConstraint(const IloEnv env,
			   const IloIntVarArray vars,
			   const IloIntTupleSet set,
			   IloBool compatible,
			   const char* name=0) :
    IloConstraint(new (env) IloIntTableSetConstraintI(env.getImpl(),
						      vars.copy(),
						      set,
						      compatible,
						      name)) {
    IloAssert(vars.getImpl(), "Using empty IloIntVarArray handle.");
    IloAssert(set.getImpl(), "Using empty IloIntTupleSet handle.");
  }
};

// Table constraint with binary predicates

class IloIntTableBinPredicateConstraintI : public IloConstraintI {
  ILOEXTRMEMBERS2DECL(IloIntTableBinPredicateConstraintI, IloConstraintI,
        IloIntVarArray, _vars,
        IloIntBinaryPredicateI*, _pred)
  virtual ~IloIntTableBinPredicateConstraintI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntBinaryPredicateI* getPredicate() const{return _pred;}
  IloIntVarArray getVars() const {return _vars;}
};

class IloIntTableBinPredicateConstraint : public IloConstraint {
  ILOEXTRHANDLE(IloIntTableBinPredicateConstraint,IloConstraint)
public:
  IloIntTableBinPredicateConstraint(const IloEnv env,
				  const IloIntVarArray vars,
				  const IloIntBinaryPredicate pred,
				  const char* name=0) :
  IloConstraint(new (env) IloIntTableBinPredicateConstraintI(env.getImpl(),
							     vars.copy(),
							     pred.getImpl(),
							     name)) {
    IloAssert(vars.getImpl(), "Using empty IloIntVarArray handle.");
  }
};

// Table constraint with ternary predicates

class IloIntTableTernaryPredicateConstraintI : public IloConstraintI {
  ILOEXTRMEMBERS2DECL(IloIntTableTernaryPredicateConstraintI, IloConstraintI,
        IloIntVarArray, _vars,
        IloIntTernaryPredicateI*, _pred)
  virtual ~IloIntTableTernaryPredicateConstraintI();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntTernaryPredicateI* getPredicate()const {return _pred;}
  IloIntVarArray getVars() const {return _vars;}
};

class IloIntTableTernaryPredicateConstraint : public IloConstraint {
  ILOEXTRHANDLE(IloIntTableTernaryPredicateConstraint,IloConstraint)
public:
  IloIntTableTernaryPredicateConstraint(const IloEnv env,
				  const IloIntVarArray vars,
				  const IloIntTernaryPredicate pred,
				  const char* name=0) :
  IloConstraint(new (env) IloIntTableTernaryPredicateConstraintI(env.getImpl(),
							     vars.copy(),
							     pred.getImpl(),
							     name)) {
    IloAssert(vars.getImpl(), "Using empty IloIntVarArray handle.");
  }
};

// Table as element constraint

class IloIntTableElementConstraint2I : public IloConstraintI {
  ILOEXTRMEMBERS3DECL(IloIntTableElementConstraint2I, IloConstraintI,
          IloIntVar, _y,
          IloIntArray, _a,
          IloNumVar, _x)
  virtual ~IloIntTableElementConstraint2I();
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntVar getYVar() const {return _y;}
  IloNumVar getXVar() const {return _x;}
  IloIntArray getArray() const {return _a;}
};

class IloIntTableElementConstraint2 : public IloConstraint {
  ILOEXTRHANDLE(IloIntTableElementConstraint2,IloConstraint)
public:
  IloIntTableElementConstraint2(const IloEnv env,
         const IloIntVar y,
         const IloIntArray a,
         const IloNumVar x,
         const char* name=0):
  IloConstraint(new (env) IloIntTableElementConstraint2I(env.getImpl(),
							y, a.copy(), x,name)) {
  }
};

// ANY table constraint

// Table constraint with TupleSet

#define IloAnyTableSetConstraintI IloIntTableSetConstraintI

class IloAnyTableSetConstraint : public IloConstraint {
  ILOEXTRHANDLE(IloAnyTableSetConstraint,IloConstraint)
public:
  IloAnyTableSetConstraint(const IloEnv env,
			   const IloAnyVarArray vars,
			   const IloAnyTupleSet set,
			   IloBool compatible,
			   const char* name=0) :
    IloConstraint(new (env) IloIntTableSetConstraintI(env.getImpl(),
						      vars.copy(),
						      set.getImpl(),
						      compatible,
						      name)) {
    IloAssert(vars.getImpl(), "Using empty IloAnyVarArray handle.");
    IloAssert(set.getImpl(), "Using empty IloAnyTupleSet handle.");
  }
};

// Table constraint with binary predicates

#define IloAnyTableBinPredicateConstraintI IloIntTableBinPredicateConstraintI

class IloAnyTableBinPredicateConstraint : public IloConstraint {
  ILOEXTRHANDLE(IloAnyTableBinPredicateConstraint,IloConstraint)
public:
  IloAnyTableBinPredicateConstraint(const IloEnv env,
				  const IloAnyVarArray vars,
				  const IloAnyBinaryPredicate pred,
				  const char* name=0) :
  IloConstraint(new (env) IloIntTableBinPredicateConstraintI(env.getImpl(),
							     vars.copy(),
							     pred.getImpl(),
							     name)) {
    IloAssert(vars.getImpl(), "Using empty IloAnyVarArray handle.");
  }
};

// Table constraint with ternary predicates

#define IloAnyTableTernaryPredicateConstraintI IloIntTableTernaryPredicateConstraintI

class IloAnyTableTernaryPredicateConstraint : public IloConstraint {
  ILOEXTRHANDLE(IloAnyTableTernaryPredicateConstraint,IloConstraint)
public:
  IloAnyTableTernaryPredicateConstraint(const IloEnv env,
				  const IloAnyVarArray vars,
				  const IloAnyTernaryPredicate pred,
				  const char* name=0) :
  IloConstraint(new (env) IloAnyTableTernaryPredicateConstraintI(env.getImpl(),
							     vars.copy(),
							     pred.getImpl(),
							     name)) {
    IloAssert(vars.getImpl(), "Using empty IloAnyVarArray handle.");
  }
};

// Table as element constraint

#define IloAnyTableElementConstraint2I IloIntTableElementConstraint2I

class IloAnyTableElementConstraint2 : public IloConstraint {
  ILOEXTRHANDLE(IloAnyTableElementConstraint2,IloConstraint)
public:
  IloAnyTableElementConstraint2(const IloEnv env,
         const IloAnyVar y,
         const IloAnyArray a,
         const IloNumVar x,
         const char* name=0);
};

#endif // __CONCERT_ilotuplesetiH
