// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilocollectioncolumniH
#define __CONCERT_ilocollectioncolumniH

#if !defined(__CONCERT_ilodatacolumniH)
# include <ilconcert/iltuple/ilodatacolumni.h>
#endif

class IloIntMap;
class IloNumMap;
class IloAnyMap;

class IloMapI;


/**
* @group optim.concert
* @show internal
* @brief A class that allow to consider an IloIntArray as an integer collection.
*
*/
class IloIntMapAsCollectionI : public IloIntCollectionI {
	ILORTTIDECL
private:
	IloMapI* _map;
public:
	virtual IloDiscreteDataCollectionI* copy() const;

	/** <p> . </p> */
	IloIntMapAsCollectionI(const IloIntMap map);
	/** <p> . </p> */
	IloIntMap getMap() const;
	/** <p> . </p> */
	IloDiscreteDataCollectionI* getIndexer() const;
	virtual ~IloIntMapAsCollectionI();
	/** <p> . </p> */
	virtual IloDataType getDataType() const;
	/** <p> . </p> */
	virtual IloBool includes(IloDataCollectionI* dom) const;
	/** <p> . </p> */
	virtual IloInt getSize() const;
	/** <p> . </p> */
	virtual IloIntArray getArray() const;
	/** <p> . </p> */
	virtual IloBool contains(IloInt e) const;
	/** <p> . </p> */
	virtual IloInt getValue(IloInt index) const;
	/** <p> . </p> */
	virtual  const IloInt& operator[] (IloInt i) const;
	/** <p> . </p> */
	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	/** <p> . </p> */
	virtual IloDataIterator* iterator() const;
	/** <p> . </p> */
	virtual void display(ILOSTD(ostream)& os) const;
	/** <p> . </p> */
	virtual void displayType(ILOSTD(ostream)& os) const;
};

/**
* @group optim.concert
* @show internal
* @brief A class that allow to consider an IloNumArray as a numeric collection.
*
*/
class IloNumMapAsCollectionI : public IloNumCollectionI {
	ILORTTIDECL
private:
	IloMapI* _map;
public:
	virtual IloDiscreteDataCollectionI* copy() const;

	/** <p> . </p> */
	IloNumMapAsCollectionI(const IloNumMap map);
	/** <p> . </p> */
	IloNumMap getMap() const;
	/** <p> . </p> */
	IloDiscreteDataCollectionI* getIndexer() const;
	virtual ~IloNumMapAsCollectionI();
	/** <p> . </p> */
	virtual IloDataType getDataType() const;
	/** <p> . </p> */
	virtual IloBool includes(IloDataCollectionI* dom) const;
	/** <p> . </p> */
	virtual IloInt getSize() const;
	/** <p> . </p> */
	virtual IloNumArray getArray() const;
	/** <p> . </p> */
	virtual const IloNum& operator[] (IloInt i) const;
	/** <p> . </p> */
	virtual IloBool contains(IloNum e) const;
	/** <p> . </p> */
	virtual IloNum getValue(IloInt index) const;
	/** <p> . </p> */
	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	/** <p> . </p> */
	virtual IloDataIterator* iterator() const;
	/** <p> . </p> */
	virtual void display(ILOSTD(ostream)& os) const;
	/** <p> . </p> */
	virtual void displayType(ILOSTD(ostream)& os) const;
};



/**
* @group optim.concert.tuple
* @show internal
* <p> IloCollectionColumnI </p>
**/
class IloCollectionColumnI : public IloAnyDataColumnI {
	IloBool _mustDelete;
	ILORTTIDECL
    //IloDiscreteDataCollectionI* _defaultValue;
protected:
        using IloAnyCollectionI::getIndex;
	virtual void updateHashForSelect(IloInt index, IloAny value, IloBool addIndex = IloTrue);
public:
	// for RS6000 because getArray is const.
	IloCollectionColumnI(IloEnvI* env, const IloAnyArray array, IloDiscreteDataCollectionI* defaultValue = 0);
	/** <p>.</p> **/
	IloCollectionColumnI(IloEnvI* env, IloInt n=0): IloAnyDataColumnI(env, n), _mustDelete(IloTrue) {}//, _defaultValue(0) {}
	virtual ~IloCollectionColumnI();
	virtual void add(IloAny elt);
	virtual void add(IloAnyDataColumnI* set);
	virtual void discard(IloAny value);
	virtual void remove(IloInt index);
	virtual void setValue(IloInt index, IloAny value);
	virtual void empty();
	virtual void setDefaultValue(const IloAny coll);
	void mustDelete(IloBool flag){ _mustDelete = flag; }
	IloBool mustDelete() const{ return _mustDelete;}
	virtual IloInt getIndex(IloAny val) const;
	virtual IloIntArray makeIndexArray(IloAny value) const;
};



/**
* @group optim.concert.tuple
* @show internal
*
* <p>An instance of this class offers a convenient way to represent a set of integer
* values as a constrained variable in Concert Technology.</p>
*
* @see IloDataIterator
* @see IloIntDataColumn::Iterator
*/
class IloTupleRefDataColumnI : public IloIntDataColumnI {
	ILORTTIDECL
private:
	IloTupleSetI* _refered;
	IloBool _checkReference;
	IloIndex2KeyHash* _hashForKeys; 
public:
        using IloIntCollectionI::getIndex;
	virtual IloDiscreteDataCollectionI* copy() const;
	virtual void setReference(IloDiscreteDataCollectionI* refered){
		throw IloWrongUsage("can not set simple reference on a TupleReference column");
	}

	/** <p> Constructor. </p> **/
	IloTupleRefDataColumnI(IloEnvI* env, IloTupleSetI* refered, IloInt n, IloBool checkReferences);

	IloBool checkReferences() const { return _checkReference;};
	void setCheckReferences(IloBool flag) { _checkReference = flag; };
	IloInt commit(IloTupleCellArray line);
	IloBool setLine(IloInt, IloTupleCellArray line);

	IloTupleCellArray getOrMakeSharedTupleCells(IloTuplePathBuffer);
	IloTupleCellArray getOrMakeEmptySharedTupleCells();
	IloTupleCellArray getOrMakeSharedTupleCells(IloInt line);
	//IloTupleCellArray getOrMakeSharedKeyCells(IloTuplePathBuffer);
	IloTupleCellArray getOrMakeSharedKeyCells(IloInt line);
	IloTupleCellArray getOrMakeSharedKeyCells(IloTupleCellArray);
	IloTupleCellArray getOrMakeEmptySharedKeyCells();
	void fillTupleCells(IloTupleCellArray array, IloInt line) const;

	IloInt getIndex(IloTupleCellArray array);

	IloInt getWidth() const; 

	/** <p> </p> **/
	virtual ~IloTupleRefDataColumnI();

	/** <p> </p> **/
	virtual void empty();

	/** <p> </p> **/
	virtual IloDataType getDataType() const;

	/** <p> </p> **/
	virtual IloBool isIntDataColumn() const{ return IloFalse;}
	virtual IloBool isTupleRefColumn() const{ return IloTrue;}

	/** <p> </p> **/
	virtual void displayType(ILOSTD(ostream)& os) const;

	/** <p> </p> **/
	virtual void display(ILOSTD(ostream)& os) const;
	virtual void displayKeys(ILOSTD(ostream)& os) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const;
	IloTupleSetI* getTupleCollection() const{return _refered;}
	IloIndex2KeyHash* getHashForKeys() const{ return _hashForKeys;}
	virtual void remove(IloInt index);
	virtual void remove(IloTupleCellArray cells);
};



/**
* @group optim.concert.tuple
* @show internal
* <p> IloIntCollectionColumnI </p>
**/
class IloIntCollectionColumnI : public IloCollectionColumnI {
	ILORTTIDECL
public:
	IloIntCollectionColumnI(IloEnvI* env, const IloAnyArray array, IloDiscreteDataCollectionI* defaultFalue = 0);
	/** <p>.</p> **/
	IloIntCollectionColumnI(IloEnvI* env, IloInt n=0): IloCollectionColumnI(env, 0) {
		if (n> 0){
			IloIntSetI* col = new (env) IloIntSetI(env);
			setDefaultValue(col);
			getArray().getImpl()->setSize(n);
			for (IloInt i=0; i< n; i++){
				getArray()[i] = col;
			}
		}
	}
	/** <p> . </p> */
	virtual IloBool isIntCollectionColumn() const;
	virtual IloDiscreteDataCollectionI* copy() const;
	virtual void enableSelectIndexes() {
		if (!_hashForSelect) _hashForSelect = new (getEnv()) IloAnyDataTableHash(getEnv(), IloIntCollectionHashFunction, IloIntCollectionCompFunction);
	}
};

/**
* @group optim.concert.tuple
* @show internal
* <p> IloNumCollectionColumnI </p>
**/
class IloNumCollectionColumnI : public IloCollectionColumnI {
	ILORTTIDECL
public:
	IloNumCollectionColumnI(IloEnvI* env, const IloAnyArray array, IloDiscreteDataCollectionI* defaultFalue = 0);
	/** <p>.</p> **/
	IloNumCollectionColumnI(IloEnvI* env, IloInt n=0): IloCollectionColumnI(env, 0) {
		if (n > 0){
			IloNumSetI* col = new (env) IloNumSetI(env);
			setDefaultValue(col);
			getArray().getImpl()->setSize(n);
			for (IloInt i=0; i< n; i++){
				getArray()[i] = _defaultValue;
			}
		}
	}
	/** <p> . </p> */
	virtual IloBool isNumCollectionColumn() const;
	virtual IloDiscreteDataCollectionI* copy() const;
	virtual void enableSelectIndexes() {
		if (!_hashForSelect) _hashForSelect = new (getEnv()) IloAnyDataTableHash(getEnv(), IloNumCollectionHashFunction, IloNumCollectionCompFunction);
	}
};

/**
* @group optim.concert.tuple
* @show internal
* <p> IloAnyCollectionColumnI </p>
**/
class IloAnyCollectionColumnI : public IloCollectionColumnI {
	ILORTTIDECL
public:
	IloAnyCollectionColumnI(IloEnvI* env, const IloAnyArray array, IloDiscreteDataCollectionI* defaultFalue = 0);
	/** <p>.</p> **/
	IloAnyCollectionColumnI(IloEnvI* env, IloInt n=0): IloCollectionColumnI(env, 0) {
		if (n > 0){
			IloAnySetI* col = new (env) IloAnySetI(env);
			setDefaultValue(col);
			getArray().getImpl()->setSize(n);
			for (IloInt i=0; i< n; i++){
				getArray()[i] = _defaultValue;
			}
		}
	}
	/** <p> . </p> */
	virtual IloBool isAnyCollectionColumn() const;
	virtual IloDiscreteDataCollectionI* copy() const;
	virtual void enableSelectIndexes() {
		if (!_hashForSelect) _hashForSelect = new (getEnv()) IloAnyDataTableHash(getEnv(), IloAnyCollectionHashFunction, IloAnyCollectionCompFunction);
	}
};


/**
* @group optim.concert.tuple
* @show internal
* <p> IloCollectionUtil </p>
**/
class IloCollectionUtil {
public:

	/** <p> . </p> */
	static IloIntMap getMap(IloIntCollection coll);
	/** <p> . </p> */
	static IloNumMap getMap(IloNumCollection coll);
};

#endif
