// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilodatacolumniH
#define __CONCERT_ilodatacolumniH

#if !defined(__CONCERT_ilodatacolumnH)
#include <ilconcert/iltuple/ilodatacolumn.h>
#endif
#if !defined(__CONCERT_iloexpressioniH)
# include <ilconcert/iloexpressioni.h>
#endif
#if !defined(__CONCERT_ilocollectioniH)
# include <ilconcert/ilocollectioni.h>
#endif

#if !defined(__CONCERT_ilotuplecollectionhashH)
# include <ilconcert/iltuple/ilotuplecollectionhash.h>
#endif


/**
* @group optim.concert.tuple
* @show internal
*
* <p>An instance of this class offers a convenient way to represent a set of integer
* values as a constrained variable in Concert Technology.</p>
*
* @see IloDataIterator
* @see IloIntDataColumn::Iterator
*/
class IloIntDataColumnI : public IloIntAbstractDataColumnI {
	ILORTTIDECL
#ifdef ILO_NEW_TUPLESET
		friend class IloIntTupleSetI;
#endif
protected:
	IloIntArray _array;
	IloIntDataTableHash* _hashForSelect;
	IloInt _defaultValue;
	IloDiscreteDataCollectionI* _refered;
	IloIntDataTableHash::Status add(IloInt value, IloIntArrayI* res){
		return _hashForSelect->add(value, res);
	}

	void updateHashForSelect(IloInt index, IloInt value, IloBool addIndex = IloTrue);

protected:

	void discard(IloInt value);
public:
	virtual void setReference(IloDiscreteDataCollectionI* refered);
	void addWithoutCheck(IloInt value, IloIntArrayI* res){
		_hashForSelect->addWithoutCheck(value, res);
	}
	virtual IloDiscreteDataCollectionI* copy() const;

	virtual void remove(IloInt index);

	/** <p> . </p> */
	void setValue(IloInt index, IloInt value);

	/** <p> . </p> */
	void createSelectIndexes();
	/** <p> . </p> */
	void clearSelectIndexes();
	/** <p> . </p> */
	IloIntArray find(IloInt value, IloBool& arrayCreated);
	/** <p> . </p> */
	IloIntArray findGe(IloInt value, IloBool& arrayCreated);
	/** <p> . </p> */
	IloIntArray findLe(IloInt value, IloBool& arrayCreated);

	/** <p> . </p> */
	void enableSelectIndexes() {
		if (!_hashForSelect) _hashForSelect = new (getEnv()) IloIntDataTableHash(getEnv());
	}
	/** <p> . </p> */
	void disableSelectIndexes() {
		if (_hashForSelect){
			_hashForSelect->clear();
			delete _hashForSelect; _hashForSelect = 0;
		}
	}
	/** <p> . </p> */
	IloBool isSelectIndexEnabled() const { return (_hashForSelect != 0); }

	/** <p> Returns an iterator. </p> **/
	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;

	/** <p> Returns an iterator. </p> **/
	virtual IloDataIterator* iterator() const;

	/** <p> Constructor. </p> **/
	IloIntDataColumnI(IloEnvI* env, const IloIntArray array, IloInt defaultValue = 0);

	/** <p> Constructor. </p> **/
	IloIntDataColumnI(IloEnvI* env, IloInt n=0);

	/** <p> </p> **/
	virtual ~IloIntDataColumnI();

	/** <p> Returns the array of values. </p> **/
	IloIntArray getArray() const {return _array;}

	/** <p> Returns the array of values. </p> **/
	virtual IloIntArray makeIndexArray(IloInt value) const;

	using IloIntAbstractDataColumnI::getIndex;

	IloInt getIndex(IloInt value) const;

	IloInt getIntValue(IloInt idx) const;
	IloNum getNumValue(IloInt idx) const;
	/** <p> Returns the size of the IntSet. </p> **/
	IloInt getSize() const;

	/** <p> Add method. </p> **/
	void add(IloInt elt);

	/** <p> Add method. </p> **/
	void add(IloIntDataColumnI* set);

	/** <p> </p> **/
	virtual void empty();

	/** <p> </p> **/
	IloBool contains(IloInt elt) const;
	IloInt getValue(IloInt index) const;

	/** <p> </p> **/
	IloBool contains(IloIntDataColumnI* set) const;

	/** <p> </p> **/
	IloBool intersects(IloIntDataColumnI* set) const;

	/** <p> </p> **/
	virtual IloDataType getDataType() const;

	/** <p> checks wether the given domain is included or not </p> **/
	IloBool includes(IloDataCollectionI* dom) const;

	/** <p> </p> **/
	IloBool isIntDataColumn() const;

	/** <p> </p> **/
	void displayType(ILOSTD(ostream)& os) const;

	/** <p> </p> **/
	void display(ILOSTD(ostream)& os) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		out << getValue(index);
	}
	virtual IloInt getLB();
	virtual IloInt getUB();

	IloInt getDefaultValue() const { return _defaultValue; }
	void setDefaultValue(const IloInt val);
	void fillWithDefaultValue(const IloInt val);
};


/**
* @group optim.concert.tuple
* @show internal
*
* <p>An instance of this class offers a convenient way to represent a set of integer
* values as a constrained variable in Concert Technology.</p>
*
* @see IloDataIterator
* @see IloNumDataColumn::Iterator
*/
class IloNumDataColumnI : public IloNumAbstractDataColumnI {
	ILORTTIDECL
#ifdef ILO_NEW_TUPLESET
		friend class IloNumTupleSetI;
#endif
private:
	IloNumArray _array;
	IloNumDataTableHash* _hashForSelect;
	IloNum _defaultValue;
	IloDiscreteDataCollectionI* _refered;

	IloNumDataTableHash::Status add(IloNum value, IloIntArrayI* res) {
		return _hashForSelect->add(value, res);
	}

	void updateHashForSelect(IloInt index, IloNum value, IloBool addIndex = IloTrue);

protected:
	void discard(IloNum value);

public:
	void setReference(IloDiscreteDataCollectionI* refered);
	void addWithoutCheck(IloNum value, IloIntArrayI* res){
		_hashForSelect->addWithoutCheck(value, res);
	}
	virtual IloDiscreteDataCollectionI* copy() const;

	void remove(IloInt index);

	/** <p> . </p> */
	void setValue(IloInt index, IloNum value);

	/** <p> . </p> */
	void createSelectIndexes();
	/** <p> . </p> */
	void clearSelectIndexes();
	/** <p> . </p> */
	IloIntArray find(IloNum value, IloBool& arrayCreated);
	/** <p> . </p> */
	IloIntArray findGe(IloNum value, IloBool& arrayCreated);
	/** <p> . </p> */
	IloIntArray findLe(IloNum value, IloBool& arrayCreated);

	/** <p> . </p> */
	void enableSelectIndexes() {
		if (!_hashForSelect) _hashForSelect = new (getEnv()) IloNumDataTableHash(getEnv());
	}
	/** <p> . </p> */
	void disableSelectIndexes() {
		if (_hashForSelect){
			_hashForSelect->clear();
			delete _hashForSelect; _hashForSelect = 0;
		}
	}
	/** <p> . </p> */
	IloBool isSelectIndexEnabled() const { return (_hashForSelect != 0); }

	/** <p> Returns an iterator. </p> **/
	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;

	/** <p> Returns an iterator. </p> **/
	virtual IloDataIterator* iterator() const;

	/** <p> Constructor. </p> **/
	IloNumDataColumnI(IloEnvI* env, const IloNumArray array, IloNum defaultValue = 0.0);

	/** <p> Constructor. </p> **/
	IloNumDataColumnI(IloEnvI* env, IloInt n=0);

	/** <p> </p> **/
	virtual ~IloNumDataColumnI();

	/** <p> Returns the array of values. </p> **/
	IloNumArray getArray() const {return _array;}

	/** <p> Returns the size of the NumSet. </p> **/
	IloInt getSize() const;

	using IloNumAbstractDataColumnI::getIndex;

	IloInt getIndex(IloNum value) const;

	IloNum getNumValue(IloInt idx) const;
	IloInt getIntValue(IloInt idx) const;

	/** <p> Returns the array of values. </p> **/
	virtual IloIntArray makeIndexArray(IloNum value) const;

	/** <p> Add method. </p> **/
	void add(IloNum elt);

	/** <p> Add method. </p> **/
	void add(IloNumDataColumnI* set);

	/** <p> </p> **/
	void empty();

	/** <p> </p> **/
	IloBool contains(IloNum elt) const;
	IloNum getValue(IloInt index) const;

	/** <p> </p> **/
	IloBool contains(IloNumDataColumnI* set) const;

	/** <p> </p> **/
	IloBool intersects(IloNumDataColumnI* set) const;

	/** <p> </p> **/
	virtual IloDataType getDataType() const;

	/** <p> checks wether the given domain is included or not </p> **/
	IloBool includes(IloDataCollectionI* dom) const;

	/** <p> </p> **/
	IloBool isNumDataColumn() const;

	/** <p> </p> **/
	void displayType(ILOSTD(ostream)& os) const;

	/** <p> </p> **/
	void display(ILOSTD(ostream)& os) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		out << getValue(index);
	}
	IloNum getDefaultValue() const { return _defaultValue; }
	void setDefaultValue(const IloNum coll);
	void fillWithDefaultValue(const IloNum val);
};



/**
* @group optim.concert.tuple
* @show internal
*
* <p>An instance of this class offers a convenient way to represent a set of integer
* values as a constrained variable in Concert Technology.</p>
*
* @see IloDataIterator
* @see IloAnyDataColumn::Iterator
*/
class IloAnyDataColumnI : public IloAnyAbstractDataColumnI {
	ILORTTIDECL
protected:
	IloAnyArray _array;
	IloAnyDataTableHash* _hashForSelect;
	IloDiscreteDataCollectionI* _refered;

	IloAnyDataTableHash::Status add(IloAny value, IloIntArrayI* res){
		return _hashForSelect->add(value, res);
	}
	virtual void updateHashForSelect(IloInt index, IloAny value, IloBool addIndex = IloTrue);

protected:
	IloAny _defaultValue;
	virtual void discard(IloAny value);

public:
	void setReference(IloDiscreteDataCollectionI* refered);
	void addWithoutCheck(IloAny value, IloIntArrayI* res){
		_hashForSelect->addWithoutCheck(value, res);
	}
	virtual IloDiscreteDataCollectionI* copy() const;

	virtual void remove(IloInt index);

	/** <p> . </p> */
	virtual void setValue(IloInt index, IloAny value);

	/** <p> . </p> */
	void createSelectIndexes();
	/** <p> . </p> */
	void clearSelectIndexes();
	/** <p> . </p> */
	IloIntArray find(IloAny value, IloBool& arrayCreated);

	/** <p> . </p> */
	virtual void enableSelectIndexes() {
		if (!_hashForSelect) _hashForSelect = new (getEnv()) IloAnyDataTableHash(getEnv(), IloAdressHashFunction, IloAdressCompFunction);
	}
	/** <p> . </p> */
	void disableSelectIndexes() {
		if (_hashForSelect){
			_hashForSelect->clear();
			delete _hashForSelect; _hashForSelect = 0;
		}
	}
	/** <p> . </p> */
	IloBool isSelectIndexEnabled() const { return (_hashForSelect != 0); }

	/** <p> Returns an iterator. </p> **/
	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;

	/** <p> Returns an iterator. </p> **/
	virtual IloDataIterator* iterator() const;

	/** <p> Constructor. </p> **/
	IloAnyDataColumnI(IloEnvI* env, const IloAnyArray array, IloAny defaultValue = 0);

	/** <p> Constructor. </p> **/
	IloAnyDataColumnI(IloEnvI* env, IloInt n=0);

	/** <p> </p> **/
	virtual ~IloAnyDataColumnI();

	/** <p> Returns the array of values. </p> **/
	IloAnyArray getArray() const {return _array;}

	/** <p> Returns the size of the AnySet. </p> **/
	IloInt getSize() const;

	using IloAnyAbstractDataColumnI::getIndex;

	/** <p> . </p> */
	virtual IloInt getIndex(IloAny val) const;

	IloAny getAnyValue(IloInt idx) const;

	/** <p> Returns the array of values. </p> **/
	virtual IloIntArray makeIndexArray(IloAny value) const;

	/** <p> Add method. </p> **/
	virtual void add(IloAny elt);

	/** <p> Add method. </p> **/
	virtual void add(IloAnyDataColumnI* set);

	/** <p> </p> **/
	virtual void empty();

	/** <p> </p> **/
	IloBool contains(IloAny elt) const;
	IloAny getValue(IloInt index) const;

	/** <p> </p> **/
	IloBool contains(IloAnyDataColumnI* set) const;

	/** <p> </p> **/
	IloBool intersects(IloAnyDataColumnI* set) const;

	/** <p> </p> **/
	virtual IloDataType getDataType() const;

	/** <p> checks wether the given domain is included or not </p> **/
	IloBool includes(IloDataCollectionI* dom) const;

	/** <p> </p> **/
	IloBool isAnyDataColumn() const;

	/** <p> </p> **/
	void displayType(ILOSTD(ostream)& os) const;

	/** <p> </p> **/
	void display(ILOSTD(ostream)& os) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		out << getValue(index);
	}
	IloAny getDefaultValue() const { return _defaultValue; }
	virtual void setDefaultValue(const IloAny val);
	void fillWithDefaultValue(const IloAny val);
};



#endif
