// --------------------------------------------------------------------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplecollectionhashH
#define __CONCERT_ilotuplecollectionhashH

#if !defined(__CONCERT_ilohashH)
# include <ilconcert/ilohash.h>
#endif

#if !defined(__CONCERT_ilotuplerequestH)
# include <ilconcert/iltuple/ilotuplerequest.h>
#endif

#if !defined(__CONCERT_ilosetH)
# include <ilconcert/iloset.h>
#endif
#if !defined(__CONCERT_iloanysetH)
# include <ilconcert/iloanyset.h>
#endif

class IloRequestSignatureI;
class IloTupleRequestI;

/**
* @group optim.concert.tuple
* @show internal
* <p> IloTupleSetHash </p>
**/
template <class KeyType>
class IloTupleSetHash : public IloEnvHashTable<KeyType, IloIntArrayI*>{
public:
	typedef IloInt (*IloHashFunction) (KeyType, IloInt);
	typedef IloBool (*IloCompFunction) (KeyType, KeyType);

	/** <p> Constructor </p> **/
	IloTupleSetHash(IloEnv env,
		IloHashFunction hashFct,
		IloCompFunction compFct,
		IloInt size=IloDefaultHashSize)
		:IloEnvHashTable<KeyType, IloIntArrayI*>(env, hashFct,compFct, size) {}
		~IloTupleSetHash(){
		}
};

extern IloInt  IloIntDataTableHashFunction(IloInt, IloInt size);
extern IloBool IloIntDataTableCompFunction(IloInt key1, IloInt key2);

extern IloInt  IloAnyCollectionHashFunction(IloAny, IloInt size);
extern IloBool IloAnyCollectionCompFunction(IloAny key1, IloAny key2);

extern IloInt  IloIntCollectionHashFunction(IloAny, IloInt size);
extern IloBool IloIntCollectionCompFunction(IloAny key1, IloAny key2);

extern IloInt  IloNumCollectionHashFunction(IloAny, IloInt size);
extern IloBool IloNumCollectionCompFunction(IloAny key1, IloAny key2);


/**
* @group optim.concert.tuple
* @show internal
* <p> IloIntDataTableHash </p>
**/
class IloIntDataTableHash : public IloTupleSetHash<IloInt> {
public:
	/** <p> Constructor </p> **/
	IloIntDataTableHash(IloEnv env, IloInt size=IloDefaultHashSize)
		:IloTupleSetHash<IloInt>(env,
		IloIntDataTableHashFunction,
		IloIntDataTableCompFunction,
		size) {}
		~IloIntDataTableHash() {}
};


extern IloInt  IloNumDataTableHashFunction(IloNum, IloInt size);
extern IloBool IloNumDataTableCompFunction(IloNum key1, IloNum key2);

/**
* @group optim.concert.tuple
* @show internal
* <p> IloNumDataTableHash </p>
**/
class IloNumDataTableHash : public IloTupleSetHash<IloNum> {
public:
	/** <p> Constructor </p> **/
	IloNumDataTableHash(IloEnv env, IloInt size=IloDefaultHashSize)
		:IloTupleSetHash<IloNum>(env,
		IloNumDataTableHashFunction,
		IloNumDataTableCompFunction,
		size) {}
		~IloNumDataTableHash() {}
};

/**
* @group optim.concert.tuple
* @show internal
* <p> IloAnyDataTableHash </p>
**/
class IloAnyDataTableHash : public IloTupleSetHash<IloAny> {
public:
	typedef IloInt (*IloHashFunction) (IloAny, IloInt);
	typedef IloBool (*IloCompFunction) (IloAny, IloAny);
	/** <p> Constructor </p> **/
	IloAnyDataTableHash(IloEnv env, IloHashFunction fct1, IloCompFunction fct2, IloInt size=IloDefaultHashSize)
		:IloTupleSetHash<IloAny>(env,
		fct1,
		fct2,
		size) {}
		~IloAnyDataTableHash() {}
};


extern IloInt  IloVBHashFunction(IloRequestSignatureI*, IloInt size);
extern IloBool IloVBCompFunction(IloRequestSignatureI* key1, IloRequestSignatureI* key2);

/**
* @group optim.concert.tuple
* @show internal
* <p> IloVBHash </p>
**/
class IloVBHash : public IloEnvHashTable<IloRequestSignatureI*, IloEnvObjectI*>{
public:
	/** <p> Constructor </p> **/
	IloVBHash(IloEnv env, IloInt size=IloDefaultHashSize)
		:IloEnvHashTable<IloRequestSignatureI*, IloEnvObjectI*>(env,
		IloVBHashFunction,
		IloVBCompFunction,
		size) {}
		~IloVBHash() {}
};



extern IloInt  IloTupleRequestHashFunction(IloTupleRequestI*, IloInt size);
extern IloBool IloTupleRequestCompFunction(IloTupleRequestI* key1,IloTupleRequestI* key2);

/**
* @group optim.concert.tuple
* @show internal
* <p> IloQueryDataTableHash </p>
**/
class IloQueryDataTableHash : public IloTupleSetHash<IloTupleRequestI*> {
public:
	/** <p> Constructor </p> **/
	IloQueryDataTableHash(IloEnv env, IloInt size=IloDefaultHashSize)
		:IloTupleSetHash<IloTupleRequestI*>(env,
		IloTupleRequestHashFunction,
		IloTupleRequestCompFunction,
		size) {}
		~IloQueryDataTableHash();
};

typedef IloInt (*IloTupleCellsHashFunction) (IloTupleCellArrayI*, IloInt);

extern IloInt  IloMixedTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloMixedBinaryTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);

extern IloBool IloMixedTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);

extern IloInt  IloMixedTupleCellsNoCollectionHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloMixedBinaryTupleCellsNoCollectionHashFunction(IloTupleCellArrayI*, IloInt size);

extern IloBool IloMixedTupleCellsNoCollectionCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);

extern IloInt  IloIntTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloIntBinaryTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloIntUnaryTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);

extern IloInt  IloNumTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloNumBinaryTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloNumUnaryTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);

extern IloInt  IloAnyTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloAnyBinaryTupleCellsHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloAnyTupleCellsNoCollectionHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloAnyBinaryTupleCellsNoCollectionHashFunction(IloTupleCellArrayI*, IloInt size);
extern IloInt  IloAnyUnaryTupleCellsNoCollectionHashFunction(IloTupleCellArrayI*, IloInt size);


typedef IloBool (*IloTupleCellsCompFunction) (IloTupleCellArrayI*, IloTupleCellArrayI*);

extern IloBool IloIntTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool IloNumTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool IloAnyTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool  IloAnyTupleCellsNoCollectionCompFunction(IloTupleCellArrayI*, IloTupleCellArrayI* key2);

extern IloBool IloIntBinaryTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool IloNumBinaryTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool IloAnyBinaryTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool  IloAnyBinaryTupleCellsNoCollectionCompFunction(IloTupleCellArrayI*, IloTupleCellArrayI* key2);

extern IloBool IloIntUnaryTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool IloNumUnaryTupleCellsCompFunction(IloTupleCellArrayI* key1,IloTupleCellArrayI* key2);
extern IloBool  IloAnyUnaryTupleCellsNoCollectionCompFunction(IloTupleCellArrayI*, IloTupleCellArrayI* key2);

/**
* @group optim.concert.tuple
* @show internal
* <p> IloTupleCellsHashBase </p>
**/
template <class KeyType>
class IloTupleCellsHashBase : public IloEnvHashTable<KeyType, IloInt>{
public:
	typedef IloInt (*IloHashFunction) (KeyType, IloInt);
	typedef IloBool (*IloCompFunction) (KeyType, KeyType);

	/** <p> Constructor </p> **/
	IloTupleCellsHashBase(IloEnv env,
		IloHashFunction hashFct,
		IloCompFunction compFct,
		IloInt size=IloDefaultHashSize)
		:IloEnvHashTable<KeyType, IloInt>(env, hashFct,compFct, size) {}
		~IloTupleCellsHashBase(){}
};

/**
* @group optim.concert.tuple
* @show internal
* <p> IloTupleCells2IndexHash </p>
**/
class IloTupleCells2IndexHash : public IloTupleCellsHashBase<IloTupleCellArrayI*> {
public:
	/** <p> Constructor </p> **/
	IloTupleCells2IndexHash(IloEnv env, IloTupleCellsHashFunction hashFct, IloTupleCellsCompFunction compFct)
		:IloTupleCellsHashBase<IloTupleCellArrayI*>(env,
		hashFct,
		compFct,
		IloDefaultHashSize) {}
		~IloTupleCells2IndexHash(){}
};


class IloIndex2KeyHash : public IloEnvHashTable<IloInt, IloTupleCellArrayI*>{
public:
	typedef IloInt (*IloHashFunction) (IloInt, IloInt);
	typedef IloBool (*IloCompFunction) (IloInt, IloInt);

	/** <p> Constructor </p> **/
	IloIndex2KeyHash(IloEnv env,
		IloHashFunction hashFct,
		IloCompFunction compFct,
		IloInt size=IloDefaultHashSize)
		:IloEnvHashTable<IloInt, IloTupleCellArrayI*>(env, hashFct,compFct, size) {}
		~IloIndex2KeyHash(){}
};



/**
* @group optim.concert.tuple
* @show internal
* <p> IloTupleCellsTupleCollectionHashBase </p>
**/
template <class KeyType>
class IloTupleCellsTupleCollectionHashBase : public IloEnvHashTable<KeyType, IloIntArrayI*>{
public:
	typedef IloInt (*IloHashFunction) (KeyType, IloInt);
	typedef IloBool (*IloCompFunction) (KeyType, KeyType);

	/** <p> Constructor </p> **/
	IloTupleCellsTupleCollectionHashBase(IloEnv env,
		IloHashFunction hashFct,
		IloCompFunction compFct,
		IloInt size=IloDefaultHashSize)
		:IloEnvHashTable<KeyType, IloIntArrayI*>(env, hashFct,compFct, size) {}
		~IloTupleCellsTupleCollectionHashBase(){}
};

/**
* @group optim.concert.tuple
* @show internal
* <p> IloTupleCells2ArrayIndexHash </p>
**/
class IloTupleCells2ArrayIndexHash : public IloTupleCellsTupleCollectionHashBase<IloTupleCellArrayI*> {
public:
	/** <p> Constructor </p> **/
	IloTupleCells2ArrayIndexHash(IloEnv env, IloTupleCellsHashFunction hashFct, IloTupleCellsCompFunction compFct)
		:IloTupleCellsTupleCollectionHashBase<IloTupleCellArrayI*>(env,
		hashFct,
		compFct,
		IloDefaultHashSize) {}
		~IloTupleCells2ArrayIndexHash(){}
};


/**
* @group optim.concert.tuple
* @show internal
*
* <p> Hash table IloInt to IloInt </p>
**/
class IloInt2IndexHashTable : public IloIntegerHashTable<IloInt> {
public:
	/** <p> Constructor </p> **/
	IloInt2IndexHashTable(IloEnv env, IloInt size=IloDefaultHashSize) : IloIntegerHashTable<IloInt>(env, size) {}
	virtual ~IloInt2IndexHashTable() {}
};

/**
* @group optim.concert.tuple
* @show internal
*
* <p> Hash table IloNum to IloInt </p>
**/
class IloNum2IndexHashTable : public IloFloatHashTable<IloInt> {
public:
	/** <p> Constructor </p> **/
	IloNum2IndexHashTable(IloEnv env, IloInt size=IloDefaultHashSize)
		: IloFloatHashTable<IloInt>(env, size) {}
		virtual ~IloNum2IndexHashTable() {}
};

/**
* @group optim.concert.tuple
* @show internal
*
* <p> Hash table Iloany to IloInt </p>
**/
class IloAny2IndexHashTable : public IloAdressHashTable<IloInt> {
public:
	/** <p> Constructor </p> **/
	IloAny2IndexHashTable(IloEnv env, IloInt size=IloDefaultHashSize)
		: IloAdressHashTable<IloInt>(env, size) {}
		virtual ~IloAny2IndexHashTable() {}
};

/**
* @group optim.concert.tuple
* @show internal
*
* <p> Hash table IloSymbil to IloInt </p>
**/
class IloSymbol2IndexHashTable : public IloSymbolHashTable<IloInt> {
public:
	/** <p> Constructor </p> **/
	IloSymbol2IndexHashTable(IloEnv env, IloInt size=IloDefaultHashSize)
		: IloSymbolHashTable<IloInt>(env, size) {}
		virtual ~IloSymbol2IndexHashTable() {}
};



// --------------------------------------------------------------
// HashTable with IloIntArrayI* keys
// --------------------------------------------------------------
extern IloInt  IloIntArrayHashFunction(IloIntArrayI* key, IloInt size);
extern IloBool IloIntArrayCompFunction(IloIntArrayI* key1, IloIntArrayI* key2);

/**
* @group optim.concert.tuple
* @show internal
* <p> IloIntArrayHashTable </p>
**/
template <class ValueType>
class IloIntArrayHashTable : public IloEnvHashTable<IloIntArrayI*, ValueType> {
public:
	IloIntArrayHashTable(IloEnv env, IloInt size=IloDefaultHashSize)
		:IloEnvHashTable<IloIntArrayI*, ValueType>(env,
		IloIntArrayHashFunction,
		IloIntArrayCompFunction,
		size) {
		}
		virtual ~IloIntArrayHashTable() {}
};

#endif


