// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplemapiH
#define __CONCERT_ilotuplemapiH


#if !defined(__CONCERT_ilomapextrH)
#include <ilconcert/ilomapextr.h>
#endif

/* 
 IloAbstractTupleMapI
*/
class IloTupleSubMapExprI;

// IloInt, IloNum, IloSymbol, IloTuple
#define DEFINE_GET_SET_ABSTRACT(_IdxType) \
  virtual void implSet(_IdxType index, IloTuple value) = 0;\
  virtual void implSet(_IdxType index, IloTupleBuffer value) = 0;\
  virtual void implGet(_IdxType index, IloTuple tuple) const = 0;\
  virtual void implGet(_IdxType index, IloTupleBuffer buffer) const = 0;

/**
 * @group optim.concert.map
 * @show internal
 * <p> IloAbstractTupleMapI. </p> **/
class IloAbstractTupleMapI : public IloAbstractMapI {
  ILORTTIDECL
protected:
  IloTupleCollectionI* _coll;
  IloAbstractTupleMapI(IloEnvI* env, const IloTupleSchema& schema, IloInt size);
  IloAbstractTupleMapI(IloEnvI* env, const IloTupleSchema& schema, IloInt size, IloTupleCellArray defaultValue);
  IloAbstractTupleMapI(IloEnvI* env, const IloTupleSchema& schema, IloInt size, IloTupleBuffer defaultValue);
  IloAbstractTupleMapI(IloEnvI* env, IloTupleCollectionI* coll);
public:
  //TEMPORARY
  IloBool isDefaultValue(IloTuple);
	void setDefaultValue(IloTupleCellArray cells);
	void setDefaultValue(IloTupleBufferI* buf);

  virtual ~IloAbstractTupleMapI();
  virtual void lock() = 0;
  virtual void unlock() = 0;
  virtual IloBool isLocked() const = 0;
  virtual IloAbstractTupleMapI* getCopy() const = 0;
  IloAbstractTupleMapI* getLazyCopy() const;
  IloTupleSchema getSchema() const { return _coll->getSchema();}
  IloBool checkSchema(IloTupleI* t) const;
  IloBool checkKeySchema(IloTupleI* t) const;
  void setTupleCollection(IloTupleCollectionI* coll){
	  _coll = coll;
  }
  IloTupleCollectionI* getTupleCollection() const { return _coll; }
  IloTupleBuffer makeTupleBuffer() const;
  IloTuple makeTuple() const;

  IloBool isIndexerIntCompatible(IloInt dim = 0) const {
    return ((getIndexer(dim)->isIntRange())
	    || (getIndexer(dim)->isIntSet())
	    || (getIndexer(dim)->isNumSet()));
  }
  IloBool isIndexerNumCompatible(IloInt dim = 0) const {
    return ((getIndexer(dim)->isNumSet())
	    || (getIndexer(dim)->isIntRange())
	    || (getIndexer(dim)->isIntSet()));
  }
  IloBool isIndexerSymbolCompatible(IloInt dim = 0) const {
    return (getIndexer(dim)->isSymbolSet());
  }
  IloBool isIndexerTupleCompatible(IloInt dim = 0) const {
    return (getIndexer(dim)->isTupleSet());
  }

  IloInt getAbsoluteIndex(IloAny idx, IloInt dim = 0) const {
    IloAssert(getIndexer(dim),"Using IloTupleMap/IloTupleMap without indexer");
    return getIndexer(dim)->getIndex(idx);
  }
  IloInt getAbsoluteIndex(IloMapIndex idx, IloInt dim = 0) const;
public:
//  using IloAbstractMapI::setAtAbsoluteIndex;
//  using IloAbstractMapI::getAtAbsoluteIndex;
  virtual void setAtAbsoluteIndex(IloInt index, IloMapItem value);
  virtual void setAtAbsoluteIndex(IloInt index, IloTuple value);
  virtual void setAtAbsoluteIndex(IloInt index, IloTupleBuffer value);
  virtual void setAtAbsoluteIndex(IloInt index, IloTupleBufferI* value);
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloTuple value) = 0;
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloTupleBuffer value)=0;
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value)=0;
  virtual void getAtAbsoluteIndex(IloInt index, IloTuple  buffer) const;
  virtual void getAtAbsoluteIndex(IloInt index, IloTupleBuffer buffer) const;
  virtual void getAtAbsoluteIndex(IloInt index, IloTupleBufferI* buffer) const;
  virtual IloMapItem getAtAbsoluteIndex(IloInt index) const;
  virtual IloMapItem getAtAbsoluteIndex(IloIntArray indices) const = 0;
  virtual void getAtAbsoluteIndex(IloIntArray indices, IloTuple tuple) const=0;
  virtual void getAtAbsoluteIndex(IloIntArray indices,
				  IloTupleBuffer buffer) const =0;
  virtual void setAt(IloMapIndexArray indices, IloTuple value) = 0;
  virtual void setAt(IloMapIndexArray indices, IloTupleBuffer value) = 0;
  virtual void setAt(IloMapIndexArray indices, IloMapItem value) = 0;
  virtual IloMapItem getAt(IloMapIndexArray indices) const = 0;
  virtual void getAt(IloMapIndexArray indices, IloTuple tuple) const = 0;
  virtual void getAt(IloMapIndexArray indices, IloTupleBuffer buffer) const =0;
  virtual IloTupleSubMapExprI* subscriptOp(IloIntExprArg idx) const =0;
  virtual IloTupleSubMapExprI* subscriptOp(IloNumExprArg idx) const =0;
  virtual IloTupleSubMapExprI* subscriptOp(IloSymbolExprArg idx) const =0;
  virtual IloTupleSubMapExprI* subscriptOp(IloTupleExprArg idx) const =0;
  virtual IloTupleSubMapExprI* subscriptOp(IloTuplePattern idx) const =0;
  DEFINE_GET_SET_ABSTRACT(IloInt)
  DEFINE_GET_SET_ABSTRACT(IloNum)
  DEFINE_GET_SET_ABSTRACT(IloSymbol)
  DEFINE_GET_SET_ABSTRACT(IloTuple)
};

//---------------------------------------------------------------------------
//   Extractables for IloTupleMap
//---------------------------------------------------------------------------
/**
 * @group optim.concert.map
 * @show internal
 * <p> IloTupleSubMapExprI. </p> **/
class IloTupleSubMapExprI : public IloTupleExprI {
  ILOEXTRDECL
protected:
  IloTupleSubMapExprI(IloEnvI* env);
public:
  virtual ~IloTupleSubMapExprI() {}
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getTupleEncoder();
  }
  virtual IloMapExtractIndexI* getIndex() const =0;
  virtual IloDiscreteDataCollectionI* getIndexer() const =0;
  virtual IloTupleSubMapExprI* makeSubMap(IloMapExtractIndexI* idx)=0;
  DEFINE_INDEX_UTILITIES_AUX()
};

/*
 IloTupleMap1I
*/

#define DEFINE_GET_SET_IMPL_1(_IdxType) \
  virtual void implSet(_IdxType index, IloTuple value) {\
    IloInt idx = this->getAbsoluteIndex(index, 0);\
    IloAssert (idx>=0 && idx<getSize(),\
               "IloTupleMap1::set Out of bounds operation");\
    ((IloTupleMap1I*)this)->setAtAbsoluteIndex(idx, value);\
  }\
  virtual void implSet(_IdxType index, IloTupleBuffer value) {\
    IloInt idx = this->getAbsoluteIndex(index, 0);\
    IloAssert (idx>=0 && idx<getSize(),\
               "IloTupleMap1::set Out of bounds operation");\
    ((IloTupleMap1I*)this)->setAtAbsoluteIndex(idx, value);\
  }\
  virtual void implGet(_IdxType index, IloTuple tuple) const {\
    IloInt idx = this->getAbsoluteIndex(index, 0);\
    IloAssert (idx>=0 && idx<getSize(),\
               "IloTupleMap1::set Out of bounds operation");\
    ((IloTupleMap1I*)this)->getAtAbsoluteIndex(idx, tuple);\
  }\
  virtual void implGet(_IdxType index, IloTupleBuffer buffer) const {\
    IloInt idx = this->getAbsoluteIndex(index, 0);\
    IloAssert (idx>=0 && idx<getSize(),\
               "IloTupleMap1::set Out of bounds operation");\
    ((IloTupleMap1I*)this)->getAtAbsoluteIndex(idx, buffer);\
  }

/**
 * @group optim.concert.map
 * @show internal
 * <p> IloTupleMap1I. </p> **/
class IloTupleMap1I : public IloAbstractTupleMapI {
  ILORTTIDECL
protected:
  IloDiscreteDataCollectionI* _indexer;
  char* _name;
  IloBool _locked;
private:
  void init(IloDiscreteDataCollectionI* indexer, IloTupleSetI* s);
public:
  IloTupleMap1I(IloEnvI*, const IloTupleMap1I*);
  virtual ~IloTupleMap1I();
  virtual IloAbstractTupleMapI* getCopy() const;
  IloTupleMap1I(IloEnvI* env,
	       IloDiscreteDataCollectionI* indexer,
	       const IloTupleSchema& schema);
  IloTupleMap1I(IloEnvI* env,
	       IloDiscreteDataCollectionI* indexer,
	       IloTupleCollectionI* coll);
  IloTupleMap1I(IloEnv env,
	       IloDiscreteDataCollection indexer,
	       IloTupleSet s);
  virtual void lock();
  virtual void unlock();
  virtual IloBool isLocked() const;
  virtual IloInt getNbDim() const { return 1; }
  virtual IloInt getTotalSize() const { return getSize(); } 
  virtual IloInt getSize() const { return _indexer->getSize(); }
  virtual IloDiscreteDataCollectionI* getIndexer() const;
  virtual IloDiscreteDataCollectionI* getIndexer(IloInt i) const;
  virtual const char* getName() const { return _name; }
  virtual void setName(const char* name);
  using IloAbstractTupleMapI::getAbsoluteIndex;
  IloInt getAbsoluteIndex(IloMapIndexArray indices) const;
  virtual void setAt(IloMapIndexArray indices, IloTuple value);
  virtual void setAt(IloMapIndexArray indices, IloTupleBuffer value);
  virtual void setAt(IloMapIndexArray indices, IloMapItem value);
  virtual IloMapItem getAt(IloMapIndexArray indices) const;
  virtual void getAt(IloMapIndexArray indices, IloTuple tuple) const;
  virtual void getAt(IloMapIndexArray indices, IloTupleBuffer buffer) const;

  using IloAbstractTupleMapI::setAtAbsoluteIndex;
  using IloAbstractTupleMapI::getAtAbsoluteIndex;
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloTuple value);
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloTupleBuffer value);
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value);
  virtual IloMapItem getAtAbsoluteIndex(IloIntArray indices) const;
  virtual void getAtAbsoluteIndex(IloIntArray indices, IloTuple tuple) const;
  virtual void getAtAbsoluteIndex(IloIntArray indices,
				  IloTupleBuffer buffer) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloIntExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloNumExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloSymbolExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloTupleExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloTuplePattern idx) const;
  DEFINE_GET_SET_IMPL_1(IloInt)
  DEFINE_GET_SET_IMPL_1(IloNum)
  DEFINE_GET_SET_IMPL_1(IloSymbol)
  DEFINE_GET_SET_IMPL_1(IloTuple)
  virtual void display(ILOSTD(ostream)& out) const;
};

//---------------------------------------------------------------------------
//   Extractable for IloTupleMap
//---------------------------------------------------------------------------
class IloTupleMapElementI : public IloTupleSubMapExprI {
  ILOEXTRDECL
private:
  IloMapExtractIndexI* _index;
  IloTupleMap1I* _map;
private:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleMapElementI(IloEnvI* env, IloMapExtractIndexI* index, IloTupleMap1I* m);
  IloTupleMapElementI(IloEnvI* env, IloIntExprI* index, IloTupleMap1I* m);
  IloTupleMapElementI(IloEnvI* env, IloNumExprI* index, IloTupleMap1I* m);
  IloTupleMapElementI(IloEnvI* env, IloAnyExprI* index, IloTupleMap1I* m);
  ~IloTupleMapElementI();
  IloTupleMap1I* getMap() const { return _map; }
  virtual IloMapExtractIndexI* getIndex() const { return _index; }
  virtual IloDiscreteDataCollectionI* getIndexer() const {
    return _map->getIndexer();
  }
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloTupleSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
};

/*
 IloTupleMapI
*/

#define DEFINE_GET_SET_IMPL(_IdxType) \
  virtual void implSet(_IdxType index, IloTuple value);\
  virtual void implSet(_IdxType index, IloTupleBuffer value);\
  virtual void implGet(_IdxType index, IloTuple tuple) const {\
    IloAssert (getNbDim() == 1,\
	       "IloTupleMap::get is not available on multi-dimensional map.");\
    ((IloTupleMapI*)this)->get(IloMapIndex(index), tuple);\
  }\
  virtual void implGet(_IdxType index, IloTupleBuffer buffer) const {\
    IloAssert (getNbDim() == 1,\
	       "IloTupleMap::get is not available on multi-dimensional map.");\
    ((IloTupleMapI*)this)->get(IloMapIndex(index), buffer);\
  }

/**
 * @group optim.concert.map
 * @show internal
 * <p> IloTupleMapI. </p> **/
class IloTupleMapI : public IloAbstractTupleMapI {
  ILORTTIDECL
protected:
  IloIntMap _indices;
private:
  void initIndices();
  IloInt initIndices(IloIntMap m, IloInt currentDim, IloInt value);
  IloMapItem getSubMapExpr(IloMapIndexArray indices) const;
#ifndef NDEBUG
public:
#endif
  IloMapI* getIndImpl() const { return _indices.getImpl(); }
  IloIntMap getIndices() const { return _indices; }
public:
  friend class IloTupleSubMapNExprI;
  friend class IloTupleSubMapRootI;
  friend class IloTupleSubMapSubI;
  IloTupleMapI(IloEnvI*, const IloTupleMapI*);
  virtual ~IloTupleMapI();
  virtual IloAbstractTupleMapI* getCopy() const;
  IloTupleMapI(IloEnvI* env, IloIntMap indices, const IloTupleSchema& schema);
  IloTupleMapI(IloEnvI* env, IloIntMap indices, const IloTupleSchema& schema, IloTupleCellArray defaultValue);
  IloTupleMapI(IloEnvI* env, IloIntMap indices, const IloTupleSchema& schema, IloTupleBuffer defaultValue);
  IloTupleMapI(IloEnvI* env, IloIntMap indices, IloTupleCollectionI* coll);

  virtual void lock();
  virtual void unlock();
  virtual IloBool isLocked() const;
  virtual IloInt getNbDim() const { return getIndImpl()->getNbDim(); }
  virtual IloInt getTotalSize() const { return getIndImpl()->getTotalSize(); } 
  virtual IloInt getSize() const { return getIndImpl()->getSize(); }
  virtual const char* getName() const { return getIndImpl()->getName(); }
  virtual void setName(const char* name) { getIndImpl()->setName(name); }
  virtual IloDiscreteDataCollectionI* getIndexer() const;
  virtual IloDiscreteDataCollectionI* getIndexer(IloInt i) const;

  using IloAbstractTupleMapI::getAbsoluteIndex;
  IloInt getAbsoluteIndex(IloIntArray indices) const;
  IloIntArray makeAbsoluteIndexArray(IloMapIndexArray indices) const;

  void set(IloMapIndex index, IloTuple value);
  void set(IloMapIndex index, IloTupleBuffer value);
  void get(IloMapIndex index, IloTuple tuple) const;
  void get(IloMapIndex index, IloTupleBuffer buffer) const;

  void setAt(IloMapIndexArray indices, IloTuple value);
  void setAt(IloMapIndexArray indices, IloTupleBuffer value);
  virtual void setAt(IloMapIndexArray indices, IloMapItem value);
  virtual IloMapItem getAt(IloMapIndexArray indices) const;
  void getAt(IloMapIndexArray indices, IloTuple tuple) const;
  void getAt(IloMapIndexArray indices, IloTupleBuffer buffer) const;

  using IloAbstractTupleMapI::setAtAbsoluteIndex;
  using IloAbstractTupleMapI::getAtAbsoluteIndex;
  void setAtAbsoluteIndex(IloIntArray indices, IloTuple value);
  void setAtAbsoluteIndex(IloIntArray indices, IloTupleBuffer value);
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value);
  virtual IloMapItem getAtAbsoluteIndex(IloIntArray indices) const;
  void getAtAbsoluteIndex(IloIntArray indices, IloTuple tuple) const;
  void getAtAbsoluteIndex(IloIntArray indices, IloTupleBuffer buffer) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloIntExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloNumExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloSymbolExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloTupleExprArg idx) const;
  virtual IloTupleSubMapExprI* subscriptOp(IloTuplePattern idx) const;
  virtual void display(ILOSTD(ostream)& out) const;
  DEFINE_GET_SET_IMPL(IloInt)
  DEFINE_GET_SET_IMPL(IloNum)
  DEFINE_GET_SET_IMPL(IloSymbol)
  DEFINE_GET_SET_IMPL(IloTuple)
};

//---------------------------------------------------------------------------
//   Extractables for IloTupleMap
//---------------------------------------------------------------------------
/**
 * @group optim.concert.map
 * @show internal
 * <p> IloTupleSubMapNExprI. </p> **/
class IloTupleSubMapNExprI : public IloTupleSubMapExprI {
  ILOEXTRDECL
protected:
  IloIntSubMapExprI* _indices;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleSubMapNExprI(IloEnvI* env,
		      IloIntSubMapExprI* ind);
  virtual ~IloTupleSubMapNExprI();
  virtual IloTupleMapI* getMap() const = 0;
  virtual IloTupleSubMapExprI* makeSubMap(IloMapExtractIndexI* idx);
  virtual IloBool isRoot() const { return IloFalse; }
  IloIntSubMapExprI* getIndices() const { return _indices; }
  virtual IloMapExtractIndexI* getIndex() const { return getIndices()->getIndex(); }
  IloInt getCurrentDim() const { return getIndices()->getCurrentDim(); }
  IloBool isLastDimension() const { return getIndices()->isLastDimension(); }
  virtual IloDiscreteDataCollectionI* getIndexer() const {
    return getIndices()->getIndexer();
  }
};

/**
 * @group optim.concert.map
 * @show internal
 * <p> IloTupleSubMapRootI. </p> **/
class IloTupleSubMapRootI : public IloTupleSubMapNExprI {
  ILOEXTRDECL
private:
  IloBool _withCopy;
  IloTupleMapI* _map;
  IloTupleSubMapRootI(IloEnvI* env,
		      IloIntSubMapRootI* index,
		      IloTupleMapI* m,
		      IloBool withCopy);
public:
  IloTupleSubMapRootI(IloEnvI* env,
		      IloMapExtractIndexI* index,
		      IloTupleMapI* m,
		      IloBool withCopy);
  IloTupleSubMapRootI(IloEnvI* env,
		      IloIntExprI* index,
		      IloTupleMapI* m,
		      IloBool withCopy);
  IloTupleSubMapRootI(IloEnvI* env,
		      IloNumExprI* index,
		      IloTupleMapI* m,
		      IloBool withCopy);
  IloTupleSubMapRootI(IloEnvI* env,
		      IloAnyExprI* index,
		      IloTupleMapI* m,
		      IloBool withCopy);
  virtual ~IloTupleSubMapRootI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual IloTupleMapI* getMap() const { return _map; }
  virtual IloBool isRoot() const { return IloTrue; }
};

/**
 * @group optim.concert.map
 * @show internal
 * <p> IloTupleSubMapSubI. </p> **/
class IloTupleSubMapSubI : public IloTupleSubMapNExprI {
  ILOEXTRDECL
private:
  IloTupleSubMapNExprI* _owner;
protected:
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleSubMapSubI(IloEnvI* env, IloIntSubMapExprI* iSub,
		     IloTupleSubMapNExprI* owner);
  IloTupleSubMapSubI(IloEnvI* env, IloMapExtractIndexI* index,
		     IloTupleSubMapNExprI* owner, IloInt dim);
  IloTupleSubMapSubI(IloEnvI* env, IloIntExprI* index,
		     IloTupleSubMapNExprI* owner, IloInt dim);
  IloTupleSubMapSubI(IloEnvI* env, IloNumExprI* index,
		     IloTupleSubMapNExprI* owner, IloInt dim);
  IloTupleSubMapSubI(IloEnvI* env, IloAnyExprI* index,
		     IloTupleSubMapNExprI* owner, IloInt dim);
  virtual ~IloTupleSubMapSubI();
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloTupleSubMapNExprI* getOwner() const { return _owner; }
  virtual IloTupleMapI* getMap() const { return _owner->getMap(); }
};

ILOSUBMAPHANDLE(IloTuple, IloTupleExprArg)


#endif
