// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplerequestiH
#define __CONCERT_ilotuplerequestiH

#if !defined(__CONCERT_ilostringH)
# include <ilconcert/ilostring.h>
#endif
#if !defined(__CONCERT_ilosymbolH)
# include <ilconcert/ilosymbol.h>
#endif
#if !defined(__CONCERT_ilotuplebufferH)
# include <ilconcert/iltuple/ilotuplebuffer.h>
#endif
#if !defined(__CONCERT_ilorequestcacheH)
# include <ilconcert/iltuple/ilorequestcache.h>
#endif


class IloTupleRequest;
class IloTupleRequestI;
class IloTupleSetI;
class IloTupleSet;
class IloTupleSchemaI;
class IloTupleSchema;
class IloTupleBuffer;
class IloTupleRequestChangeListenerI;

/**
* @group optim.concert.tuple
* @show internal
* <p> IloRequestSignatureI </p> **/
class IloRequestSignatureI : public IloRttiEnvObjectI {
	ILORTTIDECL
private:
	IloIntArray2 _signature;
  IloArray<IloTuplePath::IloTupleRequestType> _reqTypes;
	void add(IloIntArray array, IloTuplePath::IloTupleRequestType);
public:
	/** <p> . </p> */
	IloInt getSize() const { return _signature.getSize(); }
	/** <p> . </p> */
	IloIntArray2 getArray() const { return _signature; }
	/** <p> . </p> */
	IloArray<IloTuplePath::IloTupleRequestType> getReqTypes() const { return _reqTypes; }
	/** <p> . </p> */
	IloRequestSignatureI(IloTupleRequestI* req);
	~IloRequestSignatureI();
	/** <p> . </p> */
	IloBool equal(IloRequestSignatureI* sign) const;
};

/**
* @group optim.concert.tuple
* @show internal
* <p>An instance of this class offers a convenient way to represent a selection in the IloTupleSet.</p>
*/
class IloTupleRequestI : public IloRttiEnvObjectI {
  friend class IloTupleRequestChangeListenerI;
private:
	ILORTTIDECL
  IloTuplePathList _list;
  IloTupleSchemaI* _schema;
  IloTupleSetI* _coll;
  IloTupleRequestCacheI* _cache;
  IloBool  _findRequest;
  IloBool  _subTupleRequest;
  IloBool _containsAllKeys;
  IloTupleRequestI* _subRequest;
  IloTupleRequestI* _keyRequest;
  IloBool _isOrd;

  IloTupleRequestChangeListenerI* _changeListener;

	IloInt getColumnIndex(const char* name) const;
	IloIntArray makeColumnIndexArray(IloStringArray name) const;

	IloTuplePath* addWithoutCopy(IloIntArray path, IloInt value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
  IloTuplePath* addWithoutCopy(IloIntArray path, IloNum value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	IloTuplePath* addWithoutCopy(IloIntArray path, IloAny value);

	void buildIntCache();
	void buildNumCache();
	void buildAnyCache();
	void buildCommonCache();

	void initSlicing() { 
		if (_cache != 0) _cache->initialize(this);
	}
	void buildCache();
protected:

  friend class IloTupleIterator;
  void setCollection(IloTupleSetI* coll) { 
	  if (_coll && _coll!=coll){
		  this->deleteAllCaches();
	  }
	  _coll = coll;
  }
  void deleteAllCaches();

public:
	IloTupleSchemaI* getSchema() const{
		return _schema;
	}
  IloInt getInternalId(IloIntArray path) const;
  void checkIsFindRequest();
  void checkIsOrd();
  IloBool isOrd() const { return _isOrd; }
  IloBool isFindRequest() const {return _findRequest;}
  void checkIsSubTupleRequest();
  IloBool isSubTupleRequest() const {return _subTupleRequest;}

  void checkContainsAllKeys();
  IloBool containsAllKeys() const {return _containsAllKeys;}
  IloTupleRequestI* getOrMakeKeyRequest();

  IloTupleRequestI* getOrMakeSubRequest();
	/** <p> . </p> */
	void createSelectIndexes();
	/** <p> . </p> */
	void clearSelectIndexes();

	/** <p> . </p> */
	IloTuplePathList& getList();

  /** <p> . </p> */
  IloTupleBuffer buildPathBuffer();

	/** <p> . </p> */
	void clear();
	/** <p> . </p> */
	void import(IloTupleRequestI* select);

	/** <p> . </p> */
	IloTuplePath* add(IloInt columnIndex, IloInt value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt columnIndex, IloNum value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt columnIndex, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt columnIndex, IloSymbol value);

	/** <p> . </p> */
	IloTuplePath* add(IloInt columnIndex, IloIntCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt columnIndex, IloNumCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt columnIndex, IloAnyCollectionI* value);


	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloInt value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloNum value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloSymbol value);

	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloInt idx3, IloInt value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloInt idx3, IloNum value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloInt idx3, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt idx1, IloInt idx2, IloInt idx3, IloSymbol value);


	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloInt value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloNum value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloSymbol value);

	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloIntCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloNumCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloIntArray path, IloAnyCollectionI* value);

	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloInt value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloNum value, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloSymbol value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloIntCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloNumCollectionI* value);
	/** <p> . </p> */
	IloTuplePath* add(IloInt size, IloInt* path, IloAnyCollectionI* value);

	/** <p> . </p> */
	IloTuplePathArray addRecursive(IloTupleSchemaI* schema, IloIntArray path);

  /** <p> . </p> */
	IloInt getSize() const;

	/** <p> . </p> */
	IloTupleSetI* getCollection() const { return _coll; }
	virtual ~IloTupleRequestI();

	/** <p> . </p> */
	IloBool isEmpty() const { return _list.getSize() == 0; }
	/** <p> . </p> */
	IloBool isSingleColumn() const { return _list.getSize() == 1; }

	/** <p> . </p> */
	IloBool isIntQuery() const;
	/** <p> . </p> */
	IloBool isNumQuery() const;
	/** <p> . </p> */
	IloBool isAnyQuery() const;

	/** <p> . </p> */
	IloTuplePath* getFirst() const { return _list.firstObject(); }

	/** <p> . </p> */
	IloBool storeCache(IloIntArray slice);
	/** <p> . </p> */
	IloIntArrayI* getCache() {
		return ((_cache == 0) ? (IloIntArrayI*)0 : _cache->execute());
	}
	/** <p> . </p> */
	IloTupleRequestI(IloEnv env, IloTupleSetI* coll);
	/** <p> . </p> */
	IloTupleRequestI(IloEnv env, IloTupleSchemaI* schema);

	/** <p> . </p> */
	IloTuplePath* add(const char* col, IloInt value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col, IloNum value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col, IloSymbol value);

	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, IloInt value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, IloNum value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, IloSymbol value);

	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, const char* col3, IloInt value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, const char* col3, IloNum value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, const char* col3, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(const char* col1, const char* col2, const char* col3, IloSymbol value);

	/** <p> . </p> */
	IloTuplePath* add(IloStringArray path, IloInt value);
	/** <p> . </p> */
	IloTuplePath* add(IloStringArray path, IloNum value);
	/** <p> . </p> */
	IloTuplePath* add(IloStringArray path, IloAny value);
	/** <p> . </p> */
	IloTuplePath* add(IloStringArray path, IloSymbol value);

	/** <p> . </p> */
	virtual void display(ILOSTD(ostream)& os) const;
	/** <p> . </p> */
	IloBool contains(IloInt value);
	/** <p> . </p> */
	IloBool contains(IloNum value);
	/** <p> . </p> */
	IloBool contains(IloAny value);
};

#endif

