// --------------------------------------------------------------------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotupleschemaiH
#define __CONCERT_ilotupleschemaiH

#if !defined(__CONCERT_ilodatacolumnH)
#include <ilconcert/iltuple/ilodatacolumn.h>
#endif

class IloTupleSchemaI;
class IloTupleSchema;

class IloTuplePattern;
/**
* @group optim.concert.tuple
* @show internal
* <p> column type definition  </p>
**/
class IloColumnDefinitionI : public IloRttiEnvObjectI{
	ILORTTIDECL
protected:
	IloDataType _type;
	const char* _name;
	IloBool _isKey;
	IloAny _data;
	void setName(const char* name);
public:
	/** <p> Constructor </p> **/
	IloColumnDefinitionI(IloEnv env, IloDataType type, const char* name = 0);

	/** <p> Returns the type of the column </p> **/
	IloDataType getType() const {
		return _type;
	}

	void setKeyProperty(IloBool type){
		_isKey = type;
	}

	IloBool getKeyProperty() const{
		return _isKey;
	}

	/** <p> . </p> */
	void setData(IloAny data){
		_data = data;
	}
	/** <p> . </p> */
	IloAny getData(){
		return _data;
	}

	/** <p> Returns the name of the column </p> **/
	const char* getName() const {
		return _name;
	}

	virtual ~IloColumnDefinitionI(){
		if (_name!=0){
			getEnv()->free((char *)_name, 1+strlen(_name) );
		}
	}

	/** <p> . </p> */
	IloBool isInt() const { return _type == IntDataColumn; }
	/** <p> . </p> */
	IloBool isNum() const { return _type == NumDataColumn; }
	/** <p> . </p> */
	IloBool isSymbol() const { return _type == SymbolDataColumn; }
	/** <p> . </p> */
	IloBool isAny() const { return _type == AnyDataColumn; }

	/** <p> . </p> */
	IloBool isIntCollection() const { return _type == IntCollectionColumn; }
	/** <p> . </p> */
	IloBool isNumCollection() const { return _type == NumCollectionColumn; }
	/** <p> . </p> */
	IloBool isAnyCollection() const { return _type == AnyCollectionColumn; }

	/** <p> . </p> */
	virtual IloBool isTuple() const;
	/** <p> Displays the column definition </p> **/
	virtual void display(ILOSTD(ostream)& out) const;
};


/**
* @group optim.concert.tuple
* @show internal
* <p> column type definition  </p>
**/
class IloColumnDefinition {
	IloColumnDefinitionI* _impl;
public:
	/** <p> Constructor </p> **/
	IloColumnDefinition():_impl(0){}
	/** <p> Constructor </p> **/
	IloColumnDefinition(IloColumnDefinitionI* impl):_impl(impl){}
	/** <p> Returns the implementation </p> **/
	IloColumnDefinitionI* getImpl() const { return _impl; }
	/** <p> Delete the implementation </p> **/
	void end(){
		if (_impl){
			delete _impl; _impl = 0;
		}
	}
};

typedef IloArray<IloColumnDefinition> IloColumnDefinitionArrayBase;

/**
* @group optim.concert.tuple
* @show internal
* <p> Array of column type definition  </p>
**/
class IloColumnDefinitionArray : public IloColumnDefinitionArrayBase {
public:
	typedef IloDefaultArrayI ImplClass;
	/** <p> Constructor </p> **/
	IloColumnDefinitionArray(IloDefaultArrayI* i=0) : IloColumnDefinitionArrayBase(i){}
	/** <p> Constructor </p> **/
	IloColumnDefinitionArray(const IloColumnDefinitionArray& copy) : IloColumnDefinitionArrayBase(copy){}
	/** <p> Constructor </p> **/
	IloColumnDefinitionArray(const IloEnv env, IloInt n = 0) : IloColumnDefinitionArrayBase(env, n){}
};

/**
* @group optim.concert.tuple
* @show internal
* <p> Class to declare a DataTable schema </p> **/
class IloTupleSchemaI : public IloRttiEnvObjectI {
private:
	IloBool _hasKey;
	IloTupleSchemaI* _keySchema;
	IloIntArray _keyTotalIdx;
	IloIntArray _keyIdx;
	IloBool _hasSubTuple;
	const char* _name;
	IloColumnDefinitionArray _array;
	IloColumnDefinitionArray _totalArray;
	IloDataType getColumnType(IloIntArray path);
	IloDataType getColumnType(IloInt index1, IloInt index2);
	IloDataType getColumnType(IloInt index1, IloInt index2, IloInt index3);
public:
    IloInt getInternalId(IloIntArray path) const;
    IloInt getInternalSubId(IloIntArray path) const;
	IloTuplePattern makeTuplePattern() const;
	IloBool hasSubTuple() const { return _hasSubTuple; }
	void setName(const char* name);
	const char* getName() { 
		return _name; 
	}

	IloColumnDefinitionArray getOrMakeTotalColumnDefinitionArray();

    IloBool isCompatibleWith(IloTupleSchemaI*) const;

	/** <p> . </p> */
	IloBool isSimpleTypedSchema() const;
	virtual ~IloTupleSchemaI();

	void setKeyProperty(const char* col);
	void setKeyProperty(IloSymbolI* col);
	void setKeyProperty(IloInt idx);
	
	IloBool hasKeyProperty(const char* col) const;
	IloBool hasKeyProperty(IloSymbolI* col) const;
	IloBool hasKeyProperty(IloInt idx) const;

	IloBool hasKey() const { return _hasKey; }

	IloTupleSchemaI* getOrMakeSharedKeySchema();
	IloIntArray getOrMakeTotalKeyIndexes();
	IloIntArray getOrMakeKeyIndexes();
	IloInt getTotalIndexFromKey(IloInt key) const;
	IloInt getIndexFromKey(IloInt key) const;

	/**
	* <p>This constructor creates an array of <code>max</code> elements, all of
	* which are empty handles.</p>
	*/
	IloTupleSchemaI(IloEnv env, const char* name = 0);

	/**
	* <p>This member function returns an integer indicating the size of the
	* invoking array. An empty array has size 0 (zero). </p>
	*/
	IloInt getSize() const;

	/** <p> returns the total number of column of the schema </p> **/
	IloInt getTotalColumnNumber() const;

	/** <p> Returns the column defintion of index i </p> **/
	IloColumnDefinitionI* getColumn(IloInt i) const {
		return _array[i].getImpl();
	}

  IloInt getColumnIndex(const char* name) const;
  IloInt getColumnIndex(const IloSymbolI* name) const;
  const char* getColumnName(IloInt idx) const;

	/** <p> . </p> */
	IloColumnDefinitionI* getColumn(IloIntArray path) const;

	/** <p> Returns array of columns definition </p> **/
	IloColumnDefinitionArray getArray() const;

	/**
	* <p>This member function adds an IloInt Column.</p>
	*/
	void addIntColumn(const char* name =0);

	/**
	* <p>This member function adds an IloInt Column.</p>
	*/
	void addNumColumn(const char* name =0);

	void addAnyColumn(const char* name =0);

	/** <p> . </p> */
	void addSymbolColumn(const char* name =0);

	/** <p> . </p> */
	void addIntCollectionColumn(const char* name =0);
	/** <p> . </p> */
	void addNumCollectionColumn(const char* name =0);
	/** <p> . </p> */
	void addAnyCollectionColumn(const char* name =0);

	/**
	* <p>This member function appends the elements in <code>ax</code> to the
	* invoking array. </p>
	*/
	void addTupleColumn(IloTupleSchema ax, const char* name=0);
  /** <p> </p> **/
  IloTupleSchemaI* getTupleColumn(IloInt colIndex) const;
  /** <p> </p> **/
  IloTupleSchemaI* getTupleColumn(IloIntArray path) const;

	/**
	* <p>This member function removes all the elements from the invoking array.
	* In other words, it produces an empty array.</p>
	*/
	void clear();

	/** <p> Displays the schema </p> **/
	void display(ILOSTD(ostream)& out) const;

	/** <p> . </p> */
	IloBool isInt(IloInt index);
	/** <p> . </p> */
	IloBool isNum(IloInt index);
	/** <p> . </p> */
	IloBool isAny(IloInt index);
	/** <p> . </p> */
	IloBool isSymbol(IloInt index);
	/** <p> . </p> */
	IloBool isTuple(IloInt index);

	/** <p> . </p> */
	IloBool isIntCollection(IloInt index);
	/** <p> . </p> */
	IloBool isNumCollection(IloInt index);
	/** <p> . </p> */
	IloBool isAnyCollection(IloInt index);

	/** <p> . </p> */
	IloBool isInt(IloIntArray path);
	/** <p> . </p> */
	IloBool isNum(IloIntArray path);
	/** <p> . </p> */
	IloBool isAny(IloIntArray path);

	/** <p> . </p> */
	IloBool isIntCollection(IloIntArray path);
	/** <p> . </p> */
	IloBool isNumCollection(IloIntArray path);
	/** <p> . </p> */
	IloBool isAnyCollection(IloIntArray path);

	/** <p> . </p> */
	IloBool isSymbol(IloIntArray path);
	/** <p> . </p> */
	IloBool isTuple(IloIntArray path);

	/** Sets the data associated with the column **/
	void setData(IloInt index, IloAny data);
	/** Sets the data associated with the column **/
	void setData(IloIntArray path, IloAny data);
	/** Returns the data associated with the column **/
	IloAny getData(IloInt index);
	/** Returns the data associated with the column **/
	IloAny getData(IloIntArray path);


	/** <p> . </p> */
	IloBool isInt(IloInt index1, IloInt index2);
	/** <p> . </p> */
	IloBool isNum(IloInt index1, IloInt index2);
	/** <p> . </p> */
	IloBool isAny(IloInt index1, IloInt index2);

	/** <p> . </p> */
	IloBool isIntCollection(IloInt index1, IloInt index2);
	/** <p> . </p> */
	IloBool isNumCollection(IloInt index1, IloInt index2);
	/** <p> . </p> */
	IloBool isAnyCollection(IloInt index1, IloInt index2);

	/** <p> . </p> */
	IloBool isSymbol(IloInt index1, IloInt index2);
	/** <p> . </p> */
	IloBool isTuple(IloInt index1, IloInt index2);

	/** <p> . </p> */
	IloBool isInt(IloInt index1, IloInt index2, IloInt index3);
	/** <p> . </p> */
	IloBool isNum(IloInt index1, IloInt index2, IloInt index3);
	/** <p> . </p> */
	IloBool isAny(IloInt index1, IloInt index2, IloInt index3);

	/** <p> . </p> */
	IloBool isIntCollection(IloInt index1, IloInt index2, IloInt index3);
	/** <p> . </p> */
	IloBool isNumCollection(IloInt index1, IloInt index2, IloInt index3);
	/** <p> . </p> */
	IloBool isAnyCollection(IloInt index1, IloInt index2, IloInt index3);

	/** <p> . </p> */
	IloBool isSymbol(IloInt index1, IloInt index2, IloInt index3);
	/** <p> . </p> */
	IloBool isTuple(IloInt index1, IloInt index2, IloInt index3);

};

#endif

