// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------


#ifndef __XML_iloxmlhashH
#define __XML_iloxmlhashH

#if !defined(__CONCERT_ilosysH)
#include <ilconcert/ilosys.h>
#endif

#if !defined(__CONCERT_iloenvH)
#include <ilconcert/iloenv.h>
#endif

#if !defined(__CONCERT_iloanyH)
#include <ilconcert/iloany.h>
#endif

#if !defined(__CONCERT_ilohashH)
#include <ilconcert/ilohash.h>
#endif

typedef IloAdressHashTable<IloInt> IdHash;

class IloXmlIdTable : public IdHash {
 public:
  IloXmlIdTable(IloEnv env, IloInt size=IloDefaultHashSize): IdHash(env, size){}
  ~IloXmlIdTable(){}
};




 /**
 * @group optim.concert.xml
 * <p> This class stores object and related Ids. </p>
 * <p> It also have an API to check the type of the objects if they are IloRtti objects. </p>
 **/
class IloXmlObjectHandler {
private:
    IloAnyArray _link2obj;
    IloEnv _env;

    IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloRtti* exprI);
    IloBool checkTypeOfObjectById(IloTypeInfo type, IloRtti* exprI);

public:
    ~IloXmlObjectHandler();

    IloAnyArray getObjects(){
	return _link2obj;
    }

 /** <p>Constructor</p> **/
    IloXmlObjectHandler(IloEnv);

 /** <p> Adds an object and a related Id </p> **/
    void addLink2Obj(IloInt id, const void* object);

 /** <p> Empties the object. </p> **/
    void clearLink2Obj();

 /** <p> Gets an object from its Id, or return null </p> **/
    IloAny getObjectById(IloInt id);

 /**
 * <p> Checks whether an object correspond or not to the given Id. </p>
 **/
    IloBool isValidId(IloInt id);

 /**
  * <p>This method checks the RTTI of the object referenced
  * by the identifier <code>Xml_Id</code> in the XML. This object must already
  * be serialized. </p>
  */
  IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloInt Xml_Id);

 /**
  * <p>This method checks the TypeInfo of the object referenced
  * by the id in the xml. This object must have been already serialized. </p>
  */
  IloBool checkTypeOfObjectById(IloTypeInfo type, IloInt Xml_Id);

};



 /**
 * @group optim.concert.xml
 * <p> This class gives unique Ids to objects. </p>
 * <p> It also have an API to check the type of the objects if they are IloRtti objects. </p>
 * <p> This class is used by the IloXmlContext when writing an object tree to XML. </p>
 * @see IloXmlContext#getIdManager()
 **/
class IloXmlIdManager {
private:
    IloXmlIdTable _idHash;
    IloInt _maxId;
    IloEnv _env;

    IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloRtti* exprI);
    IloBool checkTypeOfObjectById(IloTypeInfo type, IloRtti* exprI);

public:
 IloXmlIdTable& getHash(){
    return _idHash;
 }

 /** <p> Constructor </p> **/
    IloXmlIdManager(IloEnv env);

 /** <p> Returns the IloEnv object. </p> **/
    IloEnv getEnv() { return _env; }
 
 /** <p> Checks whether an object has already been given an Id.
 * It does not check whether the object still exists or not. </p> 
 **/
    IloBool isValidId(IloInt id){
	if (_maxId >= id){
            return IloTrue;
        }
        return IloFalse;
    }

 /** <p> Empties the object </p> **/
    void clear();

 /**
 * <p>Determines whether an object has already been assigned an ID.</p>
 * <p><code>firstTime</code> is set to 
 * true if obj was not previously known to the IdManager; false otherwise. </p>
 * <p> If the object has already been assigned an ID, the ID is returned  
 * and firstTime is set to false; 
 * otherwise, -1 is returned and firstTime is set to true. 
 * hasId differs from getId in that it never creates an ID for an object that has not already been seen by the IdManager. </p>
 **/
    IloInt hasId(const void* object, IloBool& firstTime);

 /**
 * <p> Returns the ID for the specified object, generating a new ID if the specified object has not already been identified by the IdManager. </p>
 * <p><code>firstTime</code> is set to 
 * true if obj was not previously known to the IdManager; false otherwise. </p>
 **/
    IloInt getId(const void* object, IloBool& firstTime);

 /** 
 * <p>Returns the Maximum Id.</p>
 **/
    IloInt getMaxId();


 /** <p> Gets an Id for the argument </p>
 * @see IloXmlIdManager#getId(const void*, IloBool&)
 **/
    IloInt getId(const void *p) {
       IloBool dummy;
       return getId(p, dummy);
    }

 /**
 * <p>Determines whether an object has already been assigned an ID.</p>
 **/
    IloBool hasId(const void *p) {
       IloBool dummy;
       return hasId(p, dummy) >= 0;
    }

    const void* getObjectById(IloInt id);

 /**
  * <p>This method checks the RTTI of the object referenced
  * by the identifier <code>Xml_Id</code> in the XML. This object must already
  * be serialized. </p>
  */
  IloBool checkRttiOfObjectById(IloTypeIndex RTTI, IloInt Xml_Id);

 /**
  * <p>This method checks the TypeInfo of the object referenced
  * by the id in the xml. This object must have been already serialized. </p>
  */
  IloBool checkTypeOfObjectById(IloTypeInfo type, IloInt Xml_Id);
    

    ~IloXmlIdManager();
};


#endif
