// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __XML_iloxmlmetaH
#define __XML_iloxmlmetaH


#if !defined(__XML_iloxmlabstractH)
#include <ilconcert/iloxmlabstract.h>
#endif


class IloAndI;
class IloOrI;
class IloRangeI;


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_And</code> enables to serialize a
 * <code>IloAndI</code> in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/


class IloXmlInfo_And : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_And::getTagName(); }
  IloXmlInfo_And(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloAndI* AndI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloAndI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  IloBool setConstraintList(IloXmlWriter writer, IloXmlElement* element, IloAndI* AndI);
  IloBool checkSubElement(IloXmlReader reader, IloXmlElement* subElement, IloAndI* exprI);

};




/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_Or</code> enables to serialize a
 * IloOrI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_Or : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_Or::getTagName(); }
  IloXmlInfo_Or(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloOrI* OrI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloOrI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  IloBool setConstraintList(IloXmlWriter writer, IloXmlElement* element, IloOrI* OrI);
  IloBool checkSubElement(IloXmlReader reader, IloXmlElement* subElement, IloOrI* exprI);

};



/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_Range</code> enables to serialize a
 * IloRangeI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_Range : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_Range::getTagName(); }
  IloXmlInfo_Range(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloRangeI* rangeI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloRangeI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);

protected:
  IloBool setBounds(IloXmlWriter writer, IloXmlElement* element, IloRangeI* rangeI);
  IloBool setRangeExpr(IloXmlWriter writer, IloXmlElement* element, IloRangeI* rangeI);


};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntGe</code> enables to read a
 * IloRangeI with IntExpr as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntGe : public IloXmlInfo {
public:
  IloXmlInfo_IntGe(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntGe::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntLe</code> enables to read a
 * IloRangeI with IntExprs as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntLe : public IloXmlInfo {
public:
  IloXmlInfo_IntLe(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntLe::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_NumGeCst</code> enables to read a
 * IloRangeI with NumExpr and double as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_NumGeCst : public IloXmlInfo {
public:
  IloXmlInfo_NumGeCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_NumGeCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_NumLeCst</code> enables to read a
 * IloRangeI with NumExpr and double as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_NumLeCst : public IloXmlInfo {
public:
  IloXmlInfo_NumLeCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_NumLeCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntGtCst</code> enables to read a
 * IloRangeI with IntExpr and int as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntGtCst : public IloXmlInfo {
public:
  IloXmlInfo_IntGtCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntGtCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntLtCst</code> enables to read a
 * IloRangeI with IntExpr and int as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntLtCst : public IloXmlInfo {
public:
  IloXmlInfo_IntLtCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntLtCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_NumEqCst</code> enables to read a
 * IloRangeI with NumExpr and double as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_NumEqCst : public IloXmlInfo {
public:
  IloXmlInfo_NumEqCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_NumEqCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntEqCst</code> enables to read a
 * IloRangeI with IntExpr and int as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntEqCst : public IloXmlInfo {
public:
  IloXmlInfo_IntEqCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntEqCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntEq</code> enables to read a
 * IloRangeI with IntExpr and IntExpr as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntEq : public IloXmlInfo {
public:
  IloXmlInfo_IntEq(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntEq::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntLeCst</code> enables to read a
 * IloRangeI with IntExpr and int as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntLeCst : public IloXmlInfo {
public:
  IloXmlInfo_IntLeCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntLeCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntGeCst</code> enables to read a
 * IloRangeI with IntExpr and int as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntGeCst : public IloXmlInfo {
public:
  IloXmlInfo_IntGeCst(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntGeCst::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


// ----------------------------------------------------------*- C++ -*-------
//    INCLUDE FOR IloXmlDistribute
// ----------------------------------------------------------*- C++ -*-------

class IloDistributeI;


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 *<p>An instance of IloXmlInfo_Distribute enables to serialize a IloDistributeI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_Distribute : public IloXmlInfo {
public:
  static const char* getTagName();
  virtual const char* getTag() {
    return IloXmlInfo_Distribute::getTagName();
  }
  IloXmlInfo_Distribute(IloXmlContextI* context);
  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloRtti* exprI){
    return getTagElement(writer, (IloDistributeI*)exprI);
  }
  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloDistributeI* exprI);

  virtual IloBool write(IloXmlWriter writer, IloDistributeI* exprI, IloXmlElement* masterElement);

  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloDistributeI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_Imply</code> enables to read a
 * LowerThanOrEqual with Constraint and Constraint as arguments in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_Imply : public IloXmlInfo {
public:
  IloXmlInfo_Imply(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_Imply::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntSum</code> enables to read a
 * sum with IntExprArray as argument in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntSum : public IloXmlInfo {
public:
  IloXmlInfo_IntSum(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntSum::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};

/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_NumSum</code> enables to read a
 * sum with NumExprArray as argument in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_NumSum : public IloXmlInfo {
public:
  IloXmlInfo_NumSum(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_NumSum::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};



/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_NumOpposite</code> enables to read the
 * opposite of a  NumExpr as argument in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_NumOpposite : public IloXmlInfo {
public:
  IloXmlInfo_NumOpposite(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_NumOpposite::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <note> For compatibility with Java and C# </note>
 * <p> An instance of <code>IloXmlInfo_IntOpposite</code> enables to read the
 * opposite of an IntExpr as argument in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/
class IloXmlInfo_IntOpposite : public IloXmlInfo {
public:
  IloXmlInfo_IntOpposite(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntOpposite::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


#endif
