// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __XML_iloxmlvariablesH
#define __XML_iloxmlvariablesH


#if !defined(__XML_iloxmlabstractH)
#include <ilconcert/iloxmlabstract.h>
#endif

class IloBoolVarI;
class IloNumVarI;
class IloConstraintIndVarI;
class IloOwnedNumVarI;
class IloIntSetVarI;
class IloSemiContVarI;
class IloIntSetCardVar;



#define getLowerBound getLB
#define getUpperBound getUB



/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_NumVar</code> enables to serialize a
 * IloNumVarI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_NumVar : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_NumVar::getTagName(); }
  IloXmlInfo_NumVar(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloNumVarI* numvarI, IloXmlElement* masterElement);
  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloRtti* exprI){
    return getTagElement(writer, (IloNumVarI*)exprI);
  }
  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloNumVarI* exprI);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloNumVarI*)extractableI,masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  virtual IloBool readSolution(IloXmlReader reader, IloSolution solution, IloXmlElement* element);
  virtual void writeSolution(IloXmlWriter writer, const IloSolution solution, IloNumVarI* numvarI);
  virtual void writeSolution(IloXmlWriter writer, const IloSolution solution, const IloExtractable extractable){
    writeSolution(writer, solution, (IloNumVarI*)extractable.getImpl());
  }

};


class IloXmlInfo_IntVar : public IloXmlInfo {
public:
  IloXmlInfo_IntVar(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntVar::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  virtual IloBool readSolution(IloXmlReader reader, IloSolution solution, IloXmlElement* element);
};


class IloXmlInfo_BoolVar : public IloXmlInfo {
public:
  IloXmlInfo_BoolVar(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_BoolVar::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  virtual IloBool readSolution(IloXmlReader reader, IloSolution solution, IloXmlElement* element);
};



class IloXmlInfo_IntEnumeratedVar : public IloXmlInfo {
public:
  IloXmlInfo_IntEnumeratedVar(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_IntEnumeratedVar::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  virtual IloBool readSolution(IloXmlReader reader, IloSolution solution, IloXmlElement* element);
};

class IloXmlInfo_NumEnumeratedVar : public IloXmlInfo {
public:
  IloXmlInfo_NumEnumeratedVar(IloXmlContextI* context);
  static const char* getTagName();
  virtual const char* getTag() { return IloXmlInfo_NumEnumeratedVar::getTagName(); }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  virtual IloBool readSolution(IloXmlReader reader, IloSolution solution, IloXmlElement* element);
};



template <class ILONUMVARI> IloBool SETBOUNDS(IloXmlWriter writer, IloXmlElement* element, ILONUMVARI* numVarI){
  const char* Ub = 0;
  const char* Lb = 0;
  IloXmlElement* subElement;

  switch (numVarI->getType()) {
  case ILOBOOL:
    subElement = writer.createElement(IloXmlTagDef::XmlBoolDomain);
    Lb = writer.Int2String((IloInt)numVarI->getLowerBound());
    Ub = writer.Int2String((IloInt)numVarI->getUpperBound());
    break;
  case ILOINT:
    subElement = writer.createElement(IloXmlTagDef::XmlIntDomain);
    Lb = writer.Int2String((IloInt)numVarI->getLowerBound());
    Ub = writer.Int2String((IloInt)numVarI->getUpperBound());
    break;
  case ILOFLOAT:
    subElement = writer.createElement(IloXmlTagDef::XmlFloatDomain);
    Lb = writer.Num2String((IloNum)numVarI->getLowerBound());
    Ub = writer.Num2String((IloNum)numVarI->getUpperBound());
    break;
  default:
    return IloFalse;
    break;
  }
  writer.addAttribute(subElement, IloXmlAttributeDef::LowerBound, Lb);
  writer.addAttribute(subElement, IloXmlAttributeDef::UpperBound, Ub);
  writer.addSubElement(element, subElement);
  writer.deleteAllocatedMemory(Ub);
  writer.deleteAllocatedMemory(Lb);
  return IloTrue;
}

template <class ILONUMVARI> IloBool SETENUMBOUNDS(IloXmlWriter writer, IloXmlElement* element, ILONUMVARI* numVarI){
  IloXmlElement* subElement;
  const char* text = 0;
  switch (numVarI->getType()) {
  case ILOINT:
    subElement = writer.createElement(IloXmlTagDef::XmlIntEnumeratedDomain);
    text = writer.IntArray2String(numVarI->getIntValues());
    writer.addText(subElement, text);
    writer.deleteAllocatedMemory(text);
    break;
  case ILOFLOAT:
    subElement = writer.createElement(IloXmlTagDef::XmlFloatEnumeratedDomain);
    text = writer.NumArray2String(numVarI->getNumValues());
    writer.addText(subElement, text);
    writer.deleteAllocatedMemory(text);
    break;
  default:
    return IloFalse;
    break;
  }
  writer.addSubElement(element, subElement);
  return IloTrue;
}


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_ConstraintIndVar</code> enables to serialize a
 * IloConstraintIndVarI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_ConstraintIndVar : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_ConstraintIndVar::getTagName(); }
  IloXmlInfo_ConstraintIndVar(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloConstraintIndVarI* varI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloConstraintIndVarI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};



/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_IntSetVar</code> enables to serialize a
 * IloIntSetVarI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_IntSetVar : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_IntSetVar::getTagName(); }
  IloXmlInfo_IntSetVar(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloIntSetVarI* IntSetVarI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloIntSetVarI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
  virtual IloBool readSolution(IloXmlReader reader, IloSolution solution, IloXmlElement* element);
  virtual void writeSolution(IloXmlWriter writer, const IloSolution solution, IloIntSetVarI* exprI);
  virtual void writeSolution(IloXmlWriter writer, const IloSolution solution, const IloExtractable extractable){
    writeSolution(writer, solution, (IloIntSetVarI*)extractable.getImpl());
  }

protected:
  IloBool setSet(IloXmlWriter writer, IloXmlElement* element, IloIntSetVarI* exprI);

};



/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_SemiContVar</code> enables to serialize a
 * IloSemiContVarI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_SemiContVar : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_SemiContVar::getTagName(); }
  IloXmlInfo_SemiContVar(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloSemiContVarI* exprI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloSemiContVarI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


/**
 * @show internal
 * @group optim.concert.xmlconcert
 *
 * <p>An instance of <code>IloXmlInfo_SemiContVar</code> enables to serialize a
 * IloSemiContVarI in xml. </p>
 * @see IloXmlContextI
 * @see IloXmlInfo
 * @see IloXmlReader
 * @see IloXmlWriter
 **/

class IloXmlInfo_IntSetCardVar : public IloXmlInfo {

public:
  static const char* getTagName();

  virtual const char* getTag() { return IloXmlInfo_IntSetCardVar::getTagName(); }
  IloXmlInfo_IntSetCardVar(IloXmlContextI* context);
  virtual IloBool write(IloXmlWriter writer, IloIntSetCardVarI* exprI, IloXmlElement* masterElement);
  virtual IloBool write(IloXmlWriter writer, const IloRtti* extractableI, IloXmlElement* masterElement){
    return write(writer, (IloIntSetCardVarI*)extractableI, masterElement);
  }
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);
};


#endif
