// --------------------------------------------------------------------------
// File: examples/src/AdMIPex6.java
// Version 10.1  
// --------------------------------------------------------------------------
//  Copyright (C) 1999-2006 by ILOG.
//  All Rights Reserved.
//  Permission is expressly granted to use this example in the
//  course of developing applications that use ILOG products.
// --------------------------------------------------------------------------
//
// AdMIPex6.java -- Solving a model by passing in a solution for the root node
//                  and using that in a solve callback
//
// To run this example, command line arguments are required:
//     java AdMIPex6  filename
// where 
//     filename  Name of the file, with .mps, .lp, or .sav
//               extension, and a possible additional .gz 
//               extension.
// Example:
//     java AdMIPex6  mexample.mps.gz

import ilog.concert.*;
import ilog.cplex.*;


public class AdMIPex6 {
   static class Solve extends IloCplex.SolveCallback {
      boolean     _done = false;
      IloNumVar[] _vars;
      double[]    _x;
      Solve(IloNumVar[] vars, double[] x) { _vars = vars; _x = x; }
    
      public void main() throws IloException {
         if ( !_done ) {
            setVectors(_x, _vars, null, null);
            _done = true;
         }
      }
   }

   public static void main(String[] args) {
      try {
         IloCplex cplex = new IloCplex();
       
         cplex.importModel(args[0]);
         IloLPMatrix lp = (IloLPMatrix)cplex.LPMatrixIterator().next();
       
         IloConversion relax = cplex.conversion(lp.getNumVars(),
                                                IloNumVarType.Float);
         cplex.add(relax);

         cplex.solve();
         System.out.println("Relaxed solution status = " + cplex.getStatus());
         System.out.println("Relaxed solution value  = " + cplex.getObjValue());

         double vals[] = cplex.getValues(lp.getNumVars());
         cplex.use(new Solve(lp.getNumVars(), vals));

         cplex.delete(relax);
       
         if ( cplex.solve() ) {
            System.out.println("Solution status = " + cplex.getStatus());
            System.out.println("Solution value  = " + cplex.getObjValue());
         }
         cplex.end();
      }
      catch (IloException e) {
         System.err.println("Concert exception caught: " + e);
      }
   }
}
