// --------------------------------------------------------------------------
// File: examples/src/LPex6.java
// Version 10.1  
// --------------------------------------------------------------------------
//  Copyright (C) 1999-2006 by ILOG.
//  All Rights Reserved.
//  Permission is expressly granted to use this example in the
//  course of developing applications that use ILOG products.
// --------------------------------------------------------------------------
//
// LPex6.java - Illustrates that optimal basis can be copied and
//              used to start an optimization.
//
 
import ilog.concert.*;
import ilog.cplex.*;


public class LPex6 {
   public static void main(String[] args) {
      try {
         IloCplex cplex = new IloCplex();
       
         IloNumVar[][] var = new IloNumVar[1][];
         IloRange[][]  rng = new IloRange[1][];
         
         populateByRow(cplex, var, rng);
       
         IloCplex.BasisStatus[] cstat = {
            IloCplex.BasisStatus.AtUpper,
            IloCplex.BasisStatus.Basic,
            IloCplex.BasisStatus.Basic
         };
         IloCplex.BasisStatus[] rstat = {
            IloCplex.BasisStatus.AtLower,
            IloCplex.BasisStatus.AtLower
         };
         cplex.setBasisStatuses(var[0], cstat, rng[0], rstat);
         
         if ( cplex.solve() ) {
            System.out.println("Solution status = " + cplex.getStatus());
            System.out.println("Solution value  = " + cplex.getObjValue());
            System.out.println("Iteration count = " + cplex.getNiterations());
          
            double[] x     = cplex.getValues(var[0]);
            double[] dj    = cplex.getReducedCosts(var[0]);
            double[] pi    = cplex.getDuals(rng[0]);
            double[] slack = cplex.getSlacks(rng[0]);
          
            int ncols = cplex.getNcols();
            for (int j = 0; j < ncols; ++j) {
               System.out.println("Column: " + j +
                                  " Value = " + x[j] +
                                  " Reduced cost = " + dj[j]);
            }
          
            int nrows = cplex.getNrows();
            for (int i = 0; i < nrows; ++i) {
               System.out.println("Row   : " + i +
                                  " Slack = " + slack[i] +
                                  " Pi = " + pi[i]);
            }
         }
         cplex.end();
      }
      catch (IloException exc) {
        System.err.println("Concert exception '" + exc + "' caught");
      }
   }

   static void populateByRow(IloMPModeler  model,
                             IloNumVar[][] var,
                             IloRange[][]  rng) throws IloException {
      double[] lb = {0.0, 0.0, 0.0};
      double[] ub = {40.0, Double.MAX_VALUE, Double.MAX_VALUE};
      var[0] = model.numVarArray(3, lb, ub);
    
      double[] objvals = {1.0, 2.0, 3.0};
      model.addMaximize(model.scalProd(var[0], objvals));
    
      rng[0] = new IloRange[2];
      rng[0][0] = model.addLe(model.sum(model.prod(-1.0, var[0][0]),
                                        model.prod( 1.0, var[0][1]),
                                        model.prod( 1.0, var[0][2])), 20.0);
      rng[0][1] = model.addLe(model.sum(model.prod( 1.0, var[0][0]),
                                        model.prod(-3.0, var[0][1]),
                                        model.prod( 1.0, var[0][2])), 30.0);
   }
}
