// --------------------------------------------------------------------------
// File: examples/src/MIPex2.java
// Version 10.1  
// --------------------------------------------------------------------------
//  Copyright (C) 2001-2006 by ILOG.
//  All Rights Reserved.
//  Permission is expressly granted to use this example in the
//  course of developing applications that use ILOG products.
// --------------------------------------------------------------------------
//
// MIPex2.java - Reading in and optimizing a MIP problem.  In fact, this
//               program also works for LP or QP problems, but is different
//               from LPex2 in that no dual solution information is queried.
//
// To run this example, command line arguments are required.
// i.e.,   java MIPex2  filename
// where 
//     filename is the name of the file, with .mps, .lp, or .sav extension
// Example:
//     java MIPex2  mexample.mps
//

import ilog.concert.*;
import ilog.cplex.*;


public class MIPex2 {
   static void usage() {
      System.out.println("usage:  MIPex2 <filename>");
   }

   public static void main(String[] args) {
      if ( args.length != 1 ) {
         usage();
         return;
      }
      try {
         IloCplex cplex = new IloCplex();
       
         cplex.importModel(args[0]);
       

//         cplex.setParam(IloCplex.IntParam.Reduce, 1);
         if ( cplex.solve() ) {
            System.out.println("Solution status = " + cplex.getStatus());
            System.out.println("Solution value  = " + cplex.getObjValue());
          
            // access IloLPMatrix object that has been read from file in order to
            // access variables which are the columns of the lp.  Method
            // importModel guarantees to the model into exactly on IloLPMatrix
            // object which is why there are no test or iterations needed in the
            // following line of code.
            IloLPMatrix lp = (IloLPMatrix)cplex.LPMatrixIterator().next();
          
            double[] x = cplex.getValues(lp);
            for (int j = 0; j < x.length; ++j) {
               System.out.println("Column: " + j + " Value = " + x[j]);
            }
         }
cplex.exportModel ("java.mipex2.lp");
         cplex.end();
      }
      catch (IloException e) {
         System.err.println("Concert exception caught: " + e);
      }
   }
}
