// --------------------------------------------------------------------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef _ILOCPLEXI_H
#define _ILOCPLEXI_H

#include <ilconcert/ilomodel.h>
#include <ilconcert/iloalgi.h>
#include <ilcplex/ilocarray.h>

#ifdef ILO_ADVANCED_CONCERT
#include <ilconcert/iloforall.h>
#include <ilconcert/ilomap.h>
#include <ilconcert/iloevaluator.h>
#endif

#define CPXNEW(x,t)     try {x=t;} catch(...) {x=0;}

extern "C" {
#include <ilcplex/cplex.h>
struct cpxenv;
struct cpxlp;
class IloCplexCache;
}

class IloCplexI;
class IloSVector;
class IloSVectorArray;
class IloLPExtractor;
class IloExprExtractor;
class IloDefaultLPExtractor;
class IloLogicalExtractor;
class IloSetConstraintExtractor;
class IloColumnExtractor;
class IloExprLinearizer;
class IloExprParser;
class IloLPExtractorManager;
class IloLinearizerI;
class IloCallbackManager;
class IloCplexCallbackManager;
class IloCplexRow;
class IloCplexColumn;
class IloCplexSOS;
class IloCplexSolutionOpsNumVarAux;
class IloCplexSolutionOpsObjectiveAux;
class IloCplexAbortI;

class IloIntSetVarI;

// For the Goal API
class CpxNodeI;
class CpxBranchAsCplexI;
// END:: For the Goal API

class IloBoundI;

class ILO_EXPORTED IloBound : public IloConstraint {
  ILOEXTRHANDLE(IloBound,IloConstraint)
public:
  enum Type { Lower, Upper };

  IloBound (IloNumVar var, Type type);

  IloNumVar getVar();

  Type getType();
};


class IloCplex : public IloAlgorithm {
public:
  class NodeEvaluatorI;
  class SearchLimitI;
  IloCplexI* getImpl() const {
    return (IloCplexI*)(void*)IloAlgorithm::getImpl();
  }

private:
  friend class IloLPExtractor;
  friend class NodeEvaluatorI;
  friend class SearchLimitI;
  friend class CpxApplyGoalI;
  friend class CpxLimitGoalI;

public:
  IloCplex(IloEnv env);

  IloCplex(const IloModel model);

  IloCplex(IloCplexI* impl = 0);

  void clearModel();
  using IloAlgorithm::clear;

  IloConstraint            addUserCut(IloConstraint con);

  const IloConstraintArray addUserCuts(const IloConstraintArray con);

  void                clearUserCuts();

  IloConstraint            addCut(IloConstraint con);

  const IloConstraintArray addCuts(const IloConstraintArray con);

  void                clearCuts();

  IloConstraint            addLazyConstraint(IloConstraint con);

  const IloConstraintArray addLazyConstraints(const IloConstraintArray con);

  void                clearLazyConstraints();

  void importModel(IloModel&      model,
                          const char*    filename,
                          IloObjective&  obj,
                          IloNumVarArray vars,
                          IloRangeArray  rngs,
                          IloSOS1Array   sos1,
                          IloSOS2Array   sos2,
                          IloRangeArray  lazy = 0,
                          IloRangeArray  cuts = 0) const;

  void importModel(IloModel&      m,
                          const char*    filename,
                          IloObjective&  obj,
                          IloNumVarArray vars,
                          IloRangeArray  rngs,
                          IloRangeArray  lazy = 0,
                          IloRangeArray  cuts = 0) const;

  void importModel(IloModel& m, const char* filename) const;

  void exportModel(const char* filename) const;

  void writeOrder (const char* filename) const;

  void writeConflict (const char* filename) const;

  void writeParam(const char* name) const;

  void writeBasis(const char* name) const;

  void writeSolution(const char* name) const;

  void writeVectors(const char* name) const;

  void writeTextSolution(const char* name) const;

  void writeMIPStart(const char* name) const;

  void readOrder (const char* filename) const;

  void readParam(const char* name) const;

  void readBasis(const char* name) const;

  void readSolution(const char* name) const;

  void readVectors(const char* name) const;

  void readMIPStart(const char* name) const;

  IloObjective getObjective() const;

  const char* getVersion() const;

  IloInt getNiterations() const;

  IloInt getNbarrierIterations() const;

  IloInt getNsiftingIterations() const;

  IloInt getNsiftingPhaseOneIterations() const;

  IloInt getNcols () const;

  IloInt getNrows () const;

  IloInt getNQCs () const;

  IloInt getNSOSs () const;

  IloInt getNNZs  () const;

  /* for compatibility with previuous versions */
  IloInt getNSOS  () const;
  IloInt getNNz   () const;

  IloInt getNintVars() const;

  IloInt getNbinVars() const;

  IloInt getNsemiContVars() const;

  IloInt getNsemiIntVars() const;

   enum BasisStatus {
     NotABasicStatus  = -1,
     Basic            =  1,
     AtLower          =  0,
     AtUpper          =  2,
     FreeOrSuperbasic =  3
   };

  typedef IloArray<BasisStatus> BasisStatusArray;

  IloBool solveFixed();

  IloBool isMIP() const;

  IloBool isQC() const;

  IloBool isQO() const;

  /* just for compatibility */
  IloBool isQuadratic()  const;

  IloNum getBestObjValue() const;

  IloNum getCutoff() const;

  IloInt getIncumbentNode() const;

  IloInt getNprimalSuperbasics() const;

  IloInt getNdualSuperbasics() const;

  IloInt getNphaseOneIterations() const;

  IloInt getNnodes() const;

  IloInt getNnodesLeft() const;

  IloInt getNcrossPPush() const;

  IloInt getNcrossPExch() const;

  IloInt getNcrossDPush() const;

  IloInt getNcrossDExch() const;

// Deprecated
  IloInt getNcliques() const;

// Deprecated
  IloInt getNcovers() const;

  /* inherited from IloAlgorithm
  IloNum getValue (const IloExtractable con) const;
  IloNum getValue (const IloNumVar con) const;
  */

  void getValues (IloNumArray val, const IloNumVarArray var) const {
    IloAlgorithm::getValues(var, val);
  }

  /*
   * <p>This method puts the solution values of the numeric variables
   * indicated by the array <code>var</code> into the array
   * <code>val</code>.  Array <code>val</code> is resized to the same size as
   * array <code>var</code>, and <code>val[i]</code> will contain the
   * solution value for variable <code>var[i]</code>.</p>
   */
  void getValues (const IloNumVarArray var, IloNumArray val) const {
    IloAlgorithm::getValues(var, val);
  }

  void getValues (IloNumArray val, const IloIntVarArray var) const {
    IloAlgorithm::getValues(var, val);
  }

  void getValues (const IloIntVarArray var, IloNumArray val) const {
    IloAlgorithm::getValues(var, val);
  }

  void getIntSetValue (const IloIntSetVar var, IloIntSet set) const;

  IloNum getDual(const IloRange range) const;

  IloNum getSlack (const IloRange range) const;

  IloNum getInfeasibility (const IloConstraint con) const;

  IloNum getAX    (const IloRange range) const;

  IloNum getReducedCost(const IloNumVar var) const;

  IloNum getReducedCost(const IloIntVar var) const;

  IloNum getInfeasibility (const IloNumVar var) const;

  IloNum getInfeasibility (const IloIntVar var) const;

  /* compatibility methods */
  BasisStatus getStatus(const IloNumVar con) const;
  BasisStatus getStatus(const IloRange  con) const;

  BasisStatus getBasisStatus(const IloNumVar var) const;

  BasisStatus getBasisStatus(const IloIntVar var) const;

  BasisStatus getBasisStatus(const IloConstraint con) const;

  void getDuals  (IloNumArray val, const IloRangeArray con) const;

  void getSlacks (IloNumArray val, const IloRangeArray con) const;

  IloNum getInfeasibilities (IloNumArray infeas, const IloConstraintArray con) const;

  void getAX(IloNumArray val, const IloRangeArray con) const;

  void getReducedCosts(IloNumArray val, const IloNumVarArray var) const;

  void getReducedCosts(IloNumArray val, const IloIntVarArray var) const;

  IloNum getInfeasibilities (IloNumArray infeas, const IloNumVarArray var) const;

  IloNum getInfeasibilities (IloNumArray infeas, const IloIntVarArray var) const;

  void getStatuses(BasisStatusArray stat,
                          const IloNumVarArray con) const;
  void getStatuses(BasisStatusArray stat,
                          const IloRangeArray  con) const;
  void getStatuses(BasisStatusArray cstat,
                          const IloNumVarArray var,
                          BasisStatusArray rstat,
                          const IloRangeArray con) const;

  void getBasisStatuses(BasisStatusArray stat,
                               const IloNumVarArray var) const;

  void getBasisStatuses(BasisStatusArray stat,
                               const IloIntVarArray var) const;

  void getBasisStatuses(BasisStatusArray stat,
                               const IloConstraintArray con) const;

  void getBasisStatuses(BasisStatusArray cstat,
                               const IloNumVarArray var,
                               BasisStatusArray rstat,
                               const IloConstraintArray con) const;
  void getBasisStatuses(BasisStatusArray cstat,
                               const IloIntVarArray var,
                               BasisStatusArray rstat,
                               const IloConstraintArray con) const;

  void getBoundSA(IloNumArray lblower, IloNumArray lbupper,
                         IloNumArray ublower, IloNumArray ubupper,
                         const IloNumVarArray vars) const;

  void getBoundSA(IloNumArray lblower, IloNumArray lbupper,
                         IloNumArray ublower, IloNumArray ubupper,
                         const IloIntVarArray vars) const;

  void getRangeSA(IloNumArray lblower, IloNumArray lbupper,
                         IloNumArray ublower, IloNumArray ubupper,
                         const IloRangeArray con) const;

  void getRHSSA  (IloNumArray lower, IloNumArray upper,
                         const IloRangeArray cons) const;

  void getObjSA  (IloNumArray lower, IloNumArray upper,
                         const IloNumVarArray vars) const;

  void getObjSA  (IloNumArray lower, IloNumArray upper,
                         const IloIntVarArray cols) const;

#ifdef ILO_ADVANCED_CONCERT
  IloNum getAX(const IloForAllRange range) const;
  IloNum getDual(const IloForAllRange range) const;
  IloNum getSlack(const IloForAllRange range) const;
  void getAX  (IloNumArray val, const IloForAllRangeArray con) const;
  void getDuals  (IloNumArray val, const IloForAllRangeArray con) const;
  void getSlacks (IloNumArray val, const IloForAllRangeArray con) const;
  void getForAllRanges (IloForAllRangeArray rows,
                               IloConstraint       forall) const;
  IloNum getLB(const IloForAllRange range) const;
  IloNum getUB(const IloForAllRange range) const;
  IloBool feasOpt(const IloRangeArray rngs,       const IloNumArray rnglb,
                                                  const IloNumArray rngub,
                  const IloForAllRangeArray rngf, const IloNumArray frnglb,
                                                  const IloNumArray frngub,
                  const IloNumVarArray vars,      const IloNumArray varlb,
                                                  const IloNumArray varub);
#endif


  /* compatibility method */
  void setStatuses(const BasisStatusArray cstat,
                          const IloNumVarArray var,
                          const BasisStatusArray rstat,
                          const IloRangeArray con);

  void setBasisStatuses(const BasisStatusArray cstat,
                               const IloNumVarArray var,
                               const BasisStatusArray rstat,
                               const IloConstraintArray con);

  void setBasisStatuses(const BasisStatusArray cstat,
                               const IloIntVarArray var,
                               const BasisStatusArray rstat,
                               const IloConstraintArray con);

  void setVectors(const IloNumArray    x,
                         const IloNumArray    dj,
                         const IloNumVarArray var,
                         const IloNumArray    slack,
                         const IloNumArray    pi,
                         const IloRangeArray  rng);

  void setVectors(const IloNumArray    x,
                         const IloNumArray    dj,
                         const IloIntVarArray var,
                         const IloNumArray    slack,
                         const IloNumArray    pi,
                         const IloRangeArray  rng);

  enum NumParam {
    EpMrk           = CPX_PARAM_EPMRK, /**< = CPX_PARAM_EPMRK */
    EpOpt           = CPX_PARAM_EPOPT, /**< = CPX_PARAM_EPOPT */
    EpPer           = CPX_PARAM_EPPER, /**< = CPX_PARAM_EPPER */
    EpRHS           = CPX_PARAM_EPRHS, /**< = CPX_PARAM_EPRHS */
    NetEpOpt        = CPX_PARAM_NETEPOPT, /**< = CPX_PARAM_NETEPOPT */
    NetEpRHS        = CPX_PARAM_NETEPRHS, /**< = CPX_PARAM_NETEPRHS */

    TiLim           = CPX_PARAM_TILIM, /**< = CPX_PARAM_TILIM */
    BtTol           = CPX_PARAM_BTTOL, /**< = CPX_PARAM_BTTOL */
    CutLo           = CPX_PARAM_CUTLO, /**< = CPX_PARAM_CUTLO */
    CutUp           = CPX_PARAM_CUTUP, /**< = CPX_PARAM_CUTUP */
    EpGap           = CPX_PARAM_EPGAP, /**< = CPX_PARAM_EPGAP */
    EpInt           = CPX_PARAM_EPINT, /**< = CPX_PARAM_EPINT */
    EpAGap          = CPX_PARAM_EPAGAP, /**< = CPX_PARAM_EPAGAP */
    EpRelax         = CPX_PARAM_EPRELAX, /**< = CPX_PARAM_EPRELAX */
    ObjDif          = CPX_PARAM_OBJDIF, /**< = CPX_PARAM_OBJDIF */
    ObjLLim         = CPX_PARAM_OBJLLIM, /**< = CPX_PARAM_OBJLLIM */
    ObjULim         = CPX_PARAM_OBJULIM, /**< = CPX_PARAM_OBJULIM */
    PolishTime      = CPX_PARAM_POLISHTIME, /**< = CPX_PARAM_POLISHTIME */
    ProbeTime       = CPX_PARAM_PROBETIME, /**< = CPX_PARAM_PROBETIME */
    RelObjDif       = CPX_PARAM_RELOBJDIF, /**< = CPX_PARAM_RELOBJDIF */
    CutsFactor      = CPX_PARAM_CUTSFACTOR, /**< = CPX_PARAM_CUTSFACTOR */
    TreLim          = CPX_PARAM_TRELIM, /**< = CPX_PARAM_TRELIM */
    WorkMem         = CPX_PARAM_WORKMEM, /**< = CPX_PARAM_WORKMEM */

    BarEpComp       = CPX_PARAM_BAREPCOMP, /**< = CPX_PARAM_BAREPCOMP */
    BarQCPEpComp    = CPX_PARAM_BARQCPEPCOMP, /**< = CPX_PARAM_BARQCPEPCOMP */
    BarGrowth       = CPX_PARAM_BARGROWTH, /**< = CPX_PARAM_BARGROWTH */
    BarObjRng       = CPX_PARAM_BAROBJRNG, /**< = CPX_PARAM_BAROBJRNG */
    EpLin           = CPX_PARAM_EPLIN /**< = CPX_PARAM_EPLIN */
  };

  enum BoolParam {
    PreInd          = CPX_PARAM_PREIND,         /**< = CPX_PARAM_PREIND */
    ReverseInd      = CPX_PARAM_REVERSEIND,     /**< = CPX_PARAM_REVERSEIND */
    XXXInd          = CPX_PARAM_XXXIND,         /**< = deprecated */
    MIPOrdInd       = CPX_PARAM_MIPORDIND,      /**< = CPX_PARAM_MIPORDIND */
    MPSLongNum      = CPX_PARAM_MPSLONGNUM,     /**< = CPX_PARAM_MPSLONGNUM */
    LBHeur          = CPX_PARAM_LBHEUR,         /**< = CPX_PARAM_LBHEUR */
    PerInd          = CPX_PARAM_PERIND,         /**< = CPX_PARAM_PERIND */
    PreLinear       = CPX_PARAM_PRELINEAR,      /**< = CPX_PARAM_PRELINEAR   */
    DataCheck       = CPX_PARAM_DATACHECK,      /**< = CPX_PARAM_DATACHECK */
    QPmakePSDInd    = CPX_PARAM_QPMAKEPSDIND,   /**< = CPX_PARAM_QPMAKEPSDIND */
    MemoryEmphasis  = CPX_PARAM_MEMORYEMPHASIS, /**< = CPX_PARAM_MEMORYEMPHASIS */
    NumericalEmphasis = CPX_PARAM_NUMERICALEMPHASIS  /**< = CPX_PARAM_NUMERICALEMPHASIS */
  };

  enum IntParam {
    AdvInd          = CPX_PARAM_ADVIND, /**< = CPX_PARAM_ADVIND */
    RootAlg         = CPX_PARAM_LPMETHOD, /**< = CPX_PARAM_STARTALG, CPX_PARAM_LPMETHOD, CPX_PARAM_QPMETHOD */
    NodeAlg         = CPX_PARAM_SUBALG, /**< = CPX_PARAM_SUBALG */
    MIPEmphasis     = CPX_PARAM_MIPEMPHASIS, /**< = CPX_PARAM_MIPEMPHASIS */
    AggFill         = CPX_PARAM_AGGFILL, /**< = CPX_PARAM_AGGFILL */
    AggInd          = CPX_PARAM_AGGIND, /**< = CPX_PARAM_AGGIND */
    BasInterval     = CPX_PARAM_BASINTERVAL, /**< = CPX_PARAM_BASINTERVAL */
    ClockType       = CPX_PARAM_CLOCKTYPE, /**< = CPX_PARAM_CLOCKTYPE */
    CraInd          = CPX_PARAM_CRAIND, /**< = CPX_PARAM_CRAIND */
    DepInd          = CPX_PARAM_DEPIND, /**< = CPX_PARAM_DEPIND */
    PreDual         = CPX_PARAM_PREDUAL, /**< = CPX_PARAM_PREDUAL */
    PrePass         = CPX_PARAM_PREPASS, /**< = CPX_PARAM_PREPASS */
    RelaxPreInd     = CPX_PARAM_RELAXPREIND, /**< = CPX_PARAM_RELAXPREIND */
    RepeatPresolve  = CPX_PARAM_REPEATPRESOLVE, /**< = CPX_PARAM_REPEATPRESOLVE */
    Symmetry        = CPX_PARAM_SYMMETRY, /**< = CPX_PARAM_SYMMETRY */
    DPriInd         = CPX_PARAM_DPRIIND, /**< = CPX_PARAM_DPRIIND */
    PriceLim        = CPX_PARAM_PRICELIM, /**< = CPX_PARAM_PRICELIM */
    SimDisplay      = CPX_PARAM_SIMDISPLAY, /**< = CPX_PARAM_SIMDISPLAY */
    ItLim           = CPX_PARAM_ITLIM, /**< = CPX_PARAM_ITLIM */
    NetFind         = CPX_PARAM_NETFIND, /**< = CPX_PARAM_NETFIND */
    PerLim          = CPX_PARAM_PERLIM, /**< = CPX_PARAM_PERLIM */
    PPriInd         = CPX_PARAM_PPRIIND, /**< = CPX_PARAM_PPRIIND */
    ReInv           = CPX_PARAM_REINV, /**< = CPX_PARAM_REINV */
    ScaInd          = CPX_PARAM_SCAIND, /**< = CPX_PARAM_SCAIND */
    Threads         = CPX_PARAM_THREADS, /**< = CPX_PARAM_THREADS */
    SingLim         = CPX_PARAM_SINGLIM, /**< = CPX_PARAM_SINGLIM */
    Reduce          = CPX_PARAM_REDUCE, /**< = CPX_PARAM_REDUCE */
    NzReadLim       = CPX_PARAM_NZREADLIM, /**< = CPX_PARAM_NZREADLIM */
    ColReadLim      = CPX_PARAM_COLREADLIM, /**< = CPX_PARAM_COLREADLIM */
    RowReadLim      = CPX_PARAM_ROWREADLIM, /**< = CPX_PARAM_ROWREADLIM */
    QPNzReadLim     = CPX_PARAM_QPNZREADLIM, /**< = CPX_PARAM_QPNZREADLIM */

    SiftDisplay     = CPX_PARAM_SIFTDISPLAY, /**< = CPX_PARAM_SIFTDISPLAY */
    SiftAlg         = CPX_PARAM_SIFTALG, /**< = CPX_PARAM_SIFTALG */
    SiftItLim       = CPX_PARAM_SIFTITLIM, /**< = CPX_PARAM_SIFTITLIM */

    BrDir           = CPX_PARAM_BRDIR, /**< = CPX_PARAM_BRDIR */
    Cliques         = CPX_PARAM_CLIQUES, /**< = CPX_PARAM_CLIQUES */
    CoeRedInd       = CPX_PARAM_COEREDIND, /**< = CPX_PARAM_COEREDIND */
    Covers          = CPX_PARAM_COVERS, /**< = CPX_PARAM_COVERS */
    MIPDisplay      = CPX_PARAM_MIPDISPLAY, /**< = CPX_PARAM_MIPDISPLAY */
    MIPInterval     = CPX_PARAM_MIPINTERVAL, /**< = CPX_PARAM_MIPINTERVAL */
    MIPThreads      = CPX_PARAM_MIPTHREADS, /**< = CPX_PARAM_MIPTHREADS */
    IntSolLim       = CPX_PARAM_INTSOLLIM, /**< = CPX_PARAM_INTSOLLIM */
    NodeFileInd     = CPX_PARAM_NODEFILEIND, /**< = CPX_PARAM_NODEFILEIND */
    NodeLim         = CPX_PARAM_NODELIM, /**< = CPX_PARAM_NODELIM */
    NodeSel         = CPX_PARAM_NODESEL, /**< = CPX_PARAM_NODESEL */
    VarSel          = CPX_PARAM_VARSEL, /**< = CPX_PARAM_VARSEL */
    BndStrenInd     = CPX_PARAM_BNDSTRENIND, /**< = CPX_PARAM_BNDSTRENIND */
    HeurFreq        = CPX_PARAM_HEURFREQ, /**< = CPX_PARAM_HEURFREQ */
    RINSHeur        = CPX_PARAM_RINSHEUR, /**< = CPX_PARAM_RINSHEUR */
    RepairTries     = CPX_PARAM_REPAIRTRIES, /**< = CPX_PARAM_REPAIRTRIES */
    SubMIPNodeLim   = CPX_PARAM_SUBMIPNODELIM, /**< = CPX_PARAM_SUBMIPNODELIM */
    MIPOrdType      = CPX_PARAM_MIPORDTYPE, /**< = CPX_PARAM_MIPORDTYPE */
    BBInterval      = CPX_PARAM_BBINTERVAL, /**< = CPX_PARAM_BBINTERVAL */
    FlowCovers      = CPX_PARAM_FLOWCOVERS, /**< = CPX_PARAM_FLOWCOVERS */
    ImplBd          = CPX_PARAM_IMPLBD, /**< = CPX_PARAM_IMPLBD */
    Probe           = CPX_PARAM_PROBE, /**< = CPX_PARAM_PROBE */
    GUBCovers       = CPX_PARAM_GUBCOVERS, /**< = CPX_PARAM_GUBCOVERS */
    StrongCandLim   = CPX_PARAM_STRONGCANDLIM, /**< = CPX_PARAM_STRONGCANDLIM */
    StrongItLim     = CPX_PARAM_STRONGITLIM, /**< = CPX_PARAM_STRONGITLIM */
    StrongThreadLim = CPX_PARAM_STRONGTHREADLIM, /**< = CPX_PARAM_STRONGTHREADLIM */
    FracCand        = CPX_PARAM_FRACCAND, /**< = CPX_PARAM_FRACCAND */
    FracCuts        = CPX_PARAM_FRACCUTS, /**< = CPX_PARAM_FRACCUTS */
    FracPass        = CPX_PARAM_FRACPASS, /**< = CPX_PARAM_FRACPASS */
    PreslvNd        = CPX_PARAM_PRESLVND, /**< = CPX_PARAM_PRESLVND */
    FlowPaths       = CPX_PARAM_FLOWPATHS, /**< = CPX_PARAM_FLOWPATHS */
    MIRCuts         = CPX_PARAM_MIRCUTS, /**< = CPX_PARAM_MIRCUTS */
    DisjCuts        = CPX_PARAM_DISJCUTS, /**< = CPX_PARAM_DISJCUTS */
    AggCutLim       = CPX_PARAM_AGGCUTLIM, /**< = CPX_PARAM_AGGCUTLIM */
    CutPass         = CPX_PARAM_CUTPASS, /**< = CPX_PARAM_CUTPASS */
    DiveType        = CPX_PARAM_DIVETYPE, /**< = CPX_PARAM_DIVETYPE */

    BarAlg          = CPX_PARAM_BARALG, /**< = CPX_PARAM_BARALG */
    BarColNz        = CPX_PARAM_BARCOLNZ, /**< = CPX_PARAM_BARCOLNZ */
    BarDisplay      = CPX_PARAM_BARDISPLAY, /**< = CPX_PARAM_BARDISPLAY */
    BarItLim        = CPX_PARAM_BARITLIM, /**< = CPX_PARAM_BARITLIM */
    BarMaxCor       = CPX_PARAM_BARMAXCOR, /**< = CPX_PARAM_BARMAXCOR */
    BarOrder        = CPX_PARAM_BARORDER, /**< = CPX_PARAM_BARORDER */
    BarThreads      = CPX_PARAM_BARTHREADS, /**< = CPX_PARAM_BARTHREADS */
    BarCrossAlg     = CPX_PARAM_BARCROSSALG, /**< = CPX_PARAM_BARCROSSALG */
    BarStartAlg     = CPX_PARAM_BARSTARTALG, /**< = CPX_PARAM_BARSTARTALG */

    NetItLim        = CPX_PARAM_NETITLIM, /**< = CPX_PARAM_NETITLIM */
    NetPPriInd      = CPX_PARAM_NETPPRIIND, /**< = CPX_PARAM_NETPPRIIND */
    NetDisplay      = CPX_PARAM_NETDISPLAY, /**< = CPX_PARAM_NETDISPLAY */

    ConflictDisplay = CPX_PARAM_CONFLICTDISPLAY, /**< = CPX_PARAM_CONFLICTDISPLAY */
    FeasOptMode     = CPX_PARAM_FEASOPTMODE /**< = CPX_PARAM_FEASOPTMODE */
  };

  enum StringParam {
    WorkDir         = CPX_PARAM_WORKDIR     /**< = CPX_PARAM_WORKDIR */
  };

  void        setDefaults();

  void        setParam(IntParam    parameter, IloInt value);

  void        setParam(BoolParam   parameter, IloBool value);

  void        setParam(NumParam  parameter, IloNum value);

  void        setParam(StringParam parameter, const char* value);

  IloInt      getParam(IntParam    parameter) const;

  IloBool     getParam(BoolParam   parameter) const;

  IloNum      getParam(NumParam  parameter) const;

  const char* getParam(StringParam parameter) const;

  IloBool     getDefault(BoolParam   parameter) const;

  IloInt      getDefault(IntParam    parameter) const;

  IloNum      getDefault(NumParam  parameter) const;

  const char* getDefault(StringParam parameter) const;

  IloInt      getMin(IntParam    parameter) const;

  IloNum      getMin(NumParam  parameter) const;

  IloInt      getMax(IntParam    parameter) const;

  IloNum      getMax(NumParam  parameter) const;

  class ParameterSetI {
    friend class IloCplexI;
    IloArray<IntParam> _intParams;
    IloIntArray _intParamValues;
    IloArray<BoolParam> _boolParams;
    IloBoolArray _boolParamValues;
    IloArray<NumParam> _numParams;
    IloNumArray _numParamValues;
    IloArray<StringParam> _stringParams;
    IloArray<char*> _stringParamValues;
  public:
    ParameterSetI(IloEnvI* env);
    ~ParameterSetI();
    void clear();
    void setParam(IntParam which, IloInt val);
    void setParam(StringParam  which, char* val);
    void setParam(BoolParam which, IloBool val);
    void setParam(NumParam  which, IloNum val);
    IloInt getParam(IntParam which) const;
    char* getParam(StringParam  which) const;
    IloBool getParam(BoolParam which) const;
    IloNum getParam(NumParam  which) const;
    void applyTo(IloCplexI* cplex);
    void extractFrom(IloCplexI* cplex);
  };

  class ParameterSet {
     ParameterSetI* _impl;
    
  public:
    ParameterSet(IloEnv env) : _impl(0) {
      _impl = new ParameterSetI(env.getImpl());
    }

    ParameterSetI* getImpl() const {return _impl;}

    void clear() {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       _impl->clear();
    }

    void end() {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       delete _impl;
    }

    void setParam(IntParam which, IloInt val) {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       _impl->setParam(which, val);
    }
  
    void setParam(StringParam  which, char* val) {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       _impl->setParam(which, val);
    }
  
    void setParam(BoolParam which, IloBool val) {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       _impl->setParam(which, val);
    }
  
    void setParam(NumParam  which, IloNum val) {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       _impl->setParam(which, val);
    }

    IloInt getParam(IntParam which) const {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       return _impl->getParam(which);
    }
  
    char* getParam(StringParam  which) const {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       return _impl->getParam(which);
    }
   
    IloBool getParam(BoolParam which) const {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       return _impl->getParam(which);
    }
  
    IloNum getParam(NumParam  which) const {
       IloAssert(_impl, "Using empty handle IloCplex::ParameterSet");
       return _impl->getParam(which);
    }

  };


   ParameterSet getParameterSet() {
     IloAssert(getImpl(), "Using empty IloCplex handle.");
     ParameterSet set(getEnv());
     set.getImpl()->extractFrom(getImpl());
     return set;
   }

   void setParameterSet(ParameterSet set) {
      IloAssert(getImpl(), "Using empty IloCplex handle.");
      IloAssert(set.getImpl(), "Using empty IloCplex.ParameterSet handle.");
      set.getImpl()->applyTo(getImpl());
   }
   
  enum MIPEmphasisType {
    MIPEmphasisBalanced    = CPX_MIPEMPHASIS_BALANCED, /**< = CPX_MIPEMPHASIS_BALANCED */
    MIPEmphasisOptimality  = CPX_MIPEMPHASIS_OPTIMALITY, /**< = CPX_MIPEMPHASIS_OPTIMALITY */
    MIPEmphasisFeasibility = CPX_MIPEMPHASIS_FEASIBILITY, /**< = CPX_MIPEMPHASIS_FEASIBILITY */
    MIPEmphasisBestBound   = CPX_MIPEMPHASIS_BESTBOUND, /**< = CPX_MIPEMPHASIS_BESTBOUND */
    MIPEmphasisHiddenFeas  = CPX_MIPEMPHASIS_HIDDENFEAS /**< = CPX_MIPEMPHASIS_HIDDENFEAS */
  };

  enum VariableSelect {
    MinInfeas     = CPX_VARSEL_MININFEAS, /**< = CPX_VARSEL_MININFEAS */
    DefaultVarSel = CPX_VARSEL_DEFAULT, /**< = CPX_VARSEL_DEFAULT */
    MaxInfeas     = CPX_VARSEL_MAXINFEAS, /**< = CPX_VARSEL_MAXINFEAS */
    Pseudo        = CPX_VARSEL_PSEUDO, /**< = CPX_VARSEL_PSEUDO */
    Strong        = CPX_VARSEL_STRONG, /**< = CPX_VARSEL_STRONG */
    PseudoReduced = CPX_VARSEL_PSEUDOREDUCED /**< = CPX_VARSEL_PSEUDOREDUCED */
  };
  enum NodeSelect {
    DFS        = CPX_NODESEL_DFS, /**< = CPX_NODESEL_DFS */
    BestBound  = CPX_NODESEL_BESTBOUND, /**< = CPX_NODESEL_BESTBOUND */
    BestEst    = CPX_NODESEL_BESTEST, /**< = CPX_NODESEL_BESTEST */
    BestEstAlt = CPX_NODESEL_BESTEST_ALT /**< = CPX_NODESEL_BESTEST_ALT */
  };

  enum PrimalPricing {
    PPriIndPartial     = CPX_PPRIIND_PARTIAL, /**< = CPX_PPRIIND_PARTIAL */
    PPriIndAuto        = CPX_PPRIIND_AUTO, /**< = CPX_PPRIIND_AUTO */
    PPriIndDevex       = CPX_PPRIIND_DEVEX, /**< = CPX_PPRIIND_DEVEX */
    PPriIndSteep       = CPX_PPRIIND_STEEP, /**< = CPX_PPRIIND_STEEP */
    PPriIndSteepQStart = CPX_PPRIIND_STEEPQSTART, /**< = CPX_PPRIIND_STEEPQSTART */
    PPriIndFull        = CPX_PPRIIND_FULL /**< = CPX_PPRIIND_FULL */
  };

  enum DualPricing {
    DPriIndAuto        = CPX_DPRIIND_AUTO, /**< = CPX_DPRIIND_AUTO */
    DPriIndFull        = CPX_DPRIIND_FULL, /**< = CPX_DPRIIND_FULL */
    DPriIndSteep       = CPX_DPRIIND_STEEP, /**< = CPX_DPRIIND_STEEP */
    DPriIndFullSteep   = CPX_DPRIIND_FULLSTEEP, /**< = CPX_DPRIIND_FULLSTEEP */
    DPriIndSteepQStart = CPX_DPRIIND_STEEPQSTART,/**< = CPX_DPRIIND_STEEPQSTART */
    DPriIndDevex       = CPX_DPRIIND_DEVEX /**< = CPX_DPRIIND_DEVEX */
  };

  enum BranchDirection {
    BranchGlobal = CPX_BRANCH_GLOBAL, /**< = CPX_BRANCH_GLOBAL */
    BranchDown   = CPX_BRANCH_DOWN, /**< = CPX_BRANCH_DOWN */
    BranchUp     = CPX_BRANCH_UP /**< = CPX_BRANCH_UP */
  };

  typedef IloArray<BranchDirection> BranchDirectionArray;

  enum Algorithm {
    NoAlg      = CPX_ALG_NONE, /**< = CPX_ALG_NONE */
    AutoAlg    = CPX_ALG_AUTOMATIC, /**< = CPX_ALG_AUTOMATIC */
    Primal     = CPX_ALG_PRIMAL, /**< = CPX_ALG_PRIMAL */
    Dual       = CPX_ALG_DUAL, /**< = CPX_ALG_DUAL */ 
    Barrier    = CPX_ALG_BARRIER, /**< = CPX_ALG_BARRIER */
    Sifting    = CPX_ALG_SIFTING, /**< = CPX_ALG_SIFTING */
    Concurrent = CPX_ALG_CONCURRENT, /**< = CPX_ALG_CONCURRENT */
    Network    = CPX_ALG_NET  /**< = CPX_ALG_NET */
  };

  void                setRootAlgorithm (Algorithm alg);
  void                setNodeAlgorithm (Algorithm alg);
  IloCplex::Algorithm getRootAlgorithm() const;
  IloCplex::Algorithm getNodeAlgorithm() const;

  enum Relaxation {
    MinSum  = CPX_FEASOPT_MIN_SUM,     /**< = CPX_FEASOPT_MIN_SUM */
    OptSum  = CPX_FEASOPT_OPT_SUM,     /**< = CPX_FEASOPT_OPT_SUM */
    MinInf  = CPX_FEASOPT_MIN_INF,     /**< = CPX_FEASOPT_MIN_INF */
    OptInf  = CPX_FEASOPT_OPT_INF,     /**< = CPX_FEASOPT_OPT_INF */
    MinQuad = CPX_FEASOPT_MIN_QUAD,    /**< = CPX_FEASOPT_MIN_QUAD */
    OptQuad = CPX_FEASOPT_OPT_QUAD     /**< = CPX_FEASOPT_OPT_QUAD */
  };

  enum CplexStatus {
    Unknown         = 0,
    Optimal         = CPX_STAT_OPTIMAL, /**< = CPX_STAT_OPTIMAL */
    Unbounded       = CPX_STAT_UNBOUNDED, /**< = CPX_STAT_UNBOUNDED */
    Infeasible      = CPX_STAT_INFEASIBLE, /**< = CPX_STAT_INFEASIBLE */
    InfOrUnbd       = CPX_STAT_INForUNBD, /**< = CPX_STAT_INForUNBD */
    OptimalInfeas   = CPX_STAT_OPTIMAL_INFEAS, /**< = CPX_STAT_OPTIMAL_INFEAS */
    NumBest         = CPX_STAT_NUM_BEST, /**< = CPX_STAT_NUM_BEST */
    FeasibleRelaxedSum = CPX_STAT_FEASIBLE_RELAXED_SUM, /**< = CPX_STAT_FEASIBLE_RELAXED_SUM */
    OptimalRelaxedSum  = CPX_STAT_OPTIMAL_RELAXED_SUM, /**< = CPX_STAT_OPTIMAL_RELAXED_SUM */
    FeasibleRelaxedInf = CPX_STAT_FEASIBLE_RELAXED_INF, /**< = CPX_STAT_FEASIBLE_RELAXED_INF */
    OptimalRelaxedInf     = CPX_STAT_OPTIMAL_RELAXED_INF, /**< = CPX_STAT_OPTIMAL_RELAXED_INF */
    FeasibleRelaxedQuad   = CPX_STAT_FEASIBLE_RELAXED_QUAD, /**< = CPX_STAT_FEASIBLE_RELAXED_QUAD */
    OptimalRelaxedQuad    = CPX_STAT_OPTIMAL_RELAXED_QUAD, /**< = CPX_STAT_OPTIMAL_RELAXED_QUAD */
    AbortRelaxed          = CPXMIP_ABORT_RELAXED, /**< = CPXMIP_ABORT_RELAXED */
    AbortObjLim           = CPX_STAT_ABORT_OBJ_LIM, /**< = CPX_STAT_ABORT_OBJ_LIM */
    AbortPrimObjLim       = CPX_STAT_ABORT_PRIM_OBJ_LIM, /**< = CPX_STAT_ABORT_PRIM_OBJ_LIM */
    AbortDualObjLim       = CPX_STAT_ABORT_DUAL_OBJ_LIM, /**< = CPX_STAT_ABORT_DUAL_OBJ_LIM */
    AbortItLim            = CPX_STAT_ABORT_IT_LIM, /**< = CPX_STAT_ABORT_IT_LIM */
    AbortTimeLim          = CPX_STAT_ABORT_TIME_LIM, /**< = CPX_STAT_ABORT_TIME_LIM */
    AbortUser       = CPX_STAT_ABORT_USER, /**< = CPX_STAT_ABORT_USER */

    OptimalFaceUnbounded  = CPX_STAT_OPTIMAL_FACE_UNBOUNDED, /**< = CPX_STAT_OPTIMAL_FACE_UNBOUNDED */

    OptimalTol            = CPXMIP_OPTIMAL_TOL, /**< = CPXMIP_OPTIMAL_TOL */
    SolLim                = CPXMIP_SOL_LIM, /**< = CPXMIP_SOL_LIM */
    NodeLimFeas           = CPXMIP_NODE_LIM_FEAS, /**< = CPXMIP_NODE_LIM_FEAS */
    NodeLimInfeas         = CPXMIP_NODE_LIM_INFEAS, /**< = CPXMIP_NODE_LIM_INFEAS */
    FailFeas              = CPXMIP_FAIL_FEAS, /**< = CPXMIP_FAIL_FEAS */
    FailInfeas            = CPXMIP_FAIL_INFEAS, /**< = CPXMIP_FAIL_INFEAS */
    MemLimFeas            = CPXMIP_MEM_LIM_FEAS, /**< = CPXMIP_MEM_LIM_FEAS */
    MemLimInfeas          = CPXMIP_MEM_LIM_INFEAS, /**< = CPXMIP_MEM_LIM_INFEAS */
    FailFeasNoTree        = CPXMIP_FAIL_FEAS_NO_TREE, /**< = CPXMIP_FAIL_FEAS_NO_TREE */
    FailInfeasNoTree      = CPXMIP_FAIL_INFEAS_NO_TREE, /**< = CPXMIP_FAIL_INFEAS_NO_TREE */
    ConflictFeasible      = CPX_STAT_CONFLICT_FEASIBLE, /**< = CPX_STAT_CONFLICT_FEASIBLE */
    ConflictMinimal       = CPX_STAT_CONFLICT_MINIMAL, /**< = CPX_STAT_CONFLICT_MINIMAL */
    ConflictAbortContradiction = CPX_STAT_CONFLICT_ABORT_CONTRADICTION, /**< = CPX_STAT_CONFLICT_ABORT_CONTRADICTION */
    ConflictAbortTimeLim  = CPX_STAT_CONFLICT_ABORT_TIME_LIM, /**< = CPX_STAT_CONFLICT_ABORT_TIME_LIM */
    ConflictAbortItLim    = CPX_STAT_CONFLICT_ABORT_IT_LIM, /**< = CPX_STAT_CONFLICT_ABORT_IT_LIM */
    ConflictAbortNodeLim  = CPX_STAT_CONFLICT_ABORT_NODE_LIM, /**< = CPX_STAT_CONFLICT_ABORT_NODE_LIM */
    ConflictAbortObjLim   = CPX_STAT_CONFLICT_ABORT_OBJ_LIM, /**< = CPX_STAT_CONFLICT_ABORT_OBJ_LIM */
    ConflictAbortMemLim   = CPX_STAT_CONFLICT_ABORT_MEM_LIM, /**< = CPX_STAT_CONFLICT_ABORT_MEM_LIM */
    ConflictAbortUser     = CPX_STAT_CONFLICT_ABORT_USER /**< = CPX_STAT_CONFLICT_ABORT_USER */
  };
  typedef CplexStatus Status;

  IloAlgorithm::Status getStatus() const;

  CplexStatus     getCplexStatus() const;

  CplexStatus     getCplexSubStatus() const;

  IloBool         isPrimalFeasible() const;

  IloBool         isDualFeasible() const;

  Algorithm       getAlgorithm() const;

  Algorithm       getSubAlgorithm() const;

  enum DeleteMode {
    LeaveBasis, 
    FixBasis
  };

  void       setDeleteMode(DeleteMode mode);

  DeleteMode getDeleteMode() const;

  IloBool feasOpt(const IloRangeArray rngs,  const IloNumArray rnglb,
                                                    const IloNumArray rngub,
                         const IloNumVarArray vars, const IloNumArray varlb,
                                                    const IloNumArray varub);

  IloBool feasOpt(const IloRangeArray rngs,  const IloNumArray rnglb,
                                                    const IloNumArray rngub,
                         const IloIntVarArray vars, const IloNumArray varlb,
                                                    const IloNumArray varub);

  IloBool feasOpt(const IloNumVarArray vars, const IloNumArray varlb,
                                                    const IloNumArray varub);

  IloBool feasOpt(const IloIntVarArray vars, const IloNumArray varlb,
                                                    const IloNumArray varub);

  IloBool feasOpt(const IloRangeArray rngs,  const IloNumArray rnglb,
                                                    const IloNumArray rngub);

  IloBool feasOpt(const IloConstraintArray cts, const IloNumArray prefs);


  enum IISStatus {
     Lower    = CPXIIS_AT_LOWER, /**< = CPXIIS_AT_LOWER */
     Both     = CPXIIS_FIXED, /**< = CPXIIS_FIXED */
     Upper    = CPXIIS_AT_UPPER /**< = CPXIIS_AT_UPPER */
  };

  typedef IloArray<IISStatus> IISStatusArray;

  void getIIS(IISStatusArray cstat, IloNumVarArray var,
                     IISStatusArray rstat, IloConstraintArray  rng) const;

  enum ConflictStatus {
     ConflictExcluded       = CPX_CONFLICT_EXCLUDED,
     ConflictPossibleMember = CPX_CONFLICT_POSSIBLE_MEMBER,
     ConflictMember         = CPX_CONFLICT_MEMBER
  };

  typedef IloArray<ConflictStatus> ConflictStatusArray;

  IloBool refineConflict(IloConstraintArray cons, IloNumArray prefs)
                                                                     const;


  ConflictStatusArray getConflict(IloConstraintArray cons) const;

  ConflictStatus getConflict(IloConstraint con) const;


  void getDnorms(IloNumArray dnorms, const IloNumVarArray vars,
                                     const IloRangeArray  cons) const;
  void getPnorms(IloNumArray cnorms, const IloNumVarArray vars,
                 IloNumArray rnorms, const IloRangeArray  cons) const;
  void setDnorms(const IloNumArray norms, const IloNumVarArray vars,
                                          const IloRangeArray  cons);
  void setPnorms(const IloNumArray cnorms, const IloNumVarArray vars,
                 const IloNumArray rnorms, const IloRangeArray  cons);
  void getDnorms(IloNumArray dnorms, const IloIntVarArray vars,
                                     const IloRangeArray  cons) const;
  void getPnorms(IloNumArray cnorms, const IloIntVarArray vars,
                 IloNumArray rnorms, const IloRangeArray  cons) const;
  void setDnorms(const IloNumArray norms, const IloIntVarArray vars,
                                          const IloRangeArray  cons);
  void setPnorms(const IloNumArray cnorms, const IloIntVarArray vars,
                 const IloNumArray rnorms, const IloRangeArray  cons);

  enum Quality {
    MaxPrimalInfeas         = CPX_MAX_PRIMAL_INFEAS, /**< = CPX_MAX_PRIMAL_INFEAS */
    MaxScaledPrimalInfeas   = CPX_MAX_SCALED_PRIMAL_INFEAS, /**< = CPX_MAX_SCALED_PRIMAL_INFEAS */
    SumPrimalInfeas         = CPX_SUM_PRIMAL_INFEAS, /**< = CPX_SUM_PRIMAL_INFEAS */
    SumScaledPrimalInfeas   = CPX_SUM_SCALED_PRIMAL_INFEAS, /**< = CPX_SUM_SCALED_PRIMAL_INFEAS */
    MaxDualInfeas           = CPX_MAX_DUAL_INFEAS, /**< = CPX_MAX_DUAL_INFEAS */
    MaxScaledDualInfeas     = CPX_MAX_SCALED_DUAL_INFEAS, /**< = CPX_MAX_SCALED_DUAL_INFEAS */
    SumDualInfeas           = CPX_SUM_DUAL_INFEAS, /**< = CPX_SUM_DUAL_INFEAS */
    SumScaledDualInfeas     = CPX_SUM_SCALED_DUAL_INFEAS, /**< = CPX_SUM_SCALED_DUAL_INFEAS */
    MaxIntInfeas            = CPX_MAX_INT_INFEAS, /**< = CPX_MAX_INT_INFEAS */
    SumIntInfeas            = CPX_SUM_INT_INFEAS, /**< = CPX_SUM_INT_INFEAS */
    MaxPrimalResidual       = CPX_MAX_PRIMAL_RESIDUAL, /**< = CPX_MAX_PRIMAL_RESIDUAL */
    MaxScaledPrimalResidual = CPX_MAX_SCALED_PRIMAL_RESIDUAL, /**< = CPX_MAX_SCALED_PRIMAL_RESIDUAL */
    SumPrimalResidual       = CPX_SUM_PRIMAL_RESIDUAL, /**< = CPX_SUM_PRIMAL_RESIDUAL */
    SumScaledPrimalResidual = CPX_SUM_SCALED_PRIMAL_RESIDUAL, /**< = CPX_SUM_SCALED_PRIMAL_RESIDUAL */
    MaxDualResidual         = CPX_MAX_DUAL_RESIDUAL, /**< = CPX_MAX_DUAL_RESIDUAL */
    MaxScaledDualResidual   = CPX_MAX_SCALED_DUAL_RESIDUAL, /**< = CPX_MAX_SCALED_DUAL_RESIDUAL */
    SumDualResidual         = CPX_SUM_DUAL_RESIDUAL, /**< = CPX_SUM_DUAL_RESIDUAL */
    SumScaledDualResidual   = CPX_SUM_SCALED_DUAL_RESIDUAL, /**< = CPX_SUM_SCALED_DUAL_RESIDUAL */
    MaxCompSlack            = CPX_MAX_COMP_SLACK, /**< = CPX_MAX_COMP_SLACK */
    SumCompSlack            = CPX_SUM_COMP_SLACK, /**< = CPX_SUM_COMP_SLACK */
    MaxX                    = CPX_MAX_X, /**< = CPX_MAX_X */
    MaxScaledX              = CPX_MAX_SCALED_X, /**< = CPX_MAX_SCALED_X */
    MaxPi                   = CPX_MAX_PI, /**< = CPX_MAX_PI */
    MaxScaledPi             = CPX_MAX_SCALED_PI, /**< = CPX_MAX_SCALED_PI */
    MaxSlack                = CPX_MAX_SLACK, /**< = CPX_MAX_SLACK */
    MaxScaledSlack          = CPX_MAX_SCALED_SLACK, /**< = CPX_MAX_SCALED_SLACK */
    MaxRedCost              = CPX_MAX_RED_COST, /**< = CPX_MAX_RED_COST */
    MaxScaledRedCost        = CPX_MAX_SCALED_RED_COST, /**< = CPX_MAX_SCALED_RED_COST */
    SumX                    = CPX_SUM_X, /**< = CPX_SUM_X */
    SumScaledX              = CPX_SUM_SCALED_X, /**< = CPX_SUM_SCALED_X */
    SumPi                   = CPX_SUM_PI, /**< = CPX_SUM_PI */
    SumScaledPi             = CPX_SUM_SCALED_PI, /**< = CPX_SUM_SCALED_PI */
    SumSlack                = CPX_SUM_SLACK, /**< = CPX_SUM_SLACK */
    SumScaledSlack          = CPX_SUM_SCALED_SLACK, /**< = CPX_SUM_SCALED_SLACK */
    SumRedCost              = CPX_SUM_RED_COST, /**< = CPX_SUM_RED_COST */
    SumScaledRedCost        = CPX_SUM_SCALED_RED_COST, /**< = CPX_SUM_SCALED_RED_COST */
    Kappa                   = CPX_KAPPA, /**< = CPX_KAPPA */
    ObjGap                  = CPX_OBJ_GAP, /**< = CPX_OBJ_GAP */
    DualObj                 = CPX_DUAL_OBJ, /**< = CPX_DUAL_OBJ */
    PrimalObj               = CPX_PRIMAL_OBJ, /**< = CPX_PRIMAL_OBJ */
    ExactKappa              = CPX_EXACT_KAPPA /**< = CPX_EXACT_KAPPA */
  };

  IloNum getQuality(Quality q, IloNumVar*     var=0,
                               IloConstraint* rng=0) const;

  IloNum getQuality(Quality q, IloConstraint* rng,
                               IloNumVar*     var=0) const;

  void setPriority  (IloNumVar var, IloNum pri);

  void setPriority  (IloIntVar var, IloNum pri);

  void setPriorities(const IloNumVarArray var, const IloNumArray pri);

  void setPriorities(const IloIntVarArray var, const IloNumArray pri);

  void setDirection (IloNumVar var, BranchDirection dir);

  void setDirection (IloIntVar var, BranchDirection dir);

  void setDirections(const IloNumVarArray var,
                            const BranchDirectionArray dir);

  void setDirections(const IloIntVarArray var,
                            const BranchDirectionArray dir);

  void delPriority  (IloNumVar var);

  void delPriority  (IloIntVar var);

  void delPriorities(const IloNumVarArray var);

  void delPriorities(const IloIntVarArray var);

  void delDirection (IloNumVar var);

  void delDirection (IloIntVar var);

  void delDirections(const IloNumVarArray var);

  void delDirections(const IloIntVarArray var);

  IloNum          getPriority  (IloNumVar var) const;

  IloNum          getPriority  (IloIntVar var) const;

  BranchDirection getDirection (IloNumVar var) const;

  BranchDirection getDirection (IloIntVar var) const;

  void            getPriorities(IloNumArray          pri,
                                       const IloNumVarArray var) const;

  void            getPriorities(IloNumArray          pri,
                                       const IloIntVarArray var) const;

  void            getDirections(BranchDirectionArray dir,
                                       const IloNumVarArray var) const;
  void            getDirections(BranchDirectionArray dir,
                                       const IloIntVarArray var) const;

  void basicPresolve(const IloNumVarArray vars,
                            IloNumArray          redlb = 0,
                            IloNumArray          redub = 0,
                            const IloRangeArray  rngs = 0,
                            IloBoolArray         redundant = 0) const;

  void basicPresolve(const IloIntVarArray vars,
                            IloNumArray          redlb = 0,
                            IloNumArray          redub = 0,
                            const IloRangeArray  rngs = 0,
                            IloBoolArray         redundant = 0) const;

  void freePresolve();

  void presolve(IloCplex::Algorithm alg);

  typedef int* Index;
  typedef IloArray<Index> IndexArray;

  // advanced API for use with user-defined extractors.

  void use(IloLPExtractor* extr);

  IloExtractable getDiverging() const;

  void getRay(IloNumArray vals, IloNumVarArray vars) const;

  void getDualNorms(IloNumArray norms, const IloExprArray expr,
                           DualPricing dpriind = DPriIndAuto) const;
  void deleteDnorms();
  void deletePnorms();

  void getDriebeekPenalties(IloNumArray          downratio,
                                   IloNumArray          upratio,
                                   const IloNumVarArray ind) const;
  void getDriebeekPenalties(IloNumArray          downratio,
                                   IloNumArray          upratio,
                                   const IloIntVarArray ind) const;
  void getDriebeekPenalties(IloNumArray          downratio,
                                   IloNumArray          upratio,
                                   const IloRangeArray  ind) const;
  void getStrongBranch(IloNumArray          downratio,
                              IloNumArray          upratio,
                              const IloRangeArray  ind,
                              IloInt               itlim) const;
  void getStrongBranch(IloNumArray          downratio,
                              IloNumArray          upratio,
                              const IloNumVarArray ind,
                              IloInt               itlim) const;
  void getStrongBranch(IloNumArray          downratio,
                              IloNumArray          upratio,
                              const IloIntVarArray ind,
                              IloInt               itlim) const;

  IloNum dualFarkas (IloConstraintArray rng, IloNumArray y);

  void qpIndefCertificate (IloNumVarArray var, IloNumArray x);

  void qpIndefCertificate (IloIntVarArray var, IloNumArray x);

  void protectVariables(const IloNumVarArray var);

  void protectVariables(const IloIntVarArray var);

  class Exception : public IloAlgorithm::Exception {
    IloInt _status;
  public:
    Exception(int status, const char* str)
      : IloAlgorithm::Exception(str), _status(status)      {}
    Exception& operator=(const Exception& cpy) {
      IloException::operator=(cpy);
      _status = cpy.getStatus();
      return *this;
    }

    IloInt getStatus() const                    { return (_status); }
  };

  class MultipleObjException : public Exception {
    IloObjective _obj;
  public:
    MultipleObjException(IloObjective obj)
      : IloCplex::Exception(-1, "MultipleObjException: IloCplex can not handle multiple objectives")
      , _obj(obj)                       {}
    IloObjective getObj() const               { return (_obj); }
  };

  class MultipleConversionException : public Exception {
    IloConversion  _conv;
    IloNumVarArray _var;
  public:
    MultipleConversionException(IloConversion conv,
                                const IloNumVarArray var)
      : IloCplex::Exception(-1,
        "MultipleConversionException: IloCplex can not handle multiple variable type conversions")
      , _conv(conv)
      , _var (var )                                   {}
    void addVar(const IloNumVar var)                  { _var.add(var); }

    IloConversion        getConversion() const        { return (_conv); }

    const IloNumVarArray getVariables() const         { return (_var); }
  };

  class UnknownExtractableException : public Exception {
    IloExtractable _ex;
  public:
    UnknownExtractableException(IloExtractable ex)
    : IloCplex::Exception(-1, "UnknownExtractableException: extractable is not known to IloCplex algorithm")
    , _ex(ex)                                 {}
    IloExtractable getExtractable() const       { return (_ex); }
  };

  class InvalidCutException : public Exception {
    IloConstraint _con;
  public:
    InvalidCutException(IloConstraintI *con=0)
      : IloCplex::Exception(-1, "InvalidCutException: invalid cut")
      , _con(con)
      {}
    IloConstraint getCut() const        { return _con; }
  };

  void flush();

  class Abort {
    friend class IloCplex;
    friend class IloCplexI;
    friend class IloCplexAbortI;
    IloCplexAbortI* _impl;
  public:
    Abort(IloEnv env);
    Abort(const Abort& copy) : _impl(copy._impl) {}
    Abort(IloCplexAbortI* impl) : _impl(impl) {}
    IloCplexAbortI* getImpl() const {return _impl;}
    void      abort();
    IloBool   isAborted() const;
    void      end();
  };

  Abort use(Abort abort);
  void  remove(Abort abort);


  class CallbackI;
  class GoalI;
  class GoalBaseI;

  class Callback {
    CallbackI* _impl;
  public:

    Callback(CallbackI* impl=0) : _impl(impl)   {}

    void end();

    enum Type {
      Presolve          = 0,
      Simplex           = 1,
      Barrier           = 2,
      Crossover         = 3,
      Network           = 4,
      MIP               = 5,
      Probing           = 6,
      FractionalCut     = 7,
      DisjunctiveCut    = 8,
      Branch            = 9,
      Cut               = 10,
      Node              = 11,
      Heuristic         = 12,
      Incumbent         = 13,
      Solve             = 14,
      FlowMIRCut        = 15,
      Continuous        = 16,
      _Number           = 17    // don't document; just name for last one
    };

    Type       getType() const;

    CallbackI* getImpl() const          { return (_impl); }
  };

  class CallbackI {
    friend class Callback;
    friend class IloCplexI;
    friend class IloCallbackManager;
    friend class IloCplexCallbackManager;
    void calldelete();

    CallbackI*          _next;
  public:
    IloCallbackManager* _data;

    IloEnvI* _env;
    void operator delete (void *, size_t);
#if defined (ILODELETEOPERATOR)
    void operator delete(void*, const IloEnvI*) {}
    void operator delete(void*, const IloEnv&) {}
#endif
  protected:
    CallbackI(IloEnv env);

    // documented API
    typedef Callback::Type Type;

    virtual void       main() = 0;   // to be implemented by user-classes

    virtual CallbackI* duplicateCallback () const = 0;


    void     abort();

    IloEnv   getEnv() const { return _env; }

    IloModel getModel() const;

    IloInt   getNcols() const;

    IloInt   getNrows() const;

    IloInt   getNQCs() const;

  public:
    /*
     * <p>This method returns the callback type of the invoking
     * callback object.</p>
     */
    virtual Callback::Type getType() const = 0;
    virtual ~CallbackI();
  };


  class PresolveCallbackI : public CallbackI {
    Callback::Type getType() const;
  protected:

    PresolveCallbackI(IloEnv env) : CallbackI(env) {}

    IloInt getNremovedRows() const;

    IloInt getNremovedCols() const;

    IloInt getNaggregations() const;

    IloInt getNmodifiedCoeffs() const;
  };

  class ContinuousCallbackI : public CallbackI {
    Callback::Type getType() const;
  protected:

    ContinuousCallbackI(IloEnv env) : CallbackI(env) {}

    IloNum  getObjValue() const;

    IloNum  getInfeasibility() const;

    IloNum  getDualInfeasibility() const;

    IloBool isFeasible() const;

    IloBool isDualFeasible() const;

    IloInt  getNiterations() const;
  };
  typedef ContinuousCallbackI LPCallbackI;

  class BarrierCallbackI : public ContinuousCallbackI {
    Callback::Type getType() const;
    BarrierCallbackI(const ContinuousCallbackI& lp) :ContinuousCallbackI(lp) {}
  protected:

    BarrierCallbackI(IloEnv env) : ContinuousCallbackI(env) {}


    IloNum  getDualObjValue() const;
  };

  class SimplexCallbackI : public ContinuousCallbackI {
    Callback::Type getType() const;
    SimplexCallbackI(const ContinuousCallbackI& lp) :ContinuousCallbackI(lp) {}
  protected:

    SimplexCallbackI(IloEnv env) : ContinuousCallbackI(env) {}
  };

  typedef SimplexCallbackI PrimalSimplexCallbackI;
  typedef SimplexCallbackI   DualSimplexCallbackI;

  class CrossoverCallbackI : public CallbackI {
    Callback::Type getType() const;
  protected:
    CrossoverCallbackI(IloEnv env) : CallbackI(env) {}

    IloInt  getNprimalPushes() const;

    IloInt  getNprimalExchanges() const;

    IloInt  getNdualPushes() const;

    IloInt  getNdualExchanges() const;

    IloInt  getNsuperbasics() const;
  };

  class NetworkCallbackI : public CallbackI {
    Callback::Type getType() const;
  protected:
    NetworkCallbackI(IloEnv env) : CallbackI(env) {}

    IloNum  getObjValue() const;

    IloNum  getInfeasibility() const;

    IloBool isFeasible() const;

    IloInt  getNiterations() const;
  };

  class MIPCallbackI : public CallbackI {
    Callback::Type getType() const;
    void doGetIncumbentValues (IloNumArray val,
                               const IloNumVarArray vars) const;
    void doGetObjCoefs(IloNumArray val, const IloNumVarArray vars) const;

  public:
    class NodeData {
      public:
        virtual ~NodeData() {}

        virtual IloAny getDataType() const { return 0; }

    };

    struct NodeId {
      int _id;
      int operator==(const NodeId& nodeid) const {
         return ( nodeid._id == _id);
      }

      int operator!=(const NodeId& nodeid) const {
         return ( nodeid._id != _id);
      }
    };

  protected:
    MIPCallbackI(IloEnv env) : CallbackI(env) {}

    IloNum  getBestObjValue() const;

    IloNum  getIncumbentObjValue() const;

    IloNum  getIncumbentValue(const IloNumVar var) const;

    IloNum  getIncumbentValue(const IloIntVar var) const {
       return  getIncumbentValue(IloNumVar(var.getImpl()));
    }

    void    getIncumbentValues (IloNumArray val,
                                const IloNumVarArray vars) const {
       IloAssert (val.getImpl(), "array is not initialized");
       IloAssert (vars.getImpl(), "array is not initialized");
       doGetIncumbentValues(val, vars);
    }

    void    getIncumbentValues (IloNumArray val,
                                const IloIntVarArray vars) const {
       IloAssert (val.getImpl(), "array is not initialized");
       IloAssert (vars.getImpl(), "array is not initialized");
       doGetIncumbentValues(val, *(IloNumVarArray*)(void*)&vars);
    }

    IloInt  getMyThreadNum() const;

    IloInt  getUserThreads() const;

    IloBool hasIncumbent() const;

    IloInt  getNnodes() const;

    IloInt  getNremainingNodes() const;

    IloInt  getNiterations() const;

    IloNum  getCutoff() const;

    IloInt  getNcliques() const;

    IloInt  getNcovers() const;

    IloInt  getNflowCovers() const;

    IloInt  getNflowPaths() const;

    IloInt  getNGUBcovers() const;

    IloInt  getNfractionalCuts() const;

    IloInt  getNdisjunctiveCuts() const;

    IloInt  getNMIRs() const;

    IloInt  getNimpliedBounds() const;


    BranchDirection getDirection(const IloNumVar var) const;

    BranchDirection getDirection(const IloIntVar var) const {
       return getDirection(IloNumVar(var.getImpl()));
    }

    IloNum  getPriority(const IloNumVar sos) const;

    IloNum  getPriority(const IloIntVar sos) const {
       return  getPriority(IloNumVar(sos.getImpl()));
    }


    IloNum getObjCoef (const IloNumVar var) const;

    IloNum getObjCoef (const IloIntVar var) const {
       return getObjCoef (IloNumVar(var.getImpl()));
    }

    void   getObjCoefs(IloNumArray val, const IloNumVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetObjCoefs(val, vars);
    }

    void   getObjCoefs(IloNumArray val, const IloIntVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetObjCoefs(val, *(IloNumVarArray*)(void*)&vars);
    }

  };

  class ProbingCallbackI : public MIPCallbackI {
    Callback::Type getType() const;
  protected:
    ProbingCallbackI(IloEnv env) : MIPCallbackI(env)  {}

    IloInt  getPhase() const;

    IloNum  getProgress() const;
  };

  class FractionalCutCallbackI : public MIPCallbackI {
    Callback::Type getType() const;
  protected:
    FractionalCutCallbackI(IloEnv env) : MIPCallbackI(env)    {}

    IloNum  getProgress() const;
  };

  class FlowMIRCutCallbackI : public MIPCallbackI {
    Callback::Type getType() const;
  protected:
    FlowMIRCutCallbackI(IloEnv env) : MIPCallbackI(env)       {}

    IloNum  getProgress() const;
  };

  class DisjunctiveCutCallbackI : public MIPCallbackI {
    Callback::Type getType() const;
  protected:
    DisjunctiveCutCallbackI(IloEnv env) : MIPCallbackI(env)   {}

    IloNum  getProgress() const;
  };

  class IncumbentCallbackI : public MIPCallbackI {
    friend class GoalI;
    friend class CpxSolutionGoalI;
    friend class CpxNodeI;

    void doGetValues (IloNumArray val, const IloNumVarArray vars) const;
    Callback::Type getType() const;
  protected:
    IncumbentCallbackI(IloEnv env) : MIPCallbackI(env)     {}

    IloNum getObjValue() const;

    IloNum getValue(const IloExprArg expr) const;

    IloNum getValue(const IloNumVar var) const;

    IloNum getValue(const IloIntVar var) const {
       return getValue(IloNumVar(var.getImpl()));
    }

    IloNum getSlack(const IloRange rng) const;

    void   getSlacks (IloNumArray val, const IloRangeArray con) const;

    NodeId getNodeId() const;

    NodeData  *getNodeData() const;

    void getValues (IloNumArray val, const IloNumVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetValues(val, vars);
    }

    void getValues (IloNumArray val, const IloIntVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetValues(val, *(IloNumVarArray*)(void*)&vars);
    }

    void reject();
  };

  class NodeCallbackI : public MIPCallbackI {
    friend class NodeEvaluatorI;
    Callback::Type getType() const;

  protected:
    NodeCallbackI(IloEnv env) : MIPCallbackI(env)     {}

    NodeId getNodeId(int node) const;

    IloNum getObjValue(int node) const;

    IloNum getEstimatedObjValue(int node) const;

    IloInt getDepth(int node) const;

    IloNum getInfeasibilitySum(int node) const;

    IloInt getNinfeasibilities(int node) const;

    NodeData  *getNodeData(int node) const;

    void selectNode(int node);

    IloNumVar getBranchVar(int node) const;

    IloInt getNodeNumber(NodeId nodeid) const;

    IloNum getObjValue(NodeId nodeid) const;

    IloNum getEstimatedObjValue(NodeId nodeid) const;

    IloInt getDepth(NodeId nodeid) const;

    IloNum getInfeasibilitySum(NodeId nodeid) const;

    IloInt getNinfeasibilities(NodeId nodeid) const;

    NodeData  *getNodeData(NodeId nodeid) const;

    void selectNode(NodeId nodeid);

    IloNumVar getBranchVar(NodeId nodeid) const;
  };

  class ControlCallbackI : public MIPCallbackI {
  public:
    class PresolvedVariableException : public Exception {
      IloNumVarArray _vars;
      void doGetPresolvedVariables(IloNumVarArray vars) const;
    public:
      PresolvedVariableException(IloEnv env)
        : Exception(-2,
       "Callback attempted operation on variable that has been presolved out"),
       _vars(env) {}
      void addVar(const IloNumVar var)                   { _vars.add(var); }

      void getPresolvedVariables(IloNumVarArray vars) const {
        IloAssert(vars.getImpl() != 0,
                  "IloNumVarArray accessed through 0 handle");
        doGetPresolvedVariables(vars);
      }

      void end();
    };

    enum IntegerFeasibility {
      ImpliedInfeasible = -1, /** Not applicable */
      Feasible          =  CPX_INTEGER_FEASIBLE, /**< = CPX_INTEGER_FEASIBLE */
      Infeasible        =  CPX_INTEGER_INFEASIBLE, /**< = CPX_INTEGER_INFEASIBLE */
      ImpliedFeasible   =  CPX_IMPLIED_INTEGER_FEASIBLE /**< = CPX_IMPLIED_INTEGER_FEASIBLE */
    };

    typedef IloArray<IntegerFeasibility> IntegerFeasibilityArray;

  private:
    void doGetFeasibilities(IntegerFeasibilityArray stat,
                            const IloNumVarArray var) const;
    void doGetLbs(IloNumArray val, const IloNumVarArray vars) const;
    void doGetUbs(IloNumArray val, const IloNumVarArray vars) const;
    void doGetValues (IloNumArray val, const IloNumVarArray vars) const;
    void doGetSlacks (IloNumArray val, const IloRangeArray con) const;

  protected:
    ControlCallbackI(IloEnv env) : MIPCallbackI(env)  {}

    IntegerFeasibility getFeasibility(const IloSOS1 sos) const;

    IntegerFeasibility getFeasibility(const IloSOS2 sos) const;

    IntegerFeasibility getFeasibility(const IloNumVar var) const;

    IntegerFeasibility getFeasibility(const IloIntVar var) const {
       return getFeasibility(IloNumVar(var.getImpl()));
    }

    void getFeasibilities(IntegerFeasibilityArray stat,
                          const IloNumVarArray var) const {
      IloAssert (stat.getImpl(), "array is not initialized");
      IloAssert (var.getImpl(), "array is not initialized");
      doGetFeasibilities(stat, var);
    }

    void getFeasibilities(IntegerFeasibilityArray stat,
                          const IloIntVarArray var) const {
      IloAssert (stat.getImpl(), "array is not initialized");
      IloAssert (var.getImpl(), "array is not initialized");
      doGetFeasibilities(stat, *(IloNumVarArray*)(void*)&var);
    }

    IloBool isSOSFeasible(const IloSOS1 sos1) const;

    IloBool isSOSFeasible(const IloSOS2 sos2) const;

    IloNum getLB (const IloNumVar var) const;

    IloNum getLB (const IloIntVar var) const {
       return getLB(IloNumVar(var.getImpl()));
    }

    IloNum getUB (const IloNumVar var) const;

    IloNum getUB (const IloIntVar var) const {
       return getUB(IloNumVar(var.getImpl()));
    }

    void   getLBs(IloNumArray val, const IloNumVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetLbs(val, vars);
    }

    void   getLBs(IloNumArray val, const IloIntVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetLbs(val, *(IloNumVarArray*)(void*)&vars);
    }

    void   getUBs(IloNumArray val, const IloNumVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetUbs(val, vars);
    }

    void   getUBs(IloNumArray val, const IloIntVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetUbs(val, *(IloNumVarArray*)(void*)&vars);
    }

    IloNum getLb (const IloNumVar var) const { return getLB(var); }
    IloNum getUb (const IloNumVar var) const { return getUB(var); }
    void   getLbs(IloNumArray val, const IloNumVarArray vars) const {
      getLBs(val, vars);
    }
    void   getUbs(IloNumArray val, const IloNumVarArray vars) const {
      getUBs(val, vars);
    }

    IloNum getObjValue() const;


    IloNum getValue(const IloExprArg expr) const;

    IloNum getValue(const IloNumVar var) const;

    IloNum getValue(const IloIntVar var) const {
       return getValue(IloNumVar(var.getImpl()));
    }

    IloNum getSlack(const IloRange rng) const;

    void getValues (IloNumArray val, const IloNumVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetValues(val, vars);
    }

    void getValues (IloNumArray val, const IloIntVarArray vars) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      doGetValues(val, *(IloNumVarArray*)(void*)&vars);
    }

    void getSlacks (IloNumArray val, const IloRangeArray con) const {
      IloAssert(val.getImpl(), "array not initialized");
      IloAssert(con.getImpl(), "array not initialized");
      doGetSlacks(val, con);
    }

    IloNum getDownPseudoCost(const IloNumVar var) const;

    IloNum getDownPseudoCost(const IloIntVar var) const {
       return getDownPseudoCost(IloNumVar(var.getImpl()));
    }

    IloNum getUpPseudoCost(const IloNumVar var) const;

    IloNum getUpPseudoCost(const IloIntVar var) const {
       return getUpPseudoCost(IloNumVar(var.getImpl()));
    }

    NodeData *getNodeData() const;
  };

  class BranchCallbackI : public ControlCallbackI {
    friend class GoalI;
    friend class CpxBranchAsCplexI;
    friend class CpxSolutionGoalI;
    friend class CpxNodeI;
    Callback::Type getType() const;

    IloNum doGetBranch (IloNumVarArray       vars,
                        IloNumArray          bounds,
                        BranchDirectionArray dirs,
                        IloInt               i) const;
    NodeId doMakeBranch(const IloConstraintArray        ranges,
                        const IloNumVarArray       var,
                        const IloNumArray          bounds,
                        const BranchDirectionArray dirs,
                        IloNum                     objestimate,
                        NodeData                      *data = 0);
    NodeId doMakeBranch(const IloNumVarArray       var,
                        const IloNumArray          bounds,
                        const BranchDirectionArray dirs,
                        IloNum                     objestimate,
                        NodeData                      *data = 0);
    NodeId doMakeBranch(const IloConstraintArray& rng,
                        IloNum               objestimate,
                        NodeData                *data = 0);

  public:
    enum BranchType {
      BranchOnVariable = CPX_TYPE_VAR, /**< = CPX_TYPE_VAR */
      BranchOnSOS1     = CPX_TYPE_SOS1, /**< = CPX_TYPE_SOS1 */
      BranchOnSOS2     = CPX_TYPE_SOS2, /**< = CPX_TYPE_SOS2 */
      BranchOnAny      = CPX_TYPE_ANY, /**< = CPX_TYPE_ANY */
      UserBranch       = CPX_TYPE_USER  /** (Not Applicable) */ // don't document UserBranch
    };

  protected:
    BranchCallbackI(IloEnv env) : ControlCallbackI(env)       {}

    NodeId     getNodeId() const;

    IloInt     getNbranches() const;

    BranchType getBranchType() const;

    IloNum getBranch(IloNumVarArray       vars,
                     IloNumArray          bounds,
                     BranchDirectionArray dirs,
                     IloInt               i) const {
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(bounds.getImpl(), "array not initialized");
      IloAssert(dirs.getImpl(), "array not initialized");
      return doGetBranch(vars, bounds, dirs, i);
    }

    IloNum getBranch(IloRangeArray cuts, IloInt i) const;

    IloBool isIntegerFeasible() const;

    NodeId makeBranch(const IloNumVarArray       vars,
                      const IloNumArray          bounds,
                      const BranchDirectionArray dirs,
                      IloNum                     objestimate,
                      NodeData                   *data = 0) {
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(bounds.getImpl(), "array not initialized");
      IloAssert(dirs.getImpl(), "array not initialized");
      return (doMakeBranch(vars, bounds, dirs, objestimate, data));
    }

    NodeId makeBranch(const IloIntVarArray       vars,
                      const IloNumArray          bounds,
                      const BranchDirectionArray dirs,
                      IloNum                     objestimate,
                      NodeData                   *data = 0) {
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(bounds.getImpl(), "array not initialized");
      IloAssert(dirs.getImpl(), "array not initialized");
      return (doMakeBranch(*(IloNumVarArray*)(void*)&vars,
                           bounds, dirs, objestimate, data));
    }

    NodeId makeBranch(const IloNumVar var,
                      IloNum          bound,
                      BranchDirection dir,
                      IloNum          objestimate,
                      NodeData        *data = 0);

    NodeId makeBranch(const IloIntVar var,
                      IloNum          bound,
                      BranchDirection dir,
                      IloNum          objestimate,
                      NodeData        *data = 0) {
       return makeBranch(IloNumVar(var.getImpl()),
                         bound, dir, objestimate, data);
    }

    NodeId makeBranch(const IloConstraintArray cons,
                      IloNum                   objestimate,
                      NodeData                 *data = 0) {
      IloAssert(cons.getImpl(), "array not initialized");
      return (doMakeBranch(cons, objestimate, data));
    }

    NodeId makeBranch(const IloConstraint con,
                      IloNum              objestimate,
                      NodeData            *data = 0);

    NodeId makeBranch(const IloConstraintArray   cons,
                      const IloNumVarArray       vars,
                      const IloNumArray          bounds,
                      const BranchDirectionArray dirs,
                      IloNum                     objestimate,
                      NodeData                   *data = 0) {
      IloAssert(cons.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(bounds.getImpl(), "array not initialized");
      IloAssert(dirs.getImpl(), "array not initialized");
      return (doMakeBranch(cons, vars, bounds, dirs, objestimate, data));
    }

    NodeId makeBranch(const IloConstraintArray   cons,
                      const IloIntVarArray       vars,
                      const IloNumArray          bounds,
                      const BranchDirectionArray dirs,
                      IloNum                     objestimate,
                      NodeData                   *data = 0) {
      IloAssert(cons.getImpl(), "array not initialized");
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(bounds.getImpl(), "array not initialized");
      IloAssert(dirs.getImpl(), "array not initialized");
      return (doMakeBranch(cons, *(IloNumVarArray*)(void*)&vars,
                           bounds, dirs, objestimate, data));
    }

    void prune();
  };

  class HeuristicCallbackI : public ControlCallbackI {
    Callback::Type getType() const;
    void setSolution(const IloNumVarArray vars, const IloNumArray vals,
                     IloNum objval, IloBool set);
  protected:
    HeuristicCallbackI(IloEnv env) : ControlCallbackI(env)     {}

    void setBounds(const IloNumVar var, IloNum lb, IloNum ub);

    void setBounds(const IloIntVar var, IloNum lb, IloNum ub) {
       setBounds(IloNumVar(var.getImpl()), lb, ub);
    }

    void setBounds(const IloNumVarArray var, const IloNumArray lb,
                                             const IloNumArray ub);

    void setBounds(const IloIntVarArray var, const IloNumArray lb,
                                             const IloNumArray ub) {
       setBounds (*(IloNumVarArray*)&var, lb, ub);
    }

    IloBool              solve(Algorithm alg = Dual);

    IloAlgorithm::Status getStatus() const;

    CplexStatus          getCplexStatus() const;

    IloBool              isPrimalFeasible() const;

    IloBool              isDualFeasible() const;

    void setSolution(const IloNumVarArray vars, const IloNumArray vals) {
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(vals.getImpl(), "array not initialized");
      setSolution(vars, vals, 0.0, IloFalse);
    }

    void setSolution(const IloNumVarArray vars, const IloNumArray vals,
                     IloNum obj) {
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(vals.getImpl(), "array not initialized");
      setSolution(vars, vals, obj, IloTrue);
    }

    void setSolution(const IloIntVarArray vars, const IloNumArray vals) {
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(vals.getImpl(), "array not initialized");
      setSolution(*(IloNumVarArray*)(void*)&vars, vals, 0.0, IloFalse);
    }

    void setSolution(const IloIntVarArray vars, const IloNumArray vals,
                     IloNum obj) {
      IloAssert(vars.getImpl(), "array not initialized");
      IloAssert(vals.getImpl(), "array not initialized");
      setSolution(*(IloNumVarArray*)(void*)&vars, vals, obj, IloTrue);
    }
  };


  class SolveCallbackI : public ControlCallbackI {
    Callback::Type getType() const;
    void doSetVectors(const IloNumArray    x,
                      const IloNumVarArray var,
                      const IloNumArray    pi,
                      const IloRangeArray  rng);
  protected:
    SolveCallbackI(IloEnv env) : ControlCallbackI(env)        {}

    IloBool solve(Algorithm alg = Dual);

    IloAlgorithm::Status getStatus() const;

    CplexStatus          getCplexStatus() const;

    IloBool              isPrimalFeasible() const;

    IloBool              isDualFeasible() const;

    void useSolution();

    void setVectors(const IloNumArray    x,
                    const IloNumVarArray var,
                    const IloNumArray    pi,
                    const IloRangeArray  rng) {
      IloAssert(var.getImpl() == 0 || var.getSize() ==  x.getSize(),
                "Array lengths don't match");
      IloAssert(rng.getImpl() == 0 || rng.getSize() == pi.getSize(),
                "Array lengths don't match");
      doSetVectors(x, var, pi, rng);
    }

    void setVectors(const IloNumArray    x,
                    const IloIntVarArray var,
                    const IloNumArray    pi,
                    const IloRangeArray  rng) {
      IloAssert(var.getImpl() == 0 || var.getSize() ==  x.getSize(),
                "Array lengths don't match");
      IloAssert(rng.getImpl() == 0 || rng.getSize() == pi.getSize(),
                "Array lengths don't match");
      doSetVectors(x, *(IloNumVarArray*)(void*)&var, pi, rng);
    }
  };

  class CutCallbackI : public ControlCallbackI {
    Callback::Type getType() const;
  protected:
    friend class IloCplexCallbackManager;
    IloBool _lazy;
    CutCallbackI(IloEnv env) : ControlCallbackI(env), _lazy(IloTrue)  {}

    IloConstraint add(IloConstraint con);

    IloConstraint addLocal(IloConstraint con);
  };

  class UserCutCallbackI : public CutCallbackI {
  protected:
    UserCutCallbackI(IloEnv env) : CutCallbackI(env) { _lazy = IloFalse; }
  };

  class LazyConstraintCallbackI : public CutCallbackI {
  protected:
    LazyConstraintCallbackI(IloEnv env) : CutCallbackI(env) {}
  };

  Callback use(Callback cb);
  Callback add(Callback cb);
  void     remove(Callback cb);
  void     clear(Callback::Type cbtype);


  class Goal {
  public:
    Goal(GoalBaseI *goalI);

    Goal(const Goal& goal);

    Goal() : _impl(0) {}

    Goal(IloConstraint cut);

    Goal(IloConstraintArray cut);

    Goal operator=(const Goal& goal);

    GoalBaseI *getImpl() const { return _impl; }

    ~Goal();

  protected:
    GoalBaseI *_impl;
  };

  class GoalBaseI {
  protected:
    IloInt    _refCnt;
    IloEnv    _env;
    CpxNodeI* _node;

    IloConstraint copyConstraint(IloConstraint rng);
  public:
    enum GoalType {
      Or,
      And,
      Branch,
      User
    };

    GoalBaseI(IloEnv env);

    Goal executeNode(CpxNodeI *node);

    virtual Goal execute() = 0;
    virtual Goal duplicateGoal() = 0;

    IloEnv getEnv() const { return _env; }

    virtual GoalType getType() const;

    void addRef() { _refCnt++; }
    void removeRef() { _refCnt--; }
    IloInt getNRefs() const { return _refCnt; }

    void setNode(CpxNodeI *node) { _node = node; }
    CpxNodeI *getNode() const { return _node; }

    virtual ~GoalBaseI();
  };

  class GoalI : public GoalBaseI {
  public:
    typedef struct MIPCallbackI::NodeId NodeId;

    enum IntegerFeasibility {
      ImpliedInfeasible = -1, /** Not applicable */
      Feasible          =  CPX_INTEGER_FEASIBLE, /**< = CPX_INTEGER_FEASIBLE */
      Infeasible        =  CPX_INTEGER_INFEASIBLE, /**< = CPX_INTEGER_INFEASIBLE */
      ImpliedFeasible   =  CPX_IMPLIED_INTEGER_FEASIBLE /**< = CPX_IMPLIED_INTEGER_FEASIBLE */
    };

    typedef IloArray<IntegerFeasibility> IntegerFeasibilityArray;

    enum BranchType {
      BranchOnVariable = CPX_TYPE_VAR, /**< = CPX_TYPE_VAR */
      BranchOnSOS1     = CPX_TYPE_SOS1, /**< = CPX_TYPE_SOS1 */
      BranchOnSOS2     = CPX_TYPE_SOS2, /**< = CPX_TYPE_SOS2 */
      BranchOnAny      = CPX_TYPE_ANY, /**< = CPX_TYPE_ANY */
      UserBranch       = CPX_TYPE_USER /** (Not Applicable) */ // don't document UserBranch
    };

    GoalI(IloEnv env) : GoalBaseI(env) {};

    IloEnv getEnv() const { return _env; }

    virtual Goal execute() = 0;

    virtual Goal duplicateGoal() = 0;

    virtual GoalType getType() const;

    void abort();

    IloModel getModel() const;

    NodeId getNodeId() const;

    IloInt getNcols() const;

    IloInt getNrows() const;

    IloNum getBestObjValue() const;

    IloNum getCutoff() const;

    BranchDirection getDirection(const IloNumVar var);

    BranchDirection getDirection(const IloIntVar var) {
       return getDirection(IloNumVar(var.getImpl()));
    }

    IloNum getIncumbentObjValue() const;

    IloNum getIncumbentValue(const IloNumVar var) const;

    IloNum getIncumbentValue(const IloIntVar var) const {
       return getIncumbentValue(IloNumVar(var.getImpl()));
    }

    void getIncumbentValues (IloNumArray val,
                             const IloNumVarArray vars) const;

    void getIncumbentValues (IloNumArray val,
                             const IloIntVarArray vars) const {
       getIncumbentValues (val, *(IloNumVarArray*)&vars);
    }

    IloInt getMyThreadNum() const;

    IloInt getUserThreads() const;

    IloInt getNcliques() const;

    IloInt getNcovers() const;

    IloInt getNdisjunctiveCuts() const;

    IloInt getNflowCovers() const;

    IloInt getNflowPaths() const;

    IloInt getNfractionalCuts() const;

    IloInt getNGUBcovers() const;

    IloInt getNimpliedBounds() const;

    IloInt getNMIRs() const;

    IloInt getNiterations() const;

    IloInt getNnodes() const;

    IloInt getNremainingNodes() const;

    IloNum getPriority(const IloNumVar var) const;

    IloNum getPriority(const IloIntVar var) const {
       return getPriority(IloNumVar(var.getImpl()));
    }

    IloBool isIntegerFeasible() const;

    IloBool hasIncumbent() const;

     IloNum getBranch(IloNumVarArray vars,
                      IloNumArray bounds,
                      BranchDirectionArray dirs,
                      IloInt i) const;

    BranchType getBranchType() const;

    IloInt getNbranches() const;

    IloNum getDownPseudoCost(const IloNumVar var) const;

    IloNum getDownPseudoCost(const IloIntVar var) const {
       return getDownPseudoCost(IloNumVar(var.getImpl()));
    }

    IntegerFeasibility getFeasibility(const IloNumVar var) const;

    IntegerFeasibility getFeasibility(const IloIntVar var) const {
       return getFeasibility(IloNumVar(var.getImpl()));
    }

    IntegerFeasibility getFeasibility(const IloSOS1 sos) const;

    IntegerFeasibility getFeasibility(const IloSOS2 sos) const;

    void getFeasibilities(IntegerFeasibilityArray stats,
                          const IloNumVarArray vars) const;

    void getFeasibilities(IntegerFeasibilityArray stats,
                          const IloIntVarArray vars) const {
       getFeasibilities(stats, *(IloNumVarArray*)&vars);
    }

    IloNum getLB (const IloNumVar var) const;

    IloNum getLB (const IloIntVar var) const {
       return getLB (IloNumVar(var.getImpl()));
    }

    void getLBs(IloNumArray vals, const IloNumVarArray vars) const;

    void getLBs(IloNumArray vals, const IloIntVarArray vars) const {
       getLBs(vals, *(IloNumVarArray*)&vars);
    }

    IloNum getLb (const IloNumVar var) const    { return getLB(var); }
    void getLbs(IloNumArray vals, const IloNumVarArray vars) const {
       getLBs(vals, vars);
    }

    IloNum getObjCoef (const IloNumVar var) const;

    IloNum getObjCoef (const IloIntVar var) const {
       return getObjCoef (IloNumVar(var.getImpl()));
    }

    void getObjCoefs(IloNumArray vals,
                     const IloNumVarArray vars) const;

    void getObjCoefs(IloNumArray vals,
                     const IloIntVarArray vars) const {
       getObjCoefs(vals, *(IloNumVarArray*)&vars);
    }

    IloNum getObjValue() const;

    IloNum getSlack(const IloRange rng) const;

    void getSlacks(IloNumArray vals, const IloRangeArray rngs) const;

    IloNum getUB(const IloNumVar var) const;

    IloNum getUB(const IloIntVar var) const {
       return getUB(IloNumVar(var.getImpl()));
    }

    void getUBs(IloNumArray vals, const IloNumVarArray vars) const;

    void getUBs(IloNumArray vals, const IloIntVarArray vars) const {
       getUBs(vals, *(IloNumVarArray*)&vars);
    }

    IloNum getUb (const IloNumVar var) const    { return getUB(var); }
    void getUbs(IloNumArray vals, const IloNumVarArray vars) const {
       getUBs(vals, vars);
    }

    IloNum getUpPseudoCost(const IloNumVar var) const;

    IloNum getUpPseudoCost(const IloIntVar var) const {
       return getUpPseudoCost(IloNumVar(var.getImpl()));
    }

    IloNum getValue(const IloExpr expr) const;

    IloNum getValue(const IloNumVar var) const;

    IloNum getValue(const IloIntVar var) const {
       return getValue(IloNumVar(var.getImpl()));
    }

    void getValues(IloNumArray vals, const IloNumVarArray vars) const;

    void getValues(IloNumArray vals, const IloIntVarArray vars) const {
       getValues(vals, *(IloNumVarArray*)&vars);
    }

    IloBool isSOSFeasible(const IloSOS1 sos1) const;

    IloBool isSOSFeasible(const IloSOS2 sos2) const;

    static Goal BranchAsCplexGoal(IloEnv env);

    static Goal OrGoal(Goal goal1, Goal goal2);

    static Goal OrGoal(Goal goal1, Goal goal2, Goal goal3);
    static Goal OrGoal(Goal goal1, Goal goal2,
                       Goal goal3, Goal goal4);
    static Goal OrGoal(Goal goal1, Goal goal2, Goal goal3,
                       Goal goal4, Goal goal5);
    static Goal OrGoal(Goal goal1, Goal goal2, Goal goal3,
                       Goal goal4, Goal goal5, Goal goal6);

    static Goal AndGoal(Goal goal1, Goal goal2);
    static Goal AndGoal(Goal goal1, Goal goal2, Goal goal3);
    static Goal AndGoal(Goal goal1, Goal goal2,
                        Goal goal3, Goal goal4);
    static Goal AndGoal(Goal goal1, Goal goal2, Goal goal3,
                        Goal goal4, Goal goal5);
    static Goal AndGoal(Goal goal1, Goal goal2, Goal goal3,
                        Goal goal4, Goal goal5, Goal goal6);

    static Goal FailGoal(IloEnv env);

    static Goal GlobalCutGoal(IloConstraint con);

    static Goal GlobalCutGoal(IloConstraintArray con);

    static Goal SolutionGoal(const IloNumVarArray vars,
                             const IloNumArray vals);
    static Goal SolutionGoal(const IloNumVarArray vars,
                             const IloNumArray vals,
                             IloNum obj);

    static Goal SolutionGoal(const IloIntVarArray vars,
                             const IloNumArray vals);
    static Goal SolutionGoal(const IloIntVarArray vars,
                             const IloNumArray vals,
                             IloNum obj);

  };

  class Node {
  public:
    Node() : _impl(0) {}
    Node(CpxNodeI *impl) : _impl(impl) {}

    CpxNodeI *getImpl() const { return _impl; }

    IloInt getLeftDepth() const;
    IloInt getRightDepth() const;
    IloInt getDepth()  const;
  protected:
    CpxNodeI *_impl;
  };

  class EvaluatorI {
  public:
    EvaluatorI() : _node(0) {}

    void setNode(CpxNodeI *node) { _node = node; }
    virtual ~EvaluatorI();

  protected:
    IloInt getLeftDepth() const;
    IloInt getRightDepth() const;
    IloNum getDepth()  const;

    CpxNodeI *_node;
  };

  class NodeEvaluatorI : public EvaluatorI {
  public:
    NodeEvaluatorI () : _refCnt(0),
                        _id(-1),
                        _isInit(IloFalse) {}

    virtual IloNum evaluate() const = 0;

    virtual NodeEvaluatorI* duplicateEvaluator() = 0;

    virtual IloBool subsume(IloNum evalBest,
                            IloNum evalCurrent) const;

    virtual void    init();

    IloBool isInit() { return _isInit; }

    void addRef() { _refCnt++; }
    void removeRef() { _refCnt--; }
    IloInt getNRefs() const { return _refCnt; }

    IloNum evaluateNode(CpxNodeI *node, IloInt nIndex);

    NodeCallbackI *getCallback() const;

    void initNode(CpxNodeI *node);
    virtual NodeEvaluatorI* duplicateEvaluatorWithId();
    IloInt getId() const { return _id; }
    void  setId(IloInt id) { _id = id; }


    virtual ~NodeEvaluatorI();

  protected:
    IloNumVar getBranchVar() const;

    IloNum getDepth() const;

    IloNum getEstimatedObjValue() const;

    IloNum getInfeasibilitySum() const;

    IloInt getNinfeasibilities() const;

    GoalI::NodeId getNodeId() const;

    IloNum getObjValue() const;

    IloInt    _refCnt;
    IloInt    _nIndex;

    IloInt    _id;

    IloBool   _isInit;
  };

  class NodeEvaluator {
  protected:

    NodeEvaluatorI *_impl;

  public:
    NodeEvaluator ();

    NodeEvaluator (NodeEvaluatorI *impl);

    NodeEvaluator (const NodeEvaluator& eval);

    NodeEvaluator operator=(const NodeEvaluator& eval);

    NodeEvaluatorI *getImpl() const { return _impl; }

    ~NodeEvaluator();
  };

  class SearchLimitI : public EvaluatorI {
  public:
    SearchLimitI();

    virtual ~SearchLimitI();

    virtual SearchLimitI* duplicateLimit()=0;

    virtual IloBool check() = 0;

    virtual void init();

    // END:: Documented API

    virtual IloBool checkNode(CpxNodeI *node);
    virtual void initNode(CpxNodeI *node);

    void setViolated() { _isViolated = IloTrue; }
    IloBool isViolated() const { return _isViolated; }

    void addRef() { _refCnt++; }
    void removeRef() { _refCnt--; }
    IloInt getNRefs() const { return _refCnt; }
  protected:
    IloInt _refCnt;
    IloBool _isViolated;
  };

  class SearchLimit {
  protected:
    SearchLimitI *_impl;

  public:
    SearchLimit();

    SearchLimit(SearchLimitI* impl);

    SearchLimit(const SearchLimit& limit);

    SearchLimit operator=(const SearchLimit& limit);


    SearchLimitI *getImpl() const { return _impl; }

    ~SearchLimit();
  };

  static Goal Apply ( IloCplex cplex, Goal goal, NodeEvaluator eval );

  static Goal LimitSearch ( IloCplex cplex, Goal goal, SearchLimit limit );

/* To be implemented:
 *
 * static NodeEvaluator DFSEvaluator();
 * static NodeEvaluator IDFSEvaluator(IloCplex cplex);
 * static NodeEvaluator DDSEvaluator(IloCplex);
 *
 * static SearchLimit FailLimit(IloCplex cplex, IloInt failLimit);
 */

  /* We are not exporting this in this release: */
  static SearchLimit OrLimit(IloInt orLimit);

  IloBool solve() { return IloAlgorithm::solve(); }

  IloBool solve(Goal goal);
  // END: For the Goal API

  /* 
   * @show internal 
   * no longer documented: setFormulationEpsValue  is deprecated
   * @include-file ilocplex.h
   * @group optim.cplex.cpp
   * @group optim.cplex.cpp.piecewiselinear
   *
   * <p>
   * This method sets the epsilon value that is used to model strict
   * relations.  In order to model a strict inequality
   * such as <code>x &lt; b</code> in a MIP, <code>IloCplex</code> uses 
   * <code>x &lt;= b - eps</code> as a surrogate, where <code>eps</code>
   * is the value controlled by this method.
   * </p>
   */
  void   setFormulationEpsValue(IloNum eps) {
     setParam(EpLin, eps);
  }

  /* 
   * @show internal
   * no longer documented: getFormulationEpsValue  is deprecated
   * @include-file ilocplex.h
   * @group optim.cplex.cpp
   * @group optim.cplex.cpp.piecewiselinear
   *
   * <p>
   * This method returns the epsilon value that is used to model strict
   * relations.  In order to model a strict inequality, 
   * such as <code>x &lt; b</code> in a MIP, <code>IloCplex</code> uses
   * <code>x &lt;= b - eps</code> as a surrogate, where <code>eps</code>
   * is the value returned by this method.
   * </p>
   */
  IloNum getFormulationEpsValue() {
     return getParam(EpLin);
  }
};


class IloCplexIndexManager;
extern "C" {
void CPXPUBLIC IloCplexErrorChannelFunction(void* arg, const char *message);
const char* CPXPUBLIC rowName (void* data, int ij, char* nameBuf);
const char* CPXPUBLIC colName (void* data, int ij, char* nameBuf);
const char* CPXPUBLIC quadName(void* data, int ij, char* nameBuf);
const char* CPXPUBLIC sosName (void* data, int ij, char* nameBuf);
}

class IloCplexMutex;

class IloCplexI : public IloAlgorithmI {
public:
  // For the Goal API
  class GoalStack;
  // END: For the Goal API
  typedef IloCplex::Index      Index;
  typedef IloCplex::IndexArray IndexArray;

  IloBool inconsistency(int i);
  IloBool isSelfConsistent();
  IloBool isConsistent(IloCplexI* copy);
  IloBool isConsistent();

  IloExprParser* popExprParser();
  void           pushExprParser(IloExprParser*);

private:
  friend class IloCplex;
  friend class IloCplexColumn;
  friend class IloFastCplexColumn;
  friend class IloCplexRow;
  friend class IloCplexSOS;
  friend class IloFastCplexSOS;
  friend class IloLPExtractor;
  friend class IloCallbackManager;
  friend class IloCutCallbackManager;
  friend class IloIncumbentCallbackManager;
  friend class IloControlCallbackManager;
  friend class IloHeuristicCallbackManager;
  friend class IloSolveCallbackManager;
  friend class IloCplexCallbackManager;
  friend class IloDefaultLPExtractor;
  friend class IloLogicalExtractor;
  friend class IloSetConstraintExtractor;
  friend void CPXPUBLIC IloCplexErrorChannelFunction(void* arg,
                                                     const char *message);

  IloDefaultLPExtractor*      _defextr;
  IloLogicalExtractor*        _logextr;
  IloSetConstraintExtractor*  _setextr;
  IloColumnExtractor*         _colextr;
  IloExprParser*              _qpextr;
  IloLPExtractorManager*      _extractorManager;
  IloCplexCallbackManager*    _callbacks;

  IloCplexIndexManager* _rowind;
  IloCplexIndexManager* _colind;
  IloCplexIndexManager* _qcpind;
  IloCplexIndexManager* _sosind;
  IloCplexIndexManager* _gcind[CPX_CON_LAST_CONTYPE];
  IloCarray<int>        _gctype;

  char _tmpname[256];
  friend const char* CPXPUBLIC rowName (void* data, int ij, char *nameBuf);
  friend const char* CPXPUBLIC colName (void* data, int ij, char *nameBuf);
  friend const char* CPXPUBLIC quadName(void* data, int ij, char *nameBuf);
  friend const char* CPXPUBLIC sosName (void* data, int ij, char *nameBuf);

  cpxenv* _env;
  cpxlp*  _lp;
  IloCplex::Exception _constructorException;

  IloBool              _solving;
  const IloObjectiveI* _obj;

  IloBool _datacheck;   // cached copy of parameter for faster access
public:
  IloBool isDataCheck() const { return _datacheck; }
  static void alwaysCheckNAN(double val) {
     if ( val != val ) {
        throw IloCplex::Exception(CPXERR_NAN,
                "Numeric entry is not a double precison number (NAN)");
     }
  }
  void checkNAN(double val) const {
     if ( _datacheck  &&  val != val ) {
        throw IloCplex::Exception(CPXERR_NAN,
                "Numeric entry is not a double precison number (NAN)");
     }
  }

  // The deleter
  void setEmptyDeleter(IloBool remove=IloFalse);
  void unsetDeleter();

  void replaceObject (Index index, const IloExtractableI* src,
                                   const IloExtractableI* dest);

private:
  char _errorstring[CPX_STR_PARAM_MAX];
  char _strparam[CPX_STR_PARAM_MAX];

  IloCplex::DeleteMode _delmode;

  IloCarray<int>    _tmpi;
  IloCarray<char>   _tmpc;
  IloCarray<double> _tmpd;
  void minTmpSize(int min) const {
    ((IloCplexI*)this)->_tmpi.setMinSize(min);
    ((IloCplexI*)this)->_tmpc.setMinSize(min);
    ((IloCplexI*)this)->_tmpd.setMinSize(min);
  }

  void initSolutionManager(IloEnvI* env);
  void finitSolutionManager();
  void init();

  void copyIndices(IloCarray<int>& col, const IndexArray& ind);

  void fixProbType();
  ::IloCplex::CplexStatus handleSolveStatus(int solstat) const;
  ::IloAlgorithm::Status CplexToAlgorithmStatus(int solstat, int pfeas,
                                                             int dfeas) const;
  void cpxthrow(int status) const;

  // input caches
  IloCplexCache* _cache;

  IloCachedArray<char> _ctype;
  IloCachedArray<char> _convtype;
  void flushCtype();
  void removeFromCtype(Index col);
  void removeFromCtype(const IndexArray cols, IloCarray<int>& mark);

  IloCachedArray<char> _sostype;
  IloSVectorArray*     _soscache;
  void flushSOS();
  void removeFromSOS(Index col);
  void removeFromSOS(const IndexArray cols, IloCarray<int>& mark);

  IloBool         _ordchanged;
  IloCarray<int>  _ordpri;
  IloCarray<int>  _orddir;
  void flushOrder();
  void unflushOrder();
  void removeFromOrder(Index col);
  void removeFromOrder(const IndexArray cols, IloCarray<int>& mark);

  void markColumns(const IndexArray cols, IloCarray<int>& mark);
  
  void doflush();
  void flush() const;

  // query caches
  int _nInts;
  int _nCols;
  int _nRows;
  int _nQCs;
  IloBool _hasQ;
  IloBool hasUserCuts() const;
  IloBool hasLazyConstraints() const;
  IloBool hasGeneralConstraints() const;

  IloCachedArray<double> _lb;
  IloCachedArray<double> _ub;

  // result caches
  IloCplex::CplexStatus  _solstat;
  IloCachedArray<double> _qcpslack;
  IloCachedArray<double> _slack;
  IloCachedArray<double> _x;
  IloCachedArray<double> _pi;
  IloCachedArray<double> _dj;
  IloCachedArray<IloCplex::BasisStatus> _rstat;
  IloCachedArray<IloCplex::BasisStatus> _cstat;

  void invalidate() const;
  void validateX() const;
  void validateLb() const;
  void validateUb() const;
  void validatePi() const;
  void validateDj() const;
  void validateSlack() const;
  void validateQCSlack() const;
  void validateRowStatus() const;
  void validateColumnStatus() const;

  // For the Goal API
  CpxNodeI         *_rootNode;
  IloCplex::Goal    _rootGoal;

  IloBool           _usesEval;
  IloInt            _evalId;

  IloArray<IloConstraintArray> _globalCuts;
  IloCplexMutex**              _gCutMutex;
  IloCplexMutex*               _idMutex;

  IloCplex::BranchCallbackI           *_branchCallback;
  IloCplex::NodeCallbackI             *_nodeCallback;
  IloCplex::IncumbentCallbackI        *_incCallback;
  IloCplex::LazyConstraintCallbackI   *_lcCallback;
  void    setRootNode(CpxNodeI *node) { _rootNode = node; }
  void    initGoals(IloBool& prevInSubtree, IloBool& prevMIPStart);
  void    finitGoals(IloBool insubtree, IloBool mipstart);
  // END: For the Goal API


  IloBool doSolve(IloBool isLP=IloTrue);

public:
  ~IloCplexI();
  IloCplexI(IloEnvI* env);

  // Every extractable extracted by to IloCplex must store a data object
  // which is a derived class of Data.  In this object index must be the
  // index of the column, row, qcprow, or SOS created to represent the
  // extracted expression.
  struct Data {
    Index index;
    Data()                      {}
    Data(Index i) : index(i)    {}

    Index getIndex()        { return index; }
    void  setIndex(Index i) { index = i; } 
  };

  void uses() {}
  void getData() {}
  void setData() {}
  Index throwNotExtracted(const IloExtractableI* obj) const;

  Index getVarIndex (int id) const;
  Index getVarIndexOrException (const IloExtractableI* obj) const;

  void getVarIndices           (IndexArray ind,
                                const IloExtractableArray& obj) const;
  void getVarIndicesOrException(IndexArray ind,
                                const IloExtractableArray& obj) const;

  void getVarIndices (IndexArray& ind, const IloNumVarArray& obj) const {
     getVarIndices(ind, *(const IloExtractableArray*)&obj);
  }
  void getVarIndicesOrException (IndexArray&           ind,
                                 const IloNumVarArray& obj) const {
     getVarIndicesOrException(ind, *(const IloExtractableArray*)&obj);
  }

  Index getConIndex (int id) const;
  Index getConIndexOrException (const IloExtractableI* obj) const;

  void getConIndices           (IndexArray                 ind,
                                const IloExtractableArray& obj) const;
  void getConIndicesOrException(IndexArray                 ind,
                                const IloExtractableArray& obj) const;

  void getConIndices (IndexArray& ind, const IloRangeArray& obj) const {
     getConIndices(ind, *(const IloExtractableArray*)&obj);
  }
  void getConIndicesOrException (IndexArray&          ind,
                                 const IloRangeArray& obj) const {
     getConIndicesOrException(ind, *(const IloExtractableArray*)&obj);
  }
  int getGCType(const IloExtractableI* obj) const;

  const IloCplexIndexManager* getColIndexManager() { return (_colind); }

  const IloObjectiveI* getObjective() const                    { return _obj; }
  void                 setObjective(const IloObjectiveI* obj)  { _obj = obj; }

  IloConstraint            addLazyConstraint(IloConstraint rng);
  const IloConstraintArray addLazyConstraints(const IloConstraintArray rng);
  void                     clearLazyConstraints();

  IloConstraint            addUserCut(IloConstraint rng);
  const IloConstraintArray addUserCuts(const IloConstraintArray rng);
  void                     clearUserCuts();

   // general constraints

   Index addAbs  (const IloExtractableI* obj,
                  Index                  absvar,
                  Index                  x,
                  const char*            name=0);

   Index addMin  (const IloExtractableI*  obj,
                  Index                   minvar,
                  const IloCarray<Index>& terms,
                  double                  cnst=IloInfinity,
                  const char*             name=0);

   Index addMax  (const IloExtractableI*  obj,
                  Index                   maxvar,
                  const IloCarray<Index>& terms,
                  double                  cnst=IloInfinity,
                  const char*             name=0);

   // pwlvar = PWL(..., x);
   Index addPWL (const IloExtractableI* obj,
                 Index                  pwlvar,
                 const IloNumArray&     point,
                 const IloNumArray&     slope,
                 double                 a,
                 double                 fa,
                 Index                  x,
                 const char*            name=0);

   Index addIndicator (const IloExtractableI *obj,
                       const Index indvar,
                       double rhs, int sense,
                       const IloCarray<Index>&  lind,
                       const IloCarray<double>& lval);

   Index addSetVar (const IloIntSetVarI *obj);

   void changeSetVar (Index setvar, const IloIntSetVarI *obj);

   void getIntSetValue (const IloIntSetVarI *obj, IloIntSet set);

   Index addSetVarCard (const IloExtractableI *obj,
                        Index cardvar, Index setvar);

   Index addSetVarMember (const IloExtractableI *obj,
                        Index indvar, Index setvar, double value);

   Index addSetVarSum(const IloExtractableI *obj,
                  Index sumvar, Index setvar, IloIntArray values);

   Index addSetVarMin(const IloExtractableI *obj,
                  Index minvar, Index setvar, IloIntArray values);

   Index addSetVarMax(const IloExtractableI *obj,
                  Index maxvar, Index setvar, IloIntArray values);

   Index addSetVarSubset(const IloExtractableI *obj,
                  Index setvar1, Index setvar2, int strict);

   Index addSetVarDomain(const IloExtractableI *obj,
                  Index setvar, IloIntArray values, 
                  int possible, int required, int forbidden,
                  int strict);

   Index addSetVarUnion(const IloExtractableI *obj,
                  Index setvar, Index setvar1, Index setvar2);

   Index addSetVarIntersection(const IloExtractableI *obj,
                  Index setvar, Index setvar1, Index setvar2);

   Index addSetVarNullIntersect(const IloExtractableI *obj,
                  Index setvar1, Index setvar2);

   Index addSetVarEq(const IloExtractableI *obj,
                  Index setvar1, Index setvar2);

   Index addSetVarNEq(const IloExtractableI *obj,
                  Index setvar1, Index setvar2);

   Index addSetVarNEqCst(const IloExtractableI *obj,
                       Index setvar, IloIntArray values);

  cpxenv* getCplexEnv() { return _env; }
  cpxlp*  getCplexLp()  { return _lp; }

  IloLPExtractorManager* getExtractorManager() { return (_extractorManager); }

  void importModel(IloModel& m, const char* filename) const;
  void importModel(IloModel& m, const char* filename,
                   IloObjective& obj, IloNumVarArray vars, IloRangeArray rngs,
                   IloRangeArray  lazy,
                   IloRangeArray  cuts) const;
  void importModel(IloModel& m, const char* filename,
                   IloObjective& obj, IloNumVarArray vars, IloRangeArray rngs,
                   IloSOS1Array sos1, IloSOS2Array sos2,
                   IloRangeArray  lazy,
                   IloRangeArray  cuts) const;

  void use(IloLPExtractor* extr);
  void use(IloExprExtractor* extr);

  void use(IloCplexAbortI* abort);
  void remove(IloCplexAbortI* abort);

  IloCplex::CallbackI* use(IloCplex::CallbackI* cb);
  IloCplex::CallbackI* add(IloCplex::CallbackI* cb);
  void                 remove(IloCplex::CallbackI* cb);
  void                 clear(IloCplex::Callback::Type cbtype);

  void setMaxId(int id);
  virtual void extract(const IloExtractableI* obj);
  virtual void remove (const IloExtractableI* obj);
  virtual void apply  (const IloChange& chg, const IloExtractableI* obj);
  virtual void apply  (const IloChange& chg);
  void extract(const IloExtractableArray& o) { IloAlgorithmI::extract(o); }
  void remove (const IloExtractableArray& o) { IloAlgorithmI::remove(o); }
  virtual IloBool doExtract(const IloExtractableI* , void**);
  virtual IloBool doRemove (const IloExtractableI*, void*);
  virtual IloBool doApply  (const IloChange* chg, void** data,
                            const IloExtractableI* obj);

  // IloLPExtractor* getLPExtractor(IloInt id) const;

  void setDeleteMode(IloCplex::DeleteMode mode) { _delmode = mode; }
  IloCplex::DeleteMode getDeleteMode() const    { return (_delmode); }

  virtual IloNum getValue(const IloObjectiveI* obj) const;
  virtual IloNum getValue(const IloNumVarI*    var) const;
  virtual IloNum getValue(const IloConstraintI* con) const;
  virtual void getValues(const IloNumVarArray, IloNumArray) const;
  virtual void getValues(const IloIntVarArray, IloNumArray) const;

  IloNum getDual  (IloRangeI* con) const;
  IloNum getSlack (IloRangeI* con) const;
  IloNum getInfeasibility (IloConstraintI* con, double* rowinfeas) const;			   
  IloNum getAX    (IloRangeI* con) const;
  IloNum getReducedCost(IloNumVarI* con) const;
  IloNum getInfeasibility (IloNumVarI* var) const;
  IloCplex::BasisStatus getStatus(IloNumVarI* con) const;
  IloCplex::BasisStatus getStatus(IloRangeI*  con) const;
  IloCplex::BasisStatus getBasisStatus(IloNumVarI* con) const;
  IloCplex::BasisStatus getBasisStatus(IloConstraintI*  con) const;

  void getDuals  (IloNumArray val, const IloRangeArray con) const;
  void getSlacks (IloNumArray val, const IloRangeArray con) const;
  IloNum getInfeasibilities (IloNumArray infeas, const IloRangeArray range) const;
  IloNum getInfeasibilities (IloNumArray infeas, const IloConstraintArray con) const;
  void getAX     (IloNumArray val, const IloRangeArray con) const;
  void getReducedCosts(IloNumArray val, const IloNumVarArray con) const;
  IloNum getInfeasibilities (IloNumArray infeas, const IloNumVarArray var) const;
  void getBasisStatuses(IloCplex::BasisStatusArray stat,
                        const IloNumVarArray con) const;
  void getBasisStatuses(IloCplex::BasisStatusArray stat,
                        const IloConstraintArray  con) const;
  void getBasisStatuses(IloCplex::BasisStatusArray cstat,
                        const IloNumVarArray var,
                        IloCplex::BasisStatusArray rstat,
                        const IloConstraintArray con) const;

  void getBoundSA(IloNumArray lblower, IloNumArray lbupper,
                  IloNumArray ublower, IloNumArray ubupper,
                  const IloNumVarArray cols) const;
  void getRangeSA(IloNumArray lblower, IloNumArray lbupper,
                  IloNumArray ublower, IloNumArray ubupper,
                  const IloRangeArray rows) const;
  void getRHSSA  (IloNumArray lower, IloNumArray upper,
                  const IloRangeArray rows) const;
  void getObjSA  (IloNumArray lower, IloNumArray upper,
                  const IloNumVarArray cols) const;

#ifdef ILO_ADVANCED_CONCERT
  IloNum getAX    (IloForAllRangeI* con) const;
  IloNum getDual  (IloForAllRangeI* con) const;
  IloNum getSlack (IloForAllRangeI* con) const;
  void   getBds   (IloForAllRangeI* con, IloNum &lb, IloNum &ub) const;
  IloNum getLB    (IloForAllRangeI* con) const;
  IloNum getUB    (IloForAllRangeI* con) const;
  void getAX  (IloNumArray val, const IloForAllRangeArray con) const;
  void getDuals  (IloNumArray val, const IloForAllRangeArray con) const;
  void getSlacks (IloNumArray val, const IloForAllRangeArray con) const;
  void getForAllRanges(IloForAllRangeArray rows,
                       IloConstraintI*     forall) const;
#endif

  void setBasisStatuses(const IloCplex::BasisStatusArray cstat,
                        const IloNumVarArray var,
                        const IloCplex::BasisStatusArray rstat,
                        const IloConstraintArray con);

  void setVectors(const IloNumArray    x,
                  const IloNumArray    dj,
                  const IloNumVarArray var,
                  const IloNumArray    slack,
                  const IloNumArray    pi,
                  const IloRangeArray  rng);

  void getIIS(IloCplex::IISStatusArray& cstat, IloNumVarArray& var,
              IloCplex::IISStatusArray& rstat, IloConstraintArray& rng) const;

   IloBool fillElement (IloExtractable e, IloCarray<int>& tind,
                        IloCarray<char>& ttype) const;
   void fillGroup  (IloExtractable e, IloCarray<int>& tind,
                    IloCarray<char>& ttype,
                    IloExtractableArray tmp,
                    IloBool             fillSub) const;

  IloCarray<int> _conflictGroupId;
  IloBool refineConflict(IloConstraintArray cons, IloNumArray prios) const;
  void    getConflict(IloCplex::ConflictStatusArray& cstat,
                      IloNumVarArray&                var,
                      IloCplex::ConflictStatusArray& rstat,
                      IloConstraintArray&            rng);
  IloCplex::ConflictStatus      getConflict(IloConstraint      con ) const;
  IloCplex::ConflictStatusArray getConflict(IloConstraintArray cons) const;

  void setPriority(const IloNumVarI* var, IloNum pri);
  void setDirection (const IloNumVarI* var, IloCplex::BranchDirection dir);
  void delPriority  (const IloNumVarI* var);
  void delDirection (const IloNumVarI* var);

  void setPriorities(const IloNumVarArray& var, const IloNumArray& pri);
  void setDirections(const IloNumVarArray& var, const IloCplex::BranchDirectionArray& dir);
  void delPriorities(const IloNumVarArray& var);
  void delDirections(const IloNumVarArray& var);

  void getPriorities(const IloNumVarArray var, IloNumArray pri) const;
  void getDirections(const IloNumVarArray var, IloCplex::BranchDirectionArray dir) const;
  IloNum getPriority(IloNumVar var) const;
  IloCplex::BranchDirection getDirection (IloNumVar var) const;

  void basicPresolve(const IloNumVarArray vars,
                     IloNumArray          redlb,
                     IloNumArray          redub,
                     const IloRangeArray  rngs,
                     IloBoolArray         redundant) const;

  void getDnorms(IloNumArray& dnorms, const IloNumVarArray& vars,
                                      const IloRangeArray&  cons) const;
  void getPnorms(IloNumArray& cnorms, const IloNumVarArray& vars,
                 IloNumArray& rnorms, const IloRangeArray&  cons) const;
  void setDnorms(const IloNumArray& norms, const IloNumVarArray& vars,
                                           const IloRangeArray&  cons);
  void setPnorms(const IloNumArray& cnorms, const IloNumVarArray& vars,
                 const IloNumArray& rnorms, const IloRangeArray&  cons);

  void getRay(IloNumArray& vals, IloNumVarArray& vars) const;
  void getDriebeekPenalties(const IloExtractableArray&  ind,
                            IloNumArray&                downratio,
                            IloNumArray&                upratio) const;

  void getStrongBranch(const IloExtractableArray&  ind,
                       IloNumArray&                downratio,
                       IloNumArray&                upratio,
                       IloInt                      itlim) const;

  IloNum getQuality(IloCplex::Quality q, IloNumVar*     var=0,
                                         IloConstraint* rng=0) const;
  IloNum getQuality(IloCplex::Quality q,
                    Index& var, Index& rng, Index& qc) const;

  IloBool feasOpt(const IloRangeArray rngs,  const IloNumArray rnglb,
                                             const IloNumArray rngub,
                  const IloNumVarArray vars, const IloNumArray varlb,
                                             const IloNumArray varub);

  IloBool feasOpt(const IloRangeArray rngs,       const IloNumArray rnglb,
                                                  const IloNumArray rngub,
                  const IloForAllRangeArray rngf, const IloNumArray frnglb,
                                                  const IloNumArray frngub,
                  const IloNumVarArray vars,      const IloNumArray varlb,
                                                  const IloNumArray varub);

  IloBool feasOpt(const IloConstraintArray cts, const IloNumArray relax);

  IloExtractable getDiverging() const;

  void presolve(IloCplex::Algorithm alg);
  void freePresolve();

  void getDualNorms(const IloExprArray& expr, IloNumArray& norms,
                    IloCplex::DualPricing dpriind) const;
  void protectVariables(const IloNumVarArray& var);


  IloNum dualFarkas (IloExtractableArray& rind, IloNumArray& y);
  void qpIndefCertificate (IloExtractableArray& ind, IloNumArray& x);

  // cplex function methods stubs

  const char* getVersion() const;

  int getNcols() const { return (_nCols); }
  int getNrows() const { return (_nRows); }
  int getNQCs() const { return (_nQCs); }
  int getNSOSs() const { return (_sostype.getSize()); }
  int getNNZs () const;
  int getNintVars() const;
  int getNbinVars() const;
  int getNsemiContVars() const;
  int getNsemiIntVars() const;

  int getCplexNcols() const;
  int getCplexNrows() const;
  int getCplexNQCs() const;
  int getNiterations() const;
  int getNbarrierIterations() const;
  int getNsiftingIterations() const;
  int getNsiftingPhaseOneIterations() const;

  enum Type {
    NoType   = '-',             // don't document "NoType"
    Cont     = CPX_CONTINUOUS,
    Bin      = CPX_BINARY,
    Int      = CPX_INTEGER,
    SemiInt  = CPX_SEMIINT,
    SemiCont = CPX_SEMICONT
  };
  typedef IloArray<Type> TypeArray;

  IloNum getLb(Index col) const         { validateLb(); return (_lb[*col]); }
  IloNum getUb(Index col) const         { validateUb(); return (_ub[*col]); }
  Type   getType(Index var) const       { return ((Type)_ctype[*var]); }
  Type   getConvType(Index var) const   { return ((Type)_convtype[*var]); }
  IloNum getCoef(Index row, Index col) const;
  void   getLbs(IloNumArray& val, const IndexArray& col) const;
  void   getUbs(IloNumArray& val, const IndexArray& col) const;
  void   getType(TypeArray& type, const IndexArray& col) const ;
  void   getConvType(TypeArray& type, const IndexArray& col) const ;
  void   getRowCoefs(Index row, IloNumArray& val, const IndexArray& col) const;
  void   getRow   (Index row, IloNumArray& val, IndexArray& col) const;

  int isMIP() const;
  int isQC() const     { return (_nQCs > 0); }
  int isQP() const      { return (_hasQ); }

  int    getObjSense() const;
  IloNum getObjCoef(Index col) const;
  void   getObjCoefs(IloNumArray& obj, const IndexArray& col) const;
  void   getObj     (IloNumArray& obj, IndexArray& col) const;
  void   getLinearObj(IloNumArray& obj, IloNumVarArray col) const;
  void   getQuadraticObj(IloNumArray& obj, IloNumVarArray col1,
                                           IloNumVarArray col2) const;

  void setObjSense(int sense);
  void setObjCoef (double val, Index col);
  void setObjCoefs(const IloNumArray& obj, const IndexArray& col);
  void setObj     (int sense, const IloNumArray& obj, const IndexArray& col);
  void setObj     (int sense, const IloCarray<double>& obj,
                              const IloCarray<Index>& col);

  void copyQ(const int* beg, const int* cnt,
             const int* ind, const double* val);
  void clearQ();

  void tightenLb(Index var, IloNum bd);
  void tightenUb(Index var, IloNum bd);

  void setBounds(Index var, IloNum lb, IloNum ub);
  void setBounds(const IndexArray& col,
                 const IloNumArray&     lb,
                 const IloNumArray&     ub);

  void setType     (const IndexArray& col, const TypeArray& type);
  void setConvType (const IndexArray& col, const TypeArray& type);
  void setType     (Index var, Type type);
  void setConvType (Index var, Type type);
  void setType (Index var, IloNumVar::Type type) {
    setType(var, (type == ILOFLOAT) ? Cont : (type == ILOINT) ? Int : Bin);
  }

  IloBool setRange(Index row, double lb, double ub);
  IloBool setRange(const IndexArray& row,
                   const IloNumArray&     lb,
                   const IloNumArray&     ub);
  IloBool setQCRange(Index row, double lb, double ub);

  void setCoef (double val, Index row, Index col);
  void setRowCoefs(Index row, const IloNumArray& val, const IndexArray& col);
  void clearRow(Index row);

  void removeRow    (Index  row);
  void removeRows   (const IndexArray& rows);
  void removeColumn (Index  col);
  void removeColumns(const IndexArray& cols);
  void removeQC     (Index  qcp);
  void removeQCs    (const IndexArray& qcps);
  void fixQCs       ();
  void removeSOS    (Index sos);
  void removeSOSs   (const IndexArray& soss);
  void removeGC     (Index index, int type);

  IloBool solve();
  IloBool solveFixed();

  IloAlgorithm::Status  getStatus() const;
  IloCplex::CplexStatus getCplexStatus() const;
  IloCplex::CplexStatus getCplexSubStatus() const;
  IloBool              isPrimalFeasible() const;
  IloBool              isDualFeasible() const;
  IloCplex::Algorithm getAlgorithm() const;
  IloCplex::Algorithm getSubAlgorithm() const;


  IloNum getObjValue() const;
  IloNum getBestObjValue() const;
  IloNum getCutoff() const;
  IloInt getIncumbentNode() const;
  IloInt getNprimalSuperbasics() const;
  IloInt getNdualSuperbasics() const;
  IloInt getNphaseOneIterations() const;
  IloInt getNnodes() const;
  IloInt getNnodesLeft() const;
  IloInt getNcrossPPush() const;
  IloInt getNcrossPExch() const;
  IloInt getNcrossDPush() const;
  IloInt getNcrossDExch() const;
  IloInt getNcliques() const;
  IloInt getNcovers() const;

  IloNum getX (Index j) const {
    flush();
    if ( !_x.isValid() ) validateX();
    return (_x[*j]);
  }
  IloNum getPi(Index i) const {
    flush();
    if ( !_pi.isValid() ) validatePi();
    return (_pi[*i]);
  }
  IloNum getDj(Index j) const {
    flush();
    if ( !_dj.isValid() ) validateDj();
    return (_dj[*j]);
  }
  IloNum getSlack(Index i) const {
    flush();
    if ( !_slack.isValid() ) validateSlack();
    return (_slack[*i]);
  }
  IloNum getQCSlack(Index i) const {
    flush();
    if ( !_qcpslack.isValid() ) validateQCSlack();
    return (_qcpslack[*i]);
  }

  IloNum getRowInfeasibility(Index i) const;
  IloNum getQCInfeasibility(Index i) const;
  IloNum getSOSInfeasibility(Index i) const;
  IloNum getIndInfeasibility(Index i) const;
  IloNum getColInfeasibility(Index i) const;

  IloCplex::BasisStatus getRowStatus(Index index) const {
    flush();
    if ( !_rstat.isValid() ) validateRowStatus();
    return (_rstat[*index]);
  }
  IloCplex::BasisStatus getColumnStatus(Index index) const {
    flush();
    if ( !_cstat.isValid() ) validateColumnStatus();
    return (_cstat[*index]);
  }

  IloNum getAX(Index row) const;
  IloNum getQCAX(Index row) const;

  void deleteDnorms();
  void deletePnorms();

  const IloNumVarI*      getColumnExtractable(Index col) const;
  const IloExtractableI* getRowExtractable(Index row) const;
  const IloExtractableI* getQCExtractable(Index qcp) const;
  const IloExtractableI* getSOSExtractable(Index sos) const;

  IloBool isQuadratic(const IloConstraintI* rng, Index index) const;

  void        setDefaults();
  void        setParam(IloCplex::IntParam    parameter, IloInt value);
  void        setParam(IloCplex::BoolParam   parameter, IloBool value);
  void        setParam(IloCplex::NumParam  parameter, IloNum value);
  void        setParam(IloCplex::StringParam parameter, const char* value);

  IloInt      getParam(IloCplex::IntParam    parameter) const;
  IloBool     getParam(IloCplex::BoolParam   parameter) const;
  IloNum      getParam(IloCplex::NumParam  parameter) const;
  const char* getParam(IloCplex::StringParam parameter) const;

  IloBool     getDefault(IloCplex::BoolParam   parameter) const;
  IloInt      getDefault(IloCplex::IntParam    parameter) const;
  IloNum      getDefault(IloCplex::NumParam  parameter) const;
  const char* getDefault(IloCplex::StringParam parameter) const;
  IloInt      getMin(IloCplex::IntParam    parameter) const;
  IloNum      getMin(IloCplex::NumParam  parameter) const;
  IloInt      getMax(IloCplex::IntParam    parameter) const;
  IloNum      getMax(IloCplex::NumParam  parameter) const;

  void applyParameterSet(IloCplex::ParameterSetI* set);
  void extractParameterSet(IloCplex::ParameterSetI* set);

  void prepareNames();
  void exportModel(const char* name);
  void writeOrder (const char* name);
  void writeConflict (const char* name);
  void writeParam(const char* name);
  void writeBasis(const char* name);
  void writeVectors(const char* name);
  void writeSolution(const char* name);
  void writeTextSolution(const char* name);
  void writeMIPStart(const char* name);

  void readOrder (const char* name);
  void readParam(const char* name);
  void readBasis(const char* name);
  void readVectors(const char* name);
  void readSolution(const char* name);
  void readMIPStart(const char* name);

  void clear();

  typedef IloCplexRow    Row;
  typedef IloCplexColumn Column;
  typedef IloCplexSOS    SOS;

  Row    getRow   (Index i);
  Column getColumn(Index j);
  Index makeRow(const IloExtractableI* obj, IloNum lb, IloNum ub);
  Index makeCol(const IloExtractableI* obj, IloNum lb,
                IloNum ub, IloNumVar::Type type);
  SOS getSOS(Index i);
  Index makeSOS(const IloExtractableI* obj, IloInt type);
  Index makeGC (const IloExtractableI* obj, int type);

  Index addQC(const IloExtractableI*   obj,
               const IloCarray<double>& linVals,
               const IloCarray<Index>&  linVars,
               const IloCarray<double>& qVals,
               const IloCarray<Index>&  qVars1,
               const IloCarray<Index>&  qVars2,
               int sense, double rhs);

  // For the Goal API
  class GoalStack {
  public:
    GoalStack(IloCplex::Goal goal,
              GoalStack *prev) : _goal(goal),
                                 _prev(prev) {}

    IloCplex::Goal          _goal;
    GoalStack    *_prev;
  };

  IloBool      solve(IloCplex::Goal goal);

  CpxNodeI    *getRootNode() const { return _rootNode; }
  IloCplex::Goal getRootGoal() const { return _rootGoal; }

  IloInt getGCutsSize() const { return _globalCuts.getSize(); }
  void  addGlobalCuts(IloInt thread, IloConstraintArray rng);
  void  clearGlobalCuts(IloInt thread);
  IloConstraintArray getGlobalCuts(IloInt thread) { return _globalCuts[thread]; }

  IloCplexMutex *getGCutMutex(IloInt thread) { return _gCutMutex[thread]; }
  IloCplexMutex *getIdMutex() const { return _idMutex; }

  IloInt        getNewEvalId() { return _evalId++; }

  void         setDefaultSearch( IloBool uses ) { _usesEval = !uses; }
  IloBool      useDefaultSearch( ) { return !_usesEval; }

  IloCplex::BranchCallbackI    *getBCallback() const { return _branchCallback;}
  IloCplex::NodeCallbackI      *getNCallback() const { return _nodeCallback; }
  IloCplex::IncumbentCallbackI *getICallback() const { return _incCallback; }
  IloCplex::LazyConstraintCallbackI *getLCCallback() const {
    return _lcCallback;
  }
  // END: For Goal API
};


class IloCplexColumn {
protected:
  CPXENVptr      _env;
  CPXLPptr       _lp;
  IloCplexCache* _cache;
  int            _col;
  friend class IloCplexI;

  IloCplexColumn(IloCplexI* cplex, int j)
    : _env(cplex->_env), _lp(cplex->_lp), _cache(cplex->_cache), _col(j) {
  }

public:
  void add(IloNum val, IloInt row);
  void add(IloNum val, IloCplex::Index row);
  void add(IloCplex::Index row, IloNum val);
};


class IloCplexRow {
protected:
  CPXENVptr      _env;
  CPXLPptr       _lp;
  IloCplexCache* _cache;
  int            _row;
  friend class IloCplexI;

  void resizeNz();
  void resize();

  IloCplexRow(IloCplexI* cplex, int j)
    : _env(cplex->_env), _lp(cplex->_lp), _cache(cplex->_cache), _row(j) {
  }

public:
  void add (IloNum val, IloInt col);
  void add (IloNum val, IloCplex::Index col);
  void add (IloCplex::Index col, IloNum val);
};


class IloCplexSOS {
  friend class IloCplexI;
  friend class IloFastCplexSOS;

  IloCplexI*  _cplex;
  IloSVector* _sos;

  IloCplexSOS(IloCplexI* cplex, IloCplex::Index j);
  IloCplexSOS(IloCplexI* cplex, IloInt type);

public:
  IloCplexSOS(const IloCplexSOS& sos);
  ~IloCplexSOS();
  IloNum getMax() const;
  void add (IloNum val, IloCplex::Index col);
  void add (IloCplex::Index col, IloNum val) { add (val, col); }
};


/////////////////////////////////////////////////////////////////////////
//
// inline implementations
//

inline IloCplexI::Column
IloCplexI::getColumn(Index j) {
  return (IloCplexI::Column(this, *j));
}

inline IloCplexI::Row
IloCplexI::getRow(Index i) {
  return (IloCplexI::Row(this, *i));
}

inline IloCplexI::SOS
IloCplexI::getSOS(Index i) {
  return (IloCplexI::SOS(this, i));
}


/////////////////////////////////////////////////////////////////////////
//
// inline implementations for IloCplex
//

inline const char* IloCplex::getVersion() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getVersion());
}

inline void
IloCplex::flush() {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->flush();
}

inline IloBool
IloCplex::solveFixed() {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->solveFixed());
}

inline IloBool
IloCplex::isMIP() const {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->flush();
  return (getImpl()->isMIP());
}

inline IloBool
IloCplex::isQC() const {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->flush();
  return (getImpl()->isQC());
}

inline IloBool
IloCplex::isQuadratic() const {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->flush();
  return (getImpl()->isQP());
}

inline IloBool
IloCplex::isQO() const {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->flush();
  return (getImpl()->isQP());
}

inline void
IloCplex::setDeleteMode(IloCplex::DeleteMode mode) {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->setDeleteMode(mode);
}

inline IloCplex::DeleteMode
IloCplex::getDeleteMode() const {
  IloAssert(getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getDeleteMode());
}

inline IloNum
IloCplex::getBestObjValue() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getBestObjValue());
}

inline IloNum
IloCplex::getCutoff() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getCutoff());
}

inline IloInt
IloCplex::getIncumbentNode() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getIncumbentNode());
}

inline IloInt
IloCplex::getNprimalSuperbasics() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNprimalSuperbasics());
}

inline IloInt
IloCplex::getNdualSuperbasics() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNdualSuperbasics());
}

inline IloInt
IloCplex::getNphaseOneIterations() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNphaseOneIterations());
}

inline IloInt
IloCplex::getNnodes() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNnodes());
}

inline IloInt
IloCplex::getNnodesLeft() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNnodesLeft());
}

inline IloInt
IloCplex::getNcrossPPush() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNcrossPPush());
}

inline IloInt
IloCplex::getNcrossPExch() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNcrossPExch());
}

inline IloInt
IloCplex::getNcrossDPush() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNcrossDPush());
}

inline IloInt
IloCplex::getNcrossDExch() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNcrossDExch());
}

inline IloInt
IloCplex::getNcliques() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNcliques());
}

inline IloInt
IloCplex::getNcovers() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNcovers());
}

inline IloInt   IloCplex::getNiterations() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNiterations());
}

inline IloInt   IloCplex::getNbarrierIterations() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNbarrierIterations());
}

inline IloInt   IloCplex::getNsiftingIterations() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNsiftingIterations());
}

inline IloInt   IloCplex::getNsiftingPhaseOneIterations() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNsiftingPhaseOneIterations());
}

inline IloInt   IloCplex::getNcols() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNcols());
}

inline IloInt   IloCplex::getNrows() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNrows());
}

inline IloInt   IloCplex::getNQCs() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNQCs());
}

inline IloInt IloCplex::getNintVars() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNintVars());
}

inline IloInt IloCplex::getNbinVars() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNbinVars());
}

inline IloInt IloCplex::getNsemiContVars() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNsemiContVars());
}

inline IloInt IloCplex::getNsemiIntVars() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNsemiIntVars());
}

inline IloInt   IloCplex::getNNz() const {
  return (getNNZs());
}

inline IloInt   IloCplex::getNNZs() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNNZs());
}

inline IloInt   IloCplex::getNSOSs() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getNSOSs());
}

inline IloInt   IloCplex::getNSOS() const {
  return (getNSOSs());
}

inline void IloCplex::setDefaults() {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->setDefaults();
}

inline void IloCplex::setParam(IntParam parameter, IloInt value) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->setParam(parameter, value);
}

inline void IloCplex::setParam(BoolParam parameter, IloBool value) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->setParam(parameter, value);
}

inline void IloCplex::setParam(NumParam parameter, IloNum value) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->setParam(parameter, value);
}

inline void IloCplex::setParam(StringParam parameter, const char* value) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->setParam(parameter, value);
}

inline IloInt IloCplex::getParam(IntParam parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getParam(parameter));
}

inline IloBool IloCplex::getParam(BoolParam parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getParam(parameter));
}

inline IloNum IloCplex::getParam(NumParam parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getParam(parameter));
}

inline const char* IloCplex::getParam(StringParam parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getParam(parameter));
}

inline IloBool     IloCplex::getDefault(BoolParam   parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getDefault(parameter));
}
inline IloInt      IloCplex::getDefault(IntParam    parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getDefault(parameter));
}
inline IloNum      IloCplex::getDefault(NumParam  parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getDefault(parameter));
}
inline const char* IloCplex::getDefault(StringParam parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getDefault(parameter));
}
inline IloInt      IloCplex::getMin(IntParam    parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getMin(parameter));
}
inline IloNum      IloCplex::getMin(NumParam  parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getMin(parameter));
}
inline IloInt      IloCplex::getMax(IntParam    parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getMax(parameter));
}
inline IloNum      IloCplex::getMax(NumParam  parameter) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getMax(parameter));
}



inline IloCplex::Algorithm IloCplex::getRootAlgorithm() const {
  return (IloCplex::Algorithm)getParam(IloCplex::RootAlg);
}

inline IloCplex::Algorithm IloCplex::getNodeAlgorithm() const {
  return (IloCplex::Algorithm)getParam(IloCplex::NodeAlg);
}

inline void IloCplex::setRootAlgorithm(Algorithm alg) {
  setParam(IloCplex::RootAlg, alg);
}

inline void IloCplex::setNodeAlgorithm(Algorithm alg) {
  setParam(IloCplex::NodeAlg, alg);
}

inline void IloCplex::exportModel(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->exportModel(name);
}

inline void IloCplex::writeOrder(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeOrder(name);
}

inline void IloCplex::writeConflict(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeConflict(name);
}

inline void IloCplex::writeParam(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeParam(name);
}

inline void IloCplex::writeBasis(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeBasis(name);
}

inline void IloCplex::writeSolution(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeSolution(name);
}

inline void IloCplex::writeVectors(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeVectors(name);
}

inline void IloCplex::writeTextSolution(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeTextSolution(name);
}

inline void IloCplex::writeMIPStart(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->writeMIPStart(name);
}

inline void IloCplex::readOrder(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->readOrder(name);
}

inline void IloCplex::readParam(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->readParam(name);
}

inline void IloCplex::readBasis(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->readBasis(name);
}

inline void IloCplex::readSolution(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->readSolution(name);
}

inline void IloCplex::readVectors(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->readVectors(name);
}

inline void IloCplex::readMIPStart(const char* name) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->readMIPStart(name);
}

inline IloAlgorithm::Status
IloCplex::getStatus() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getStatus());
}

inline IloCplex::CplexStatus    
IloCplex::getCplexStatus() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getCplexStatus());
}

inline IloCplex::CplexStatus    
IloCplex::getCplexSubStatus() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getCplexSubStatus());
}

inline IloBool
IloCplex::isPrimalFeasible() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->isPrimalFeasible());
}

inline IloBool
IloCplex::isDualFeasible() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->isPrimalFeasible());
}

inline IloCplex::Algorithm    
IloCplex::getAlgorithm() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getAlgorithm());
}

inline IloCplex::Algorithm    
IloCplex::getSubAlgorithm() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getSubAlgorithm());
}

inline void
IloCplex::deleteDnorms() {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->deleteDnorms();
}

inline void
IloCplex::deletePnorms() {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->deletePnorms();
}

inline IloConstraint
IloCplex::addUserCut(IloConstraint rng) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->addUserCut(rng));
}

inline const IloConstraintArray
IloCplex::addUserCuts(const IloConstraintArray rng) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  IloAssert (rng.getImpl(), "array not initialized");
  return (getImpl()->addUserCuts(rng));
}

inline void
IloCplex::clearUserCuts() {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->clearUserCuts();
}

inline IloConstraint
IloCplex::addCut(IloConstraint rng) {
  return (addUserCut(rng));
}

inline const IloConstraintArray
IloCplex::addCuts(const IloConstraintArray rng) {
  return (addUserCuts(rng));
}

inline void
IloCplex::clearCuts() {
  clearUserCuts();
}


inline IloConstraint
IloCplex::addLazyConstraint(IloConstraint rng) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->addLazyConstraint(rng));
}

inline const IloConstraintArray
IloCplex::addLazyConstraints(const IloConstraintArray rng) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  IloAssert (rng.getImpl(), "array not initialized");
  return (getImpl()->addLazyConstraints(rng));
}

inline void
IloCplex::clearLazyConstraints() {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->clearLazyConstraints();
}

inline void
IloCplex::importModel(IloModel& m, const char* filename) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->importModel(m, filename);
}

inline void
IloCplex::importModel(IloModel& m, const char* filename,
                           IloObjective& obj,
                           IloNumVarArray vars, IloRangeArray  rngs,
                           IloRangeArray  lazy,
                           IloRangeArray  cuts) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->importModel(m, filename, obj, vars, rngs, lazy, cuts);
}

inline void
IloCplex::importModel(IloModel& m, const char* filename,
                           IloObjective& obj,
                           IloNumVarArray vars, IloRangeArray rngs,
                           IloSOS1Array sos1, IloSOS2Array sos2,
                           IloRangeArray  lazy,
                           IloRangeArray  cuts) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->importModel(m, filename, obj, vars, rngs, sos1, sos2, lazy, cuts);
}

inline IloExtractable
IloCplex::getDiverging() const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  return (getImpl()->getDiverging());
}

inline void
IloCplex::getDualNorms(IloNumArray norms, const IloExprArray expr,
                            IloCplex::DualPricing dpriind) const {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  IloAssert (expr.getImpl(), "array not initialized");
  IloAssert (norms.getImpl(), "array not initialized");
  getImpl()->getDualNorms(expr, norms, dpriind);
}

inline IloCplex::Callback
IloCplex::use(Callback cb) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  IloAssert (cb.getImpl(), "accessing callback through 0 handle");
  return (getImpl()->use(cb.getImpl()));
}

inline IloCplex::Callback
IloCplex::add(Callback cb) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  IloAssert (cb.getImpl(), "accessing callback through 0 handle");
  return (getImpl()->add(cb.getImpl()));
}

inline void
IloCplex::remove(Callback cb) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  IloAssert (cb.getImpl(), "accessing callback through 0 handle");
  getImpl()->remove(cb.getImpl());
}

inline void
IloCplex::clear(Callback::Type cbtype) {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->clear(cbtype);
}

inline void
IloCplex::clearModel() {
  IloAssert (getImpl(), "accessing IloCplex through 0 handle");
  getImpl()->clear();
}

inline IloBool
IloCplex::solve(Goal goal) {
  return getImpl()->solve(goal);
}

class CpxEmptyDeleterI : public IloBaseDeleterI {
  IloBool _remove;
public:
  CpxEmptyDeleterI(IloEnvI *env, IloBool remove = IloFalse);
  virtual void addToBeExtracted(const IloExtractable extractable) {}
  virtual void inhibit(IloBool b) {}
  virtual void end(IloExtractableI* object, IloBool);
  virtual void end(const IloExtractableArray objects, IloBool);
  virtual ~CpxEmptyDeleterI() {}
  virtual void setVerbosityLevel(IloInt l) {}
  virtual void apply(const IloChange& chg, const IloExtractableI* object) {}

  virtual void removeFromAll(IloInt id) {}
  virtual void removeToBeExtracted(const IloExtractable extractable)    {}
  virtual void fastEnd(IloExtractableI* object) { end(object, IloFalse); }
  virtual void addLink(IloExtractableI* super, IloExtractableI* sub)    {}
  virtual void removeLink(IloExtractableI* super, IloExtractableI* sub) {}
  virtual void addLink(IloExtractableI* super, IloExtractableArray sub) {}
  virtual void removeLink(IloExtractableI* super, IloExtractableArray sub) {}
  virtual IloBool hasLink(IloExtractableI* user, IloExtractableI* object) {
    return IloFalse;
  }
  virtual IloBool sendFullChange() const        { return IloFalse; }
  virtual void setSendFullChange(IloBool c)     {}
  virtual void setSendFullChange()      {}

protected:
  IloEnvI* _env;
};

/////////////////////////////////////
// class CpxNodeI
/////////////////////////////////////

class CpxNodeI : public IloCplex::MIPCallbackI::NodeData {
public:
  class GoalStack {
  public:
    GoalStack(IloCplex::Goal goal,
                 GoalStack *prev) : _goal(goal),
                                       _prev(prev) {}

    IloCplex::Goal   _goal;
    GoalStack       *_prev;
  };

  class LimitStack {
  public:
    LimitStack(IloCplex::SearchLimit limit,
                  LimitStack *prev) : _limit(limit),
                                         _prev(prev) {}

    IloCplex::SearchLimit   _limit;
    LimitStack   *_prev;
  };

  class EvaluatorList {
  public:
    EvaluatorList(IloCplex::NodeEvaluator eval,
                  EvaluatorList          *next) : _eval(eval),
                                                  _value(0),
                                                  _next(next) {
    }

    IloCplex::NodeEvaluator  _eval;
    IloNum                   _value;
    EvaluatorList           *_next;
  };


  CpxNodeI(IloCplexI *cplex, IloNum ext=IloInfinity);
  virtual ~CpxNodeI();

  IloInt     getNNewNodes() const { return _nNewNodes; }
  CpxNodeI  *addNewNode(IloNum est);
  CpxNodeI **getNewNodes()  const { return _newNodes; }

  void solve();

  IloNum              getEstimate() const { return _est; }
  IloConstraintArray  getCuts() const { return _ranges; }
  IloConstraintArray  getGlobalCuts() const { return _globalRanges; }
  void                initCuts(IloBool create);
  void                addCut(IloConstraint rng) { _ranges.add(rng); }
  void                addCuts(IloConstraintArray rng) { _ranges.add(rng); }
  void                addGlobalCut(IloConstraint rng) { _globalRanges.add(rng); }
  void                addGlobalCuts(IloConstraintArray rng) {
    _globalRanges.add(rng);
  }

  void performBranch();

  void        delBranchInfo();
  void        prepareToBranch();

  // Related with the Goal Stack:
  IloCplex::Goal popGoalStack();
  void addGoalStack(IloCplex::Goal goal);
  void copyGoalStack(CpxNodeI *node);

  // Related with the Limit Stack:
  IloCplex::SearchLimit popLimitStack();
  void addLimitStack(IloCplex::SearchLimit limit);
  void copyLimitStack(CpxNodeI *node);
  IloBool checkLimits();

  void setNodeCallback(IloCplex::NodeCallbackI *cb) { _ndCallback = cb; }
  void setBranchCallback(IloCplex::BranchCallbackI *cb) { _brCallback = cb; }
  void setIncumbentCallback(IloCplex::IncumbentCallbackI *cb) {
    _incCallback = cb;
  }

  IloCplex::NodeCallbackI      *getNodeCallback() const { return _ndCallback; }
  IloCplex::BranchCallbackI    *getBranchCallback() const {return _brCallback;}
  IloCplex::IncumbentCallbackI *getIncumbentCallback() const {
    return _incCallback;
  }

  IloCplexI *getCplex() const { return _cplex; }
  IloEnv     getEnv() const { return _cplex->getEnv(); }

  CpxNodeI *makeBranch(IloCplex::Goal goal, IloNum est=IloInfinity);

  IloBool   isFail() const { return _fail; }
  void      setFail(IloBool fail) { _fail = fail; }

  // Methods to get the information
  void      increaseLDepth() { _lDepth++; _depth++; }
  void      increaseRDepth() { _depth++; }
  IloInt    getLeftDepth() const { return _lDepth; }
  IloInt    getRightDepth() const { return _depth - _lDepth; }
  IloInt    getDepth() const { return _depth; }

  IloAny getDataType() const;
  IloBool hasGoals() const { return _goalStack != 0; }

  IloCplexMutex *getMutex() const { return _mutex; }
  void evaluate(IloInt nodeId);
  IloBool subsume(CpxNodeI *node);
  IloBool isPhathomed() const { return _phathomed; }

  // For the evaluators.
  void cleanEvaluators();
  void copyEvaluators(CpxNodeI *node);
  void addEvaluator(IloCplex::NodeEvaluator newEval);

  // Duplication
  CpxNodeI *duplicate();

  // Feasibility checker
  IloBool checkFeasibility(IloConstraintI* con, IloNum eps) const;
  IloBool checkFeasibility(IloNum eps) const;

protected:
  IloCplexI     *_cplex;
  IloCplexMutex *_mutex;

  GoalStack                  *_goalStack;
  LimitStack                 *_limitStack;
  EvaluatorList              *_evaluators;
  EvaluatorList              *_toEvaluate;

  IloNum                     _est;
  IloInt                     _nSons;

  IloBool                    _phathomed;

  IloConstraintArray         _ranges;
  IloConstraintArray         _globalRanges;

  CpxNodeI                  **_newNodes;
  IloInt                      _nNewNodes;

  IloBool                     _fail;

  IloCplex::NodeCallbackI      *_ndCallback;
  IloCplex::BranchCallbackI    *_brCallback;
  IloCplex::IncumbentCallbackI *_incCallback;

  // For the limits
  IloInt _depth;
  IloInt _lDepth;

  // Ordering
  void sort(IloNumVarArray vars, IloIntArray index,
            IloInt start, IloInt end);
};


inline IloInt
IloCplex::Node::getLeftDepth() const {
  return _impl->getLeftDepth();
}

inline IloInt
IloCplex::Node::getRightDepth() const {
  return _impl->getRightDepth();
}

inline IloInt
IloCplex::Node::getDepth() const {
  return _impl->getDepth();
}

inline IloInt
IloCplex::EvaluatorI::getLeftDepth() const {
  return _node->getLeftDepth();
}

inline IloInt
IloCplex::EvaluatorI::getRightDepth() const {
  return _node->getRightDepth();
}

inline IloNum
IloCplex::EvaluatorI::getDepth()  const {
  return (IloNum)_node->getDepth();
}

inline IloCplex::NodeCallbackI
*IloCplex::NodeEvaluatorI::getCallback() const {
  return _node->getNodeCallback();
}

inline IloObjective
IloCplex::getObjective() const {
   return (IloObjective((IloObjectiveI*)(getImpl()->getObjective())));
}

#endif
