// --------------------------------------------------------------------------
//  Copyright (C) 1990-2006 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef _ILOLPEXTRMGR_H
#define _ILOLPEXTRMGR_H

#include <ilconcert/ilomodel.h>
#include <ilconcert/iloalgi.h>
#include <ilcplex/ilocplexi.h>

class IloCpxExtractor;
class IloLPExtractor;
class IloExprExtractor;

// --------------------------------------------------------------------------
//
// IloLPExtractorManager
//
// This class manages the list of extractors for IloCplex.
//
#ifdef ILO_IZMIR
class IloLabelCallbackI;
#endif
class IloLPExtractorManager : public IloPlugIn {
  ILORTTIDECL

  typedef IloCplexI::Data Data;

  IloCplexI*          _alg;
#ifdef ILO_IZMIR
  IloLabelCallbackI* _labelCB;
#endif
  IloExtractableArray _added;    // added during extraction
  IloExtractableArray _failed;   // extractables for which extraction failed

  struct ExList {
     IloCpxExtractor* _extr;
     Data*            _data;
     int              _usecnt;
     ExList(IloCpxExtractor*e=0) : _extr(e), _data(0), _usecnt(0) {}
     void clear() {
        _extr   = 0;
        _data   = 0;
        _usecnt = 0;
     }
  };

  IloLPExtractor*  _first;
  IloArray<ExList> _which;

  IloExprExtractor*  _firstExpr;
  IloArray<ExList>   _whichExpr;

  IloBool _recordExtraction;

  void setMaxID (int maxId);
#ifdef ILO_IZMIR
  void checkLabelCB();
#endif

  Data* makeData (void* data, int firstadded, int lastadded);
  void  freeData (void* data, IloBool remove=IloTrue);

  void popConData (IloInt id, IloExtractableArray added);
  void pushConData (IloInt id, IloExtractableArray added, void *data = 0);
  void pushConData (IloInt id, int firstadded, int lastadded, void *data =0);
  void moveConData (const IloExtractableI* from, const IloExtractableI* to);

  void getSubExtractables(IloExtractable e, IloExtractableArray cons) const;

  IloCplexI::Index doMakeIndex  (const IloNumExprI* expr);
  IloBool          doRemoveIndex(const IloNumExprI* expr);

  IloBool doExtract(const IloExtractableI* obj);
  IloBool doRemove (const IloExtractableI* obj);

public:
  IloLPExtractorManager(IloCplexI* alg);
  ~IloLPExtractorManager();

  IloCplexI* getCplex() const   { return (_alg); }
  IloEnvI*   getEnv() const     { return (_alg->getEnv()); }

  IloBool isAccounting() const  { return _recordExtraction; }
  void startAccounting()        { _recordExtraction = IloTrue; }
  void stopAccounting ()        { _recordExtraction = IloFalse; }

  void clear();
  void setMaxId(int maxid);

  IloExtractable add(IloExtractable e);
  IloExtractableArray add(IloExtractableArray e);
  void endAdded();

  IloInt getConNadded(int id) const;
  IloInt getConNadded(const IloExtractableI *obj) const {
     return getConNadded(obj->getId());
  }
  IloInt getConNadded(IloExtractable obj) const {
     return getConNadded(obj.getImpl());
  }
  IloExtractable getConAdded(int id, int n) const;
  IloExtractable getConAdded(const IloExtractableI* obj, int n) const {
     return (getConAdded(obj->getId(), n));
  }
  IloExtractable getConAdded(IloExtractable obj, int n) const {
     return (getConAdded(obj.getImpl(), n));
  }
  void setConAdded(const IloExtractableI* obj, int n, IloExtractable set);
  void setConAdded(IloExtractable obj, int n, IloExtractable set) {
     setConAdded(obj.getImpl(), n, set);
  }
  IloExtractable getConLastAdded(IloExtractable obj);

  IloCplexI::Index getConIndex (int id) const {
     return ((id >= 0 && id < _which.getSize() && _which[id]._data != 0)
             ? _which[id]._data->index : 0);
  }
  IloCplexI::Index getConIndex (const IloExtractableI* extr) const {
     return (getConIndex (extr->getId()));
  }
  IloCplexI::Index getConIndex (const IloExtractable extr) const {
     return (getConIndex (extr.getId()));
  }
  void setConIndex (const IloExtractableI* extr, IloCplexI::Index index) const {
     _which[extr->getId()]._data->index = index;
  }

  IloInt getVarNadded(int id) const;
  IloInt getVarNadded(const IloExtractableI* obj) const {
     return (getVarNadded(obj->getId()));
  }
  IloInt getVarNadded(const IloExtractable obj) const {
     return (getVarNadded(obj.getImpl()));
  }

  IloExtractable getVarAdded(int id, int n) const;
  IloExtractable getVarAdded(const IloExtractableI* obj, int n) const {
     return (getVarAdded(obj->getId(), n));
  }
  IloExtractable getVarAdded(IloExtractable obj, int n) const {
     return (getVarAdded(obj.getImpl(), n));
  }
  void setVarAdded(const IloExtractableI* obj, int n, IloExtractable set);
  void setVarAdded(IloExtractable obj, int n, IloExtractable set) {
     setVarAdded(obj.getImpl(), n, set);
  }
  IloExtractable getVarLastAdded(IloExtractable obj) const;

  IloCplexI::Index getVarIndex(int id) const {
     return (_whichExpr[id]._data != 0 ? _whichExpr[id]._data->index : 0);
  }
  IloCplexI::Index getVarIndex(const IloNumExprI* expr) const {
     return getVarIndex(expr->getId());
  }
  IloCplexI::Index getVarIndex(const IloNumExpr expr) const {
     return getVarIndex(expr.getId());
  }

  IloCplexI::Index makeIndex(const IloNumExprI* expr) {
     int id = expr->getId();
     return (_whichExpr[id]._data != 0  &&  ++_whichExpr[id]._usecnt)
             ? _whichExpr[id]._data->index
             : doMakeIndex (expr);
  }
  IloCplexI::Index makeIndex(const IloNumExpr expr) {
     return makeIndex (expr.getImpl());
  }

   IloNumVar makeNumVar (const IloNumExprI* expr) {
      IloCplexI::Index index = makeIndex(expr);
      return (index
              ? IloNumVar((IloNumVarI*)_alg->getColumnExtractable(index))
              : IloNumVar(0)                                             );
   }


  IloBool removeIndex (const IloNumExprI* expr) {
     return (--_whichExpr[expr->getId()]._usecnt < 1) ? doRemoveIndex (expr)
                                                      : IloTrue;
  }
  IloBool removeIndex (const IloNumExpr expr) {
     return removeIndex (expr.getImpl());
  }

  void addExtractor   (IloExprExtractor* e);
  void removeExtractor(IloExprExtractor* e);

  void addExtractor   (IloLPExtractor* e);
  void removeExtractor(IloLPExtractor* e);

  IloBool extract(const IloExtractableI* obj) {
     return ((++_which[obj->getId()]._usecnt > 1) ? IloTrue : doExtract(obj));
  }
  IloBool remove (const IloExtractableI* obj) {
     return ((--_which[obj->getId()]._usecnt > 0) ? IloTrue : doRemove(obj));
  }
  void apply (const IloChange& chg, const IloExtractableI* obj);
  void apply (const IloChange& chg);

  IloBool isExtracted (IloExtractableI* obj) const;
  IloBool isExprExtracted (IloNumExprI* obj) const;


  IloBool hasExtracted (IloInt id, IloLPExtractor* extr) const {
     return (id >= 0  &&  _which[id]._extr == (IloCpxExtractor*)extr);
  }
  IloBool hasExtracted (IloInt id, IloExprExtractor* extr) const {
     return (id >= 0  &&  _whichExpr[id]._extr == (IloCpxExtractor*)extr);
  }
  void unuse     (int id);
  void unuseExpr (int id);

  IloBool change (const IloChange* chg);
  IloBool change (IloInt id, const IloChange* chg);

  IloBool isConsistent() const;
  IloBool isConsistent(IloLPExtractorManager* copy) const;
};

#endif
